<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2007, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

/**
 * Application class is a main class of the system
 *
 * @package Core
 * @author Alexey Florinsky
 * @access  public
 */
class Application
{
//------------------------------------------------
//               PUBLIC DECLARATION
//------------------------------------------------

    /**#@+
     * @access public
     */

    /**
     * Application constructor.
     */
    function Application()
    {
        $this->pFactory = &new Factory();
        $this->appIni = array();
        $this->TagFatalErrors = array();
        $this->TagWarnings = array();
        $this->protocol = (isset($_SERVER["HTTPS"]) && (strtolower($_SERVER["HTTPS"]) == "on" || $_SERVER["HTTPS"] == 1 || $_SERVER["HTTPS"] === true))? "https":"http";
    }


    /**
     * Initialization.
     */
    function init()
    {
        # Create an object and connect to a database.
        $this->db = &$this->getInstance( 'DB_MySQL' );
        $this->db->DB_Connect();

        # Create a session class and start with the current session.
        $session = &$this->getInstance( 'Session' );
        $request = &$this->getInstance( 'Request' );
        $request->importHTTPRequestData();
        $sid = ($request->getValueByKey('CZSESSID')? $request->getValueByKey('CZSESSID'):($request->getValueByKey('AZSESSID')? $request->getValueByKey('AZSESSID'):""));
        $session->start($sid);

        #                                    !
        #                 GET   POST       ,                                         .
        #                   (20      2007),                                                    .
        $request->restoreGETPOSTData();

        # Parse a current query string for further usage, and
        # POST data are parsed in Request::parseQueryString().
        $request->prepareGetPostData();

        # ModuleManager - has already been loaded, an object can be created.
        # MM will load all the main module classes and create their objects, after
        # that getInstance([ModuleName]) can be called for any module.
        # Besides, MM must load an action file, based on the information
        # from Request. Warning: don't create its object and not to direct
        # the control to it, this will be done later in Application::ProcessAction().
        # View files will be loaded in Application::output(), if necessary,
        # that will increase productivity.
        $mm = &$this->getInstance( 'Modules_Manager' );
        $mm->initModules(); # to load and create objects of all modules

        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] After Modules_Manager::initModules()');

        #Load a resource file of system messages
        $this->MessageResources = &$this->getInstance('MessageResources');

        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] System Messages Loaded');

        # register information tags
        $this->registerTags();

        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] After Application::registerTags()');

        global $zone;
        if ($zone == 'CustomerZone')
        {
            log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Customer Zone: Starting Layout & Configs Loading');
            # check tags
            $res = LayoutConfigurationManager::static_checkLayoutFile();
            //                        -        -               .
            if(!empty($res["MAIN_ERROR_PARAMETERS"]))
            {
                $_fatal_params = array();
                //                                            _fatal
                $_fatal_params[] = $res["MAIN_ERROR_PARAMETERS"];
                unset($res["MAIN_ERROR_PARAMETERS"]);
                foreach($res as $value)
                {
                    $_fatal_params[] = $value;
                }

                call_user_func_array("_fatal", $_fatal_params);
            }


            //Copy 3 3ini of the ini-file to the temporary folder convert them
            //  to the old format.
            $this->ini_copy_3_ini_and_convert_them_to_old_ini_format_3ini();

            # read the configuration file layouts-config.ini
            $this->readLayoutsINI();

            log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Customer Zone: Done');
        }

        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End');
    }

    /**
     * Adds prefix to the field names in the meta description of the module
     * database list.
     *
     * @author Alexandr Girin
     * @todo
     * @param reference $tables the reference to the array of the meta
     * description list
     * @return the changed array of the meta discription
     */
    function addTablePrefix(&$tables)
    {
        $table_prefix = $this->getAppIni("DB_TABLE_PREFIX");
        if ($table_prefix == NULL)
        {
            return $tables;
        }
        foreach ($tables as $table => $table_structure)
        {
            foreach ($tables[$table]['columns'] as $col_name => $column)
            {
                $tables[$table]['columns'][$col_name] = $table_prefix.$column;
            }
        }
        return $tables;
    }

    /**
     * Output the view by name.
     * $Viewname is the first required parameter. It is the name of the view
     * class, which the method output() will be invoked. After the $Viewname
     * parameter the unlimitted number of parameters can follow. All of them
     * will be passed in to the method output() being invoked at
     * the view class.
     *
     * An example of using:
     * <code>
     * $html = $application->output( 'Catalog', $par1, $par2);
     *
     * $html = modApiFunc('application', 'output', 'Catalog', $par1, $par2);
     * </code>
     *
     * @param string $Viewname The name of view class
     * @return generated HTML code of view
     */
    function output( $Viewname )
    {
        # Get a list of the passed parameters
        $arg_list = func_get_args();

        # Remove the first parameter from the list
        array_shift($arg_list);

        # To get an object from the representation name, enable the Modules_Manager
        # object. It is the only one that has metainformation
        # about modules.
        $mm = &$this->getInstance( 'Modules_Manager' );
        $viewObj = &$mm->getViewObject( $Viewname );

        $HTMLCode = call_user_func_array( array( &$viewObj, 'output' ), $arg_list);
        return $HTMLCode;
    }


    /**
     * Gets a pointer to object.
     *
     * It takes an unlimited number of parameters.
     * The required parameter is the clas name. Others will be sent to
     * the constructor "as is".
     *
     * @param string $Classname Name of class
     * @return &object Pointer to object of the class
     */
    function &getInstance($Classname)
    {
        // parameters for the constructor
        $_args_list = func_get_args();
        array_shift($_args_list);

        // if the file of the class hasn't been loaded yet, then call
        // Modules Manager to load the required file
        if ( !class_exists($Classname) )
        {
            $mm = &$this->getInstance('Modules_Manager');
            $mm->includeAPIFileOnce($Classname);
        }

        // if the object has the loadState method, then call it
        // only if the object was created
        $obj = &$this->pFactory->getInstance($Classname, $_args_list, 'loadState');

        return $obj;
    }


    /**
     * Saves the state of all loaded system modules/classes.
     * It calls the function saveState for each of the module, if
     * such function is declared.
     */
    function saveStateofInstancedClasses()
    {
    	$this->pFactory->callMethodToAllObjects('saveState');
    }


    /**
     * Calls the controller to the process current action.
     * It has to use the Request object to read action and other parameters
     * from Query String.
     */
    function processAction()
    {
        global $application;
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Starting');

        $request = $this->getInstance( 'Request' );
        $asc_actionName = $request->getCurrentAction();

        if ($asc_actionName === NULL)
        {
            $asc_actionName = "ActionIsNotSetAction";
        }

        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Action Name = '.$asc_actionName);

        # To get an object from the asc_action'  name, enable the Modules_Manager
        # object. It's the only one that has metainformation
        # about modules.
        $mm = &$this->getInstance('Modules_Manager');
        $asc_actionObj = &$mm->getActionObject($asc_actionName);

        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Action Object Created');

        if (is_object($asc_actionObj))
        {
            $this->processActionStarted = true;
            # Pass the control to the selected action.
            $asc_actionObj->onAction();

            log_debug('['.__CLASS__.'::'.__FUNCTION__.'] onAction Done');

            # Query MM about the list of hook-classes names of the current action,
            # create their objects and direct the control.
            $hooksList = $mm->getHooksClassesList($asc_actionName);
            if ($hooksList)
            {
                foreach ($hooksList as $hookClass)
                {
                    $hookObj = $this->getInstance( $hookClass );
                    $hookObj->onHook($asc_actionObj);
                    log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Hook "'.$hookClass.'" Done');
                }
            }

            // Save the module state (only for loaded modules)
            $this->saveStateofInstancedClasses();

            if ($this->haveToExit)
            {
                log_debug('['.__CLASS__.'::'.__FUNCTION__.'] EXIT');
                exit;
            }

            $this->processActionStarted = false;
            if ($this->redirectURL)
            {
                log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Redirect');
                $request = new Request($this->redirectURL);
                $application->redirect($request);
            }

            if ($this->js_redirectURL)
            {
                log_debug('['.__CLASS__.'::'.__FUNCTION__.'] jsRedirect');
                $js_redirect = new Request($this->js_redirectURL);
                $application->jsRedirect($js_redirect);
            }
        }

        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End');
    }

      function isImageFileValid($filename)
      {
                      $full_filename = $this->getAppIni('PATH_IMAGES_DIR').$filename;
                      return (@getimagesize($full_filename) != FALSE);
      }

    /**
     * Method for recalculation the height and the width of the image.
     *
     * @todo the method is not finished. Return an empty image, if the file
     * doesn't exist
     * @param string $filename the image filename
     * @param int $max_img_height max image heght
     * @param int $max_img_width max image width
     * @return array image size - the structure of the array matches the array
     * returned PHP by the getimagesize() function
     * @todo !!! Delete this method
     */
    function img_size($filename, $max_img_height=50, $max_img_width=50)
    {
        if ($image_file=@fopen($filename, "r"))
        {
            fclose($image_file);
            $img_size = getimagesize($filename);
            if ($img_size[0] > $max_img_width)
            {
                $kof = $max_img_width/$img_size[0];
                $img_size[0] = $max_img_width;
                $img_size[1] = $kof*$img_size[1];
            }
            if ($img_size[1] > $max_img_height)
            {
                $kof = $max_img_height/$img_size[1];
                $img_size[1] = $max_img_height;
                $img_size[0] = $kof*$img_size[0];
            }
            $img_size[3] = 'width="'.$img_size[0].'" height="'.$img_size[1].'"';
        }
        else
        {
            $img_size = array($max_img_width, $max_img_height, '3', 'width="'.$max_img_width.'" height="'.$max_img_height.'"', '8', 'image/png');
        }
        return $img_size;
    }

    /**
     * Defines the possibility of uploading images by file type.
     *
     * @param $file The array consists of the $_FILES variable, for
     * the current file.
     * @return boolean
     */
    function isAllowedImageType($file)
    {
/*        $type = strtolower($file['type']);
        switch ($type)
        {
            case 'image/gif':
            case 'image/jpeg':
            case 'image/jpg':
            case 'image/jpe':
            case 'image/jfif':
            case 'image/pjpeg':
            case 'image/pjp':
            case 'image/png':
            case 'image/x-png':
                return true;
            default:
                return false;
        }
*/
        return true;
    }

    /**
     * Defines the extension of the image file of the file type to be used in the system.
     *
     * @param $file - The array consists of the $_FILES variable, for
     * the current file.
     * @return mixed false - this image type can't be used in the system.
     */
    function getImageTypeExtension($file)
    {
        $type = strtolower($file['type']);
        $ext = false;
        switch ($type)
        {
            case 'image/gif':
                $ext = "gif";
                break;
            case 'image/jpeg':
            case 'image/jpg':
            case 'image/jpe':
            case 'image/jfif':
            case 'image/pjpeg':
            case 'image/pjp':
                $ext = "jpg";
                break;
            case 'image/png':
            case 'image/x-png':
                $ext = "png";
                break;
            default:
                $ext = false;
                break;
        }
        return $ext;
    }

    function _img_doesnot_exist_and_creatable($basename)
    {
        $path = $this->_img_path($basename);
        if(file_exists($path))
        {
            return false;
        }
        else
        {
            $fh = fopen($path, "w");
            if($fh === FALSE)
            {
                return false;
            }
            else
            {
                fclose($fh);
                if(@unlink($path) === FALSE)
                {
                    return false;
                }
                else
                {
                    return true;
                }
            }
        }

    }

    function _img_path($basename)
    {
        return $this->getAppIni('PATH_IMAGES_DIR') . $basename;
    }

    function _img_exists($basename)
    {
        global $application;
        return file_exists($this->_img_path($basename));
    }

    /**
     * Defines a unique filename and creates it for temporary storing in
     * the image catalog.
     *
     * @param $prefix string the filename prefix.
     * @param $postfix string the file extension.
     * @return mixed false - the file can't be created.
     */
    function getUploadImageName($orig_path)
    {
        $path_parts = pathinfo($orig_path);
        $basename = $path_parts['basename'];
        $ext = $path_parts['extension'];

        //                                  
        //http://tools.ietf.org/html/rfc1630
        $safe = "a-zA-Z0-9\$\-\_\@\.\&";
        $basename = preg_replace("/[^" . $safe. "]/","", $basename);

        $basename_without_ext = substr($basename, 0, -1 * (strlen("." . $ext)));
        if($basename_without_ext == "")
        {
        	$basename_without_ext = "image";
        }
        $basename = $basename_without_ext . "." . $ext;

        if(!$this->_img_exists($basename))
        {
            if($this->_img_doesnot_exist_and_creatable($basename))
            {
                return $this->_img_path($basename);
            }
            else
    {
            return false;
        }
        }
        else
        {
        // find a temporary name
            $tries = 0;
        do {
            // get a known, unique temporary file name
                $new_name = $basename_without_ext . "_" . $tries . "." . $ext;
                if(!$this->_img_exists($new_name))
                {
                    if($this->_img_doesnot_exist_and_creatable($new_name))
                    {
                        return $this->_img_path($new_name);
                    }
                    else
                    {
                return false;
            }
            }
                $tries++;
            } while (true);
            }
    }

    /**
     * Saves the temporary image file to the catalog as a permanent one.
     * If the names match, appends a number "_XXXXXXXXX" to the end of the filename.
     *
     * @param $prefix string the permanent filename prefix.
     * @param $tmp_file string the temporary filename.
     * @return Array (['name'], ['width'], ['height'])
     */
    function saveUploadImage($prefix, $tmp_file)
    {
        $dir = $this->getAppIni('PATH_IMAGES_DIR');

        if(is_file($dir . $tmp_file))
        {
            $image_info = getimagesize($dir . $tmp_file);
        return array(
                'name' => $tmp_file,
            'width' => $image_info[0],
            'height' => $image_info[1]
        );
    }
        else
        {
            return array(
                'name' => '',
                'width' => 0,
                'height' => 0
            );
        };
    }


    /**
     * Returns the section name based on the current filename.
     */
    function getSectionByCurrentPagename()
    {
        $zone = modApiFunc('Users', 'getZone');

        $sections = array();
        if ($zone == "CustomerZone")
        {
            foreach ($this->Configs_array["Layouts"] as $section => $params)
            {
                if (strtoupper($section) == "SITE" || strtoupper($section) == "TEMPLATES")
                {
                    continue;
                }
                foreach ($params as $key => $val)
                {
                    if (strtoupper($key)!="HTTPS" && $val != "")
                    {
                        if (!(strpos($_SERVER["PHP_SELF"], $val) === false))
                        {
                            $sections[] = $section;
                        }
                    }
                }
            }
        }
        elseif ($zone == "AdminZone")
        {
            $current_file = basename($_SERVER["PHP_SELF"]);
            switch ($current_file)
            {
                case "signin.php":
                case "signin_password_update.php":
                case "signin_password_recovery.php":
                case "admin_members.php":
                case "admin_member_info.php":
                case "admin_member_add.php":
                case "admin_member_edit.php":
                case "admin_member_passwd_reset.php":
                case "admin_member_delete.php":
                    $sections[] = "SignIn_AdminMembers";
                    break;
                case "orders.php":
                case "checkout_delete_orders.php":
                case "orders_info.php":
                case "customers.php":
                case "customers_info.php":
                    $sections[] = "Orders_Customers";
                    break;
                case "payment_modules.php":
                case "checkout_payment_module_settings.php":
                case "shipping_modules.php":
                case "checkout_shipping_module_settings.php":
                case "scc_settings.php":
                case "shipping_tester_window.php":
                    $sections[] = "Payment_Shipping";
                    break;
                default:
                    break;
            }
            if ($this->appIni["AllAdminArea"])
            {

            }
        }

        return $sections;
    }

    /**
     * Returns the protocol, by which the section must be loaded.
     */
    function getSectionProtocol($section)
    {
        $section = strtoupper($section);
        $layout = array_change_key_case($this->Configs_array["Layouts"], CASE_UPPER);
        foreach ($layout as $key => $sectionInfo)
        {
            $layout[$key] = array_change_key_case($sectionInfo, CASE_UPPER);
        }
        if (isset($this->appIni['SITE_HTTPS_URL']) && isset($layout[$section]["HTTPS"]))
        {
            if (strtolower($layout[$section]["HTTPS"]) == "yes" ||
                           $layout[$section]["HTTPS"] == 1)
            {
                return "https";
            }
        }
        return "http";
    }

    /**
     * Returns the protocol, by which the current page is loaded.
     */
    function getCurrentProtocol()
    {
        return $this->protocol;
    }

    /**
     * Redirects to another protocol, if necessary.
     * The method is called in 'admin.php' and 'store.php', after
     * executing the current action.
     *
     *                                              ,                   
     *                             action' .
     */
    function redirectToAnotherProtocol()
    {
        $sections = $this->getSectionByCurrentPagename();

        $zone = modApiFunc('Users', 'getZone');

        if ($zone == "CustomerZone")
        {
            /*
                                                                  storefront

                ----------------------------------------------------------------
                |                   |                                          |
                |                   |------------------------------------------|
                |                   |     HTTP           |     HTTPS           |
                ----------------------------------------------------------------
                |                   |                    |                     |
                |                   |                    |                 HTTP|
                |layout.ini         |                    |                     |
                ----------------------------------------------------------------
                |                   |                    |                     |
                |                   |                    |                 HTTP|
                |layout.ini         |                    |                     |
                |           HTTP    |                    |                     |
                ----------------------------------------------------------------
                |                   |                    |                     |
                |                   |                    |                     |
                |layout.ini         |        HTTPS       |                     |
                |           HTTPS   |                    |                     |
                ----------------------------------------------------------------
                |                   |                    |                     |
                |                   |                    |                     |
                |layout.ini         |        HTTPS       |                     |
                |      HTTPS,       |                    |                     |
                |      HTTP         |                    |                     |
                |                   |                    |                     |
                ----------------------------------------------------------------

            */


            /*
                                      HTTPS                          layout      
                           ,                       HTTP.
                 . .     "       "         ,                           HTTP.
            */
            if (!sizeof($sections) && $this->protocol == "https")
            {
                $request = &new Request();
                $request->setView(CURRENT_REQUEST_URL);
                $request->setKey(session_name(), session_id());
                $request->saveCurrentGETPOSTData();
                $this->redirect($request, "http");
            }

            $redirect_to_http_protocol = true;

            /*
                                           ,                          .
            */
            foreach ($sections as $section)
            {
                /*
                                          HTTP,                                         
                                              HTTPS -                       HTTPS.
                     . .           ,                                                   
                            HTTPS -                                     .
                */
                if ($this->getSectionProtocol($section) == "https" && $this->protocol == "http")
                {
                    $redirect_to_http_protocol = false;
                    $request = &new Request();
                    $request->setView($this->getViewBySection($section));
                    $request->setKey(session_name(), session_id());
                    $request->saveCurrentGETPOSTData();
                    $this->redirect($request, "https");
                }


                /*
                                          HTTPS,                                     
                                          HTTP           -                       HTTP.
                                   ,                                                     
                                     HTTPS           -                      ,          
                    HTTPS         .
                */
                if ($this->getSectionProtocol($section) == "https" && $this->protocol == "https")
                {
                    $redirect_to_http_protocol = false;
                }
            }

            if ($redirect_to_http_protocol && $this->protocol == "https")
            {
                $request = &new Request();
                $request->setView($this->getViewBySection($sections[0]));
                $request->setKey(session_name(), session_id());
                $request->saveCurrentGETPOSTData();
                $this->redirect($request, "http");
            }
        }
        elseif ($zone == "AdminZone")
        {
            if ($this->appIni["AllAdminArea"])
            {
                if ($this->protocol == "http")
                {
                    $request = &new Request();
                    $request->setView(CURRENT_REQUEST_URL);
                    $request->setKey(session_name(), session_id());
                    $request->saveCurrentGETPOSTData();
                    $this->redirect($request, "https");
                }
            }
            elseif (!sizeof($sections) && $this->protocol == "https")
            {
                $request = &new Request();
                $request->setView(CURRENT_REQUEST_URL);
                $request->setKey(session_name(), session_id());
                $request->saveCurrentGETPOSTData();
                $this->redirect($request, "http");
            }
            elseif (isset($sections[0]))
            {
                if ($this->appIni[$sections[0]] && $this->protocol == "http")
                {
                    $request = &new Request();
                    $request->setView(CURRENT_REQUEST_URL);
                    $request->setKey(session_name(), session_id());
                    $request->saveCurrentGETPOSTData();
                    $this->redirect($request, "https");
                }
                if (!$this->appIni[$sections[0]] && $this->protocol == "https")
                {
                    $request = &new Request();
                    $request->setView(CURRENT_REQUEST_URL);
                    $request->setKey(session_name(), session_id());
                    $request->saveCurrentGETPOSTData();
                    $this->redirect($request, "http");
                }
            }

        }
    }

    /**
     * Returns the filename (a web site page) for the specified view.
     *
     * @todo method isn't finished. There must be a selection from the database.
     * What kind of module must work on it?
     */
    function getPagenameByViewname($viewName, $category = -1, $product = -1, $zone = '')
    {
        $file = '';
        if ($zone == '')
        {
            $zone = modApiFunc('Users', 'getZone');
        }

        if ($zone == "CustomerZone")
        {
            return $this->getViewLayoutPage($viewName, $category, $product);
        }
        switch ($viewName)
        {
            case 'AdminSignIn':
                $file = "signin.php";
                break;
            case 'AdminZoneBlocked':
                $file = "signin_blocked.php";
                break;
            case 'AdminPasswordUpdate':
                $file = "signin_password_update.php";
                break;
            case 'AdminPasswordRecovery':
                $file = "signin_password_recovery.php";
                break;
            # Commented by AF:
            # I think, it should be removed in the release
            case 'Maximize':
                $file = "maximize.html";
                break;
            case 'HomeTab':
                $file = "index.php";
                break;
            case 'NavigationBar':
                $file = "catalog_manage_categories.php";
                break;
            case 'CustomerZoneBlocked':
                $file = "customer_blocked.php";
                break;
            case 'ProductList':
                $file = "catalog_manage_products.php";
                break;
            case 'AddCategory':
                $file = "catalog_add_category.php";
                break;
            case 'MoveCategory':
                $file = "catalog_move_category.php";
                break;
            case 'DeleteCategory':
                $file = "catalog_del_category.php";
                break;
            case 'EditCategory':
                $file = "catalog_edit_category.php";
                break;
            case 'ViewCategory':
                $file = "catalog_view_category.php";
                break;
            case 'SortCategories':
                $file = "catalog_sort_category.php";
                break;
            case 'Catalog_AddProduct':
                $file = "catalog_addproduct.php";
                break;
            case 'Catalog_EditProduct':
                $file = "catalog_editproduct.php";
                break;
            case 'MoveProducts':
                $file = "catalog_move_product.php";
                break;
            case 'CopyProducts':
                $file = "catalog_copy_product.php";
                break;
            case 'DeleteProducts':
                $file = "catalog_delete_products.php";
                break;
            case 'SortProducts':
                $file = "catalog_sort_products.php";
                break;
            case 'Catalog_ProdInfo':
                $file = "catalog_product.php";
                break;
            case 'SelectProductType':
                $file = "catalog_select_type.php";
                break;
            case 'ManageProductTypes':
                $file = "catalog_manage_product_types.php";
                break;
            case 'AddProductType':
                $file = "catalog_new_product_type.php";
                break;
            case 'EditProductType':
                $file = "catalog_edit_product_type.php";
                break;
            case 'EditCustomAttribute':
                $file = "catalog_edit_custom_attribute.php";
                break;
            case 'DeleteProductType':
                $file = "catalog_delete_product_type.php";
                break;
            case 'CheckoutPaymentModulesList':
                $file = "payment_modules.php";
                break;

            case 'CheckoutInfoList':
                $file = "checkout_info_list.php";
                break;
            case 'CheckoutInfoSortGroup':
                $file = "checkout-info-sort-group.php";
                break;
            case 'CheckoutInfoAttributeEdit':
                $file = "checkout-info-attribute-edit.php";
                break;

            case 'CheckoutPaymentModuleSettings':
                $file = "checkout_payment_module_settings.php";
                break;

            case 'CheckoutShippingModulesList':
                $file = "shipping_modules.php";
                break;
            case 'CheckoutShippingModuleSettings':
                $file = "checkout_shipping_module_settings.php";
                break;

            case 'CountriesList':
                $file = "store_settings_countries.php";
                break;
            case 'StatesList':
                $file = "store_settings_states.php";
                break;

            case 'PHPInfo':
                $file = 'admin_php_info.php';
                break;

            case 'AdminMembersList':
                $file = 'admin_members.php';
                break;
            case 'AdminMemberInfo':
                $file = 'admin_member_info.php';
                break;
            case 'AdminMemberAdd':
                $file = 'admin_member_add.php';
                break;
            case 'AdminMemberEdit':
                $file = 'admin_member_edit.php';
                break;
            case 'AdminPasswordChange':
            case 'AdminMemberPasswordReset':
                $file = 'admin_member_passwd_reset.php';
                break;
            case 'AdminMemberDelete':
                $file = 'admin_member_delete.php';
                break;

            case 'DateTimeFormat':
                $file = 'store_settings_local_date.php';
                break;

            case 'NumberFormat':
                $file = 'store_settings_local_number.php';
                break;
            case 'CurrencyFormat':
                $file = 'store_settings_local_currency.php';
                break;
            case 'WeightUnit':
                $file = 'store_settings_local_weight.php';
                break;

            case 'ProductTaxClassSettings':
                $file = 'store_settings_prod_tax_classes.php';
                break;

            case 'TaxSettings':
                $file = 'store_settings_taxes.php';
                break;

            case 'AddTaxName':
                $file = 'store_settings_tax_add_name.php';
                break;

            case 'EditTaxName':
                $file = 'store_settings_tax_edit_name.php';
                break;

            case 'AddTaxDisplayOption':
                $file = 'store_settings_tax_add_display.php';
                break;

            case 'EditTaxDisplayOption':
                $file = 'store_settings_tax_edit_display.php';
                break;

            case 'AddTaxClass':
                $file = 'store_settings_tax_add_class.php';
                break;

            case 'EditTaxClass':
                $file = 'store_settings_tax_edit_class.php';
                break;

            case 'AddTaxRate':
                $file = 'store_settings_tax_add_rate.php';
                break;

            case 'EditTaxRate':
                $file = 'store_settings_tax_edit_rate.php';
                break;

            case 'TaxCalculator':
                $file = 'store_settings_tax_calculator.php';
                break;

            case 'Hint':
                $file = 'hint.php';
                break;

            case 'ShippingTaxes':
                $file = 'store_settings_tax_edit_shipping_modules.php';
                break;

            case 'Orders':
                $file = 'orders.php';
                break;

            case 'DeleteOrders':
                $file = 'checkout_delete_orders.php';
                break;

            case 'OrderInfo':
                $file = 'orders_info.php';
                break;

            case 'OrderInvoice':
                $file = 'orders_invoice.php';
                break;

            case 'OrderPackingSlip':
                $file = 'orders_packing_slip.php';
                break;

            case 'Customers':
                $file = 'customers.php';
                break;

            case 'CustomerInfo':
                $file = 'customers_info.php';
                break;

            case 'AdminBackup':
                $file = 'admin_backup.php';
                break;

            case 'AdminBackupInfo':
                $file = 'admin_backup_info.php';
                break;

            case 'AdminBackupCreate':
                $file = 'admin_backup_create.php';
                break;

            case 'AdminBackupProgress':
                $file = 'admin_backup_progress.php';
                break;

            case 'AdminBackupDelete':
                $file = 'admin_backup_delete.php';
                break;

            case 'AdminBackupDeleteProgress':
                $file = 'admin_backup_delete_progress.php';
                break;

            case 'AdminRestoreProgress':
                $file = 'admin_restore_progress.php';
                break;

            case 'Downdoad':
                $file = 'download.php';
                break;

            case 'AdminBackupRestore':
                $file = 'admin_backup_restore.php';
                break;

            case 'Notifications':
                $file = 'store_settings_notifications.php';
                break;

            case 'NotificationInfo':
                $file = 'store_settings_notific_info.php';
                break;

            case 'SearchResult':
                $file = 'catalog_search_result.php';
                break;

            case 'SystemUpdate':
                $file = 'update.php';
                break;

            case 'SystemUpdateProgress':
                $file = 'update_progress.php';
                break;

            case 'SystemUpdateProgressDownloadFile':
                $file = 'download_update_file_start.php';
                break;

            case 'LicenseInfo':
                $file = 'license_info.php';
                break;

            case 'HTTPSSettings':
                $file = 'https_settings.php';
                break;

            // begin vews for ProductOptions module
            case 'PO_OptionsList':
                $file = 'po_options_list.php';
                break;

            case 'PO_AddOption':
                $file = 'po_new_option.php';
                break;

            case 'PO_EditOption':
                $file = 'po_edit_option.php';
                break;

            case 'PO_EditExs':
                $file = 'po_edit_exs.php';
                break;

            case 'PO_CRulesEditor':
                $file = 'po_crules_editor.php';
                break;

            case 'PO_InvEditor':
                $file = 'po_inv_editor.php';
                break;
            // end views for ProductOptions module

            case 'CSVImportProducts':
                $file = "csv_import_products.php";
                break;

            case 'CSVImportNextStepValidation':
                $file = "csv_import_product_next_step_validation.php";
                break;

            case 'CSVImportStartValidation':
                $file = "csv_import_product_validation.php";
                break;

            case 'AdminCSVExport':
                $file = "csv_export_products_exporting.php";
                break;

            case 'AdminCSVInfo':
                $file = 'admin_csv_info.php';
                break;

            case 'AdminCSVDelete':
                $file = 'admin_csv_delete.php';
                break;

            case 'AdminCSVDeleteProgress':
                $file = 'admin_csv_delete_progress.php';
                break;

            case 'CSVExportSetParamsExporting':
                $file = "csv_export_set_params.php";
                break;

            case 'discounts_manage_global_discounts_az':
                $file = "marketing_manage_discounts.php";
                break;

            case 'AddPromoCode':
                $file = "marketing_add_promo_code.php";
                break;

            case 'EditPromoCode':
                $file = "marketing_edit_promo_code.php";
                break;

            case 'PromoCodesNavigationBar':
                $file = "marketing_manage_promo_codes.php";
                break;

            //views for Product Files module
            case 'PF_FilesList':
                $file = 'pf_files_list.php';
                break;
            //end views for Product Files module

            //views for Product Images module
            case 'PI_ImagesList':
                $file = 'pi_images_list.php';
                break;

            case 'manage_quantity_discounts_az':
                $file = 'manage_quantity_discounts_az.php';
                break;
            //end views for Product Files module

            case 'MngProductCats':
                $file = 'mng_product_cats.php';
                break;

            case 'RegisterFormEditor':
                $file = 'register_form_editor.php';
                break;

            case 'CustomersList':
                $file = 'customers.php';
                break;

            case 'PopupWindow':
                $file = 'popup_window.php';
                break;

            case 'CreditCardSettings':
                $file = 'credit_cards_editor.php';
                break;

            case 'SortCreditCardTypes':
                $file = 'sort_credit_card_types.php';
                break;

            # Commented by AF: Is the page CartContent really contained in the Admin Zone??
            #case 'CartContent':
            #    $file = "cart.php";
            #    break;

            default:
                $file = 'index.php';
                break;
        }
        return $file;
    }

    /**
     * Determines the dependence between a block tag and a filename, which contains
     * this tag. Depending on the passed parameters $category and $product and
     * on the settings in layouts-config.ini, replaces the file with the
     * proper configurations.
     */
    function getViewLayoutPage($viewName, $category = -1, $product = -1)
    {
        $section = "";
        $section = $this->getSectionByViewName($viewName);
        if ($section == 'UNDEFINED')
            return 'index.php';

        if ($product != -1 && array_key_exists('products', $this->layout[$section]))
        {
            if (array_key_exists($product, $this->layout[$section]['products']))
            {
                return $this->layout[$section]['products'][$product];
            }
        }
        if ($category != -1 && array_key_exists('categories', $this->layout[$section]))
        {
            if (array_key_exists($category, $this->layout[$section]['categories']))
            {
                return $this->layout[$section]['categories'][$category];
            }
        }
        return $this->layout[$section]['default'];
    }

    function getSectionByViewName($viewName)
    {
        switch ($viewName)
        {
            case 'NavigationBar':
            case 'Subcategories':
            case 'ProductList':
                $section = 'productlist';
                break;

            case 'ProductInfo':
                $section = 'productinfo';
                break;

            case 'CartContent':
                $section = 'cart';
                break;

            case 'CheckoutView':
                $section = 'checkout';
                break;

            case 'Closed':
                $section = 'closed';
                break;

            case 'SearchResult':
                $section = 'searchresult';
                break;

            case 'Download':
                $section = 'download';
                break;

            case 'Registration':
                $section = 'registration';
                break;

            case 'AccountActivation':
                $section = 'accountactivation';
                break;

            case 'CustomerPersonalInfo':
                $section = 'customerpersonalinfo';
                break;

            case 'CustomerOrdersHistory':
                $section = 'customerordershistory';
                break;

            case 'CustomerOrderInfo':
                $section = 'customerorderinfo';
                break;

            case 'CustomerOrderInvoice':
                $section = 'customerorderinvoice';
                break;

            case 'CustomerOrderDownloadLinks':
                $section = 'customerorderdownloadlinks';
                break;

            case 'CustomerSignIn':
                $section = 'customersignin';
                break;

            case 'CustomerNewPassword':
                $section = 'customernewpassword';
                break;

            case 'CustomerChangePassword':
                $section = 'customerchangepassword';
                break;

            case 'CustomerForgotPassword':
                $section = 'customerforgotpassword';
                break;

            case 'CustomerAccountHome':
                $section = 'customeraccounthome';
                break;

            default:
                $section = 'UNDEFINED';
                break;
        }
        return $section;
    }

    /**
     * Determines the dependence between a filename and a block tag.
     * The inverse function to Application::getViewLayoutPage()
     */
    function getViewBySection($section)
    {
        $view = "";
        switch (strtolower($section))
        {
            case 'productlist':
                $view = 'ProductList';
                break;

            case 'productinfo':
                $view = 'ProductInfo';
                break;

            case 'cart':
                $view = 'CartContent';
                break;

            case 'checkout':
                $view = 'CheckoutView';
                break;

            case 'closed':
                $view = 'Closed';
                break;

            case 'searchresult':
                $view = 'SearchResult';
                break;

            case 'download':
                $view = 'Download';
                break;

            case 'registration':
                $view = 'Registration';
                break;

            case 'accountactivation':
                $view = 'AccountActivation';
                break;

            case 'customerpersonalinfo':
                $view = 'CustomerPersonalInfo';
                break;

            case 'customerordershistory':
                $view = 'CustomerOrdersHistory';
                break;

            case 'customerorderinfo':
                $view = 'CustomerOrderInfo';
                break;

            case 'customerorderinvoice':
                $view = 'CustomerOrderInvoice';
                break;

            case 'customerorderdownloadlinks':
                $view = 'CustomerOrderDownloadLinks';
                break;

            case 'customersignin':
                $view = 'CustomerSignIn';
                break;

            case 'customernewpassword':
                $view = 'CustomerNewPassword';
                break;

            case 'customerchangepassword':
                $view = 'CustomerChangePassword';
                break;

            case 'customerforgotpassword':
                $view = 'CustomerForgotPassword';
                break;

            case 'customeraccounthome':
                $view = 'CustomerAccountHome';
                break;

            default:
                return 'Index';
                break;
        }
        return $view;
    }

    /**
     * Activates redirect if necessary.
     *
     * @todo finish the functions on this page
     */
    function redirect($request, $force_redirect_to = "")
    {
        $url = $request->getURL($force_redirect_to);

        if ($this->processActionStarted)
        {
            $this->redirectURL = $url;
        }
        elseif ($url)
        {
            Header("Location: " . $url);
            exit;
        }
    }

    function jsRedirect($request, $force_redirect_to = "")
    {
        $url = $request->getURL($force_redirect_to);

        if ($this->processActionStarted)
        {
            $this->js_redirectURL = $url;
        }
        elseif ($url)
        {
//            Header("Location: " . $url);
            echo str_replace("%NEW_LOCATION%",$url,file_get_contents('js_redirect.html'));
            exit;
        }
    }

    function _exit()
    {
        $this->haveToExit = true;
    }

    function closeChild_UpdateParent()
    {
        echo file_get_contents("close_child_update_parent.html");
        exit;
    }

    /**
     * Reads system settings from config.php.
     *
     *
     * All paths are returned with the slash at the end.
     *
     * The parameter values can be got by calling Application::getAppIni($key).
     * An example:
     * <code>
     *  $core_path = $application->getAppIni('PATH_MODULES_DIR');
     *  $db_server = $application->getAppIni('DB_SERVER');
     *  $db_user   = $application->getAppIni('DB_USER');
     *  $db_paswd  = $application->getAppIni('DB_PASSWORD');
     * </code>
     *
     * @see Application::getAppIni()
     */
    function readAppINI()
    {
   	    global $zone;
        # Predefined Application directories and files
        $_web_root = dirname( dirname( dirname(__FILE__) ) );
        $_web_root = strtr($_web_root,'\\','/');

        $this->appIni["PATH_ASC_ROOT"]         = $_web_root.'/';
        # Path to license file
        $this->appIni['PATH_LICENSE_KEY_FILE'] = $_web_root.'/avactis-system/license.key';

        $this->appIni['PATH_CORE_DIR']         = $_web_root.'/avactis-system/core/';
        $this->appIni['PATH_MODULES_DIR']      = $_web_root.'/avactis-system/modules/';
        $this->appIni['PATH_CACHE_DIR']        = $_web_root.'/avactis-system/cache/';
        $this->appIni['PATH_SYSTEM_DIR']       = $_web_root.'/avactis-system/';
        $this->appIni['PATH_LAYOUTS_DIR']      = $_web_root.'/avactis-layouts/';
        $this->appIni['PATH_BACKUP_DIR']       = $_web_root.'/avactis-system/backup/';
        $this->appIni['PATH_EXPORT_DIR']       = $_web_root.'/avactis-exports/';
        $this->appIni['PRODUCT_FILES_DIR']     = $_web_root.'/avactis-downloads/';

        $this->appIni['PATH_ADMIN_RESOURCES']  = $_web_root.'/avactis-system/admin/templates/resources/';
        $this->appIni['PATH_ADMIN_TPLS_VIEWS'] = $_web_root.'/avactis-system/admin/templates/modules/';

        # Path to TAGS file
        # TODO: should be deprecated
        $this->appIni['PATH_TAGS_FILE']        = $_web_root.'/avactis-system/tags';

        # Path to temporary config files
        # TODO: should be deprecated
        $this->appIni['PATH_BLOCK_CONFIG']     = $_web_root.'/avactis-system/admin/blocks_ini/';

        # Path to update files
        # TODO: Should be changed
        $this->appIni['PATH_UPDATES_DIR']      = $_web_root.'/updates/';

        $this->appIni['PATH_CONFIG_FILE']      = $_web_root.'/avactis-system/config.php';

        # Reading system settings
        $cfg_file = $this->appIni['PATH_CONFIG_FILE'];
        if ( is_readable($cfg_file) )
        {
            $ini_array = @_parse_ini_file( $cfg_file );
            $this->appIni = array_merge( $this->appIni,  $ini_array);
            if (file_exists($this->appIni['PATH_SYSTEM_DIR']."restore"))
            {
                $this->appIni['DB_TABLE_PREFIX'] = "restore_".$this->appIni['DB_TABLE_PREFIX'];
            }
        }
        else
        {
            _fatal( array( "CODE"      => "CORE_030",
                           "FILE"      => $cfg_file   ) );
        }

        $this->appIni['PATH_HTTPS_CONFIG_FILE']      = $_web_root.'/avactis-system/https_config.php';

        if (!isset($this->appIni["HTTP_URL"]) && isset($this->appIni["WWWADDRESS"]))
        {
            $this->appIni["HTTP_URL"] = $this->appIni["WWWADDRESS"];
        }
        $http_url = $this->appIni["HTTP_URL"];
        # Reading system settings
        $https_cfg_file = $this->appIni['PATH_HTTPS_CONFIG_FILE'];
        if ( is_readable($https_cfg_file) )
        {
            $ini_array = @_parse_ini_file( $https_cfg_file );
            $this->appIni = array_merge( $this->appIni,  $ini_array);
        }
        else
        {
            $this->appIni["HTTPS_URL"] = $this->appIni["HTTP_URL"];
            $this->appIni["AllAdminArea"] = "";
            $this->appIni["SignIn_AdminMembers"] = "";
            $this->appIni["Orders_Customers"] = "";
            $this->appIni["Payment_Shipping"] = "";
        }

        # The variable $layout_file_path must be defined in Page Init String
        global $layout_file_path;

        if (!isset($layout_file_path))
        {
            if($zone != 'AdminZone')
            {
                _fatal( array( "CODE" => "CORE_056") );
            }
        }

        #            $layout_file_path                                 :
        # 1)                 $layout_file_path                          ,
        #                                                       avactis-layouts.
        # 2)                 $layout_file_path                             ,
        #                                  .
        #
        # NOTE:                 $layout_file_path                             ,
        #                             ,                                        
        #                   avactis-layouts         .                      
        #       ,                        .

        #                                      avactis-layouts,
        #                                            ,                  
        #                    .
        #             ,              1.5.3           1.6.0             
        #             layout        avactis-layouts,           
        # storefront/init.php                                  layout      .
        $new_layout_file_path = $this->appIni['PATH_LAYOUTS_DIR'] . basename($layout_file_path);
        if (file_exists($new_layout_file_path) && is_readable($new_layout_file_path) && is_file($new_layout_file_path))
        {
            $layout_file_path = $new_layout_file_path;
        }
        #              avactis-layouts                ,                  
        #                   layout                    1.5.3
        elseif (file_exists($layout_file_path) && is_readable($layout_file_path) && is_file($layout_file_path))
        {
            #                          ,                      
            #            $layout_file_path                
        }
        else
        {
            if($zone != 'AdminZone')
            {
                _fatal( array( "CODE" => "CORE_056") );
            }
        }

        $layout_config_info = LayoutConfigurationManager::static_parse_layout_config_file($layout_file_path);
        //                        -        -               .
        if(!empty($layout_config_info["MAIN_ERROR_PARAMETERS"]))
        {
        	$_fatal_params = array();
            //                                            _fatal
            $_fatal_params[] = $layout_config_info["MAIN_ERROR_PARAMETERS"];
            unset($layout_config_info["MAIN_ERROR_PARAMETERS"]);
            foreach($layout_config_info as $value)
            {
            	$_fatal_params[] = $value;
            }

        	call_user_func_array("_fatal", $_fatal_params);
        }
        else
        {
        	//Merge parse results with appIni
            $this->appIni = array_merge($this->appIni, $layout_config_info);
        }

        # Path to Installer
        $this->appIni['INSTALLER_FILE_DAT'] = $_web_root . '/install.dat';
        $this->appIni['INSTALLER_FILE_PHP'] = $_web_root . '/install.php';

        # Path and URL to images directory
        $this->appIni['PATH_IMAGES_DIR'] = $_web_root.'/avactis-images/';
        $this->appIni['URL_IMAGES_DIR']  = $http_url.'avactis-images/';
        if (isset($this->appIni['HTTPS_URL']))
        {
            $this->appIni['HTTPS_URL_IMAGES_DIR']  = $this->appIni['HTTPS_URL'].'avactis-images/';
        }

        $this->appIni['SHOW_NO_PICTURE_IMAGE'] = 'DISABLED';
        $this->appIni['PATH_NO_PICTURE_IMAGE'] = $this->appIni['URL_IMAGES_DIR'].'no_picture.gif';
        $this->appIni['NO_PICTURE_IMAGE']      = 'no_picture.gif';

        /**
         * URL to AZ like http[s]://domain/path/
         *  E.g. http://www.avactis.com/avactis-system/admin/
         */
        $this->appIni['SITE_AZ_URL'] = $this->appIni['HTTP_URL'].'avactis-system/admin/';
        if (isset($this->appIni['HTTPS_URL']))
        {
            $this->appIni['SITE_AZ_HTTPS_URL']  = $this->appIni['HTTPS_URL'].'avactis-system/admin/';
        }

        #
        # Read config.def.php
        #
        $this->appIni['PATH_LOCAL_CONFIG_FILE'] = $_web_root.'/avactis-system/config.def.php';
        if (file_exists($this->appIni['PATH_LOCAL_CONFIG_FILE']) && is_readable($this->appIni['PATH_LOCAL_CONFIG_FILE']))
        {
            $ini_array = @_parse_ini_file( $this->appIni['PATH_LOCAL_CONFIG_FILE'] );
            $this->appIni = array_merge( $this->appIni,  $ini_array);
        }
    }

    /**
     * Checks whether the ini file syntax is correct, in comparison with regular
     * expressions.
     *
     * @author Alexander Girin
     * @todo it might be necessary to make changes in defining regular
     * expressions
     *
     * @param string $ini_file_name - ini filename
     * @return array - the array of errors found in the format:
     *                      Array (
     *                             Array ("OBJ_XXX" => "line_num"),
     *                             ...
     *                            )
     *                            )
     */
    function isIniFileCorrect($ini_file_name)
    {
        $Errors = array();
        $fp = fopen($ini_file_name, "r");
        $str_num = 0;
        while (! feof($fp))
        {
            $str = trim(fgets($fp));
            $str_num++;
            //If the string is a comment, then it is skipped
            if (empty($str) || $str[0] == ";")
            {
                continue;
            }
            //If there is character ";" in the string, everything has left after it
            //is truncated
            if ($pos = strpos($str, ";"))
            {
                $str = substr($str, 0, $pos)."\n";
            }
            //Check whether the rest of the strings match regular expressions
            //If the first string character is "[", then check whether the
            //the section declaration is valid
            if (!(($pos = strpos($str, "[")) === false))
            {
                # the name of the section must be well-defined
                if (!preg_match('/^\[[a-z]+[a-z0-9]*\]/i', $str))
                {
                    array_push($Errors, array("CORE_020" => $str_num));      // Mismatching with the regular expression of the
                                                                             // [<Section Name>] type
                }
                continue;
            }



            # if the directive contains "ProductType", check the syntax by the special template {1[,2,3]}
            if (preg_match('/^\s*producttype.*/', strtolower($str)) && !preg_match('/^TemplateFile-[0-9a-z]+-ProductType\s*{\s*\d+\s*(,\s*\d+\s*?)*\}\s*=\s*/i', strtolower($str)))
            {
                array_push($Errors, array("CORE_022_3INI" => $str_num));      // Mismatching with the regular expression of the type
                                                                         // <directive> [1[,2,3]] = <value>
            }
            # if the directive contains "Categories", check it by the special template {1[+][,2,3[+]]}
            elseif (preg_match('/^\s*categories.*/', strtolower($str)) && !preg_match('/^categories\s*\{\s*\d+\+{0,1}(\s*,\s*\d+\+{0,1}\s*?)*\}\s*=\s*/i', strtolower($str)))
            {
                array_push($Errors, array("CORE_021" => $str_num));      //  Mismatching with the regular expression of the type
                                                                         // <directive> [{1[+][, 2+, 3]}] = <value>
            }
            # if the directive contains "Products", check it by the special template {1[+][,2,3[+]]}
            elseif (preg_match('/^\s*products.*/', strtolower($str)) && !preg_match('/^products\s*\{\s*\d+\+{0,1}(\s*,\s*\d+\+{0,1}\s*?)*\}\s*=\s*/i', strtolower($str)))
            {
                array_push($Errors, array("CORE_021" => $str_num));      //   Mismatching with the regular expression of the type
                                                                         // <directive> [{1[+][, 2+, 3]}] = <value>
            }
        }
        fclose($fp);
        return $Errors;
    }

    /**
     * Emulation of the PHP function file_put_contents().
     */
    function asc_file_put_contents($filename, $text)
    {
        $handle = fopen($filename, "w");
        if($handle !== false)
        {
            fwrite($handle, $text);
            fclose($handle);
        }
    }
    /**
     * Copies three new ini files to the temporary folder. It converts them
     * to the old format. It returns the number of the strings,
     * appended to the beginning of the file - 2. It is used for error
     * mapping. While building, no error can exist in the first two strings.
     *
     * @author Alexander Girin
     *
     */
    function ini_copy_3_ini_and_convert_them_to_old_ini_format_3ini()
    {
        global $application;

        $from = $application->getAppIni('PATH_TEMPLATES') . 'catalog/product-info/default/product-info-config.ini';
        $to = $application->getAppIni('PATH_CACHE_DIR') . 'product-info-config.ini';
        $text = @file_get_contents($from);
        if($text !== FALSE)
        {
            $text = str_replace("Template", "TemplateFile-Item", $text);
            $text = str_replace("ProductType{default}", "Default", $text);
            $text = str_replace("TemplateFile-Item-OutOfStock-", "TemplateFile-ItemOutOfStock-", $text);

            $text =
                "[ProductInfo]" . "\n" .
                "TemplateDirectory = catalog/product-info/default" . "\n" .
                $text;
            $this->asc_file_put_contents($to, $text);
        }

        $from = $application->getAppIni('PATH_TEMPLATES') . 'catalog/product-list/default/product-list-config.ini';
        $to = $application->getAppIni('PATH_CACHE_DIR') . 'product-list-config.ini';
        $text = @file_get_contents($from);
        if($text !== FALSE)
        {
            $text = str_replace("Template", "TemplateFile", $text);
            $text = str_replace("Container-Empty", "ContainerEmpty", $text);
            $text = str_replace("ProductType{default}", "Item-Default", $text);
            $text = str_replace("ProductType", "Item-ProductType", $text);
            $text = str_replace("TemplateFile-OutOfStock-Item-", "TemplateFile-ItemOutOfStock-", $text);

            $text =
                "[ProductList]" . "\n" .
                "TemplateDirectory = catalog/product-list/default" . "\n" .
                $text;
            $this->asc_file_put_contents($to, $text);
        }

        $from = $application->getAppIni('PATH_TEMPLATES') . 'catalog/subcategory-list/default/subcategory-list-config.ini';
        $to = $application->getAppIni('PATH_CACHE_DIR') . 'subcategory-list-config.ini';
        $text = @file_get_contents($from);
        if($text !== FALSE)
        {
            $text = str_replace("Template", "TemplateFile", $text);
            $text = str_replace("Container-Empty", "ContainerEmpty", $text);
            $text =
                "[Subcategories]" . "\n" .
                "TemplateDirectory = catalog/subcategory-list/default" . "\n" .
                $text;
            $this->asc_file_put_contents($to, $text);
        }

        $from = $application->getAppIni('PATH_TEMPLATES') . 'catalog/shopping-cart/default/shopping-cart-config.ini';
        $to = $application->getAppIni('PATH_CACHE_DIR') . 'cart-content-config.ini';
        $text = @file_get_contents($from);
        if($text !== FALSE)
        {
            $text = str_replace("Template", "TemplateFile", $text);
            $text = str_replace("Container-Empty", "ContainerEmpty", $text);
            $text = str_replace("ProductType{default}", "Item-Default", $text);
            $text = str_replace("ProductType", "Item-ProductType", $text);
            $text =
                "[ShoppingCart]" . "\n" .
                "TemplateDirectory = catalog/shopping-cart/default" . "\n" .
                $text;
            $this->asc_file_put_contents($to, $text);
        }
    }

    /**
     * If one of the three ini files name is passed, replaces it with the name
     * of the matching copy in the temporary folder.
     * (See ini_copy_3_ini_and_convert_them_to_old_ini_format_3ini).
     * It returns without changes otherwise.
     *
     * @author Alexander Girin
     *
     */
    function ini_filename_old_to_copy_3ini($path_config, $ini_filename)
    {
        switch($ini_filename)
        {
            case "product-info-config.ini":
            case "product-list-config.ini":
            case "subcategory-list-config.ini":
            case "cart-content-config.ini":
            {
                return   $this->getAppIni('PATH_CACHE_DIR') . $ini_filename;
                break;
            }
            default:
            {
                return $path_config.$ini_filename;
            }
        }
    }

    /**
     * If one of the three ini files name is passed, replaces it with the name
     * of the new corresponding file.
     * (See ini_copy_3_ini_and_convert_them_to_old_ini_format_3ini).
     * It returns without changes otherwise.
     *
     * If in the converted configuration file an error occurs,
     * the message must specify a not converted file yet.
     * The nature of conversions and checkings allows at the same time to save
     * valid the error message semantic.
     *
     * @author Alexander Girin
     */
    function ini_filename_old_to_new_3ini($ini_filename)
    {
        switch($ini_filename)
        {
            case "product-info-config.ini":
            {
                return "../../../avactis-templates/catalog/product-info/default/" . $ini_filename;
                break;
            }
            case "product-list-config.ini":
            {
                return "../../../avactis-templates/catalog/product-list/default/" . $ini_filename;
                break;
            }
            case "cart-content-config.ini":
            {
                return "../../../avactis-templates/catalog/shopping-cart/default/" . "shopping-cart-config.ini";
                break;
            }
            default:
            {
                return $ini_filename;
            }
        }
    }

    /**
     * Checks all kinds of errors connected with defining tags in the
     * customer area at the stage of initializing the application.
     *
     * @author Alexander Girin
     * @todo
     * @return
     */
    function checkTags($view_name,$view_file)
    {
        $path_templates = $this->getAppIni('PATH_TEMPLATES');
        $path_config = $this->getAppIni('PATH_BLOCK_CONFIG');

            // if in the view the getTemplateFormat function isn't defined, then skip it.
            if(!is_callable(array($view_name, 'getTemplateFormat'))) return;

            // get information about configuration file format
            $format = eval("return $view_name::getTemplateFormat();");

            //FIXME: If at the template viewing stage the configuration file
            //  for ViewClassName was found, then it's better use it, rather than
            //  the default file in system/admin/blocks_ini

            //FIXME: Create a virtual group "Default" for a configuration file,
            // which in fact doesn't contain any sections at all.
//            $config_file = $path_templates . $format['layout-file'];

            $new_config_file = $path_config . $this->ini_filename_old_to_new_3ini ($format['layout-file']);
            $config_file     = $this->ini_filename_old_to_copy_3ini($path_config, $format['layout-file']);
            //2 lines were added in the beginning of a file while converging NEW configs. 0 - otherwise.
            $config_is_3ini = ($config_file == ($path_config . $format['layout-file']) ? true : false);
            $line_shift = $config_is_3ini ? -2 : 0;


            #check if the configuration file exists
            if (!file_exists($config_file))
            {
                $err_mes = new ActionMessage(array("CORE_001"));
                $err_params = array(
                                    "CODE"    => "CORE_001",
                                    "FILE"    => $new_config_file, //$config_file,
                                    "MESSAGE" => $this->MessageResources->getMessage($err_mes)
                                    );
                $this->addErrorWarningMessage($view_name, "Error", $err_params);
                return; // continue the next view
            }

            #check the configuration file syntax
            if (sizeof($ini_errors = $this->isIniFileCorrect($config_file))>0)
            {
                foreach ($ini_errors as $error)
                {
                    list($error_code, $error_line) = each($error);
                    $err_mes = new ActionMessage(array($error_code));
                    $err_params = array(
                                        "CODE"    => $error_code,
                                        "FILE"    => $new_config_file, //$config_file,
                                        "LINE"    => $error_line + $line_shift,
                                        "MESSAGE" => $this->MessageResources->getMessage($err_mes)
                                        );
                    $this->addErrorWarningMessage($view_name, "Error", $err_params);
                }
                return; //  continue the next view
            }

            //FIXME: see the comments to parse_ini_file above.
            $this->Configs_array[$view_name] = @_parse_ini_file($config_file, true);
            foreach ($this->Configs_array[$view_name] as $section_name => $section_array)
            {
                $reserved_words = array("if", "else", "elseif", "while", "do", "for", "foreach", "break", "continue",
                                        "switch", "case", "declare", "return", "require", "include", "require_once",
                                        "include_once", "and", "or", "xor", "exit", "echo", "print", "die", "list",
                                        "as", "array", "function", "class", "var", "eval", "layouts", "");
                # check the matching reserved words
                if (array_search(strtolower($section_name), $reserved_words))
                {
                    $err_mes = new ActionMessage(array('CORE_006'));
                    $err_params = array(
                                        "CODE"    => 'CORE_006',
                                        "FILE"    => $new_config_file, //$config_file,
                                        "SECTION" => $section_name,
                                        "MESSAGE" => $this->MessageResources->getMessage($err_mes)
                                        );
                    _fatal($err_params);
                }

                # check the template description
                # check if the required directive "TemplateDirectory" exists
                if (($template_directory = getKeyIgnoreCase('TemplateDirectory', $section_array)) == null)
                {
                    $err_mes = new ActionMessage(array('CORE_015'));
                    $err_params = array(
                                        "CODE"      => 'CORE_015',
                                        "FILE"      => $new_config_file, //$config_file,
                                        "SECTION"   => $section_name,
                                        "DIRECTIVE" => "TemplateDirectory",
                                        "MESSAGE"   => $this->MessageResources->getMessage($err_mes)
                                        );
                    $this->addErrorWarningMessage($view_name, "Error", $err_params);
                }
                else
                {
                    # the "TemplateDirectory" directive exists.
                    # check if the spesified catalog exists
                    $dir = $path_templates . $template_directory;
                    if (!is_dir($dir))
                    {
                        $err_mes = new ActionMessage(array('CORE_016', $dir));
                        $err_params = array(
                                            "CODE"      => 'CORE_016',
                                            "FILE"      => $new_config_file, //$config_file,
                                            "SECTION"   => $section_name,
                                            "DIRECTIVE" => "TemplateDirectory = ".$template_directory,
                                            "MESSAGE"   => $this->MessageResources->getMessage($err_mes)
                                            );
                        $this->addErrorWarningMessage($view_name, "Error", $err_params);
                    }
                }

                # check if all the described directives in the format exist
                foreach ($format['files'] as $template_file => $template_file_type)
                {
                    if ($template_file_type == TEMPLATE_FILE_SIMPLE && getKeyIgnoreCase('TemplateFile-'.$template_file, $section_array) == null)
                    {
                        $err_mes = new ActionMessage(array('CORE_0151', 'TemplateFile-'.$template_file));
                        $err_params = array(
                                            "CODE"      => 'CORE_0151',
                                            "FILE"      => $new_config_file,//$config_file,
                                            "SECTION"   => $section_name,
                                            "DIRECTIVE" => 'TemplateFile-'.$template_file,
                                            "MESSAGE"   => $this->MessageResources->getMessage($err_mes)
                                            );
                        $this->addErrorWarningMessage($view_name, "Error", $err_params);
                        continue;
                    }
                    elseif ($template_file_type == TEMPLATE_FILE_PRODUCT_TYPE)
                    {
                        # The description must contain several directives for the specified file:
                        # TemplateFile-<name>-Default - one directive
                        # TemplateFile-<name>-ProductType{1,2,3} - 0, 1 or a few for different product types
                        $default = false;
                        foreach ($section_array as $directive => $value)
                        {
                            if (strpos(strtolower($directive), strtolower('templatefile-'.$template_file.'-Default')) === 0)
                            {
                                $default = true;
                            }
                            # if it is a ProductType description, it must conform with the regulation
                            elseif (strpos(strtolower($directive), strtolower('templatefile-'.$template_file.'-ProductType')) === 0)
                            {
                                if (!preg_match('/{\s*\d+\s*(,\s*\d+\s*?)*\}/', $directive))
                                {
                                    $error_code = $config_is_3ini ? 'CORE_0154_3INI' : 'CORE_0154';
                                    $err_mes = new ActionMessage(array($error_code, $directive));
                                    $err_params = array(
                                                        "CODE"      => $error_code,
                                                        "FILE"      => $new_config_file, //$config_file,
                                                        "SECTION"   => $section_name,
                                                        "DIRECTIVE" => $directive,
                                                        "MESSAGE"   => $this->MessageResources->getMessage($err_mes)
                                                        );
                                    $this->addErrorWarningMessage($view_name, "Error", $err_params);
                                }
                            }
                        }
                        if (!$default)
                        {
                            # if the Default directive did not occur, then output an error
                            $err_mes = new ActionMessage(array('CORE_0155', 'TemplateFile-'.$template_file.'-Default'));
                            $err_params = array(
                                                "CODE"      => 'CORE_0155',
                                                "FILE"      => $new_config_file, //$config_file,
                                                "SECTION"   => $section_name,
                                                "DIRECTIVE" => 'TemplateFile-'.$template_file.'-Default',
                                                "MESSAGE"   => $this->MessageResources->getMessage($err_mes)
                                                );
                            $this->addErrorWarningMessage($view_name, "Error", $err_params);
                        }

                        continue;
                    }
                    # check if the described file exists
                    $template_file = getKeyIgnoreCase('TemplateFile-'.$template_file, $section_array);
                    $template_file = $path_templates.$template_directory."/".$template_file;
                    #check if the files declared in the directives exist
                    if (!file_exists($template_file))
                    {
                        $err_mes = new ActionMessage(array('CORE_017', $template_file));
                        $err_params = array(
                                            "CODE"      => 'CORE_017',
                                            "FILE"      => $new_config_file, //$config_file,
                                            "SECTION"   => $section_name,
                                            "DIRECTIVE" => $directive." = ".$value,
                                            "MESSAGE"   => $this->MessageResources->getMessage($err_mes)
                                            );
                        $this->addErrorWarningMessage($view_name, "Error", $err_params);
                    }
                }

                # check if all the described options in the format exist
                foreach ($format['options'] as $option => $option_type)
                {
                    # check the required options only
                    if ($option_type == TEMPLATE_OPTION_REQUIRED && getKeyIgnoreCase('Option-'.$option, $section_array) == null)
                    {
                        $err_mes = new ActionMessage(array('CORE_0152', 'Option-'.$option));
                        $err_params = array(
                                            "CODE"      => 'CORE_0152',
                                            "FILE"      => $new_config_file, //$config_file,
                                            "SECTION"   => $section_name,
                                            "DIRECTIVE" => 'Option-'.$option,
                                            "MESSAGE"   => $this->MessageResources->getMessage($err_mes)
                                            );
                        $this->addErrorWarningMessage($view_name, "Error", $err_params);
                    }
                }

                # check the settings of the references to other templates
                foreach ($section_array as $directive => $value)
                {
                    # skip all directives except 'Categories' and 'Products'
                    if (strpos(strtolower($directive), 'categories') === false && strpos(strtolower($directive), 'products') === false) continue;
                    #check if the declaration of other directives is valid
                    if (!eregi("categories|products([\t\ ]*)({[0-9]+\+?(,?[\t\ ]*[0-9]\+?)*})?", $directive))
                    {
                        $err_mes = new ActionMessage(array('CORE_013', $directive));
                        $err_params = array(
                                            "CODE"      => 'CORE_013',
                                            "FILE"      => $new_config_file,
                                            "SECTION"   => $section_name,
                                            "DIRECTIVE" => $directive." = ".$value,
                                            "MESSAGE"   => $this->MessageResources->getMessage($err_mes)
                                            );
                        $this->addErrorWarningMessage($view_name, "Warning", $err_params);
                    }
                    #check whether the sections declared in the directives exist in the section[layouts]
                    if (getKeyIgnoreCase($value, $this->Configs_array[$view_name]) == null)
                    {
                        $err_mes = new ActionMessage(array('CORE_012', $value));
                        $err_params = array(
                                            "CODE"      => 'CORE_012',
                                            "FILE"      => $new_config_file,
                                            "SECTION"   => $section_name,
                                            "DIRECTIVE" => $directive." = ".$value,
                                            "MESSAGE"   => $this->MessageResources->getMessage($err_mes)
                                            );
                        $this->addErrorWarningMessage($view_name, "Warning", $err_params);
                    }
                    continue;
                }
            }
        return ;
    }

    /**
     * Checks if the "Default" directive exists in each of the sections,
     * declared in the layouts-config.ini file and if matching it template files
     * exist. It also checks the files, defined for other directives.
     *
     * @author Alexander Girin
     * @param string $section_name - the name of checked section
     * @param array $section_arr - the array of values, defined in this section
     * @param array $necessary_checks - reference to the array of necessary
     * checkings
     * @return
     * @todo do the tag registration where the error occurred
     */
    function checkDefaultFileAndOther ($section_name, $section_arr, &$necessary_checks, $SITE_PATH = NULL, $PATH_LAYOUTS_CONFIG_FILE = NULL)
    {
        if($SITE_PATH === NULL)
        {
            $SITE_PATH = $this->getAppIni('SITE_PATH');
        }

        if($PATH_LAYOUTS_CONFIG_FILE === NULL)
        {
            $PATH_LAYOUTS_CONFIG_FILE = $this->getAppIni('PATH_LAYOUTS_CONFIG_FILE');
        }

        $necessary_checks[$section_name]["section"] = true;
        foreach ($section_arr as $param => $value)
        {
            if (strtolower($param) == 'default')
            {
                $necessary_checks[$section_name]["default"] = true;
                #check if the file declared in the "default" directive exists
                if (file_exists($SITE_PATH.$value))
                {
                    $necessary_checks[$section_name]["template_file"] = true;
                }
                else
                {
                    $necessary_checks[$section_name]["template_file"] = $SITE_PATH.$value;
                }
            }
            else
            {
                switch ($section_name)
                {
                    case "productlist":
                        #check if the directive declaration in the section [Category] is valid
                        if (!eregi("categories([\t\ ]*)(\{[0-9]+\+?(,?[\t\ ]*[0-9]\+?)*\})?", $param))
                        {
                            $err_mes = new ActionMessage(array('CORE_007', $param));
                            $err_params = array(
                                                "CODE"      => 'CORE_007',
                                                "FILE"      => $PATH_LAYOUTS_CONFIG_FILE,
                                                "SECTION"   => "ProductList",
                                                "DIRECTIVE" => $param." = ".$value,
                                                "MESSAGE"   => $this->MessageResources->getMessage($err_mes)
                                                );
                            $this->addErrorWarningMessage("Core", "Warning", $err_params);
                        }
                        break;
                    case "productinfo":
                        #check if the directive declaration in the section [ProductInfo] is valid
                        if (!eregi("categories|products([\t\ ]*)(\{[0-9]+\+?(,?[\t\ ]*[0-9]\+?)*\})?", $param))
                        {
                            $err_mes = new ActionMessage(array('CORE_008', $param));
                            $err_params = array(
                                                "CODE"      => 'CORE_008',
                                                "FILE"      => $PATH_LAYOUTS_CONFIG_FILE,
                                                "SECTION"   => "ProductInfo",
                                                "DIRECTIVE" => $param." = ".$value,
                                                "MESSAGE"   => $this->MessageResources->getMessage($err_mes)
                                                );
                            $this->addErrorWarningMessage("Core", "Warning", $err_params);
                        }
                        break;
                    case "cart":
                    case "closed":
                        #check if the directive declaration in the section [cart], [checkout], [closed] is valid
                        $err_mes = new ActionMessage(array('CORE_009', $param));
                        $err_params = array(
                                            "CODE"      => 'CORE_009',
                                            "FILE"      => $PATH_LAYOUTS_CONFIG_FILE,
                                            "SECTION"   => $section_name,
                                            "DIRECTIVE" => $param." = ".$value,
                                            "MESSAGE"   => $this->MessageResources->getMessage($err_mes)
                                            );
                        $this->addErrorWarningMessage("Core", "Warning", $err_params);
                        break;
                    default:

                        break;
                }
                #check if the files declared in the directives exist
                if (!file_exists($SITE_PATH.$value))
                {
                    $err_mes = new ActionMessage(array('CORE_005', $SITE_PATH.$value));
                    $err_params = array(
                                        "CODE"      => 'CORE_005',
                                        "FILE"      => $PATH_LAYOUTS_CONFIG_FILE,
                                        "SECTION"   => $section_name,
                                        "DIRECTIVE" => $param,
                                        "MESSAGE"   => $this->MessageResources->getMessage($err_mes)
                                        );
                    $this->addErrorWarningMessage("Core", "Warning", $err_params);
                }
            }
        }
        if ($necessary_checks[$section_name]["default"] == false)
        {
            $necessary_checks[$section_name]["template_file"] = true;
        }
    }

    /**
     * Adds fatal error or warning message into the proper arrays $this->TagFatalErrors,
     * $this->TagWarnings.
     *
     * @author Alexander Girin
     * @param string $message - error or warning message
     * @param string $message_level - error level: the core or the block
     * @param string $message_type - message type: the error or warning
     * @return
     * @todo make the tag registration where the error occurred
     */
    function addErrorWarningMessage($message_level="Core", $message_type="Error", $params=array())
    {
        #register the fatal errors of core and block level
        if ($message_type=="Error")
        {
            if ($message_level=="Core")
            {
                if (!isset($this->TagFatalErrors['Core']))
                {
                    $this->TagFatalErrors['Core'] = array();
                }
                $this->TagFatalErrors['Core'][] = $params;
            }
            else
            {
                if (!isset($this->TagFatalErrors[$message_level]))
                {
                    $this->TagFatalErrors[$message_level] = array();
                }
                $this->TagFatalErrors[$message_level][] = $params;
            }
        }
        #register the warnings of core and block level
        elseif ($message_type=="Warning")
        {
            if ($message_level=="Core")
            {
                if (!isset($this->TagWarnings['Core']))
                {
                    $this->TagWarnings['Core'] = array();
                }
                $this->TagWarnings['Core'][] = $params;
            }
            else
            {
                if (!isset($this->TagWarnings[$message_level]))
                {
                    $this->TagWarnings[$message_level] = array();
                }
                $this->TagWarnings[$message_level][] = $params;
            }
        }
    }

    /**
     * Checks if fatal errors of the block level exist.
     *
     * @author Alexandr Girin
     * @param string $block - the block name
     * @return bool Returns true if fatal errors of the block level have been
     * detected, false otherwise
     */
    function issetBlockTagFatalErrors($block)
    {
        if (isset($this->TagFatalErrors[$block]))
        {
            return true;
        }
        return false;
    }

    /**
     * Gets the array read from the *.ini file.
     *
     * @author Alexander Girin
     * @param string $ini_array - the array name, e.g. "Layouts" or the
     * representation name
     * @return array - the array of values, read from the *.ini file
     */
    function getConfigArray($ini_array)
    {
        return getKeyIgnoreCase($ini_array, $this->Configs_array);
    }

    /**
     * Outputs the list of errors and warnings, occurred during the process of
     * checkings, connected with user Tags.
     *
     * @author Alexander Girin
     * @return
     */
    function outputTagErrors($in_block = "", $block_name = "", $type = "")
    {
        global $application;
        $session = &$application->getInstance("Session");
        if (!$session->is_Set("DEBUG_MODE"))
        {
            return;
        }
        if (sizeof($this->TagFatalErrors)==0&&sizeof($this->TagWarnings)==0)
        {
            if (file_exists("TagErrors.html"))
            {
                @unlink("TagErrors.html");
            }
            return;
        }
        $link = "";
        if ($in_block)
        {
            $URL = $this->getAppIni("HTTP_URL")."/avactis-system/TagErrors.html";
            if ($type == "Errors" && isset($this->TagFatalErrors[$block_name]))
            {
                $link = "<span style=\"font-family: verdana; font-size: 10px; font-weight: bold;\">";
                $link.= "<a href=\"$URL\" target=_blank>The are Fatal Errors in Block: \"$block_name\"</a></span><br>";
            }
            if ($type == "Warnings" && isset($this->TagWarnings[$block_name]))
            {
                $link = "<span style=\"font-family: verdana; font-size: 10px; font-weight: bold\">";
                $link.= "<a href=\"$URL\" target=_blank>The are Warnings in Block: \"$block_name\"</a></span><br>";
            }
            if ($link)
            {
                echo $link;
            }
        }
        $output = "";
        $header = "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\"\n";
        $header.= "\"http://www.w3.org/TR/html4/loose.dtd\">\n";
        $header.= "<HTML><HEAD><TITLE>Tags Errors and Warnings</TITLE>";
        $header.= "<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=windows-1251\">\n";
        $header.= "<LINK HREF=\"style/style.css\" TYPE=\"text/css\" REL=\"stylesheet\">\n";
        $header.= "<script src=\"javascript/menu.js\" type=\"text/javascript\"></SCRIPT>\n";
        $header.= "</HEAD><BODY>\n";
        $header.= "<style type=\"text/css\">\n";
        $header.= "<!--\n";
        $header.= ".title{background-color: #C0C0C0; font-size: 12px; font-family: verdana; font-weight: bold; \n";
        $header.= "       border-right: solid #505050 1px; border-bottom: solid #505050 1px; \n";
        $header.= "       border-left: solid #FFFFFF 1px; border-top: solid #FFFFFF 1px;}\n";
        $header.= ".subtitle{background-color: #E0E0E0; font-size: 11px; font-family: verdana; font-weight: bold; \n";
        $header.= "       border-right: solid #C0C0C0 1px; border-bottom: solid #C0C0C0 1px; \n";
        $header.= "       border-left: solid #FFFFFF 1px; border-top: solid #FFFFFF 1px;}\n";
        $header.= ".message{background-color: #FAFAFA; font-size: 10px;\n";
        $header.= "       border-right: solid #E0E0E0 1px; border-bottom: solid #E0E0E0 1px; \n";
        $header.= "       border-left: solid #FFFFFF 1px; border-top: solid #FFFFFF 1px;}\n";
        $header.= "-->\n";
        $header.= "</style>\n";
        $header.= "<TABLE width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">\n";

        $subtitle = "<tr><td width=\"100%%\" class=\"title\">%s</td></tr>\n";
        $subtitle.= "<tr><td>\n<table width=\"100%%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">\n";
        $subtitle.= "<tr><td class=\"subtitle\"";
        $subtitle.= "width=\"10%%\">Error Code</td>\n";
        $subtitle.= "<td class=\"subtitle\"";
        $subtitle.= "width=\"90%%\">Error Source</td></tr>%s</table></td></tr>\n";

        $tablecontent = "<tr><td class=\"subtitle\" style=\"background-color: #FAFAFA;\">%s</td>\n";
        $tablecontent.= "<td class=\"subtitle\">";
        $tablecontent.= "<table width=\"100%%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">%s</table></td></tr>\n";

        $source_line = "<tr><td class=\"message\" width=\"10%%\">%s</td>\n";
        $source_line.= "<td class=\"message\" style=\"font-weight: normal;\" width=\"90%%\">%s</td></tr>\n";

        $output = $header;
        if (isset($this->TagFatalErrors['Core']))
        {
            $content = "";
            foreach ($this->TagFatalErrors['Core'] as $error_source)
            {
                $content.= $this->outputErrorSource($tablecontent, $source_line, $error_source);
            }
            $output.= sprintf($subtitle, "FATAL Core Errors", $content);
        }
        foreach ($this->TagFatalErrors as $block => $block_errors)
        {
            if ($block != "Core")
            {
                $content = "";
                foreach ($this->TagFatalErrors[$block] as $error_source)
                {
                    $content.= $this->outputErrorSource($tablecontent, $source_line, $error_source);
                }
                $output.= sprintf($subtitle, $block." Block: Fatal Errors", $content);
            }
        }
        if (isset($this->TagWarnings['Core']))
        {
            $content = "";
            foreach ($this->TagWarnings['Core'] as $error_source)
            {
                $content.= $this->outputErrorSource($tablecontent, $source_line, $error_source);
            }
            $output.= sprintf($subtitle, "Core: Warnings", $content);
        }
        foreach ($this->TagWarnings as $block => $block_errors)
        {
            if ($block != "Core")
            {
                $content = "";
                foreach ($this->TagWarnings[$block] as $error_source)
                {
                    $content.= $this->outputErrorSource($tablecontent, $source_line, $error_source);
                }
                $output.= sprintf($subtitle, $block." Block: Warnings", $content);
            }
        }

        $footer = "</table></BODY></HTML>";
        $output.= $footer;
        $fp = fopen($this->getAppIni("PATH_SYSTEM_DIR")."TagErrors.html", "w");
        fwrite($fp, $output);
        fclose($fp);

        $javascript = "<SCRIPT LANGUAGE=\"JavaScript\">\n";
        $javascript.= "<!--\n";
        $javascript.= "var newwin;\n";
        $javascript.= "var w = 800;\n";
        $javascript.= "var h = 600;\n";
        $javascript.= "var bars_width = 47;\n";
        $javascript.= "var winl = (screen.width - w) / 2;\n";
        $javascript.= "var wint = ((screen.height - h) - bars_width) / 2;\n";
        $javascript.= "var params = 'top='+wint+',left='+winl+',width='+w+',height='+h+',directories=no,location=no,menubar=no,scrollbars=yes,status=no,toolbar=no,resizable=yes';\n";
        $javascript.= "newwin = window.open('".$this->getAppIni("HTTP_URL")."/avactis-system/TagErrors.html', 'Errors', params);\n";
        $javascript.= "newwin.focus();\n";
        $javascript.= "//-->\n";
        $javascript.= "</SCRIPT>\n";

        echo $javascript;
    }

    /**
     * Outputs the error or warning message.
     *
     * @author Alexander Girin
     * @param string $tablecontent - HTML template of Table content
     * @param string $source_line - HTML string template of Error source
     * @param array $error_source - Array of Error Source
     * @return string HTML code
     */
    function outputErrorSource($tablecontent, $source_line, $error_source)
    {
        $source = "";
        if (isset($error_source["FILE"]))
        {
            $source.= sprintf($source_line, "File:", $error_source["FILE"]);
        }
        if (isset($error_source["LINE"]))
        {
            $source.= sprintf($source_line, "Line:", $error_source["LINE"]);
        }
        if (isset($error_source["SECTION"]))
        {
            $source.= sprintf($source_line, "Section:", "[".$error_source["SECTION"]."]");
        }
        if (isset($error_source["DIRECTIVE"]))
        {
            $source.= sprintf($source_line, "Directive:", $error_source["DIRECTIVE"]);
        }
        if (isset($error_source["MESSAGE"]))
        {
            $source.= sprintf($source_line, "Message:", $error_source["MESSAGE"]);
        }
        return sprintf($tablecontent, $error_source["CODE"], $source);
    }

    /**
     * Reads out the settings from the layouts-config.ini file.
     */
    function readLayoutsINI()
    {
        # read the configuration file
        $ini_array = $this->getConfigArray('Layouts');

        $this->row_layout_ini_array = $ini_array;

        # read the hierarchy of categories
        $obj = &$this->getInstance('Catalog');
        $categories = $obj->getSubcategoriesFullListWithParent(1, false);
        $tree = array();
        foreach ($categories as $cat)
        {
            $tree[$cat['id']] = $cat;
        }
        # create an anonymous function to determine children categories
        $func = create_function('$arr, $cid', '$parent = $arr[$cid]; $children = array(); foreach ($arr as $cat) { if ($cat[\'c_left\'] >= $parent[\'c_left\'] && $cat[\'c_right\'] <= $parent[\'c_right\']) $children[] = $cat;} return $children;');

        $layout = array();
        $direct_categories = array();
        foreach ($ini_array as $section_name => $section_arr)
        {
            $section_name = strtolower($section_name);
            $layout[$section_name] = $this->readLayout($section_arr, $tree);
        }
        $this->layout = $layout;
    }

    /**
     * Reads out the block settings from configuration files.
     */
    function readBlocksINI($block)
    {
        static $tree = null;
        if ($tree == null)
        {
        #  read the hierarchy of categories
        $obj = &$this->getInstance('Catalog');
        $categories = $obj->getSubcategoriesFullListWithParent(1, false);
        $tree = array();
        foreach ($categories as $cat)
        {
            $tree[$cat['id']] = $cat;
        }
        }
        #  create an anonymous function to define children categories
        $func = create_function('$arr, $cid', '$parent = $arr[$cid]; $children = array(); foreach ($arr as $cat) { if ($cat[\'c_left\'] >= $parent[\'c_left\'] && $cat[\'c_right\'] <= $parent[\'c_right\']) $children[] = $cat;} return $children;');

        if (!isset($this->Configs_array[$block]))
        {
            return;
        }
        $block_arr = $this->Configs_array[$block];

            if (strtolower($block) == 'layouts') return;
            # read the configuration file for this block
            $config = $this->readBlockINI($block, $tree);
            # register functions for derived blocks.
            foreach ($config as $custom_tag => $template)
            {
                # skip the special tag layouts
                if ($custom_tag == "layout") continue;
                # skip the tags, which are representation names
                if (class_exists($custom_tag) && ($custom_tag != "Checkout")) continue;
                if(function_exists($custom_tag)) continue;
                $func = '
                function '. $custom_tag .'() {
                    global $application;
                    $application->setBlockOverride(\''.$block.'\', \''.$custom_tag.'\');
                    $arg0 = @func_get_arg(0);
                    if ($arg0 === false) {
                        $out = '. $block .'();
                    } else {
                        $out = '. $block .'($arg0);
                    }
                    $application->resetBlockOverride(\''.$block.'\');
                    return $out;
                }';
                eval($func);
            }
            # save information.
            $this->block_config[$block] = $config;
        }

    /**
     * Reads out the block settings from the given file.
     * @return array
     */
    function readBlockINI($viewname, $tree)
    {
        #  read the configuration file
        $ini_array = $this->getConfigArray($viewname);

        $block = array();
        # parse settings for each section
        foreach ($ini_array as $section_name => $section_arr)
        {
            # it is the block description.
            $block[$section_name] = array('template' => array(), 'options' => array(), 'layout' => array( 'categories' => array(), 'products' => array()));
            $block_section = &$block[$section_name];
            # parse every option for this block.
            foreach ($section_arr as $key => $value)
            {
                // keyword: TemplateDirectory, TemplateFile, Option
                $_key = strtolower($key);
                // describe the references to the other templates
                if (strpos($_key, 'categories') === 0 || strpos($_key, 'products') === 0)
                {
                    $layout_line = $this->readLayoutLine($_key, $value, $tree);
                    # add the references to the array
                    if (is_array($layout_line) && array_key_exists('categories', $layout_line))
                    {
                        $block_section['layout']['categories'] = $block_section['layout']['categories'] + $layout_line['categories'];
                    }
                    elseif (is_array($layout_line) && array_key_exists('products', $layout_line))
                    {
                        $block_section['layout']['products'] = $block_section['layout']['products'] + $layout_line['products'];
                    }
                }
                // the keyword variation, that may contain branches: ProductType
                $_subkey = "";
                if (strpos($_key, '-') > 0)
                {
                    $_subkey = substr($_key, strpos($_key, '-') + 1);
                    $_key = substr($_key, 0, strpos($_key, '-'));
                }
                # if the option has branches, then parse it.
                if (strpos($_subkey, '-') > 0)
                {
                    $_vary = substr($_subkey, strpos($_subkey, '-') + 1);
                    $_subkey = substr($_subkey, 0, strpos($_subkey, '-'));
                    if ($_vary == "default")
                    {
                        $value = array('default' => $value);
                    }
                    else
                    {
                        $value = $this->readLayoutLine($_vary, $value, $tree);
                    }
                }
                switch ($_key)
                {
                    case 'templatedirectory':
                        $block_section['template']['directory'] = $value;
                        break;

                    case 'templatefile':
                        if (array_key_exists($_subkey, $block_section['template']) && is_array($block_section['template'][$_subkey]))
                        {
                            $block_section['template'][$_subkey] = _array_merge_recursive($block_section['template'][$_subkey], $value);
                        }
                        else
                        {
                            $block_section['template'][$_subkey] = $value;
                        }
                        break;

                    case 'option':
                        $block_section['options'][$_subkey] = $value;
                        break;

                    default:
                        break;
                }
            }
        }
        return $block;
    }

    /**
     * Determines the layout name, depending on the current PHP page.
     * @return Array each array key - layout, to which this page refers.
     */
    function getLayout()
    {
        global $application;

        $page_name = $_SERVER["PHP_SELF"]; // the current executing page address from the root of the site
        $parsed_site_address = parse_url($application->getAppIni('SITE_URL')); // parse the base site address
        // remove the path from the root of the site,if it exists
        if (strpos($page_name, $parsed_site_address['path']) === 0)
        {
            $page_name = substr($page_name, strlen($parsed_site_address['path']));
        }

        $result = array();
        foreach ($this->layout as $layout => $arr)
        {
            $result[$layout] = multi_array_search($page_name, $arr);
        }
        return $result;
    }

    /**
     * Returns the current category id and the current product id, if the page
     * displayed at this moment, refers to Category Layout or ProductInfo Layout.
     *
     * @return Array
     */
    function getTemplateParameters()
    {
        $result = array('category_id' => -1, 'product_id' => -1);
        if (modApiFunc('Users', 'getZone') == 'AdminZone')
            return $result;

        $layout_arr = $this->getLayout();

        $category_id = modApiFunc('Catalog', 'getCurrentCategoryID');
        $product_id = modApiFunc('Catalog', 'getCurrentProductID');

        if ($layout_arr['productinfo'])
        {
            $result['category_id'] = $category_id;
            $result['product_id'] = $product_id;
        }
        if ($layout_arr['productlist'])
        {
            $result['category_id'] = $category_id;
        }
        return $result;
    }

    function prepareStorefrontBlockTag($viewname)
    {
        global $zone;
        if ($zone == 'CustomerZone' and !class_exists($viewname))
        {
            $mm = &$this->getInstance( 'Modules_Manager' );
            $mm->includeViewFileOnce($viewname);
            $this->checkTags($viewname,$mm->czViewList[$viewname]);
            $this->readBlocksINI($viewname);
        }
    }


    /**
     * Returns the description of the template for the given block.
     */
    function getBlockTemplate($block_name)
    {
        $this->prepareStorefrontBlockTag($block_name);

        $params = $this->getTemplateParameters();
        $category = $params['category_id'];
        $product = $params['product_id'];
        # if such block doesn't exist, return null.
        if (!array_key_exists($block_name, $this->block_config))
        {
            return null;
        }

        $block = $this->block_config[$block_name];
        # check if the tag overrides
        $override = $this->getBlockOverride($block_name);
        if ($override != null)
        {
            # if the description for the specified block doesn't exist, return null
            if (getKeyIgnoreCase($override, $block) === null)
            {
                return null;
            }
            # if the overriding of the main tag is specified, extract the template for the overriding tag
            $block = $block[$override];
        }
        else
        {
            # if the description for the basic block doesn't exist, return null
            if (getKeyIgnoreCase($block_name, $block) === null)
            {
                return null;
            }
            # extract the template for the basic tag otherwise
            $block = $block[$block_name];
        }
        $layout = $block['layout'];

        if ($product != -1 && array_key_exists('products', $layout))
        {
            if (array_key_exists($product, $layout['products']))
            {
                $block = $this->block_config[$block_name][$layout['products'][$product]];
            }
        }
        if ($category != -1 && array_key_exists('categories', $layout))
        {
            if (array_key_exists($category, $layout['categories']))
            {
                $block = $this->block_config[$block_name][$layout['categories'][$category]];
            }
        }
        return $block;
    }

    /**
     * Returns the whole path of the specified file in this block template.
     */
    function getBlockTemplateFile($template, $file, $product_type = -1)
    {
        if (getKeyIgnoreCase($file, $template['template']) == null)
        {
            return null;
        }
        # define a base folder.
        $dir = $this->appIni['PATH_TEMPLATES'] . $template['template']['directory'] . '/';

        # if the product type is defined, then search one of the variation by product_type
        if ($product_type != -1)
        {
            if (is_array($template['template'][strtolower($file)]))
            {
                if (array_key_exists('producttype', $template['template'][strtolower($file)]) && is_array($template['template'][strtolower($file)]['producttype']) && array_key_exists($product_type, $template['template'][strtolower($file)]['producttype']))
                {
                    # if the variation array is defined, then select a required one.
                    return $dir . $template['template'][strtolower($file)]['producttype'][$product_type];
                }
                else
                {
                    # if the required variation isn't found or the array isn't specified at all, return the default value.
                    # warning: the default value must be specified!
                    return $dir . $template['template'][strtolower($file)]['default'];
                }
            }
        }
        return $dir . $template['template'][strtolower($file)];
    }

    /**
     * Returns the value of the option for this block template.
     */
    function getBlockOption($template, $option)
    {
        if (!array_key_exists(strtolower($option), $template['options']))
        {
            return null;
        }
        return $template['options'][strtolower($option)];
    }

    /**
     * Reads out the settings from the Layout section of any configuration file.
     */
    function readLayout($section_arr, $tree)
    {
        #   create an anonymous function to define children categories
        $func = create_function('$arr, $cid', '$parent = $arr[$cid]; $children = array(); foreach ($arr as $cat) { if ($cat[\'c_left\'] >= $parent[\'c_left\'] && $cat[\'c_right\'] <= $parent[\'c_right\']) $children[] = $cat;} return $children;');

        $layout = array();
        $direct_categories = array();
        foreach ($section_arr as $option => $value)
        {
            $option = strtolower($option);
            if ($option == "default")
            {
                $layout['default'] = $value;
            }
            else
            {
                # find out a keyword
                $end = strpos($option, '{');
                $key = trim(substr($option, 0, $end));

                # parse the ids
                $begin = strpos($option, '{') + 1;
                $end = strpos($option, '}');
                $ids = str_replace(" ", "", trim(substr($option, $begin, $end - $begin)));
                $ids = split(",", $ids);

                #  create a key, if it hasn't been created yet
                if (!array_key_exists($key, $layout))
                {
                    $layout[$key] = array();
                }
                foreach ($ids as $id)
                {
                    # check if the nesting symbol exists
                    if (strpos($id, '+'))
                    {
                        $id = substr($id, 0, strlen($id) - 1);
                        if ($key == "categories" && !array_key_exists($id, $tree))
                            continue;
                        $id = $func($tree, $id);
                    }
                    else
                    {
                        if ($key == "categories" && !array_key_exists($id, $tree))
                            continue;
                        $direct_categories[] = $id;
                    }
                    if (is_array($id))
                    {
                        foreach ($id as $_id)
                        {
                            if (!in_array($_id['id'], $direct_categories))
                            {
                                $layout[$key][$_id['id']] = $value;
                            }
                        }
                    }
                    else
                    {
                        $layout[$key][$id] = $value;
                    }
                }
            }
        }
        return $layout;
    }

    /**
     * @todo the description for the Application-> function.
     */
    function readLayoutLine($option, $value, $tree)
    {
        # create an anonymous function to define children categories
        $func = create_function('$arr, $cid', '$parent = $arr[$cid]; $children = array(); foreach ($arr as $cat) { if ($cat[\'c_left\'] >= $parent[\'c_left\'] && $cat[\'c_right\'] <= $parent[\'c_right\']) $children[] = $cat;} return $children;');

        $layout = array();
        #  find out a keyword
        $end = strpos($option, '{');
        $key = trim(substr($option, 0, $end));

        #  parse the ids
        $begin = strpos($option, '{') + 1;
        $end = strpos($option, '}');
        $ids = str_replace(" ", "", trim(substr($option, $begin, $end - $begin)));
        $ids = split(",", $ids);

        # create a key, if it hasn't been created yet
        if (!array_key_exists($key, $layout))
        {
            $layout[$key] = array();
        }
        $direct_categories = array();
        foreach ($ids as $id)
        {
            #  check if the nesting symbol exists
            if (strpos($id, '+'))
            {
                $id = substr($id, 0, strlen($id) - 1);
                if ($key == "categories" && !array_key_exists($id, $tree))
                    continue;
                $id = $func($tree, $id);
            }
            else
            {
                if ($key == "categories" && !array_key_exists($id, $tree))
                    continue;
                $direct_categories[] = $id;
            }
            if (is_array($id))
            {
                foreach ($id as $_id)
                {
                    if (!in_array($_id['id'], $direct_categories))
                    {
                        $layout[$key][$_id['id']] = $value;
                    }
                }
            }
            else
            {
                $layout[$key][$id] = $value;
            }
        }
        return $layout;
    }
    /**
     * Gets Application setting value by a key.
     * If the key does not exist, then return NULL.
     *
     * @see Application::readAppINI()
     * @param string $key Key
     * @return mixed Value of key or NULL if key is not exists
     */
    function getAppIni($key)
    {
        $val = '';
        if (array_key_exists($key, $this->appIni))
        {
            $val = $this->appIni[$key];
        }
        else
        {
            $val = NULL;
        }

        return $val;
    }

    /**
     * Registers all the information files in the system/tags file.
     */
    function registerTags()
    {
        $tags = file($this->appIni['PATH_TAGS_FILE']);
        if(!is_array($tags)) return;

        $func_tpl = file_get_contents($this->getAppIni('PATH_CORE_DIR') . '/block_tag_info.tpl');

        foreach ($tags as $line_num => $tag)
        {
            if (!function_exists(trim($tag)))
            {
                $func = str_replace ('$tag', rtrim($tag), $func_tpl);
                eval($func);
                unset($func);
            }
        }
        unset($tags);
        unset($func_tpl);
    }

    /**
     * Registers all the tags specified in the paramaters.
     * The input array can be the regular or associative one (it is necessary for compatability).
     */
    function registerAttributes($arr)
    {
        if(!is_array($arr)) return;

        // suppose, that tags are keys in the array
        $tags = array_keys($arr);
        // if the first key is a number, then suppose, that tags are values in the array
        if (count($tags) > 0 && is_int($tags[0]))
        {
            $tags = array_values($arr);
        }

        $func_tpl = file_get_contents($this->getAppIni('PATH_CORE_DIR') . '/block_tag_info.tpl');

        foreach ($tags as $tag) {
            $tag = trim(str_replace('{', '', str_replace('}', '', $tag)));
            // skip the the invalid tag names
            if (!preg_match("/^[a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff]*$/", $tag))
            {
//                $tag = "Product".$tag;
                continue;
            }
            // skip already existing functions.
            if (function_exists($tag))
            {
                continue;
            }
            $func = str_replace ('$tag', rtrim($tag), $func_tpl);
            eval($func);
        }
    }

    /**
     * Saves the tag in the callstack.
     */
    function pushTag($tag)
    {
        array_push($this->tag_stack, $tag);
    }

    /**
     * Extracts the tag from the callstack.
     *
     * @return string tag name
     */
    function popTag()
    {
        return array_pop($this->tag_stack);
    }

    /**
     * Returns the last tag name from the stack.
     *
     * @return string the last tag name from the stack
     */
    function getLatestTag()
    {
        $cnt = sizeof($this->tag_stack);
        if ($cnt == 0) return null;
        return $this->tag_stack[$cnt - 1];
    }

    /**
     * Searches the tag occurrence in the stack.
     *
     * @return boolean TRUE - if the tag has been already called, FALSE otherwise.
     */
    function isTagInStack($tag)
    {
        return in_array($tag, $this->tag_stack);
    }

    /**
     * Sets the overriding of the block tag at the customer one.
     */
    function setBlockOverride($block, $custom_tag)
    {
        $this->block_override[$block] = $custom_tag;
    }

    /**
     * Resets the overriding of the block tag.
     */
    function resetBlockOverride($block)
    {
        unset($this->block_override[$block]);
    }

    /**
     * Checks the overriding of the block tag.
     */
    function getBlockOverride($block)
    {
        if (array_key_exists($block, $this->block_override))
        {
            return $this->block_override[$block];
        }
        return null;
    }

    /**
     * Returns the statistics of using the database for the defined block tag.
     */
    function getDbStat($block = "")
    {
        $time = 0;
        $result = array();
        foreach ($this->db_stat as $entry)
        {
            $last = count($entry['stack']);
            if ($last > 0)
            {
                $last = $entry['stack'][$last - 1];
            }
            else {
                $last = "";
            }
            if ($block != "" && $block != $last)
            {
                continue;
            }
            $time += $entry['time'];
            $result[] = $entry['query'];
        }
        $result['time'] = $time;
        return $result;
    }

    /**
     * Returns the time, that has passed from the start of the system.
     */
    function getTimeSpent()
    {
        global $_avactis_start_time;

        list($usec, $sec) = explode(" ", microtime());
        $_time = ((float)$usec + (float)$sec);

        return ($_time - $_avactis_start_time);
    }

    /**
     * Prints the time, that has passed from the start of the system.
     */
    function printTimeSpent()
    {
        echo "Time spent: " . round($this->getTimeSpent(),4) . " s.";
    }

    /**
     * Prints the number of executed SQL quieries, from the start of the system.
     */
    function printNumberOfQueries()
    {
        echo "Queries Executed: " . sizeof($this->db_stat) ;
    }

    /**
     * Prints the total time of the executed SQL queries, from the start of
     * the system.
     */
    function printQueriesTime()
    {
        $time = 0;
        foreach ($this->db_stat as $entry)
        {
            $time += $entry['time'];
        }

        echo "Queries Time: " . round($time,4) ." s." ;
    }

    /**
     * Prints the list of all the executed SQL queries, from the start
     * of the system.
     */
    function printQueriesList($block = '')
    {
        print_r($this->getDbStat($block));
    }

    function areInstallerFilesNotRemoved()
    {
        return (@file_exists($this->getAppIni("INSTALLER_FILE_DAT")) ||
                @file_exists($this->getAppIni("INSTALLER_FILE_PHP")));
    }

    /**
     * Checks if the file recording with cached pages is available.
     *
     * @author Vadim Lyalikov
     * @todo
     * @return
     */
    function isCacheFolderNotWritable()
    {
        global $application;
        $dir_fs_name = $application->getAppIni("PATH_CACHE_DIR");
        return !is_dir_writable($dir_fs_name);
    }

    /**#@-*/

//------------------------------------------------
//              PRIVATE DECLARATION
//------------------------------------------------

    /**#@+
     * @access private
     */

    /**
     * The pointer to Factory object
     */
    var $pFactory;

    /**
     * The array for storing the application settings.
     */
    var $appIni;

    /**
     * @var DB_MySQL
     */
    var $db;

    /**
     * @var Array stack for storing tag callings.
     */
    var $tag_stack = array();

    /**
     * @var Array                                                       layouts
     *                  layouts-config.ini
     */
    var $layout = array();

    var $block_config = array();

    var $block_override = array();

    /**
     * The temporary variable to optimize the system perfomance.
     * It contains the information about the number of database queries and
     * the time of their executing.
     * @var Array
     */
    var $db_stat = array();

    /**
     * The array of fatal errors, connected with defining tags in the customer area.
     */
    var $TagFatalErrors;
    var $TagWarnings;

    var $Configs_array = array();

    /**
     * System messages.
     */
    var $MessageResources;

    var $processActionStarted = false;

    var $redirectURL = NULL;

    var $js_redirectURL = NULL;

    var $haveToExit = NULL;

    var $row_layout_ini_array = null;
    /**#@-*/
}
?>