<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2007, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

_use( dirname(__FILE__).'/ccache_memory.php' );


class CCacheFile extends CCacheMemory
{
    /**
     * Constructor of the class.
     * Constructor has been optimized. It creates object CCacheFile without
     * any values, that means empty. Values are loaded as required.
     * The object saves all its files in folder
     * $application->getAppIni('PATH_CACHE_DIR'),
     * the files have mask names "_cache.{id}", where id is a hashed key.
     *
     * @return CCacheFile
     */
    function CCacheFile($file_mask)
    {
        parent::CCacheMemory();

        global $application;
        $this->_fCacheDir = $application->getAppIni('PATH_CACHE_DIR');
        $this->_fCacheGlobalFileMask = '_cache.';
        $this->_fCacheFileMask = $this->_fCacheGlobalFileMask.$file_mask;
    }

    /**
     * Saves value $value into cache with key $key.
     * If key $key already exists in cache, the value of the key will be
     * changed to a new one.
     *
     * @param string $key Cache key
     * @param mixed $value Key value
     */
    function save($key, $value)
    {
        parent::save($key, $value);

        $this->_save_data($key);
    }

    /**
     * Adds the value to the key in cache. You can save several values for
     * one key.
     * In this case method get($key) returns the array of saved values.
     *
     * If the key doesn't exist in the cache, the key will be created. In this
     * case, method get($key) returns the current value of the key.
     *
     * Warning: using method get($key), you need to check
     * whether a new value is an array or a scalar value.
     *
     * @param string $key Cache key
     * @param mixed $value Key value
     */
    function append($key, $value)
    {
        if ( !array_key_exists($key, $this->_fData) )
        {
            $this->_load_data($key);
        }

        parent::append($key, $value);

        $this->_save_data($key);
    }

    /**
     * Returns the value of the key from cache.If the key doesn't exist,
     * value null will be returned.
     *
     * If only one value has been saved for the key, this value will be
     * returned "as is". If several values have been saved for the key, then
     * the array of all values will be returned.
     *
     * @param string $key Cache key
     * @return Cache kay value or null
     */
    function get($key)
    {
        if ( !array_key_exists($key, $this->_fData) )
        {
            $this->_load_data($key);
        }

        return parent::get($key);
    }

    /**
     * Deletes the value from cache. It removes the key and its value.
     *
     * @param string $key Cache key
     */
    function reset($key)
    {
        parent::reset($key);

        if (file_exists($this->_getFileName($key)))
	    {
	    	unlink($this->_getFileName($key));
	    }
    }

    function free_memory($key)
    {
        parent::reset($key);
    }

    function _save_data($key)
    {
        if ( !array_key_exists($key, $this->_fData) )
        {
            return false;
        }

        $data = serialize($this->_fData[$key]);

        if (!$handle = @fopen($this->_getFileName($key), 'w'))
        {
            return false;
        }

        if (@fwrite($handle, $data) === FALSE)
        {
            return false;
        }

        @fclose($handle);

        return true;
    }

    function _load_data($key)
    {
		if (!file_exists($this->_getFileName($key)))
		{
			return;
		}

		if (!is_readable($this->_getFileName($key)))
		{
			return;
        }

        $this->_fData[$key] = unserialize(file_get_contents($this->_getFileName($key)));
    }

    function clear()
    {
        $dir = @dir($this->_fCacheDir);
        if ($dir == null)
        {
            return;
        }
        while ($file = $dir->read())
        {
            if ( $file == '..' || $file == '.' || $file[0] == '.' || $file == "index.php" )
            {
                continue;
            }
            elseif ( !(strpos($file, $this->_fCacheGlobalFileMask)===false) )
            {
                @unlink($this->_fCacheDir . $file);
            }
        }
        $dir->close();
        $this->_fData = array();
    }

    function _getFileName($key)
    {
        $name = $this->_fCacheFileMask.$key;
        return $this->_fCacheDir.$name;
    }

    var $_fCacheDir;
    var $_fCacheFileMask;
    var $_fCacheGlobalFileMask;
}

?>