<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2007, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

_use( dirname(__FILE__).'/ccache_interface.php' );


class CCacheMemory extends CCacheInterface
{
    /**
     * Constructor of the class. It creates object CCacheMemory
     * without any values, that means empty.
     *
     * @return CCacheMemory
     */
    function CCacheMemory()
    {
        $this->_fData = array();
    }

    /**
     * Saves in the cache value $value with key $key.
     * If key $key already exists in the cache, the value of the key will
     * be changed to a new one.
     *
     * @param string $key Cache key
     * @param mixed $value Key value
     */
    function save($key, $value)
    {
        $this->_fData[$key] = array($value);
    }

    /**
     * Adds value to the key in the cache. You can save several values for
     * one key. In this case method get($key) returns the array of saved
     * values.
     *
     * If the key doesn't exist in the cache, the key will be created. In this
     * case, method get($key) returns the current value of the key.
     *
     * Warning: using method get($key), you need to check
     * whether a new value is an array or a scalar value.
     *
     * @param string $key Cache key
     * @param mixed $value Key value
     */
    function append($key, $value)
    {
        if ( array_key_exists($key, $this->_fData) )
        {
            array_push($this->_fData[$key], $value);
        }
        else
        {
            $this->save($key, $value);
        }
    }

    /**
     * Returns the value of the key from the cache. If the key doesn't exist,
     * value null will be returned.
     *
     * If only one value has been saved for the key, this value will be
     * returned "as is". If several values have been saved for the key, then
     * the array of all values will be returned.
     *
     * @param string $key Cache key
     * @return Cache kay value or null
     */
    function get($key)
    {
        if ( array_key_exists($key, $this->_fData) )
        {
            if ( sizeof($this->_fData[$key]) == 1 )
            {
                return $this->_fData[$key][0];
            }
            else
            {
                return $this->_fData[$key];
            }
        }
        else
        {
            return null;
        }
    }

    /**
     * Deletes the value from cache. It removes the key and its value.
     *
     * @param string $key Cache key
     */
    function reset($key)
    {
        if ( array_key_exists($key, $this->_fData) )
        {
            unset($this->_fData[$key]);
        }
    }

    var $_fData;
}

?>