<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2007, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php
/**
 * @copyright Copyright &copy; 2005, Pentasoft Corp.
 * @package Core
 * @author ag
 */
_use(dirname(__FILE__).'/dbquery.php');

/**
 * DB_Insert class is used to generate the SQL query of type Insert.
 *
 * Example of usage DB_Insert, to generate INSERT<br> query.
 * When inserting the value of only one field, the values of other ones are
 * on by default:
 *<code>
 *  $db_insert= &new DB_Insert('categories');
 *  $db_insert->addInsertValue('Games', 'category_name');
 *  $application->db->PrepareSQL($db_insert);
 *  $application->db->DB_Exec();
 *</code><br>
 * When inserting values of all the fields, i.e. when calling the addInsertValue()
 * function, the second parameter - a field name might not be specified.
 * But it is important to keep the sequence of fields in the database:
 *<code>
 *  $db_insert= &new DB_Insert('categories');
 *  $db_insert->addInsertValue(2);                   //  category_id
 *  $db_insert->addInsertValue(1);                   // the value of the field parent_category_id
 *  $db_insert->addInsertValue('Games');             // the value of the field category_name
 *  $db_insert->addInsertValue('Games Description'); // the value of the field category_descr
 *  $application->db->PrepareSQL($db_insert);
 *  $application->db->DB_Exec();
 *</code>
 *
 * @access  public
 * @author Alexandr Girin
 * @package Core
 */
class DB_Insert extends DBQuery
{
//------------------------------------------------
//               PUBLIC DECLARATION
//------------------------------------------------

    /**#@+
     * @access public
     */

    /**
     * DB_Insert class constructor.
     *
     * @return
     * @param string $table the table name
     */
    function DB_Insert($table)
    {
        global $application;
        $this->table_prefix = $application->getAppIni('DB_TABLE_PREFIX');
        $this->QueryType=DBQUERY_TYPE_INSERT;
        $this->InsertTable=$this->table_prefix.$table;
        $this->InsertFields=array();
        $this->InsertValues=array();
        $this->CopyFromTable="";
    }

    /**
     * Adds the field value.
     *
     * @return
     * @param string $value the value
     * @param string $field the field name
     */
    function addInsertValue($value, $field='')
    {
        if ($field!='')
        {
            $field = $this->parseFieldName($field);
            array_push($this->InsertFields, $field);
        }
        $value = "'".DBQuery::DBAddSlashes($value)."'";
        array_push($this->InsertValues, $value);
    }

    /**
     * Adds the field value in the form of expression.
     *
     * @return
     * @param string $value the value
     * @param string $field the field name
     */
    function addInsertExpression($value, $field='')
    {
        if ($field!='')
        {
            $field = $this->parseFieldName($field);
            array_push($this->InsertFields, $field);
        }
        $value = DBQuery::DBAddSlashes($value);
        array_push($this->InsertValues, $value);
    }

    /**
     * Gets the table name.
     *
     * @return string the table name
     */
    function getInsertTable()
    {
        return $this->InsertTable;
    }

    /**
     * Gets a list of fields, to which values are assigned.
     *
     * @return array the field array
     */
    function getInsertFields()
    {
        return $this->InsertFields;
    }

    /**
     * Gets a list of values.
     *
     * @return array the value array
     */
    function getInsertValues()
    {
        return $this->InsertValues;
    }

    /**
     * Copies from the other table.
     *
     * @return array the value array
     */
    function setCopyFromTable($tableName)
    {
        $this->CopyFromTable = $this->table_prefix.$tableName;
    }

    /**
     * Gets the name of the table which is copied from.
     *
     * @return array the value array
     */
    function getCopyFromTable()
    {
        return $this->CopyFromTable;
    }


    /**#@-*/

//------------------------------------------------
//              PRIVATE DECLARATION
//------------------------------------------------

    /**#@+
     * @access private
     */

    /**
     * The table name, where a record is inserted.
     *
     * @var  string
     */
    var $InsertTable;


    /**
     * The array of fields, where values are inserted.
     *
     * @var  array
     */
    var $InsertFields;

    /**
     * The array of inserted values.
     *
     * @var  array
     */
    var $InsertValues;

    /**
     * The array of inserted values.
     *
     * @var  string
     */
    var $CopyFromTable;
}
?>