<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2007, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php
/**
 * @copyright Copyright &copy; 2005, Pentasoft Corp.
 * @package Core
 * @author ag
 */
_use(dirname(__FILE__).'/dbquery.php');

/**
 * DB_Update class is used to generate the SQL query of type Update.
 *
 * Example of usage the DB_Update class to generate the UPDATE query.
 *<code>
 *  $db_update= &new DB_Update('categories');
 *  $db_update->addUpdateValue('category_name', 'FILM');
 *  $db_update->WhereValue('category_id', DB_EQ, 1);
 *  $application->db->PrepareSQL($db_update);
 *  $application->db->DB_Exec();
 *</code>
 * When generating the query conditions functions WhereField() or WhereValue() can be used along with WhereAND(), WhereOR(),
 * WhereNOT, addWhereOpenSection(), addWhereCloseSection(), for example:
 * <code>
 * $db_update->addWhereOpenSection();
 * $db_update->WhereField('field_name1', DB_NEQ, 'field_name2');
 * $db_update->WhereOR();
 * $db_update->WhereValue('field_name3', DB_EQ, 4);
 * $db_update->addWhereCloseSection();
 * </code>
 * The query condition will be the following:<br>
 * WHERE (field_name1<>field_name2 OR field_name3='4')
 *
 * @access  public
 * @author Alexandr Girin
 * @package Core
 */
class DB_Update extends DBQuery
{
//------------------------------------------------
//               PUBLIC DECLARATION
//------------------------------------------------

    /**#@+
     * @access public
     */

    /**
     * DB_Update class constructor.
     *
     * @return
     * @param string $table table name
     */
    function DB_Update($table)
    {
        global $application;
        $table_prefix = $application->getAppIni('DB_TABLE_PREFIX');
        $this->QueryType=DBQUERY_TYPE_UPDATE;
        $this->UpdateTable=$table_prefix.$table;
        $this->UpdateValues=array();
        $this->WhereList = array();
    }

    /**
     * Adds the field, whose value will be updated.
     *
     * @return
     * @param string $field field name
     * @param string $value update value
     */
    function addUpdateValue($field, $value)
    {
        $value = "'".$this->DBAddSlashes($value)."'";
        array_push($this->UpdateValues, $field.' = '.$value);
    }

    /**
     * Adds the field, whose value will be updated. As a value, an expression
     * is passed.
     *
     * @return
     * @param string $field field name
     * @param string $expression update value
     */
    function addUpdateExpression($field, $expression)
    {
        array_push($this->UpdateValues, $field.' = '.$expression);
    }

    /**
     * Gets a table name.
     *
     * @return string table name
     */
    function getUpdateTable()
    {
        return $this->UpdateTable;
    }

    /**
     * Gets a list of update values.
     *
     * @return array value array.
     */
    function getUpdateValues()
    {
        return $this->UpdateValues;
    }

    /**#@-*/

//------------------------------------------------
//              PRIVATE DECLARATION
//------------------------------------------------

    /**#@+
     * @access private
     */

    /**
     * The table name, which is updated.
     *
     * @var  string variable
     */
    var $UpdateTable;
    /**
     * Tha array of fields and update values.
     *
     * @var  string variable
     */
    var $UpdateValues;

    /**#@-*/
}
?>