<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2007, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php
/**
 * Class Logger is used to create a log file. It provides an easier application
 * debugging.
 *
 * To invoke methods of the Logger class, function _debug($line,
 * $type = 'message', $mark='NOMARK') is declared in the file functions.php.
 *
 * @see _debug
 *
 * @access private
 * @package Core
 */
class Logger
{
//------------------------------------------------
//               PUBLIC DECLARATION
//------------------------------------------------

    /**#@+
     * @access public
     */

    /**
     * The Logger class constructor.
     *
     * @return
     */
    function Logger($file_name='')
    {
        $write_log = modApiFunc('application', 'getAppIni', 'WRITE_LOG');
        if ($write_log == 'DISABLED')
        {
            return;
        }
        $this->starttime = $this->GetMicroTime();
        $this->lastmark = $this->starttime;
        $this->deltas = array();
        $this->messages = array();
        $this->prefix = '';
        $this->deltatime = array();

        if ($file_name=='')
        {
            $this->file_name = 'log.html';
        }
        else
        {
            $this->file_name = $file_name;
        }
        if (!defined('_APPEND_'))
        {
            $file = fopen($this->file_name,"w");
        }
        else
        {
            $file = fopen($this->file_name,"a");
        }
        $HTMLCode = '<HTML>\n';
        $HTMLCode.= '<HEAD>\n';
        $HTMLCode.= '<TITLE> Log File </TITLE>\n';
        $HTMLCode.= '</HEAD>\n\n';
        $HTMLCode.= '<BODY>\n';
        $HTMLCode.= '</TABLE><hr>\n';
        $HTMLCode.= date ('d.m.Y H:i:s').'<br>\n';
        $HTMLCode.= '<TABLE width="100%" cellspacing="0" cellpadding="0" border="0" nowrap>\n';
        $HTMLCode.= '<tr><td width="50%"><td width="50%">\n';
        fwrite($file, stripcslashes($HTMLCode));
        fclose($file);
    }

    /**
     * Registers the time change associated with the user message.
     *
     * @return
     * @param string $message -  the user message
     */
    function BlockExecuteTime($message)
    {
        $this->deltas[$message] = $this->GetDeltaTime();
    }

    /**
     * Outputs time stamps in the log file.
     *
     * @return
     */
    function DisplayTime()
    {
        $this->TotalTime();
        $this->deltas['Total:'] = $this->totaltime;
        $this->Display('Executing blocks in seconds', array('Block','Seconds'),$this->deltas);
    }

    /**
     * Outputs the name and the value of the variable in the log file.
     *
     * @return
     * @param mixed $var - the array of defined variables
     */
    function DisplayVar($var, $var_name)
    {
        $this->Display('', '', array($var_name => '<pre>'.print_r($var, true).'</pre>'));
    }

    /**
     * Outputs the array of defined variables in the log file.
     *
     * @return
     * @param array $vars - the array of defined variables
     */
    function DisplayDefinedVars($vars, $var_name)
    {
        if (!is_array($vars))
        {
            if ($vars==NULL)
            {
                $vars='<i>NULL</i>';
            }
            $vars = array($var_name => $vars);
        }
        $this->DefinedVars($vars);
        $this->Display('',array('',''),$this->vars, 2);
    }

    /**
     * Outputs the array of defined user and standard functions in the log file.
     *
     * @return
     */
    function DisplayDefinedFunctions()
    {
        $this->DefinedFunctions();
        $this->Display('Defined Functions','Function Name',$this->functions['user'], 1);
    }

    /**
     * Outputs the array of defined user and standard constants in the log file.
     *
     * @return
     */
    function DisplayDefinedConstants()
    {
        $this->DefinedConstants();
        $this->Display('Defined Constants',array('Constant Name','Constant Value'),$this->constants, 2);
    }

    /**
     * Outputs the array of defined user and standard classes in the log file.
     *
     * @return
     */
    function DisplayDeclaredClasses()
    {
        $this->DeclaredClasses();
        $this->Display('Declaired Classes','Class Name',$this->classes);
    }

    /**
     * Outputs any message in the log file, which specifies starting time of
     * the script execution.
     *
     * @return
     * @param string $message - any message
     */
    function DisplayMessage($message, $mark)
    {
        $time = $this->getTime();
        if (strtolower($mark)!='nomark')
        {
            $message.=': '.$mark;
            if (strtolower($mark) == 'end')
            {
                $this->prefix =substr($this->prefix, 0, (strlen($this->prefix)-19));
                $time.= '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;dt='.($time-array_pop($this->deltatime));
            }
        }
        array_push($this->messages,array($this->prefix.$message, $this->prefix.$time));
        $this->Display('','',array($this->prefix.$message => $this->prefix.$time));
        if (strtolower($mark)!='nomark')
        {
            if (strtolower($mark) == 'begin')
            {
                $this->prefix.='|&nbsp;&nbsp;&nbsp;';
                array_push($this->deltatime, $time);
            }
        }
    }

    /**
     * Outputs the log file in a new browser window.
     *
     * @return
     */
    function ShowLog()
    {
        if (defined('_DEBUG_'))
        {
            echo '<SCRIPT LANGUAGE="JavaScript">';
            echo 'function NewWindow(){';
            echo '    window.open("log.html","Log","left=100,top=100,width=800,height=500,scrollbars=1");';
            echo '}';
            echo 'NewWindow();';
            echo '</SCRIPT>';
        }
    }

    /**#@-*/

//------------------------------------------------
//              PRIVATE DECLARATION
//------------------------------------------------

    /**#@+
     * @access private
     */

    /**
    * The log file name.
    *
    * @var string
    */
    var $file_name;
    /**
    * Starting time of the script execution.
    *
    * @var float
    */
    var $starttime;
    /**
    * The last time stamp.
    *
    * @var float
    */
    var $lastmark;
    /**
    * Total time of the script execution.
    *
    * @var float
    */
    var $totaltime;
    /**
    * The array of time stamps.
    *
    * @var array
    */
    var $deltas;
    /**
    * The array of defined variables.
    *
    * @var array
    */
    var $vars;
    /**
    * The array of defined functions.
    *
    * @var array
    */
    var $functions;
    /**
    * The array of defined constants.
    *
    * @var array
    */
    var $constants;
    /**
    * The array of defined classes.
    *
    * @var array
    */
    var $classes;
    /**
    * The array of user messages.
    *
    * @var array
    */
    var $messages;
    /**
    * Value of indentation.
    *
    * @var string
    */
    var $prefix;
    /**
    * Execution time between 'BEGIN' and 'END'.
    *
    * @var array
    */
    var $deltatime;
    /**
    * The first color of the color scheme.
    *
    * @var int
    */
    var $C1;
    /**
    * The second color of the color scheme.
    *
    * @var int
    */
    var $C2;


    /**
     * Gets current time in seconds.
     *
     * @return float time in microseconds
     */
    function GetMicroTime()
    {
        list($usec, $sec) = explode(" ",microtime());
        return ((float)$usec + (float)$sec);
    }

    /**
     * Gets how much time has passed since the moment of the previous method
     * call. At the first call the method returns how much time has passed
     * since the beginning of the script execution.
     *
     * @return float the time change in microseconds
     */
    function GetDeltaTime()
    {
        $time = $this->GetMicroTime();
        $delta = $time - $this->lastmark;
        $this->lastmark = $time;
        return $delta;
    }

    /**
     * Gets time in seconds, that has passed from the beginning of the script
     * execution.
     *
     * @return float passed time
     */
    function getTime()
    {
        return $this->GetMicroTime() - $this->starttime;
    }

    /**
     * Calculates total time in seconds for the script execution, the value
     * is assigned
     *
     * @return
     */
    function TotalTime()
    {
        $this->totaltime = $this->GetMicroTime() - $this->starttime;
    }

    /**
     * Gets the array of defined variables in the function call domain -
     * get_defined_vars().
     *
     * @return
     * @param array $vars -the array of defined variables
     */
    function DefinedVars($vars)
    {
        $this->vars = $vars;
    }

    /**
     * Gets the array of defined user functions.
     *
     * @return
     */
    function DefinedFunctions()
    {
        $this->functions = get_defined_functions();
    }

    /**
     * Gets the array of user and standard constants.
     *
     * @return
     */
    function DefinedConstants()
    {
        $this->constants = get_defined_constants();
    }

    /**
     * Gets the array of user and standard classes.
     *
     * @return
     */
    function DeclaredClasses()
    {
        $this->classes = get_declared_classes();
    }

    /**
     * Sets a color scheme.
     *
     * @return
     * @param int $color_schem -  number of color scheme
     */
    function setColorSchem($color_schem)
    {
        $this->C1 = '#EEEEEE';
        $this->C2 = '#DDDDDD';
        switch ($color_schem)
        {
            case 0:
                    $this->C1 = '#EEEEEE';
                    $this->C2 = '#DDDDDD';
                    break;
            case 1:
                    $this->C1 = '#BBBBBB';
                    $this->C2 = '#CCCCCC';
            case 2:
                    $this->C1 = '#999999';
                    $this->C2 = '#AAAAAA';
                  break;
        }
    }

    /**
     * Writes log to the log.html file.
     *
     * @return
     * @param string $title - the title of the table
     * @param array $coltitle - the column title of the table
     * @param array $content - the content of the table
     * @param int $color_schem - the number of color scheme
     */
    function Display($title,$coltitle,$content, $color_schem=0)
    {
        $this->setColorSchem($color_schem);
        $HTMLCode = '';
//        $HTMLCode = '<TABLE width="100%" cellspacing="0" cellpadding="0" border="0">';
        $HTMLCode.= '<tr>\n\t<td nowrap colspan="2" align="left"  bgcolor="#707070">';
        $HTMLCode.= '<FONT color="#FFFFFF"><b><i>'.$title.'</i></b></FONT>\n';
        $HTMLCode.= '<tr>\n\t';
        if (is_array($coltitle))
        foreach($coltitle as $value)
        {
            $HTMLCode.= '<td nowrap align="left" bgcolor="#808080" width="50%">';
            $HTMLCode.= '<FONT color="#FFFFFF">'.$value.'</FONT>\n';
        }
        else
        {
            $HTMLCode.= '<td nowrap align="left" bgcolor="#808080" colspan="2">';
            $HTMLCode.= '<FONT color="#FFFFFF">'.$coltitle.'</FONT>\n';
        }
        $bgcolor=$this->C1;
        foreach ($content as $key => $value)
        {
            if ($bgcolor==$this->C2)
                $bgcolor=$this->C1;
            else
                $bgcolor=$this->C2;
            $HTMLCode.= '<tr>\n\t';
            if (is_int($key))
            {
                $HTMLCode.= '<td nowrap align="left" valign="top" bgcolor="'.$bgcolor.'" colspan="2">'.$value.'\n';
            }
            elseif ($key!='this'&&$key!='log')
            {
                $HTMLCode.= '<td nowrap align="left" valign="top" bgcolor="'.$bgcolor.'">'.$key.'\n';
                if (is_array($value)||is_object($value))
                {
                    $HTMLCode.= '<td nowrap align="left" valign="top" bgcolor="'.$bgcolor.'"><pre>';
                    $HTMLCode.= print_r($value, true);
                    $HTMLCode.= '</pre>\n';
                }
                else
                {
                    $HTMLCode.= '<td nowrap align="left" valign="top" bgcolor="'.$bgcolor.'">'.$value.'\n';
                }
            }
        }
//        $HTMLCode.= '</TABLE>';
        $file = fopen($this->file_name, "a+");
        fwrite($file, stripcslashes($HTMLCode));
        fclose($file);
    }

    /**#@-*/
}
?>