<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2007, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

/**
 * Controls the output process of text messages from the resource file.
 *
 * @package Core
 * @access  public
 * @author Alexey Kolesnikov
 */
class MessageResources
{

    /**
     * Constructs an object for the specified resource file.
     * Reads the resource file as the INI-file from the catalog PATH_USERS_RESOURCES.
     *
     *               $filename -                            ,         
     *                          .                 ,                                 
     *                              .
     *
     * @param string $filename - the resource filename without extension
     * @param string $zone  is necessary to specify the resource file path
     * @access public
     */
	function MessageResources($filename = "", $zone = "CustomerZone")
	{
	    global $application;
        $lang = strtolower($application->getAppIni('LANGUAGE'));

        if (file_exists($filename))
        {
            $full_filename = $filename;
        	$this->messages = _parse_ini_file($full_filename);
        }
        else
        {
            if ($filename == "")
            {
                $path = $application->getAppIni('PATH_ADMIN_RESOURCES');
                $filename = 'system-messages-'.$lang.'.ini';
            }
            elseif($zone == "CustomerZone")
            {
                $filename .= '.ini';
                $path = $application->getAppIni('PATH_USERS_RESOURCES');
            }
            else
            {
                $path = $application->getAppIni('PATH_ADMIN_RESOURCES');
                $filename = $filename.'-'.$lang.'.ini';
            }
            $full_filename = $path . $filename;
            if (@file_exists($full_filename))
            {
                $this->messages = _parse_ini_file($full_filename);
            }
            else
            {
                $this->messages = _parse_ini_file($path . "system-messages-eng.ini");
            }
        }

        if (isset($this->messages['COPYRIGHT_TEXT']))
        {
            $this->messages['COPYRIGHT_TEXT'] = sprintf($this->messages['COPYRIGHT_TEXT'], date("Y"));
        }

        $replace_resources = $application->getAppIni('REPLACE_RESOURCES');
        if (isset($replace_resources)&&$replace_resources=='ENABLED')
        {
            $fp = fopen($path."system-messages-zero.ini", "w");
            $fp2 = fopen($path."system-messages-zero2.ini", "w");
            foreach ($this->messages as $key => $val)
            {
                $newstr = "";
                for ($i=0; $i<strlen($val); $i++)
                {
                    switch ($val[$i])
                    {
                        case ' ':
                        case '.':
                        case ',':
                        case '(':
                        case ')':
                        case ':':
                        case ';':
                        case '-':
                        case '\'':
                        case '"':
                            $newstr.= $val[$i];
                            break;
                        case '{':
                            $i = $this->escapeSymbols($val, $i, '}', $newstr);
                            break;
                        case '<':
                            $i = $this->escapeSymbols($val, $i, '>', $newstr);
                            break;
                        case '&':
                            $i = $this->escapeSymbols($val, $i, ';', $newstr);
                            break;
                        default:
                            $val[$i] = "0";
                            $newstr.= "00";
                            break;
                    }
                }
                fwrite($fp, $key." = \"".$val."\"\n");
                fwrite($fp2, $key." = \"".$newstr."\"\n");
//                $this->messages[$key] = $val;
            }
            fclose($fp);
            fclose($fp2);
        }
    }

    function escapeSymbols(&$str, $pos, $symb, &$newstr)
    {
        while ($str[$pos]!=$symb)
        {
            $newstr.= $str[$pos];
            $pos++;
        }
        $newstr.= $str[$pos];
        return $pos++;
    }

	/**
	 * Returns a message from the resource file.
	 * $key can be either an object of the class ActionMessage or a string
	 * associated with the key from the resource file. If it is a string, then $objs is used
	 * to generate the message. If it is an object, $objs is skipped.
         *
	 * @param mixed $key
	 * @param array $objs
	 * @return string
	 * @access public
	 */
	function getMessage($key, $objs = array())
	{
	    $result = "";
	    $msg = "";
	    $values = "";
	    // if it is an object, its type must be ActionMessage
		if (is_object($key))
		{
			if (strtolower(get_class($key)) == 'actionmessage')
			{
				$msg = $key->getMessage();
				$values = $key->getValues();
			}
		}
		else
		{
			$msg = $key;
			$values = $objs;
		}

		if ($msg == "" || $msg == null || !array_key_exists($msg, $this->messages))
		{
			return "???" . $msg . "???";
		}
		return $this->format($this->messages[$msg], $values);
	}

    /**
     *
     */
    function isDefined($key)
    {
        return array_key_exists($key, $this->messages);
    }

	/**
	 * Formates a specified message with defined values.
         *
	 * @param string $msg resource file message
	 * @param array $values value array to format
	 * @return string
	 * @access private
	 */
	function format($msg, $values)
	{
	    // had to do the additional encoding in base64
	    $values = base64_encode(serialize($values));
	    return preg_replace("/\{(\d)\}/e", "MessageResources::replaceValue('\\1', '$values')", $msg);
	}

	/**
	 * The internal function is used to replace values in the text message.
         *
         * @param integer $id the identifier of replacement value.
	 * @param array $values the value array.
	 * @return string
	 * @access private
	 */
	function replaceValue($id, $values)
	{
	    $values = unserialize(base64_decode($values));
		if ($id == null || $id == "" || !is_array($values) || !array_key_exists($id, $values))
		{
			return '';
		}
		return $values[$id];
	}

	/**
	 * The array of resource file messages.
         *
	 * @access private
	 */
	var $messages = array();
}
?>