<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2007, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

/**
 * Standard Template Engine used in the system.
 *
 * @access private
 * @author Alexey Kolesnikov
 * @package Core
 */
class TemplateFiller
{
//------------------------------------------------
//               PUBLIC DECLARATION
//------------------------------------------------

    /**#@+
     * @access public
     */

    /**
     * @todo a description for the function TemplateFiller->.
     */
    function TemplateFiller()
    {
    	global $application;
        $this->MessageResources = &$application->getInstance('MessageResources');
    }

    /**
     * Sets a current template.
     */
    function setTemplate($template)
    {
    	$this->template = $template;
    }

    /**
     * Checks if the template with the specified name exists.
     */
    function isTemplateReadable($file)
    {
        global $application;
        $file = $application->getBlockTemplateFile($this->template, $file);
        if ($file === null)
        {
        	return false;
        }
        return is_readable($file);
    }

    /**
     * A main function for filling the template.
     *
     * @param string $file the template filename.
     * @param integer $product_type the product type identifier
     * @return string the filled template
     */
    function fill($file, $product_type = -1)
    {
        global $application;

        $tmpl_file = $application->getBlockTemplateFile($this->template, $file, $product_type);
        if ($tmpl_file === null)
        {
            $err_mes = new ActionMessage(array("TMPL_001", $this->template . $file));
        	return $this->MessageResources->getMessage($err_mes);
        }

        //Open file, read contents, replace tags.
        $this->CurrentTemplateFilename = $tmpl_file;
        $text = file_get_contents($this->CurrentTemplateFilename);
        if($text === FALSE)
        {
            _fatal("TMPL_FILLER_ERROR_CANNOT_READ_FILE", $this->CurrentTemplateFilename);
        }

        $text = $this->removeTemplateWrapper($text);

        // find and modify all the image references.
        // for customer zone only
        $zone = modApiFunc('Users', 'getZone');
        if ($zone == 'CustomerZone')
        {
            $text = $this->replaceImages($text);
        }
        // find and replace all the tags
        $text = $this->replaceTags($text);

        return $text;
    }
    /**#@-*/

//------------------------------------------------
//              PRIVATE DECLARATION
//------------------------------------------------

    /**#@+
     * @access private
     */

    /**
     * Extracts from the template text only the part, which must be used for
     * mapping.
     *
     * @param string $text raw contents of the template file.
     * @return string template contents which contain between the seperators.
     */
    function removeTemplateWrapper($text)
    {
        static $begin_tpl_tag_expr;
        static $end_tpl_tag_expr;
//        $begin_tpl_tag_expr = '/<hr>.*<!--.*BEGIN.*TPL.*\(DO.*NOT.*REMOVE!\).*-->/im';
        $begin_tpl_tag_expr = '/<\s*!\s*-\s*-\s*B\s*E\s*G\s*I\s*N\s*T\s*P\s*L\s*\(\s*D\s*O\s*N\s*O\s*T\s*R\s*E\s*M\s*O\s*V\s*E\s*!\s*\)\s*-\s*-\s*>\s*/im';
//        $end_tpl_tag_expr = '/<hr>.*<!--.*END.*TPL.*\(DO NOT REMOVE!\).*-->/im';
        $end_tpl_tag_expr = '/\s*<\s*!\s*-\s*-\s*E\s*N\s*D\s*T\s*P\s*L\s*\(\s*D\s*O\s*N\s*O\s*T\s*R\s*E\s*M\s*O\s*V\s*E\s*!\s*\)\s*-\s*-\s*>/im';

        if (preg_match($begin_tpl_tag_expr, $text, $begin, PREG_OFFSET_CAPTURE) != 1)
        {
            _fatal(array( "CODE" => "CORE_046"), $this->CurrentTemplateFilename);
        }

        if (preg_match($end_tpl_tag_expr, $text, $end, PREG_OFFSET_CAPTURE) != 1)
        {
            _fatal(array( "CODE" => "CORE_047"), $this->CurrentTemplateFilename);
        }

        $begin_pos = $begin[0][1] + strlen($begin[0][0]);
        $end_pos = $end[0][1];
        if($begin_pos > $end_pos+1)
        {
            _fatal(array( "CODE" => "CORE_048"), $this->CurrentTemplateFilename);
        }

        return substr($text, $begin_pos, $end_pos - $begin_pos);
    }

    /**
     * Fills the template with values.
     * The function searches tags within the template, defining them by the
     * standard php constructions: <code>&lt;?php function_code(); ?&gt;</code>.
     * Each function into the template will be called and will
     * be replaced  with the returned value.
     *
     * @param string $text the template with tags
     * @return string the template filled with tags.
     * @return string                                    .
     */
    function replaceTags($text)
    {
        return  preg_replace_callback("/<\?php(.*?)\?>/",array("TemplateFiller", "evalTag"),$text);
    }

    /**
     * Calls the function by Info-tags names.
     *
     * @param array $m the value array returned by the preg_replace_callback
     * function.
     * @return string the string created by Info-tag.
     */
    function evalTag($m)
    {
    	return eval("return $m[1]");
    }

    /**
     * Finds and replaces all references to images in the template, mapping
     * relative references to absolute ones.
     * All references to images in the template are relative references to
     * the template itself.
     */
    function replaceImages($text)
    {
        $image_subpattern = '([a-zA-Z0-9\\\\\/\-_\.:]+)';
    	$img_pattern = "/(\s+src=['\"]{0,1})".$image_subpattern."(['\"]{0,1}\s*)/msi";
    	$table_pattern = "/(\s+background=['\"]{0,1})".$image_subpattern."(['\"]{0,1}\s*)/msi";
    	$css_image_pattern = "/(background-image:\s+url\s*\(\s*['\"]{0,1})".$image_subpattern."(['\"]{0,1}\s*)/msi";
    	$css_image_pattern1 = "/(background:.*?\s+url\s*\(\s*['\"]{0,1})".$image_subpattern."(['\"]{0,1}\s*)/msi";

    	$result = $text;
    	$result = preg_replace_callback($img_pattern, array($this, "evalImage"), $result);
    	$result = preg_replace_callback($table_pattern, array($this, "evalImage"), $result);
    	$result = preg_replace_callback($css_image_pattern, array($this, "evalImage"), $result);
    	$result = preg_replace_callback($css_image_pattern1, array($this, "evalImage"), $result);
//    	echo "<pre>result: ".htmlspecialchars($result)."kkk</pre>";
        return $result;
    }

    function evalImage($image)
    {
        global $application;
        static $www_address, $site_url, $templates_url;

        if (!$www_address)
        {
            $www_address = $application->getAppIni('HTTP_URL');
            if ($application->getCurrentProtocol() == "https" && $application->getAppIni('HTTPS_URL'))
            {
                $www_address = $application->getAppIni('HTTPS_URL');
            }
        }
        if (!$templates_url)
        {
        	$templates_url = $application->getAppIni('URL_TEMPLATES');
            if ($application->getCurrentProtocol() == "https" && $application->getAppIni('HTTPS_URL_TEMPLATES'))
            {
                $templates_url = $application->getAppIni('HTTPS_URL_TEMPLATES');
            }
        }
        if (!$site_url)
        {
            $site_url = $application->getAppIni('SITE_URL');
            if ($application->getCurrentProtocol() == "https" && $application->getAppIni('SITE_HTTPS_URL'))
            {
                $site_url = $application->getAppIni('SITE_HTTPS_URL');
            }
        }
        $avactis_url = $templates_url . $this->template['template']['directory'];

    	$replace = "";
        if (strpos($image[2],'://') != 0)
        {
        	$replace = $image[0];
        }
        elseif (substr($image[2], 0, 1) == '/')
        {
            $replace = $site_url . substr($image[2], 1);
        }
        else
        {
    	   $replace = $image[1].$avactis_url.'/'.$image[2].$image[3];
        }
//      $count = count($image);
//    	echo "<pre>!!!";
//    	echo ($image[$count - 1]);
//    	echo "!!!<br/>";
//    	foreach ($image as $value)
//    	{
//    	    echo "<li>";
//    		print_r(htmlspecialchars($value));
//    		echo "<br/>";
//    	}
//
//    	echo "replace: ".$replace."<br/>";
//    	echo "end</pre>";
    	return $replace;
    }

    /**
     * Template file currenly being filled.
     */
    var $CurrentTemplateFilename = "";

    var $template;

    var $MessageResources;
    /**#@-*/
}
?>