<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2007, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

class NavigationBarHTMLCache extends CCacheFile
{
    function NavigationBarHTMLCache()
    {
        global $application;
        parent::CCacheFile(md5($application->getAppIni('SITE_PATH')).'HTML_Block_NavigationBar_cid');
    }
}



class ProductListHTMLCache extends CCacheFile
{
    function ProductListHTMLCache()
    {
        global $application;
        parent::CCacheFile(md5($application->getAppIni('SITE_PATH')).'HTML_Block_ProductList_');
    }
}



class CacheProductAttributes extends CCacheFile
{
    function CacheProductAttributes()
    {
        global $application;
        parent::CCacheFile('ProductAttributesInfo_pid');
    }
}



/**
 * The class is used to save the product list of the category in cache,
 * taking into account the paginator parameters.
 * This cache is used in Catalog::getProdsListInCategory
 *
 * @package Catalog
 * @author Alexey Florinsky
 * @access public
 */
class CacheProductListInCategories extends CCacheFile
{
    /**
     * A constructor of the class.
     *
     * @return CacheProductListInCategories
     */
    function CacheProductListInCategories()
    {
        parent::CCacheFile('ProductListInCategory_');
        $this->_fCacheIDsList = &new CCacheFile('ProductListInCategory_Ids_List_cid_');
    }

    /**
     * Saves data in cache.
     * The $key parameter is the $data data id in cache. It should be an array:
     * array($cid, $paginator_page_number, $paginator_products_per_page, $use_peginator)
     *
     * @param array $key The data id in cache should be the following:
     * array($cid, $paginator_page_number, $paginator_products_per_page, $use_peginator)
     * @param mixed $data Random data, which should be saved in cache.
     */
    function save( $key, $data )
    {
        /*
        When saving data in cache, a file is created for each key. As the file
        names depend on a set of parameters, then for one category will be
        created a lot of such files.Then comes a question: how to clear out
        the cache, i.e. the product list, in this category? There is no way
        to find out which of the cache files was created for the specified
        category.

        So it is necessary to create a cache to save in it cache file ids
        for each category.
        It should be done to clear out the cache for the specified category.
        */

        list($cid, $paginator_page_number, $paginator_products_per_page, $use_peginator) = $key;
        $cache_id = $this->_getCacheKey($cid, $paginator_page_number, $paginator_products_per_page, $use_peginator);

        //save into $cache_id, in the append mode
        $this->_fCacheIDsList->append($cid, $cache_id);

        parent::save($cache_id, $data);
    }

    /**
     * Returns data from cache by the key.
     * If the specified key doesn't exist in cache, it returns null.
     * The $key parameter is the $data data id in cache. It should be an array:
     * array($cid, $paginator_page_number, $paginator_products_per_page, $use_peginator)
     *
     * @param array $key The data id in cache should be the following:
     * array($cid, $paginator_page_number, $paginator_products_per_page, $use_peginator)
     * @return mixed Return cached data or NULL if $key not exists
     */
    function get( $key )
    {
       /* TODO
        May be it's better to check the cache integrity. It means the following:
        If the $key key exists in cache, then it should exist in the cache file
        'cache_ids_list_cid_'.$cid.
        If this statement is not fulfilled, then clear out the cache and return NULL.
        */
        list($cid, $paginator_page_number, $paginator_products_per_page, $use_peginator) = $key;
        return parent::get( $this->_getCacheKey($cid, $paginator_page_number, $paginator_products_per_page, $use_peginator) );
    }

    /**
     * Deletes a product list for the specified category from cache.
     * This method should be always invoked, when the number of products changes
     * in the category or they were selected:
     * - Add product to the category
     * - Delete product(s) from the category
     * - Copy product(s) to the category
     * - Sort products in the category
     *
     * @param int $category_id The category id, to clear the cache for.
     */
    function reset($category_id)
    {
       /*
        To clear out cache for the specified category, it is necessary to get
        a cache files list, created for this category.
        */
        $cache_ids_list = $this->_fCacheIDsList->get($category_id);
        if ( !($cache_ids_list === null) )
        {
            if (!is_array($cache_ids_list))
            {
                $cache_ids_list = array($cache_ids_list);
            }
            foreach ($cache_ids_list as $cache_id)
            {
                parent::reset( $cache_id );
            }
            $this->_fCacheIDsList->reset($category_id);
        }
    }

    function clear()
    {
        parent::clear();
        $this->_fCacheIDsList->clear();
    }


    /**
     * Returns a string, which identifies the record in cache.
     *
     * @param int $cid Category ID
     * @param int $paginator_page_number Page number in a paginator
     * @param int $paginator_products_per_page Products per page
     * @param bool $use_peginator Use or not paginator
     * @return string Cache key
     */
    function _getCacheKey( $cid, $paginator_page_number, $paginator_products_per_page, $use_peginator )
    {
        return  modApiFunc('Users', 'getZone').'_'.
                'cid'.$cid.'_'.
                'pn'.$paginator_page_number.'_'.
                'ppp'.$paginator_products_per_page.'_'.
                ($use_peginator ? 'T' : 'F');
    }

    var $_fCacheData = null;
    var $_fCacheIDsList = null;
}



/**
 * The class is used to cache the number of products in the category.
 *
 * This cache should cleared out in following cases:
 * - Add a product in the category
 * - Delete product(s) from the category
 * - Copy products to the category
 * - Move products to the category.
 *
 */
class CacheProductsNumberInCategories extends CCacheInterface
{
    function CacheProductsNumberInCategories()
    {
        $this->_fCacheAZ = &new CCacheSingleFile('ProductsNumberInCategoriesAZ');
        $this->_fCacheCZ = &new CCacheSingleFile('ProductsNumberInCategoriesCZ');
    }

    function save( $key, $prods_number )
    {
        list($cid, $mode) = $key;

        if (modApiFunc('Users', 'getZone') == "CustomerZone")
        {
            return $this->_fCacheCZ->save( $this->_getCacheKey($cid, $mode), $prods_number);
        }
        else
        {
            return $this->_fCacheAZ->save( $this->_getCacheKey($cid, $mode), $prods_number);
        }
    }

    function get($key)
    {
        list($cid, $mode) = $key;

        if (modApiFunc('Users', 'getZone') == "CustomerZone")
        {
            return $this->_fCacheCZ->get( $this->_getCacheKey($cid, $mode) );
        }
        else
        {
            return $this->_fCacheAZ->get( $this->_getCacheKey($cid, $mode) );
        }
    }

    function reset($category_id)
    {
        # TODO do it more effective
        $this->clear();
    }

    function clear()
    {
        $this->_fCacheAZ->clear();
        $this->_fCacheCZ->clear();
    }

    function _getCacheKey( $category_id, $calc_mode)
    {
        return 'cid'.$category_id.'_mode'.$calc_mode.'_'.modApiFunc('Users', 'getZone');
    }

    var $_fCacheAZ = null;
    var $_fCacheCZ = null;
}



class CatalogCacheManager
{
    function CatalogCacheManager()
    {
        global $application;

        if (modApiFunc('Configuration','getValue',SYSCONFIG_CACHE_LEVEL)==3)
        {
            $this->NavigationBarHTMLCache = &new NavigationBarHTMLCache();
            $this->ProductListHTMLCache = &new ProductListHTMLCache();
            $this->CacheProductAttributes = &new CacheProductAttributes();
            $this->CacheProductListInCategories = &new CacheProductListInCategories();
            $this->CacheProductsNumberInCategories = &new CacheProductsNumberInCategories();
        }
        elseif (modApiFunc('Configuration','getValue',SYSCONFIG_CACHE_LEVEL)==2)
        {
            $this->NavigationBarHTMLCache = &new CCacheNull();
            $this->ProductListHTMLCache = &new CCacheNull();
            $this->CacheProductAttributes = &new CacheProductAttributes();
            $this->CacheProductListInCategories = &new CacheProductListInCategories();
            $this->CacheProductsNumberInCategories = &new CacheProductsNumberInCategories();
        }
        elseif (modApiFunc('Configuration','getValue',SYSCONFIG_CACHE_LEVEL)==1)
        {
            $this->NavigationBarHTMLCache = &new CCacheNull();
            $this->ProductListHTMLCache = &new CCacheNull();
            $this->CacheProductAttributes = &new CCacheNull();
            $this->CacheProductListInCategories = &new CacheProductListInCategories();
            $this->CacheProductsNumberInCategories = &new CacheProductsNumberInCategories();
        }
        else /* modApiFunc('Configuration','getValue',SYSCONFIG_CACHE_LEVEL)==0 */
        {
            $this->NavigationBarHTMLCache = &new CCacheNull();
            $this->ProductListHTMLCache = &new CCacheNull();
            $this->CacheProductAttributes = &new CCacheNull();
            $this->CacheProductListInCategories = &new CCacheNull();
            $this->CacheProductsNumberInCategories = &new CCacheNull();
        }
    }

    function disableCache()
    {
        $this->NavigationBarHTMLCache = &new CCacheNull();
        $this->ProductListHTMLCache = &new CCacheNull();
        $this->CacheProductAttributes = &new CCacheNull();
        $this->CacheProductListInCategories = &new CCacheNull();
        $this->CacheProductsNumberInCategories = &new CCacheNull();
    }

    function eventProductInfoChanged($pid)
    {
        $this->ProductListHTMLCache->clear();
        $this->CacheProductAttributes->reset($pid);
    }

    function eventProductTypeChanged($ptid)
    {
        $this->NavigationBarHTMLCache->clear();
        $this->ProductListHTMLCache->clear();
        $this->CacheProductAttributes->clear();
        $this->CacheProductListInCategories->clear();
        $this->CacheProductsNumberInCategories->clear();
    }

    function eventCategoryInfoChanged($cid)
    {
        $this->NavigationBarHTMLCache->clear();
    }

    function eventCategoryProductsContentChanged($cid)
    {
        $this->ProductListHTMLCache->clear();
        $this->CacheProductListInCategories->reset($cid);
        $this->CacheProductsNumberInCategories->reset($cid);
    }

    function eventCategorySubcategoriesContentChanged($cid)
    {
        $this->NavigationBarHTMLCache->clear();
    }

    function clearAllCache()
    {
        $this->NavigationBarHTMLCache->clear();
        $this->ProductListHTMLCache->clear();
        $this->CacheProductAttributes->clear();
        $this->CacheProductListInCategories->clear();
        $this->CacheProductsNumberInCategories->clear();
    }

    var $NavigationBarHTMLCache;
    var $ProductListHTMLCache;
    var $CacheProductAttributes;
    var $CacheProductListInCategories;
    var $CacheProductsNumberInCategories;
}


?>