<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2007, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

include_once(dirname(dirname(__FILE__)).'/catalog_api.php');
include_once(dirname(dirname(__FILE__)).'/const.php');
include_once(dirname(__FILE__).'/product_class.php');

/**
 * The words in the searching request shorter than PRODUCT_SEARCH_MIN_WORD_LENGHT
 * are skipped.
 */
define('PRODUCT_SEARCH_MIN_WORD_LENGHT', 1);

/**
 * The words in the searching request longer than PRODUCT_SEARCH_MAX_WORD_LENGHT
 * are skipped.
 */
define('PRODUCT_SEARCH_MAX_WORD_LENGHT', 20);

/**
 * The number of the first words in the searching request, which will be used
 * in the search. The rest of the words are skipped.
 */
define('PRODUCT_SEARCH_MAX_WORD_NUMBER', 10);

/**
 * The max length of the searching request.
 * If the string of the searching request is longer than max length, then it will
 * be truncated up to the specified length.
 */
define('PRODUCT_SEARCH_MAX_PATTERN_LENGHT', 4096);

/**
 * The size of the selection from the DB on each relevance level.
 */
define('PRODUCT_SEARCH_RESULT_LIMIT',300);

/**
 * Time in seconds, after that the search result will be deleted from
 * the database.
 */
define('PRODUCT_SEARCH_RESULT_LIFE_TIME',3600);

/**
 * The constant, which is used for working the search algorithm.
 * It means at least one word should be searched in the text.
 */
define('PRODUCT_SEARCH_ANY_WORD',1);

/**
 * The constant, which is used for working the search algorithm.
 * It means all the words should be searched in the text.
 */
define('PRODUCT_SEARCH_ALL_WORD',2);

/**
 * The class is used to search the products in the catalog by any specified
 * text string.
 *
 * The current realization (Nov 19, 2007).
 * 1. To search in the fields:
 *       - ProductName,
 *       - ProductSKU,
 *       - ShortDescription and DetailedDescription
 *       - Meta Keywords and Meta Description
 * 2. Five relevance levels are realized:
 *    - searching in ProductName with the condition ALL_WORD - relevance 0 (the greatest)
 *    - searching in ProductSKU with the condition ALL_WORD - relevance 1
 *    - searching in Description (Short and Detailed) with the condition ALL_WORD - relevance 2
 *    - searching in Description (Short and Detailed) with the condition ANY_WORD - relevance 3
 *    - searching in Meta fields (Keywords and Description) with the condition ANY_WORD - relevance 4
 * 3. Output the result by the relevance level.
 *
 * The class usage:
 * 1. The search should be performed first, using the prepareProductsSearch($pattern)
 *    method.
 * 2. This method saves the searching result - the array of products id  - in
 *    the DB and returns the searching id.
 * 3. Using getProdsListInSearchResult($search_id), one can get the searching
 *    result as the array of references to the objects CProductInfo.
 *
 *
 * @package Catalog
 * @author Alexey Florinsky
 */
class CatalogSearch
{

    /**#@+
     * @access public
     */

    /**
     * A constructor of the class.
     */
    function CatalogSearch()
    {
        loadCoreFile('cstring.php');
        $this->cstring = new CString();
    }

    /**
     * Searches the products in the DB.
     * The searching result is saved in the DB.
     * The method returns the search id.
     * The searching result can be null.
     *
     * @param string $pattern Search request
     * @return int Search ID
     */
    function prepareProductsSearch($pattern)
    {
        /* Extract the words from the string, to be searched by. */
        $words = array_values($this->getIndexWordList($pattern));

       /*
        If the searching words are empty, then save the search in the DB without
        its results.
        It can be used, to output the request string to the customer area search
        and process the empty searching result.
        */
        if (empty($words))
        {
            return $this->saveSearchResult($pattern,array(),array());;
        }

        /* Perform the relevant searching */
        $result_relevance_00 = $this->searchProductsByName($words, PRODUCT_SEARCH_ALL_WORD );

        $_to_skip = $result_relevance_00;
        $result_relevance_10 = $this->searchProductsBySKU($words, PRODUCT_SEARCH_ALL_WORD, $_to_skip );

        $_to_skip = array_merge($result_relevance_00, $result_relevance_10);
        $result_relevance_20 = $this->searchProductsByDescriptions($words, PRODUCT_SEARCH_ALL_WORD, $_to_skip );

        $_to_skip = array_merge($_to_skip, $result_relevance_20);
        $result_relevance_30 = $this->searchProductsByDescriptions($words, PRODUCT_SEARCH_ANY_WORD, $_to_skip );

        $_to_skip = array_merge($_to_skip, $result_relevance_30);
        $result_relevance_40 = $this->searchProductsByMetaData($words, PRODUCT_SEARCH_ANY_WORD, $_to_skip);

        $result = $this->filterProducts(array_merge($_to_skip, $result_relevance_40));

        /* The searching result should contain only unique products */
        $result = array_unique($result);

        $search_id = $this->saveSearchResult($pattern,$result,$words);

        return $search_id;
    }

    /**
     * Returns the array of the references to CProductInfo products from the
     * searching result, using paginator.
     *
     * @param int $search_id Search ID
     * @return array A list of CProductInfo objects
     */
    function getProdsListInSearchResult($search_id, $return_as_pids_array = false)
    {
        global $application;

        $tables = Catalog::getTables();

	    $t_products_search_result = $tables['products_search_result']['columns'];

        $query = new DB_Select();
        $query->addSelectField($t_products_search_result['p_id'], 'product_id');
        $query->WhereValue($t_products_search_result['id'], DB_EQ, $search_id );
        $query->SelectOrder($t_products_search_result['relevance']);
        $query = modApiFunc('paginator', 'setQuery', $query);
        $result = $application->db->getDB_Result($query);

        if($return_as_pids_array)
        {
            return $result;
        };

        $products_listing = array();

        for ($i=0; $i<sizeof($result); $i++)
        {
            $offset = modApiFunc('paginator', 'getCurrentPaginatorOffset');
            if (!is_numeric($offset))
            {
                $offset=0;
            }

            $product_info = &new CProductInfo($result[$i]['product_id']);
            $product_info->setAdditionalProductTag('N', ($i+1+$offset) );
            array_push($products_listing, $product_info);
        }

        $this->updateSearchResultAccessTime($search_id);
        return $products_listing;
    }

    /**
     * Returns the initial request string for the specified search id.
     *
     * @param int $search_id Search ID
     * @return string Search request string
     */
    function getSearchPatternInSearchResult($search_id)
    {
        global $application;

        $tables = Catalog::getTables();

	    $t_products_search = $tables['products_search']['columns'];

        $query = new DB_Select();
        $query->addSelectField($t_products_search['pattern'], 'pattern');
        $query->WhereValue($t_products_search['id'], DB_EQ, $search_id );
        $result = $application->db->getDB_Result($query);
        if (isset($result[0]['pattern']))
        {
            $this->updateSearchResultAccessTime($search_id);
            return $result[0]['pattern'];
        }
        else
        {
            return '';
        }
    }

    /**
     * Returns a list of index words, to be searched by, for the specified
     * search id.
     * It is used to highlight in the searching results.
     *
     * @param int $search_id Search ID
     * @return array A list of index words
     */
    function getSearchIndexWordsInSearchResult($search_id)
    {
        global $application;

        $tables = Catalog::getTables();

	    $t_products_search = $tables['products_search']['columns'];

        $query = new DB_Select();
        $query->addSelectField($t_products_search['words'], 'words');
        $query->WhereValue($t_products_search['id'], DB_EQ, $search_id );
        $result = $application->db->getDB_Result($query);
        if (isset($result[0]['words']))
        {
            $this->updateSearchResultAccessTime($search_id);
            return unserialize($result[0]['words']);
        }
        else
        {
            return array();
        }
    }

    /**
     * Deletes searching results from the DB, the time of the last access
     * to which was more than PRODUCT_SEARCH_RESULT_LIFE_TIME seconds ago.
     *
     */
    function deleteOldSearches()
    {
        global $application;

        $tables = Catalog::getTables();
        $t_products_search = $tables['products_search']['columns'];
	    $t_products_search_result = $tables['products_search_result']['columns'];

        $query = new DB_Select();
        $query->addSelectField($t_products_search['id'], 'search_id');
        $query->WhereValue($t_products_search['time'], DB_LTE, date('Y-m-d H:i:s', time()-3600) );
        $result = $application->db->getDB_Result($query);

        $search_ids = array();
        foreach($result as $element)
        {
            array_push($search_ids, $element['search_id']);
        }

        if (!empty($search_ids))
        {
            $query = &new DB_Delete('products_search');
            $query->WhereField($t_products_search['id'], DB_IN,"('" . implode("','", $search_ids) . "')");
            $application->db->getDB_Result($query);

            $query = &new DB_Delete('products_search_result');
            $query->WhereField($t_products_search_result['id'], DB_IN,"('" . implode("','", $search_ids) . "')");
            $application->db->getDB_Result($query);
        }
    }


    /**#@-*/



    /**#@+
     * @access private
     */


    /**
     * Saves the searching result in the DB.
     * The parameters $result and $words can be empty. It means the searching
     * does not have any results.
     *
     * @param string $pattern Search pattern string (user input, for example)
     * @param array $result A list of product ids, plain list (examle: 31,25,33,42), sorted by relevance
     * @param array $words A list of index words
     * @return int Search ID
     */
    function saveSearchResult($pattern,$result,$words)
    {
        global $application;

        $tables = Catalog::getTables();

	    $t_products_search = $tables['products_search']['columns'];
	    $t_products_search_result = $tables['products_search_result']['columns'];

        $db_insert = new DB_Insert('products_search');
        $db_insert->addInsertValue($pattern, $t_products_search['pattern']);
        $db_insert->addInsertValue(date('Y-m-d H:i:s', time()), $t_products_search['time']);
        $db_insert->addInsertValue(serialize($words), $t_products_search['words']);
        $application->db->PrepareSQL($db_insert);
        $application->db->DB_Exec();
        $search_id = $application->db->DB_Insert_Id();

        $relevance = 0;
        foreach($result as $pid)
        {
            $db_insert = new DB_Insert('products_search_result');
            $db_insert->addInsertValue($search_id, $t_products_search_result['id']);
            $db_insert->addInsertValue($pid, $t_products_search_result['p_id']);
            $db_insert->addInsertValue($relevance, $t_products_search_result['relevance']);
            $application->db->PrepareSQL($db_insert);
            $application->db->DB_Exec();
            unset($db_insert);
            $relevance++;
        }

        return $search_id;
    }

    /**
     * Filters the searching results, i.e. truncates
     * product ids, which should not be outputted to the Store Front.
     *
     * @param array $products_id_list Plain list of product ids wich should be filtered
     * @return array A list of filtered product ids
     */
    function filterProducts($products_id_list)
    {
        global $application;

        if (modApiFunc('Users', 'getZone') == "CustomerZone") # Check Available attribute
        {
            $tables = Catalog::getTables();

            $t_products                = $tables['products']['columns'];
            $t_product_type_attributes = $tables['product_type_attributes']['columns'];
            $t_product_attributes      = $tables['product_attributes']['columns'];
            $p2c = $tables['products_to_categories']['columns'];

            $query = &new DB_Select();
            $query->addSelectField($t_products['id'], 'id');
            $query->WhereField($t_products['id'], DB_IN,"('" . implode("','", $products_id_list) . "')");

            if (modApiFunc('Users', 'getZone') == "CustomerZone")
            {
                CCategoryInfo::_applyProductsFilter($query);

                //                     online          :
                $cat = &new CCategoryInfo(HOME_CATEGORY_ID);
                $online_subcat_ids = $cat->_getOnlineSubCategories();

                $query->addSelectTable('products_to_categories');

                $query->WhereAND();
                $query->WhereField($p2c['category_id'], DB_IN,"('" . implode("','", $online_subcat_ids) . "')");
                $query->WhereAND();
                $query->WhereField($t_products['id'], DB_EQ, $p2c['product_id']);
            }

            $product_number = $application->db->getDB_Result($query);

            $plain_result = array();
            for($i=0; $i<count($product_number); $i++)
            {
                array_push($plain_result, $product_number[$i]['id']);
            }

            return $plain_result;
        }
        else
        {
            return $products_id_list;
        }
    }

    /**
     * Searches the products in the DB by the name, using index words.
     * For the optimization used a direct searching in the table products
     * by the attributes available and stock level without filtering.
     * The searching result is limited by the const value PRODUCT_SEARCH_RESULT_LIMIT.
     *
     * @param array $words A plain list of index words (example: ('item','flower','day') )
     * @param int $condition Constant: PRODUCT_SEARCH_ANY_WORD or PRODUCT_SEARCH_ALL_WORD
     * @return array A plain list of product ids
     */
    function searchProductsByName($words, $condition)
    {
        global $application;

        $tables = Catalog::getTables();
	    $t_products = $tables['products']['columns'];

        $query = new DB_Select();
        $query->addSelectField($t_products['id'], 'id');

        for($i=0; $i<count($words); $i++)
        {
            $query->WhereValue($t_products['name'], DB_LIKE, '%'.$words[$i].'%');

            if ($i < count($words)-1)
            {
                if ($condition == PRODUCT_SEARCH_ALL_WORD)
                {
                    $query->WhereAnd();
                }
                else
                {
                    $query->WhereOr();
                }
            }
        }

        $query->SelectLimit(0,PRODUCT_SEARCH_RESULT_LIMIT);

        $result = $application->db->getDB_Result($query);

        $plain_result = array();
        for($i=0; $i<count($result); $i++)
        {
            array_push($plain_result, $result[$i]['id']);
        }

        return $plain_result;
    }

    /**
     * Searches the products in the DB by the attributes $attributes_id_list
     *
     * For the optimization used a direct searching in the table product_attributes
     * without filtering by the attributes available and stock level.
     *
     * To filter result you must call CatalogSearch::filterProducts
     *
     * The searching result is limited by the const value PRODUCT_SEARCH_RESULT_LIMIT.
     *
     * @param array $words A plain list of inde words (example: ('item','flower','day') )
     * @param int $condition Constant: PRODUCT_SEARCH_ANY_WORD or PRODUCT_SEARCH_ALL_WORD
     * @param array $attributes_id_list Search products by these attributes
     * @param array $prod_id_list_to_skip Skip these products in the search
     * @return array A plain list of product ids
     */
    function searchProductsByAttributes($words, $condition, $attributes_id_list, $prod_id_list_to_skip = array())
    {
        global $application;

        $tables = Catalog::getTables();
	    $t_product_attributes = $tables['product_attributes']['columns'];

        $query = new DB_Select();
        $query->addSelectField($t_product_attributes['p_id'], 'id');
        if (!empty($prod_id_list_to_skip))
        {
            $query->WhereField($t_product_attributes['p_id'], DB_NIN, "('" . implode("','", $prod_id_list_to_skip) . "')");
            $query->WhereAND();
        }
        $query->addWhereOpenSection();
            for($i=0; $i<count($attributes_id_list); $i++)
            {
                $query->WhereValue($t_product_attributes['a_id'], DB_EQ, $attributes_id_list[$i]);
                if ($i < count($attributes_id_list)-1)
                {
            $query->WhereOr();
                }
            }
        $query->addWhereCloseSection();
        $query->WhereAnd();
        $query->addWhereOpenSection();
            for($i=0; $i<count($words); $i++)
            {
                $query->WhereValue($t_product_attributes['attr_value'], DB_LIKE, '%'.$words[$i].'%');

                if ($i < count($words)-1)
                {
                    if ($condition == PRODUCT_SEARCH_ALL_WORD)
                    {
                        $query->WhereAnd();
                    }
                    else
                    {
                        $query->WhereOr();
                    }
                }
            }
        $query->addWhereCloseSection();

        $query->SelectLimit(0,PRODUCT_SEARCH_RESULT_LIMIT);

        $result = $application->db->getDB_Result($query);

        $plain_result = array();
        for($i=0; $i<count($result); $i++)
        {
            array_push($plain_result, $result[$i]['id']);
        }

        return $plain_result;
    }

    /**
     * Short alias to search by Short and Detailed Description.
     */
    function searchProductsByDescriptions($words, $condition, $prod_id_list_to_skip = array())
    {
        return $this->searchProductsByAttributes($words,
                                                 $condition,
                                                 array(SHORT_DESCRIPTION_PRODUCT_ATTRIBUTE_ID,
                                                       DETAILED_DESCRIPTION_PRODUCT_ATTRIBUTE_ID),
                                                 $prod_id_list_to_skip
                                                );
    }

    /**
     * Short alias to search by Meta Keywords and Meta Description.
     */
    function searchProductsByMetaData($words, $condition, $prod_id_list_to_skip = array())
    {
        return $this->searchProductsByAttributes($words,
                                                 $condition,
                                                 array(META_KEYWORDS_PRODUCT_ATTRIBUTE_ID,
                                                       META_DESCRIPTION_PRODUCT_ATTRIBUTE_ID),
                                                 $prod_id_list_to_skip
                                                );
    }

    /**
     * Short alias to search by SKU.
     */
    function searchProductsBySKU($words, $condition, $prod_id_list_to_skip = array())
    {
        return $this->searchProductsByAttributes($words,
                                                 $condition,
                                                 array(SKU_PRODUCT_ATTRIBUTE_ID),
                                                 $prod_id_list_to_skip
                                                );
    }

    /**
     * Returns a list of the index words from the random string $string,
     * which can be used for searching.
     * From the random string $string the following words will be deleted:
     * - shorter than PRODUCT_SEARCH_MIN_WORD_LENGHT
     * - longer than PRODUCT_SEARCH_MAX_WORD_LENGHT
     * - all HTML tags and HTML-characters
     *
     * @param string $string Search pattern
     * @return array Index words list
     */
    function getIndexWordList($string)
    {
        # TODO optimize
        $string = substr($string,0,PRODUCT_SEARCH_MAX_PATTERN_LENGHT);
        $string = $this->cstring->stripHTML($string);
        $string = $this->cstring->mergeWhiteSpace($string);
        $words = explode(" ",$string);

        $chars = array("$",".","!","?","@",",","#","%","^","&","*","(",")","_","+","=","(",")","{","}","[","]","\\","|",";",":","\"","<",">","/","~","-","'");

        $search = array();
        $replace = array();
        $search[] = "/([" . implode("\\", $chars) . "]+)$/";
        $search[] = "/^([" . implode("\\", $chars) . "]+)/";
        $replace[]= "\\2";
        $replace[]= "\\2";

        // remove wrong-length words in the array
        foreach($words as $key => $word)
        {
            if(0==strlen($word) ||
               strlen($word)<PRODUCT_SEARCH_MIN_WORD_LENGHT ||
               strlen($word)>PRODUCT_SEARCH_MAX_WORD_LENGHT)
            {
                unset($words[$key]);
            }
        }

        // keep memory usage low
        if(PRODUCT_SEARCH_MAX_WORD_NUMBER<count($words))
        {
            $words = array_splice($words, 0, PRODUCT_SEARCH_MAX_WORD_NUMBER);
        }

        // strip punctuation at the beginning and end of words
        $words = preg_replace($search, $replace, $words);

        // one more, remove wrong-length words
        foreach($words as $key => $word)
        {
            if(0==strlen($word) ||
               strlen($word)<PRODUCT_SEARCH_MIN_WORD_LENGHT ||
               strlen($word)>PRODUCT_SEARCH_MAX_WORD_LENGHT)
            {
                unset($words[$key]);
            }
        }

        return $words;
    }


    /**
     * Updates the time from the last access to the searching result to the
     * current one.
     *
     * @param int $search_id Search ID
     */
	function updateSearchResultAccessTime($search_id)
    {
        global $application;

        $tables = Catalog::getTables();
	    $t_products_search = $tables['products_search']['columns'];

        $query = &new DB_Update('products_search');
        $query->addUpdateValue($t_products_search['time'], date('Y-m-d H:i:s', time()));
        $query->WhereValue($t_products_search['id'], DB_EQ, $search_id);
        $application->db->getDB_Result($query);
    }


    var $cstring;
    /**#@-*/

}

?>