<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2007, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php
include_once(dirname(dirname(__FILE__)).'/catalog_api.php');
include_once(dirname(dirname(__FILE__)).'/const.php');


define ('CATEGORYINFO_DEFAULT_LOCALIZED_MODE', 0);
define ('CATEGORYINFO_LOCALIZED_DATA', 1);
define ('CATEGORYINFO_NOT_LOCALIZED_DATA', 2);

define ('IN_CATEGORY_ONLY', 3);
define ('IN_CATEGORY_RECURSIVELY', 4);
define ('IN_CATEGORY_RECURSIVELY_WITHOUT_CURRENT', 5);

define ('IMAGE_SMALL', 6);
define ('IMAGE_LARGE', 7);


/**
 * The class gives complete information about category.
 * To get this information, it's necessary to create the class object
 * and invoke the method getCategoryTagValue, specifying the tag.
 *
 * The class interface:
 * CCategoryInfo::CCategoryInfo($cid, $localized = CATEGORYINFO_DEFAULT_LOCALIZED_MODE)
 * CCategoryInfo::turnOnDebugMode()
 * CCategoryInfo::turnOffDebugMode()
 * CCategoryInfo::getCategoryTagValue($tag, $localized = CATEGORYINFO_DEFAULT_LOCALIZED_MODE)
 *
 * Example of usage:
 * <code>
 *  $category = new CCategoryInfo(8);
 *  $category_name = $category->getCategoryTagValue('name');
 * </code>
 *
 * @package Catalog
 * @author Alexey Florinsky
 */
class CCategoryInfo
{

    /**#@+
     * @access public
     */


    /**
	 * The class constructor.
	 * The object will be created only if the specified $cid (Category ID)
	 * is a correct value and such category exists in the DB.
	 *
	 * @param int $cid Category ID
	 * @param int $localized The parameter defines, whether to return localized
	 * values or to send as values the constants CATEGORYINFO_LOCALIZED_DATA or
	 * CATEGORYINFO_NOT_LOCALIZED_DATA directly from the DB
     * @return CCategoryInfo object or null
	 */
    function CCategoryInfo($cid, $localized = CATEGORYINFO_DEFAULT_LOCALIZED_MODE)
    {
		# If the product id is not correct, then the object will return the empty tag values
	    if ($cid == null || !is_numeric($cid) || !modApiFunc('Catalog', 'isCorrectCategoryId', $cid))
		{
		    $this->_fCategoryIDIsIncorrect = true;
		}

		$this->_fCategoryID = $cid;

		if  ($localized == CATEGORYINFO_LOCALIZED_DATA)
	    {
	        $this->localized = true;
	    }
	    else
	    {
	        $this->localized = false;
	    }

	    $this->debugmode = false;

	    # Flip the keys of the array _fArtificialTagList, i.e.
	    # the key values will equal the key itself.
	    # It is necessary to make it easier to use the array
	    $_tmp = $this->_fArtificialTagList;
	    $this->_fArtificialTagList = array();
	    foreach ($_tmp as $tag)
	    {
	        $this->_fArtificialTagList[$tag] = $tag;
	    }

        # Load base category info. It is a fast SQL query.
	    $this->_loadBaseCategoryInfo();
    }

    /**
	 * Turns on the debug mode. For the attributes whose values are not defined
	 * will be returned a text message instead of empty string, explaining why
	 * the values are not defined.
	 *
	 * @see CCategoryInfo::turnOffDebugMode()
	 *
	 */
	function turnOnDebugMode()
	{
	    $this->debugmode = true;
	}

    /**
	 * Turns off the debug mode.
	 *
	 * @see CCategoryInfo::turnOnDebugMode()
	 *
	 */
	function turnOffDebugMode()
	{
	    $this->debugmode = false;
	}

    /**
     * Returns Category info tag value.
     *
     * @param string $tag Short tag name
	 * @param int $localized The parameter defines, whether to return localized
	 * values or to send as values the constants CATEGORYINFO_LOCALIZED_DATA or
	 * CATEGORYINFO_NOT_LOCALIZED_DATA directly from the DB
	 * @return string Tag value
	 **/
    function getCategoryTagValue($tag, $localized = CATEGORYINFO_DEFAULT_LOCALIZED_MODE)
    {
	   global $application;

      /*
        If creating an object the incorrect id was passed to the category, then
        it will return an empty value of any tag.
       */
       if ($this->_fCategoryIDIsIncorrect == true)
       {
           return ($this->debugmode ? 'Category ID is incorrect' : '');
       }

       # Check additional tags
       if (array_key_exists($tag, $this->_fAdditionalTagList))
       {
           return $this->_fAdditionalTagList[$tag];
       }

       # Check localization request
       if  ($localized == CATEGORYINFO_LOCALIZED_DATA)
	   {
	       $this->localized = true;
	   }
	   elseif ($localized == CATEGORYINFO_NOT_LOCALIZED_DATA)
	   {
	       $this->localized = false;
	   }

       # Get Tag value
	   $tag = strtolower(trim($tag));
       switch ($tag)
       {
          #
          # Process artificial tags
          #
          case $this->_fArtificialTagList['id']:
              return $this->_fCategoryID;
              break;

          case $this->_fArtificialTagList['name']:
              return $this->localized ? prepareHTMLDisplay($this->_fBaseCategoryInfo[0]['name']) : $this->_fBaseCategoryInfo[0]['name'];
              break;

          case $this->_fArtificialTagList['status']:
              $cat_status = "";
              $MessageResources = &$application->getInstance('MessageResources');
              switch($this->_fBaseCategoryInfo[0]['status'])
              {
                  case CATEGORY_STATUS_ONLINE:
                      $cat_status = $MessageResources->getMessage('CAT_STATUS_ONLINE');
                      break;
                  case CATEGORY_STATUS_OFFLINE:
                      $cat_status = $MessageResources->getMessage('CAT_STATUS_OFFLINE');
                      break;
                  default:
                      //TODO: report error
                      $cat_status = "";
              }
              return $this->localized ? prepareHTMLDisplay($cat_status) : $this->_fBaseCategoryInfo[0]['status'];

              break;

          case $this->_fArtificialTagList['description']:
              return $this->localized ? prepareHTMLDisplay($this->_fBaseCategoryInfo[0]['descr']) : $this->_fBaseCategoryInfo[0]['descr'];
              break;

          case $this->_fArtificialTagList['largeimage']:
              if ($this->_fImagesData === null)
              {
                $this->_getImageInfo();
              }
              $_src = $this->_fImagesData['largeimage_file']['URL'];
              $_width = $this->_fImagesData['largeimage_file']['WIDTH'];
              $_height = $this->_fImagesData['largeimage_file']['HEIGHT'];
              $_alttext = $this->getCategoryTagValue('imagealttext');
              return '<img src="'.$_src.'" width="'.$_width.'" height="'.$_height.'" alt="'.$_alttext.'" border="0">';
              break;

          case $this->_fArtificialTagList['largeimagesrc']:
              if ($this->_fImagesData === null)
              {
                $this->_getImageInfo();
              }
              return $this->_fImagesData['largeimage_file']['URL'];
              break;

          case $this->_fArtificialTagList['largeimagewidth']:
              if ($this->_fImagesData === null)
              {
                $this->_getImageInfo();
              }
              return $this->_fImagesData['largeimage_file']['WIDTH'];
              break;

          case $this->_fArtificialTagList['largeimageheight']:
              if ($this->_fImagesData === null)
              {
                $this->_getImageInfo();
              }
              return $this->_fImagesData['largeimage_file']['HEIGHT'];
              break;

          case $this->_fArtificialTagList['smallimage']:
              if ($this->_fImagesData === null)
              {
                $this->_getImageInfo();
              }
              $_src = $this->_fImagesData['smallimage_file']['URL'];
              $_width = $this->_fImagesData['smallimage_file']['WIDTH'];
              $_height = $this->_fImagesData['smallimage_file']['HEIGHT'];
              $_alttext = $this->getCategoryTagValue('imagealttext');
              return '<img src="'.$_src.'" width="'.$_width.'" height="'.$_height.'" alt="'.$_alttext.'" border="0">';
              break;

          case $this->_fArtificialTagList['smallimagesrc']:
              if ($this->_fImagesData === null)
              {
                $this->_getImageInfo();
              }
              return $this->_fImagesData['smallimage_file']['URL'];
              break;

          case $this->_fArtificialTagList['smallimagewidth']:
              if ($this->_fImagesData === null)
              {
                $this->_getImageInfo();
              }
              return $this->_fImagesData['smallimage_file']['WIDTH'];
              break;

          case $this->_fArtificialTagList['smallimageheight']:
              if ($this->_fImagesData === null)
              {
                $this->_getImageInfo();
              }
              return $this->_fImagesData['smallimage_file']['HEIGHT'];
              break;

          case $this->_fArtificialTagList['imagealttext']:
              return $this->localized ? prepareHTMLDisplay($this->_fBaseCategoryInfo[0]['image_descr']) : $this->_fBaseCategoryInfo[0]['image_descr'];
              break;

          case $this->_fArtificialTagList['pagetitle']:
              return $this->localized ? prepareHTMLDisplay($this->_fBaseCategoryInfo[0]['page_title']) : $this->_fBaseCategoryInfo[0]['page_title'];
              break;

          case $this->_fArtificialTagList['metakeywords']:
              return $this->localized ? prepareHTMLDisplay($this->_fBaseCategoryInfo[0]['meta_keywords']) : $this->_fBaseCategoryInfo[0]['meta_keywords'];
              break;

          case $this->_fArtificialTagList['metadescription']:
              return $this->localized ? prepareHTMLDisplay($this->_fBaseCategoryInfo[0]['meta_descr']) : $this->_fBaseCategoryInfo[0]['meta_descr'];
              break;

          case $this->_fArtificialTagList['productsnumber']:
              return $this->_getProductsNumberInCategory(IN_CATEGORY_ONLY);
              break;

          case $this->_fArtificialTagList['productsnumberrecursively']:
              return $this->_getProductsNumberInCategory(IN_CATEGORY_RECURSIVELY);
              break;

          case $this->_fArtificialTagList['offlineandonlinesubcategoriesnumber']:
              if ($this->_fSubcategoriesNumber == null)
              {
                  $this->_loadSubcategoriesNumber();
              }
              return $this->_fSubcategoriesNumber;

          case $this->_fArtificialTagList['subcategoriesnumber']:
              $OnlineSubCategories = $this->_getOnlineSubcategories();
              $n_subcategories_including_current = sizeof($OnlineSubCategories);
              //                 (Navigator AZ)                                      ,   
              //                              .
              //             _loadOnlineSubCategories               Id      Online             ,
              //       id                  ,          Online,        .
              return $n_subcategories_including_current > 0 ?
                  $n_subcategories_including_current - 1 : // current is online
                  $n_subcategories_including_current; // current is itself offline

              break;

          case $this->_fArtificialTagList['left']:
              return $this->_fBaseCategoryInfo[0]['left1'];
              break;

          case $this->_fArtificialTagList['right']:
              return $this->_fBaseCategoryInfo[0]['right1'];
              break;

          case $this->_fArtificialTagList['level']:
              return $this->_fBaseCategoryInfo[0]['level'];
              break;

          case $this->_fArtificialTagList['parentid']:
              if ($this->_fParentCategoryID == null)
              {
                  $this->_loadParentID();
              }
              return $this->_fParentCategoryID;
              break;

          case $this->_fArtificialTagList['recursivestatus']:
              if ($this->_fRecursiveStatus == null)
              {
                  $this->_loadRecursiveStatus();
              }
              $cat_status = "";
              $MessageResources = &$application->getInstance('MessageResources');
              switch($this->_fRecursiveStatus)
              {
                  case CATEGORY_STATUS_ONLINE:
                      $cat_status = $MessageResources->getMessage('CAT_STATUS_ONLINE');
                      break;
                  case CATEGORY_STATUS_OFFLINE:
                      $cat_status = $MessageResources->getMessage('CAT_STATUS_OFFLINE');
                      break;
                  default:
                      //TODO: report error
                      $cat_status = "";
              }
              return $this->localized ? prepareHTMLDisplay($cat_status) : $this->_fRecursiveStatus;
              break;

          case $this->_fArtificialTagList['link']:
              $_request = &new Request();
              $_request->setView  ( 'NavigationBar' );
              $_request->setAction( "SetCurrCat" );
              $_request->setKey   ( "category_id", $this->_fCategoryID );
              $_request->setCategoryID($this->_fCategoryID);
              $_url = $_request->getURL();
              /*
              $_seo = $this->_fBaseCategoryInfo[0]['seo_url_prefix'];
              if($_seo != '')
              {
                list($_ss, $_qs) = explode('?',$_url);
                $_qs = $_seo.'-'.$_qs;
                $_url = implode('?',array($_ss, $_qs));
              }
              */
              return $_url;
              break;

          case $this->_fArtificialTagList['seo_url_prefix']:
              return $this->_fBaseCategoryInfo[0]['seo_url_prefix'];
              break;

          # If the tag is not defined, return an empty string.
          default:
              return $this->debugmode ? 'The tag is undefined:'.$tag : '';
       }

    }

    /**
     *                           :
     * CATEGORY_SHOW_PRODUCTS_RECURSIVELY     CATEGORY_DONOTSHOW_PRODUCTS_RECURSIVELY.
     *
     *           CATEGORY_DONOTSHOW_PRODUCTS_RECURSIVELY         ,             
     *                           "                                                     
     *                         ,                                           ".
     *
     *            CATEGORY_SHOW_PRODUCTS_RECURSIVELY         ,             
     *                           "                                              
     *                            ".
     *
     *                   (18      2007)                                    storefront
     *                               .
     */
    function getShowProductsRecursivelyStatus()
    {
        return $this->_fBaseCategoryInfo[0]['show_prod_recurs'];
    }

    /**
     * Defines additional tags and their values in this object.
     * To get a specified value, it is used getProductTagValue(). The tags, defined
     * by this method are more priority, then the others, i.e. the tag values
     * can be overridden.
     *
     * Example of usage:
     * <code>
     *
     * // Create a product object with ID=1
     * $prd = new CProductInfo(1);
     *
     * if ($prd === null) die('Product does not exists');
     *
     * // Get a product name
     * $product_name = $prd->getProductTagValue('Name');
     *
     * // Create a new tag 'Number' in the product with value '1'
     * $prd->setAdditionalProductTag('Number','1');
     *
     * // Override the product name
     * $prd->setAdditionalProductTag('Name','Product with ID = 1');
     *
     * </code>
     *
     * @param string $key Tag name
     * @param string $value Tag value
     */
    function setAdditionalCategoryTag($key, $value)
    {
        $this->_fAdditionalTagList[$key] = $value;
    }

    /**
     * Returns the associative array of the additional tags:
     * array( 'TagName' => 'TagValue', ...)
     *
     * This method is used to log the additional tags in the system.
     *
     * An example:
     * <code>
     * // Define additional tags
     * $productInfo->setAdditionalProductTag('Checked', true);
     * $productInfo->setAdditionalProductTag('Number', $i);
     *
     * // Log the additional tags, after logging they are available in the
     * // templates as PHP functions.
     * $application->registerAttributes( $productInfo->getAdditionalProductTagList() );
     * </code>
     *
     * @return array array( 'TagName' => 'TagValue', ...)
     */
    function getAdditionalCategoryTagList()
    {
        return $this->_fAdditionalTagList;
    }

    /**
     * Returns true if the specified tag exists in the given product object.
     * It searches among all the available tags:
     * - additional
     * - artificial
     * - in the attribute database.
     *
     * @param string $tag Tag name
     * @return boolean Return true if tag name exists in the object
     */
    function isTagExists($tag)
    {
       /*
        If creating an object the incorrect id was passed to the category, then
        the method will return false for any tag.
       */
       if ($this->_fCategoryIDIsIncorrect == true)
       {
           return ($this->debugmode ? 'Category ID is incorrect' : false);
       }

       # Check additional tag list
       if ( array_key_exists( $tag, $this->_fAdditionalTagList ) )
       {
           return true;
       }

       # Check artificial tag list
       if ( array_key_exists( strtolower($tag), $this->_fArtificialTagList ) )
       {
           return true;
       }

       return false;
    }


    /**#@-*/



    /**#@+
     * @access private
     */

    /**
     * Initializes the field of the class _fImagesData.
     * After executing this method the field $this->_fImagesData looks as follows:
     *
     * $this->_fImagesData = array('largeimage_file' => array('WIDTH' => XXX,
     *                                                        'HEIGHT' => XXX,
     *                                                        'URL' => '[url]'),
     *                             'smallimage_file' => array('WIDTH' => XXX,
     *                                                        'HEIGHT' => XXX,
     *                                                        'URL' => '[url]')
     *                                  );
     */
    function _getImageInfo()
    {
        global $application;

        if ($this->_fImagesData === null)
        {

            /* Define default values, in case the category doesn't have
               any images   */
            $this->_fImagesData = array('largeimage_file' => array('WIDTH' => 1,
                                                                   'HEIGHT' => 1,
                                                                   'URL' => ''),
                                        'smallimage_file' => array('WIDTH' => 1,
                                                                   'HEIGHT' => 1,
                                                                   'URL' => '')
                                       );

            /* Create a cycle, not to duplicate the following code*/
            $_cycle = array('largeimage_file', 'smallimage_file');

            $imagesUrl = $application->getAppIni('URL_IMAGES_DIR');
            if ($application->getCurrentProtocol() == "https" && $application->getAppIni('HTTPS_URL_IMAGES_DIR'))
            {
                $imagesUrl = $application->getAppIni('HTTPS_URL_IMAGES_DIR');
            }
            foreach ($_cycle as $_image_file_type)
            {
                if ( !empty($this->_fBaseCategoryInfo[0][$_image_file_type])
                     &&
                     $application->isImageFileValid($this->_fBaseCategoryInfo[0][$_image_file_type]) )
                {
                    $size = @getimagesize($application->getAppIni('PATH_IMAGES_DIR')
                            .$this->_fBaseCategoryInfo[0][$_image_file_type]);

                    $this->_fImagesData[$_image_file_type]['WIDTH']  = $size[0];
                    $this->_fImagesData[$_image_file_type]['HEIGHT'] = $size[1];
                    $this->_fImagesData[$_image_file_type]['URL']    = $imagesUrl.$this->_fBaseCategoryInfo[0][$_image_file_type];
                }
            }
        }
    }

    /**
     * Returns the product number in the category.
     * The parameter $calc_mode defines the method of calculating the product number:
     * IN_CATEGORY_ONLY - calculating the product number in the current category only,
     * IN_CATEGORY_RECURSIVELY - calculating the product number in the current
     * category and in all the nested ones
     * IN_CATEGORY_RECURSIVELY_WITHOUT_CURRENT - calculating the product number
     * in the nested categories only
     *
     * It invokes the method $this->_loadProductsNumberInCategory() to execute
     * SQL queries. The data are saved in the field $this->_fProductsNumberInCategory.
     * That means the next invocations of this method will not call the database
     * access.
     *
     * The product number, which will be returned, depends on the current zone.
     * If the current zone is Customer Zone, then it will be calculated only the
     * products, which:
     * - attribute Available is installed in Yes,
     * - attribute Available is installed in the empty string,
     * - attribute Available can't be seen in the product type.
     *
     * If the current zone is Admin Zone, then all products will be calculated.
     *
     * @see CCategoryInfo::_loadProductsNumberInCategory()
     * @param int $calc_mode The constant IN_CATEGORY_ONLY, IN_CATEGORY_RECURSIVELY
     *        or IN_CATEGORY_RECURSIVELY_WITHOUT_CURRENT
     * @return int the number of the products in the current category
     */
    function _getProductsNumberInCategory($calc_mode = IN_CATEGORY_ONLY)
    {
        switch ($calc_mode)
        {
            case IN_CATEGORY_RECURSIVELY:
                if ( !isset($this->_fProductsNumberInCategory[IN_CATEGORY_RECURSIVELY]) )
                {
                    $this->_fProductsNumberInCategory[IN_CATEGORY_RECURSIVELY] = $this->_loadProductsNumberInCategory(IN_CATEGORY_RECURSIVELY);
                }
                return $this->_fProductsNumberInCategory[IN_CATEGORY_RECURSIVELY];
                break;

            case IN_CATEGORY_RECURSIVELY_WITHOUT_CURRENT:
                if ( !isset($this->_fProductsNumberInCategory[IN_CATEGORY_RECURSIVELY_WITHOUT_CURRENT]) )
                {
                    $this->_fProductsNumberInCategory[IN_CATEGORY_RECURSIVELY_WITHOUT_CURRENT] = $this->_loadProductsNumberInCategory(IN_CATEGORY_RECURSIVELY_WITHOUT_CURRENT);
                }
                return $this->_fProductsNumberInCategory[IN_CATEGORY_RECURSIVELY_WITHOUT_CURRENT];
                break;

            default:
                if ( !isset($this->_fProductsNumberInCategory[IN_CATEGORY_ONLY]) )
                {
                    $this->_fProductsNumberInCategory[IN_CATEGORY_ONLY] = $this->_loadProductsNumberInCategory(IN_CATEGORY_ONLY);
                }
                return $this->_fProductsNumberInCategory[IN_CATEGORY_ONLY];
                break;
        }
    }

    /**
     * Executes SQL query, which loads base category info in the field
     * $this->_fBaseCategoryInfo.
     *
     */
    function _loadBaseCategoryInfo()
    {
        global $application;
	    $tables = Catalog::getTables();

        $c     = $tables['categories']['columns'];
        $cd    = $tables['categories_descr']['columns'];
        $p     = $tables['products']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($c['left'], 'left1');
        $query->addSelectField($c['right'], 'right1');
        $query->addSelectField($c['level'], 'level');
        $query->addSelectField($c['status'], 'status');
        $query->addSelectField($cd['name'], 'name');
        $query->addSelectField($cd['descr'], 'descr');
        $query->addSelectField($cd['image_file'], 'largeimage_file');
        $query->addSelectField($cd['image_small_file'], 'smallimage_file');
        $query->addSelectField($cd['image_descr'], 'image_descr');
        $query->addSelectField($cd['page_title'], 'page_title');
        $query->addSelectField($cd['meta_keywords'], 'meta_keywords');
        $query->addSelectField($cd['meta_descr'], 'meta_descr');
        $query->addSelectField($cd['show_prod_recurs'], 'show_prod_recurs');
        $query->addSelectField($cd['seo_url_prefix'], 'seo_url_prefix');

        $query->WhereValue($c['id'], DB_EQ, $this->_fCategoryID);
        $query->WhereAnd();
        $query->WhereField($c['id'], DB_EQ, $cd['id']);

        $this->_fBaseCategoryInfo = $application->db->getDB_Result($query);

/*      At the current moment (July, 2006), it isn't clear, if such a rigid
        checking should be done.

        if (sizeof($this->_fBaseCategoryInfo) != 1)
        {
            _fatal(array( "CODE" => "CORE_052"), __CLASS__, __FUNCTION__);
        }
 */
    }

    /**
     * Executes SQL query, which loads a parent category id in the field
     * $this->_fParentCategoryID.
     *
     */
    function _loadParentID()
    {
	    global $application;
	    $tables = Catalog::getTables();

        $c     = $tables['categories']['columns'];

        # define the parent category id
        //FIXME "Magic Number"
        if($this->_fCategoryID == 1)
        {
            $this->_fParentCategoryID = 1;
        }
        else
        {
            $query = &new DB_Select();
            $query->addSelectField($c['id'], 'id');
            $query->WhereValue($c['left'],  DB_LTE, $this->_fBaseCategoryInfo[0]['left1']);
            $query->WhereAnd();
            $query->WhereValue($c['right'], DB_GTE, $this->_fBaseCategoryInfo[0]['right1']);
            $query->WhereAnd();
            $query->WhereValue($c['level'], DB_EQ, $this->_fBaseCategoryInfo[0]['level']-1);
            $par = $application->db->getDB_Result($query);
            $this->_fParentCategoryID = $par[0]['id'];
        }

    }

    /**
     * Executes SQL query, which recursively computes category status: Online/Offline.
     * If parent category is Offline then child's status will be offline too.
     * $this->_fRecursiveStatus.
     *
     */
    function _loadRecursiveStatus()
    {
        global $application;
        $tables = Catalog::getTables();

        $c     = $tables['categories']['columns'];

        $query = &new DB_Select();
        $query->addSelectField("(COUNT(*) = 0)", 'b_online');
        $query->WhereValue($c['left'],  DB_LTE, $this->_fBaseCategoryInfo[0]['left1']);
        $query->WhereAnd();
        $query->WhereValue($c['right'], DB_GTE, $this->_fBaseCategoryInfo[0]['right1']);
        $query->WhereAnd();
        $query->WhereValue($c['status'], DB_EQ, CATEGORY_STATUS_OFFLINE);
        $par = $application->db->getDB_Result($query);

        $this->_fRecursiveStatus = $par[0]['b_online'] == 1 ? CATEGORY_STATUS_ONLINE : CATEGORY_STATUS_OFFLINE;

    }

    /**
     * Executes SQL query, which returns the product number in the current
     * category.
     *
     * The product number, which will be returned, depends on the current zone.
     * If the current zone is Customer Zone, then it will be calculated only the
     * products, which:
     * - attribute Available is installed in Yes,
     * - attribute Available is installed in the empty string, (                ,
     *                        ,                       ,     Yes)
     * - attribute Available can't be seen in the product type.
     *
     * If the current zone is Admin Zone, then all products will be calculated.
     *
     * @see CCategoryInfo::_loadProductsNumberInCategory()
     * @param int $calc_mode The constant IN_CATEGORY_ONLY, IN_CATEGORY_RECURSIVELY
     *        or IN_CATEGORY_RECURSIVELY_WITHOUT_CURRENT
     * @return int the number of the products in the current category
     */
    function _loadProductsNumberInCategory($calc_mode)
    {
	    global $application;

	    $cache_id = array($this->_fCategoryID, $calc_mode);
	    $cache = &$application->getInstance('CatalogCacheManager');
	    $result = $cache->CacheProductsNumberInCategories->get($cache_id);
	    if ( !($result === null) )
	    {
	        return $result;
	    }

	    $tables = Catalog::getTables();

        $t_products                = $tables['products']['columns'];
        //$t_product_type_attributes = $tables['product_type_attributes']['columns'];
        //$t_product_attributes      = $tables['product_attributes']['columns'];
        $t_categories              = $tables['categories']['columns'];

        $p2c = $tables['products_to_categories']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($query->fCount($p2c['product_id']), 'products_number');

        if ( $calc_mode == IN_CATEGORY_ONLY )
        {
            //          ,                                                           
            //                     .                  Customer Zone
            if (modApiFunc('Users', 'getZone') == "CustomerZone")
            {
                $objCurrCategory = &new CCategoryInfo($this->_fCategoryID);
                if ($objCurrCategory->getShowProductsRecursivelyStatus() == CATEGORY_SHOW_PRODUCTS_RECURSIVELY)
                {
                    $online_subcat_ids = $objCurrCategory->_getOnlineSubCategories();
                    $query->WhereField($p2c['category_id'], DB_IN," ('" . implode("','", $online_subcat_ids) . "') ");
                }
                else /* CATEGORY_DONOTSHOW_PRODUCTS_RECURSIVELY */
                {
                    $query->WhereValue($p2c['category_id'], DB_EQ, $this->_fCategoryID );
                }
            }
            else
            {
                $query->WhereValue($p2c['category_id'], DB_EQ, $this->_fCategoryID );
            }
        }
        else
        {
            if (modApiFunc('Users', 'getZone') == "CustomerZone")
            {
                $online_subcat_ids = $this->_getOnlineSubCategories();
                $query->Where($t_products['id'], DB_EQ, $p2c['product_id']);
                $query->WhereAND();
                if ( $calc_mode == IN_CATEGORY_RECURSIVELY )
                {
                    $query->WhereField($p2c['category_id'], DB_IN,"('" . implode("','", $online_subcat_ids) . "')");
                }
                else # IN_CATEGORY_RECURSIVELY_WITHOUT_CURRENT
                {
                    $key = array_search($this->_fCategoryID, $online_subcat_ids);
                    if($key)
                    {
                        unset($online_subcat_ids[$key]);
                    }
                    $query->WhereField($p2c['category_id'], DB_IN,"('" . implode("','", $online_subcat_ids) . "')");
                }
                $this->_applyProductsFilter($query);
            }
            else
            {
                $query->WhereField($t_categories['id'], DB_EQ, $p2c['category_id'] );
                $query->WhereAND();
                $query->WhereValue($t_categories['left'], DB_GTE, $this->_fBaseCategoryInfo[0]['left1'] );
                $query->WhereAND();
                if ( $calc_mode == IN_CATEGORY_RECURSIVELY )
                {
                    $query->WhereValue($t_categories['right'], DB_LTE, $this->_fBaseCategoryInfo[0]['right1'] );
                }
                else # IN_CATEGORY_RECURSIVELY_WITHOUT_CURRENT
                {
                    $query->WhereValue($t_categories['right'], DB_LT,  $this->_fBaseCategoryInfo[0]['right1'] );
                }
            }
        }

        $product_number = $application->db->getDB_Result($query);
        $result = $product_number[0]['products_number'];

	    $cache->CacheProductsNumberInCategories->save($cache_id, $result);

        return $result;
    }

    /**
     * Executes SQL query to calculate the number of subcategories in the current
     * category. The value is saved in the field $this->_fSubcategoriesNumber.
     *
     */
    function _loadSubcategoriesNumber()
    {
	    global $application;
	    $tables = Catalog::getTables();

        $c     = $tables['categories']['columns'];

        # calculate the number of subcategories for the given category
        $query = &new DB_Select();
        $query->addSelectField($query->fCount($c['id']), 'count_c_id');
        $query->WhereValue($c['left'], DB_GT, $this->_fBaseCategoryInfo[0]['left1']);
        $query->WhereAnd();
        $query->WhereValue($c['left'], DB_LT, $this->_fBaseCategoryInfo[0]['right1']);
        $sub = $application->db->getDB_Result($query);

        $this->_fSubcategoriesNumber = $sub[0]['count_c_id'] == NULL ? 0 : $sub[0]['count_c_id'];
    }

    /*
     *                   cid Online                  $root_cid.
     *                                             Offline          ,
     *                                           Offline                                .
     *                        Online,                        ,                             
     *                          id.
     *                               Offline,                              .
     */
    function _loadOnlineSubCategories(/*$root_cid*/)
    {
        if ($this->_fRecursiveStatus == null)
        {
            $this->_loadRecursiveStatus();
        }
        if($this->_fRecursiveStatus == CATEGORY_STATUS_OFFLINE)
        {
            //                            Offline (                     ),
            //                       .
            $this->OnlineSubCategories = array();
        }
        else
        {
            //                   Offline,         status = Offline,
            //                                       - Offline.           Online.

            //                              $root_cid             .
            //                            Online                        .
            //                  id.
            global $application;
            $tables = Catalog::getTables();

            $c     = $tables['categories']['columns'];

            /*
                                           ,                        .
                           , offline                     .
            */
            $query = &new DB_Select();
            $query->addSelectField($c['id'], 'id');
            $query->addSelectField($c['left'], 'left1');
            $query->addSelectField($c['right'], 'right1');
            $query->addSelectField($c['status'], 'status');
            $query->WhereValue($c['left'],  DB_GTE, $this->_fBaseCategoryInfo[0]['left1']);
            $query->WhereAnd();
            $query->WhereValue($c['right'], DB_LTE, $this->_fBaseCategoryInfo[0]['right1']);
            $result = $application->db->getDB_Result($query);

            //                      :
            //        -                left     "nested sets"        
            //          -       : id          , left, right, status.
            //                                    .
            $categories = array();
            foreach ($result as $key => $value)
            {
                $categories[$value["left1"]] = $value;
            }

            $online_categories = array();
            if(sizeof($categories) >= 1)
            {
                //Depth First Search (DFS) in the subtree. Skip all Offline vertices and their children.
                //Add all Online vertices into resulting list.
                //              left.                          DFS       .
                ksort($categories);
                //               ,             offline                      .
                $indices = array_keys($categories);
                $i = 0;
                for(;;)
                {
                    $left = $indices[$i];

                    if($categories[$left]["status"] == CATEGORY_STATUS_ONLINE)
                    {
                        //        Online -                                                              .
                        $online_categories[] = $categories[$left]["id"];
                        $i++;
                    }
                    else
                    {
                        //        Offline -                        (                   
                        //        DFS        .
                        //                    :                                 ,        
                        //                      .
                        //  ( . .      left                          right                
                        //                       ).
                        for(; $i < sizeof($indices) &&
                              $categories[$indices[$i]]["left1"] <= $categories[$left]["right1"];)
                        {
                            $i++;
                        }
                    }

                    if($i >= sizeof($indices))
                    {
                        //                .
                        break;
                    }
                }
            }
            else
            {
                //                .
            }
            $this->OnlineSubCategories = $online_categories;
        }
    }

    function _getOnlineSubCategories(/*$root_cid*/)
    {
        if ($this->OnlineSubCategories == null)
        {
            $this->_loadOnlineSubCategories();
        }
        return $this->OnlineSubCategories;
    }

    /**
     * Filters the products by attributess "Available" and "QuantityInStock"
     * to output a product list to the category in CZ.
     *
     * @param object $query -the initial Select query to filter the products in
     *        the category
     * @return object - the changed query with the applied filter
     */
    function _applyProductsFilter(&$query)
    {
        $tables = Catalog::getTables();

        $p = $tables['products']['columns'];
        $pa = $tables['product_attributes']['columns'];
        $pta = $tables['product_type_attributes']['columns'];
        $store_show_absent = modApiFunc("Configuration", "getValue", "store_show_absent");
        if ($store_show_absent == STORE_SHOW_ABSENT_NOT_SHOW_NOT_BUY)
        {
            $pa1 = $query->addTableAlias($pa, 'pa1');
            $pta1 = $query->addTableAlias($pta, 'pta1');
            $pa2 = $query->addTableAlias($pa, 'pa2');
            $pta2 = $query->addTableAlias($pta, 'pta2');

//        $query->WhereValue($p['c_id'], DB_EQ, 12);

            $query->WhereAND();
            $query->WhereField($p['pt_id'], DB_EQ, $pta1['pt_id']);
            $query->WhereAND();
            $query->WhereField($pa1['a_id'], DB_EQ, $pta1['a_id']);
            $query->WhereAND();
            $query->WhereField($pa1['p_id'], DB_EQ, $p['id']);

            $query->WhereAND();
            $query->WhereField($p['pt_id'], DB_EQ, $pta2['pt_id']);
            $query->WhereAND();
            $query->WhereField($pa2['a_id'], DB_EQ, $pta2['a_id']);
            $query->WhereAND();
            $query->WhereField($pa2['p_id'], DB_EQ, $p['id']);
            $query->WhereAND();
            $query->WhereField($pa1['a_id'], DB_NEQ, $pa2['a_id']);

            $query->WhereAND();
            $query->addWhereOpenSection();
                $query->addWhereOpenSection();
                    $query->WhereValue($pa1['a_id'], DB_EQ, 3); # stock level attribute
                    $query->WhereAND();
                    $query->addWhereOpenSection();
                        $query->addWhereOpenSection();
                            $query->WhereValue($pta1['type_attr_visible'], DB_EQ, 1);
                            $query->WhereAND();
                            $query->addWhereOpenSection();
                                $query->WhereValue($pa1['attr_value'], DB_GT, 0);
                                $query->WhereOR();
                                $query->WhereValue($pa1['attr_value'], DB_EQ, '');
                            $query->addWhereCloseSection();
                        $query->addWhereCloseSection();
                        $query->WhereOR();
                        $query->addWhereOpenSection();
                            $query->WhereValue($pta1['type_attr_visible'], DB_NEQ, 1);
                        $query->addWhereCloseSection();
                    $query->addWhereCloseSection();
                $query->addWhereCloseSection();
                $query->WhereAND();
                $query->addWhereOpenSection();
                    $query->WhereValue($pa2['a_id'], DB_EQ, 7);  # available attribute
                    $query->WhereAND();
                    $query->addWhereOpenSection();
                        $query->addWhereOpenSection();
                            $query->WhereValue($pta2['type_attr_visible'], DB_EQ, 1);
                            $query->WhereAND();
                            $query->addWhereOpenSection();
                                $query->WhereValue($pa2['attr_value'], DB_EQ, PRODUCT_STATUS_ONLINE);
                                $query->WhereOR();
                                $query->WhereValue($pa2['attr_value'], DB_EQ, '');
                            $query->addWhereCloseSection();
                        $query->addWhereCloseSection();
                        $query->WhereOR();
                        $query->addWhereOpenSection();
                            $query->WhereValue($pta2['type_attr_visible'], DB_NEQ, 1);
                        $query->addWhereCloseSection();
                    $query->addWhereCloseSection();
                $query->addWhereCloseSection();
            $query->addWhereCloseSection();
        }
        else
        {
            $query->WhereAND();
            $query->WhereField($p['pt_id'], DB_EQ, $pta['pt_id'] );
            $query->WhereAND();
            $query->WhereField($pa['a_id'], DB_EQ, $pta['a_id'] );
            $query->WhereAND();
            $query->WhereField($pa['p_id'], DB_EQ, $p['id'] );
            $query->WhereAND();
            $query->addWhereOpenSection();
            $query->WhereValue($pa['a_id'], DB_EQ, 7 ); # Check Available attribute
            $query->WhereAND();
            $query->addWhereOpenSection();
                $query->addWhereOpenSection();
                    $query->WhereValue($pta['type_attr_visible'], DB_EQ, '1' );
                    $query->WhereAND();
                    $query->addWhereOpenSection();
                        $query->WhereValue($pa['attr_value'], DB_EQ, PRODUCT_STATUS_ONLINE );
                        $query->WhereOR();
                        $query->WhereValue($pa['attr_value'], DB_EQ, '' );
                    $query->addWhereCloseSection();
                $query->addWhereCloseSection();
                $query->WhereOR();
                $query->addWhereOpenSection();
                    $query->WhereValue($pta['type_attr_visible'], DB_NEQ, '1' );
                $query->addWhereCloseSection();
            $query->addWhereCloseSection();
            $query->addWhereCloseSection();
        }
        return $query;
    }


    var $debugmode = false;
    var $_fCategoryID = null;
    var $_fCategoryIDIsIncorrect = false;
    var $_fParentCategoryID = null;
    var $_fRecursiveStatus = null;
    var $localized = true;
    var $_fBaseCategoryInfo = null;
    var $_fProductsNumberInCategory = null;
    var $_fSubcategoriesNumber = null;
    var $_fImagesData = null;

    var $_fAdditionalTagList = array();

    var $_fArtificialTagList = array( 'id'
                                     ,'name'
                                     ,'status'
                                     ,'description'
                                     ,'largeimage'
                                     ,'largeimagesrc'
                                     ,'largeimagewidth'
                                     ,'largeimageheight'
                                     ,'smallimage'
                                     ,'smallimagesrc'
                                     ,'smallimagewidth'
                                     ,'smallimageheight'
                                     ,'imagealttext'
                                     ,'pagetitle'
                                     ,'metakeywords'
                                     ,'metadescription'
                                     ,'productsnumber'
                                     ,'productsnumberrecursively'
                                     ,'offlineandonlinesubcategoriesnumber'
                                     ,'subcategoriesnumber'
                                     ,'left'
                                     ,'right'
                                     ,'level'
                                     ,'parentid'
                                     ,'recursivestatus'
                                     ,'link'
                                     ,'seo_url_prefix'
                                    );

    var $OnlineSubCategories = null;

    /**#@-*/

}

?>