<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2007, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

_use(dirname(__FILE__).'/abstract/catalog_search.php');
_use(dirname(__FILE__).'/abstract/product_class.php');
_use(dirname(__FILE__).'/abstract/category_class.php');
_use(dirname(__FILE__).'/abstract/cache_manager.php');

/**
 * Catalog module.
 * It works with inventory.
 *
 * @package Catalog
 * @author Alexander Girin
 * @access public
 */
class Catalog
{
//------------------------------------------------
//               PUBLIC DECLARATION
//------------------------------------------------

    /**#@+
     * @access public
     */

    /**
     * Catalog module constructor.
     */
    function Catalog()
    {
    }

    /**
     *                                          .
     */
    function loadState()
    {
        // restore the current selected category
        if(modApiFunc('Session', 'is_Set', 'currentCategoryID'))
        {
            $this->setCurrentCategoryID(modApiFunc('Session', 'get', 'currentCategoryID'));
        }
        else
        {
            $this->currentCategoryID = 1;
        }

        // restore the current selected product
        if(modApiFunc('Session', 'is_Set', 'moveToCategoryID'))
        {
            $this->setMoveToCategoryID(modApiFunc('Session', 'get', 'moveToCategoryID'));
        }
        else
        {
            $this->moveToCategoryID = $this->currentCategoryID;
        }

        // restore the current selected category
        if(modApiFunc('Session', 'is_Set', 'editableCategoryID'))
        {
            $this->setEditableCategoryID(modApiFunc('Session', 'get', 'editableCategoryID'));
        }
        else
        {
            $this->editableCategoryID = NULL;
        }

        //  restore the current selected edited product
        if(modApiFunc('Session', 'is_Set', 'editableProductsID'))
        {
            $this->setEditableProductsID(modApiFunc('Session', 'get', 'editableProductsID'));
        }
        else
        {
            $this->editableProductsID = array();
        }

        // restore the current selected product
        if(modApiFunc('Session', 'is_Set', 'currentProductID'))
        {
            $this->currentProductID = modApiFunc('Session', 'get', 'currentProductID');
        }
        else
        {
            $this->currentProductID = NULL;
        }

        //   restore the current selected product type
        if(modApiFunc('Session', 'is_Set', 'currentProductTypeID'))
        {
            $this->currentProductTypeID = modApiFunc('Session', 'get', 'currentProductTypeID');
        }

        /**
         * Restore the variable, wich is used to define the product info output
         * when deleting and browsing its attributes.
         */
        if(modApiFunc('Session', 'is_Set', 'DisplayDeleteInfo'))
        {
            $this->DisplayDeleteInfo = modApiFunc('Session', 'get', 'DisplayDeleteInfo');
            modApiFunc('Session', 'un_Set', 'DisplayDeleteInfo');
        }

        $this->registerAttributes();
    }

    /**
     * Saves the module state.
     */
    function saveState()
    {
        modApiFunc('Session', 'set', 'currentCategoryID', $this->currentCategoryID);
        modApiFunc('Session', 'set', 'currentProductID', $this->currentProductID);
        modApiFunc('Session', 'set', 'currentProductTypeID', $this->currentProductTypeID);
        modApiFunc('Session', 'set', 'editableCategoryID', $this->editableCategoryID);
        modApiFunc('Session', 'set', 'editableProductsID', $this->editableProductsID);
        modApiFunc('Session', 'set', 'moveToCategoryID', $this->moveToCategoryID);
        modApiFunc('Session', 'set', 'DisplayDeleteInfo', $this->DisplayDeleteInfo);
    }

    /**
     * Returns the meta description of the database tables, specified for storing
     * the Catalog module data.
     *
     * @return array tables meta info
     */
    function getTables ()
    {
        static $tables;

        if (is_array($tables))
        {
        	return $tables;
        }

        $tables = array ();

        $categories = 'categories';
        $tables[$categories] = array();
        $tables[$categories]['columns'] = array
            (
                'id'                => 'categories.category_id'
               ,'left'              => 'categories.category_left'
               ,'right'             => 'categories.category_right'
               ,'level'             => 'categories.category_level'
               ,'status'            => 'categories.category_status'
               ,'sort_order'        => 'categories.category_sort_order'
            );
        $tables[$categories]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'left'              => DBQUERY_FIELD_TYPE_INT .' NOT NULL'
               ,'right'             => DBQUERY_FIELD_TYPE_INT .' NOT NULL'
               ,'level'             => DBQUERY_FIELD_TYPE_INT .' NOT NULL'
               ,'status'            => DBQUERY_FIELD_TYPE_INT .' NOT NULL default \'1\'' //1 - online, 2 - offline
               ,'sort_order'        => DBQUERY_FIELD_TYPE_INT .' NOT NULL'
            );
        $tables[$categories]['primary'] = array
            (
                'id'
            );
        $tables[$categories]['indexes'] = array
            (
                'IDX_lrl' => 'left, right, level'
               ,'IDX_left' => 'left'
               ,'IDX_right' => 'right'
               ,'IDX_level' => 'level'
               ,'IDX_lr' => 'left, right'
            );

        $categories_descr = 'categories_descr';
        $tables[$categories_descr] = array();
        $tables[$categories_descr]['columns'] = array
            (
                'id'                => 'categories_descr.category_id'
               ,'name'              => 'categories_descr.category_name'
               ,'descr'             => 'categories_descr.category_descr'
               ,'image_file'        => 'categories_descr.category_image_file'
               ,'image_small_file'  => 'categories_descr.category_image_small_file'
               ,'image_descr'       => 'categories_descr.category_image_descr'
               ,'page_title'        => 'categories_descr.category_page_title'
               ,'meta_keywords'     => 'categories_descr.category_meta_keywords'
               ,'meta_descr'        => 'categories_descr.category_meta_descr'
               ,'show_prod_recurs'  => 'categories_descr.category_show_prod_recurs'
               ,'seo_url_prefix'    => 'categories_descr.category_seo_url_prefix'
            );
        $tables[$categories_descr]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL'
               ,'name'              => DBQUERY_FIELD_TYPE_CHAR255 .' NOT NULL'
               ,'descr'             => DBQUERY_FIELD_TYPE_TEXT
               ,'image_file'        => DBQUERY_FIELD_TYPE_CHAR255
               ,'image_small_file'  => DBQUERY_FIELD_TYPE_CHAR255
               ,'image_descr'       => DBQUERY_FIELD_TYPE_CHAR255
               ,'page_title'        => DBQUERY_FIELD_TYPE_CHAR255
               ,'meta_keywords'     => DBQUERY_FIELD_TYPE_TEXT
               ,'meta_descr'        => DBQUERY_FIELD_TYPE_TEXT
               ,'show_prod_recurs'  => DBQUERY_FIELD_TYPE_INT . ' NOT NULL default '.CATEGORY_DONOTSHOW_PRODUCTS_RECURSIVELY
               ,'seo_url_prefix'    => DBQUERY_FIELD_TYPE_CHAR255
            );
        $tables[$categories_descr]['indexes'] = array
            (
                'IDX_ci' => 'id'
            );

        $products = 'products';
        $tables[$products] = array();
        $tables[$products]['columns'] = array
            (
                'id'                => 'products.product_id'
               ,'pt_id'             => 'products.product_type_id'
               ,'name'              => 'products.product_name'
               ,'date_added'        => 'products.product_date_added'
               ,'date_updated'      => 'products.product_date_updated'
               ,'date_available'    => 'products.product_date_available'
            );
        $tables[$products]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'pt_id'             => DBQUERY_FIELD_TYPE_INT .' NOT NULL'
               ,'name'              => DBQUERY_FIELD_TYPE_CHAR255
               ,'date_added'        => DBQUERY_FIELD_TYPE_DATE . ' default \'0000-00-00\''
               ,'date_updated'      => DBQUERY_FIELD_TYPE_DATE . ' default \'0000-00-00\''
               ,'date_available'    => DBQUERY_FIELD_TYPE_DATE . ' default \'0000-00-00\''
            );
        $tables[$products]['primary'] = array
            (
                'id'
            );
        $tables[$products]['indexes'] = array
            (
                'IDX_pti' => 'pt_id'
            );

        $p_to_c = 'products_to_categories';
        $tables[$p_to_c] = array();
        $tables[$p_to_c]['columns'] = array(
            'record_id'     =>  $p_to_c.'.record_id'
           ,'product_id'    =>  $p_to_c.'.product_id'
           ,'category_id'   =>  $p_to_c.'.category_id'
           ,'sort_order'    =>  $p_to_c.'.sort_order'
        );
        $tables[$p_to_c]['types'] = array(
            'record_id'     =>  DBQUERY_FIELD_TYPE_INT.' NOT NULL auto_increment'
           ,'product_id'    =>  DBQUERY_FIELD_TYPE_INT.' NOT NULL'
           ,'category_id'   =>  DBQUERY_FIELD_TYPE_INT.' NOT NULL'
           ,'sort_order'    =>  DBQUERY_FIELD_TYPE_INT.' NOT NULL default 0'
        );
        $tables[$p_to_c]['primary'] = array(
            'record_id'
        );
        $tables[$p_to_c]['indexes'] = array(
            'UNIQUE KEY IDX_pc' => 'product_id,category_id'
           ,'IDX_cid' => 'category_id'
        );

        $product_types = 'product_types';
        $tables[$product_types] = array();
        $tables[$product_types]['columns'] = array
            (
                'id'                => 'product_types.product_type_id'
               ,'name'              => 'product_types.product_type_name'
               ,'descr'             => 'product_types.product_type_descr'
            );
        $tables[$product_types]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'name'              => DBQUERY_FIELD_TYPE_CHAR255
               ,'descr'             => DBQUERY_FIELD_TYPE_TEXT
            );
        $tables[$product_types]['primary'] = array
            (
                'id'
            );

        $attributes = 'attributes';
        $tables[$attributes] = array();
        $tables[$attributes]['columns'] = array
            (
                'id'                => 'attributes.attribute_id'
               ,'ag_id'             => 'attributes.attribute_group_id'
               ,'it_id'             => 'attributes.input_type_id'
               ,'ut'                => 'attributes.unit_type'
               ,'view_tag'          => 'attributes.attribute_view_tag'
               ,'name'              => 'attributes.attribute_name'
               ,'descr'             => 'attributes.attribute_descr'
               ,'type'              => 'attributes.attribute_type'
               ,'allow_html'        => 'attributes.attribute_allow_html'
               ,'min'               => 'attributes.attribute_min_value'
               ,'max'               => 'attributes.attribute_max_value'
               ,'size'              => 'attributes.attribute_html_size'
               ,'sort_order'        => 'attributes.attribute_sort_order'
            );
        $tables[$attributes]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'ag_id'             => DBQUERY_FIELD_TYPE_INT .' NOT NULL'
               ,'it_id'             => DBQUERY_FIELD_TYPE_INT
               ,'ut'                => DBQUERY_FIELD_TYPE_CHAR10
               ,'view_tag'          => DBQUERY_FIELD_TYPE_CHAR100
               ,'name'              => DBQUERY_FIELD_TYPE_CHAR255
               ,'descr'             => DBQUERY_FIELD_TYPE_TEXT
               ,'type'              => DBQUERY_FIELD_TYPE_CHAR10
               ,'allow_html'        => DBQUERY_FIELD_TYPE_INT
               ,'min'               => DBQUERY_FIELD_TYPE_CHAR255
               ,'max'               => DBQUERY_FIELD_TYPE_CHAR255
               ,'size'              => DBQUERY_FIELD_TYPE_CHAR255
               ,'sort_order'        => DBQUERY_FIELD_TYPE_INT
            );
        $tables[$attributes]['primary'] = array
            (
                'id'
            );
        $tables[$attributes]['indexes'] = array
            (
                'IDX_pagi' => 'ag_id'
               ,'IDX_iti'  => 'it_id'
               ,'IDX_aso'  => 'sort_order'
            );

        $attribute_groups = 'attribute_groups';
        $tables[$attribute_groups] = array();
        $tables[$attribute_groups]['columns'] = array
            (
                'id'                => 'attribute_groups.attribute_group_id'
               ,'name'              => 'attribute_groups.attribute_group_name'
               ,'sort_order'        => 'attribute_groups.attribute_group_sort_order'
            );
        $tables[$attribute_groups]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'name'              => DBQUERY_FIELD_TYPE_CHAR255
               ,'sort_order'        => DBQUERY_FIELD_TYPE_INT
            );
        $tables[$attribute_groups]['primary'] = array
            (
                'id'
            );
        $tables[$attribute_groups]['indexes'] = array
            (
                'IDX_agso'  => 'sort_order'
            );

        $product_type_attributes = 'product_type_attributes';
        $tables[$product_type_attributes] = array();
        $tables[$product_type_attributes]['columns'] = array
            (
                'id'                => 'product_type_attributes.product_type_attr_id'
               ,'pt_id'             => 'product_type_attributes.product_type_id'
               ,'a_id'              => 'product_type_attributes.attribute_id'
               ,'type_attr_visible' => 'product_type_attributes.product_type_attr_visibility'
               ,'type_attr_required'=> 'product_type_attributes.product_type_attr_required'              //new field
               ,'type_attr_def_val' => 'product_type_attributes.product_type_attr_default_value'
            );
        $tables[$product_type_attributes]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'pt_id'             => DBQUERY_FIELD_TYPE_INT .' NOT NULL'
               ,'a_id'              => DBQUERY_FIELD_TYPE_INT
               ,'type_attr_visible' => DBQUERY_FIELD_TYPE_BOOL
               ,'type_attr_required'=> DBQUERY_FIELD_TYPE_BOOL
               ,'type_attr_def_val' => DBQUERY_FIELD_TYPE_TEXT
            );
        $tables[$product_type_attributes]['primary'] = array
            (
                'id'
            );
        $tables[$product_type_attributes]['indexes'] = array
            (
                'IDX_pti' => 'pt_id'
               ,'IDX_ai'  => 'a_id'
               ,'IDX_aid_vis'  => 'a_id, type_attr_visible'
               ,'IDX_ptid_aid_vis'  => 'pt_id, a_id, type_attr_visible'
               ,'IDX_ptid_aid'  => 'pt_id, a_id'
            );

        $product_attributes = 'product_attributes';
        $tables[$product_attributes] = array();
        $tables[$product_attributes]['columns'] = array
            (
                'id'                => 'product_attributes.product_attr_id'
               ,'p_id'              => 'product_attributes.product_id'
               ,'a_id'              => 'product_attributes.attribute_id'
               ,'attr_value'        => 'product_attributes.product_attr_value'
            );
        $tables[$product_attributes]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'p_id'              => DBQUERY_FIELD_TYPE_INT .' NOT NULL'
               ,'a_id'              => DBQUERY_FIELD_TYPE_INT .' NOT NULL'
               ,'attr_value'        => DBQUERY_FIELD_TYPE_TEXT
            );
        $tables[$product_attributes]['primary'] = array
            (
                'id'
            );
        $tables[$product_attributes]['indexes'] = array
            (
                'IDX_pi'  => 'p_id'
               ,'IDX_ai'  => 'a_id'
               ,'IDX_pid_aid'  => 'p_id, a_id'
            );

        $product_images = 'product_images';
        $tables[$product_images] = array();
        $tables[$product_images]['columns'] = array
            (
                'id'                => 'product_images.image_id'
               ,'pa_id'             => 'product_images.product_attr_id'
               ,'name'              => 'product_images.image_name'
               ,'width'             => 'product_images.image_width'
               ,'height'            => 'product_images.image_height'
            );
        $tables[$product_images]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'pa_id'             => DBQUERY_FIELD_TYPE_INT .' NOT NULL'
               ,'name'              => DBQUERY_FIELD_TYPE_CHAR255 .' NOT NULL'
               ,'width'             => DBQUERY_FIELD_TYPE_INT .' NOT NULL'
               ,'height'            => DBQUERY_FIELD_TYPE_INT .' NOT NULL'
            );
        $tables[$product_images]['primary'] = array
            (
                'id'
            );
        $tables[$product_images]['indexes'] = array
            (
                'IDX_pi'  => 'pa_id'
            );

        $input_types = 'input_types';
        $tables[$input_types] = array();
        $tables[$input_types]['columns'] = array
            (
                'id'                => 'input_types.input_type_id'
               ,'ut_id'             => 'input_types.unit_type_id'
               ,'name'              => 'input_types.input_type_name'
            );
        $tables[$input_types]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'ut_id'             => DBQUERY_FIELD_TYPE_INT .' NOT NULL DEFAULT 0'
               ,'name'              => DBQUERY_FIELD_TYPE_CHAR50
            );
        $tables[$input_types]['primary'] = array
            (
                'id'
            );
        $tables[$input_types]['indexes'] = array
            (
                'IDX_uti' => 'ut_id'
            );

        $input_type_values = 'input_type_values';
        $tables[$input_type_values] = array();
        $tables[$input_type_values]['columns'] = array
            (
                'id'                => 'input_type_values.input_type_value_id'
               ,'it_id'             => 'input_type_values.input_type_id'
               ,'value'             => 'input_type_values.input_type_value'
            );
        $tables[$input_type_values]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'it_id'             => DBQUERY_FIELD_TYPE_INT
               ,'value'             => DBQUERY_FIELD_TYPE_CHAR50
            );
        $tables[$input_type_values]['primary'] = array
            (
                'id'
            );
        $tables[$input_type_values]['indexes'] = array
            (
                'IDX_iti' => 'it_id'
            );

        $catalog_temp = 'catalog_temp';
        $tables[$catalog_temp] = array();
        $tables[$catalog_temp]['columns'] = array
            (
                'id'                => 'catalog_temp.catalog_temp_id'
               ,'form_id'           => 'catalog_temp.catalog_temp_form_id'
               ,'value'             => 'catalog_temp.catalog_temp_value'
            );
        $tables[$catalog_temp]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'form_id'           => DBQUERY_FIELD_TYPE_INT
               ,'value'             => DBQUERY_FIELD_TYPE_TEXT
            );
        $tables[$catalog_temp]['primary'] = array
            (
                'id'
            );

        $products_search = 'products_search';
        $tables[$products_search] = array();
        $tables[$products_search]['columns'] = array
            (
                'id'                => 'products_search.products_search_id'
               ,'pattern'           => 'products_search.products_search_pattern'
               ,'time'              => 'products_search.products_search_time'
               ,'words'             => 'products_search.products_search_words'
            );
        $tables[$products_search]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'pattern'           => DBQUERY_FIELD_TYPE_TEXT
               ,'time'              => DBQUERY_FIELD_TYPE_DATETIME
               ,'words'             => DBQUERY_FIELD_TYPE_TEXT
            );
        $tables[$products_search]['primary'] = array
            (
                'id'
            );

        $products_search_result = 'products_search_result';
        $tables[$products_search_result] = array();
        $tables[$products_search_result]['columns'] = array
            (
                'id'                => 'products_search_result.products_search_id'
               ,'p_id'              => 'products_search_result.product_id'
               ,'relevance'         => 'products_search_result.relevance'
            );
        $tables[$products_search_result]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL '
               ,'p_id'              => DBQUERY_FIELD_TYPE_INT .' NOT NULL '
               ,'relevance'         => DBQUERY_FIELD_TYPE_INT .' NOT NULL '
            );
        $tables[$products_search_result]['primary'] = array
            (
                'id', 'p_id'
            );
        $tables[$products_search_result]['indexes'] = array
            (
                'IDX_relevance' => 'relevance'
            );


        global $application;
        return $application->addTablePrefix($tables);
    }

    /**
     * Returns the category technical specifications: left, right, level.
     * Caching is not used.
     */
    function fetchCategoryInfo($cid)
    {
        global $application;
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Start, Category ID = '.$cid);
        $retval = false;
        if ($this->isCorrectCategoryId($cid))
        {
            $tables = $this->getTables();   #meta description of the Catalog module tables

            $field = $tables['categories']['columns'];

            $query = &new DB_Select();
            $query->addSelectField($field['left'], 'left1');
            $query->addSelectField($field['right'], 'right1');
            $query->addSelectField($field['status'], 'status');
            $query->addSelectField($field['level'], 'level');
            $query->WhereValue($field['id'], DB_EQ, $cid);

            $result = $application->db->getDB_Result($query);
            if (sizeof($result) != 1)
            {
                _fatal(array( "CODE" => "CORE_052"), __CLASS__, __FUNCTION__);
            }
            $retval = array (
                "left"           => $result[0]['left1']
               ,"right"          => $result[0]['right1']
               ,"status"         => $result[0]['status']
               ,"level"          => $result[0]['level']
            );
        }
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End, Result returned');
        return $retval;
    }

    /**
     * Adds a filter to select only the products, available in the category
     * and, if necessary, those products, which number is grater than 0.
     *
     * @param ogject $query - the Select type query
     * @return object $query
     */
    function getAvailableProductsQuery($query)
    {
        if (modApiFunc('Users', 'getZone') == "CustomerZone")
        {
            global $application;
            $tables = $this->getTables();
            $p     = $tables['products']['columns'];

            $query->addSelectField($p['id'], 'id');
            CCategoryInfo::_applyProductsFilter($query);
            $query->SelectGroup($p['id']);
        }
        return $query;
    }

    /**
     * Returns category info with the specified id.
     * If it is allowed to use cache and it contains required data, then info
     * is restored from cache.
     * The array of returned attributes:
     * <code>
     * array (
     *   "ID"             => // the category id
     *  ,"Name"           => // the category name
     *  ,"Status"         => // the category status
     *  ,"Description"    => // the category description
     *  ,"LargeImage"     => // the large image of the category
     *  ,"SmallImage"     => // the small image of the category
     *  ,"ImageAltText"   => // the description of the category image
     *  ,"PageTitle"      => // the title page fro the given category
     *  ,"MetaKeywords"   => // the list of keywords for the category
     *  ,"MetaDescription"=> // the category meta description
     *  ,"ProductsNumber" => // the product number in the category
     *  ,"ProductsNumberRecursively" => // the product number in this category
     *                                  // and in all subcategories
     *  ,"SubcategoriesNumber" => // the subcategoies number
     *  ,"Link"           => // the URL reference to the given category
     *  ,"ParentID"       => // the parent category id
     *  ,"left"           => // specific parameters
     *  ,"right"          => //
     *  ,"level"          => //
     * )
     * </code><FONT SIZE="" COLOR=""></FONT>
     * @param integer $cid the category id.
     * @return array the array with the category description.
     */
    function getCategoryInfo($cid)
    {
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Start, Category ID = '.$cid);
        global $application;

        if (!$this->isCorrectCategoryId($cid))
        {
            log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Category ID is incorrect, NULL Returned');
        	return null;
        }

        $tables = $this->getTables();
        $c     = $tables['categories']['columns'];
        $cd    = $tables['categories_descr']['columns'];
        $p     = $tables['products']['columns'];
        $p2c   = $tables['products_to_categories']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($c['left'], 'left1');
        $query->addSelectField($c['right'], 'right1');
        $query->addSelectField($c['level'], 'level');
        $query->addSelectField($c['status'], 'status');
        $query->addSelectField($cd['name'], 'name');
        $query->addSelectField($cd['descr'], 'descr');
        $query->addSelectField($cd['image_file'], 'largeImage_file');
        $query->addSelectField($cd['image_small_file'], 'smallImage_file');
        $query->addSelectField($cd['image_descr'], 'image_descr');
        $query->addSelectField($cd['page_title'], 'page_title');
        $query->addSelectField($cd['meta_keywords'], 'meta_keywords');
        $query->addSelectField($cd['meta_descr'], 'meta_descr');
        $query->addSelectField($cd['show_prod_recurs'], 'show_prod_recurs');
        $query->addSelectField($cd['seo_url_prefix'], 'seo_url_prefix');

        $query->WhereValue($c['id'], DB_EQ, $cid);
        $query->WhereAnd();
        $query->WhereField($c['id'], DB_EQ, $cd['id']);

        $result = $application->db->getDB_Result($query);
        if (sizeof($result) != 1)
        {
            _fatal(array( "CODE" => "CORE_052"), __CLASS__, __FUNCTION__);
        }

        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] After Select common category info');

        # determine the parent category id
        //FIXME "Magic Number"
        if($cid == 1)
        {
            $parent = $cid;
        }
        else
        {
            $query = &new DB_Select();
            $query->addSelectField($c['id'], 'id');
            $query->WhereValue($c['left'], DB_LTE, $result[0]['left1']);
            $query->WhereAnd();
            $query->WhereValue($c['right'], DB_GTE, $result[0]['right1']);
            $query->WhereAnd();
            $query->WhereValue($c['level'], DB_EQ, $result[0]['level']-1);
            $par = $application->db->getDB_Result($query);
            $parent = $par[0]['id'];
        }

        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Parent Category ID determined');

        # determine the product number in the category
        $query = &new DB_Select();
        $query->addSelectField($query->fCount($p2c['product_id']), 'count_p_id');
        $query->WhereValue($p2c['category_id'], DB_EQ, $cid);
        $query = $this->getAvailableProductsQuery($query);
        $prods = $application->db->getDB_Result($query);

		if (modApiFunc('Users', 'getZone') == "CustomerZone")
        {
            $prods_number = sizeof($prods);
        }
        else
        {
            $prods_number = $prods[0]['count_p_id'];
        }

        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Products number in category calculated');

        # deftermine the product number in this category and in all subcategories
        //FIXME2 there may be linked products in several subcategories
        $query = &new DB_Select();
        $query->addSelectField($query->fCount($p2c['product_id']), 'count_p_id');
        $query->WhereValue($c['left'], DB_GTE, $result[0]['left1']);
        $query->WhereAnd();
        $query->WhereValue($c['left'], DB_LT, $result[0]['right1']);
        $query->WhereAnd();
        $query->WhereField($c['id'], DB_EQ, $p2c['category_id']);
        $query = $this->getAvailableProductsQuery($query);
        $prods = $application->db->getDB_Result($query);

		if (modApiFunc('Users', 'getZone') == "CustomerZone")
        {
            $prods_number_recurse = sizeof($prods);
        }
        else
        {
            $prods_number_recurse = $prods[0]['count_p_id'];
        }

        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Recursive Products number in category calculated');

        # determine the number of subcategories for the given category
        $query = &new DB_Select();
        $query->addSelectField($query->fCount($c['id']), 'count_c_id');
        $query->WhereValue($c['left'], DB_GT, $result[0]['left1']);
        $query->WhereAnd();
        $query->WhereValue($c['left'], DB_LT, $result[0]['right1']);

        $sub = $application->db->getDB_Result($query);
        $sub_number = $sub[0]['count_c_id'] == NULL ? 0 : $sub[0]['count_c_id'];

        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Number of subcategories calculated');

        $_request = &new Request();
        $_request->setView  ( 'NavigationBar' );
        $_request->setAction( "SetCurrCat" );
        $_request->setKey   ( "category_id", $cid );
        $_request->setCategoryID($cid);
        $info_link = $_request->getURL();

        // image processing
        $images = array(
            'smallImage' => array(
                 'show_that_file_is_not_uploaded' => '0'
                ,'img' => ''
                ,'src' => ''
                ,'width' => 1
                ,'height' => 1
            ),
            'largeImage' => array(
                 'show_that_file_is_not_uploaded' => '0'
                ,'img' => ''
                ,'src' => ''
                ,'width' => 1
                ,'height' => 1
            )
        );
        foreach ($images as $image => $imageInfo)
        {
            $imgAddress = $application->getAppIni('URL_IMAGES_DIR');
            if ($application->getCurrentProtocol() == "https" && $application->getAppIni('HTTPS_URL_IMAGES_DIR'))
            {
                $imgAddress = $application->getAppIni('HTTPS_URL_IMAGES_DIR');
            }
            if (!empty($result[0][$image.'_file']) &&
                ($application->isImageFileValid($result[0][$image.'_file'])))
            {
                $size = @getimagesize($application->getAppIni('PATH_IMAGES_DIR').$result[0][$image.'_file']);
            	$imageInfo['src'] = $imgAddress.$result[0][$image.'_file'];
            	$imageInfo['width'] = $size[0];
            	$imageInfo['height'] = $size[1];
        	    $imageInfo['img'] = '<img src="'.$imageInfo['src'].'" width="'.$imageInfo['width'].'" height="'.$imageInfo['height'].'"     alt="'.$result[0]['image_descr'].'" border="0">';
            }
            //Should not be used.
            elseif (false /*$application->getAppIni('SHOW_NO_PICTURE_IMAGE') == 'ENABLED' */)
            {
                //FIXME: remove
                //    $imageInfo['show_that_file_is_not_uploaded'] = '1';
                $imageInfo['show_that_file_is_not_uploaded'] = '1';

            	//$noPictureInfo = @getimagesize($application->getAppIni('PATH_NO_PICTURE_IMAGE'));
            	$imageInfo['src'] = "";//$imgAddress.$application->getAppIni('NO_PICTURE_IMAGE');
            	$imageInfo['width'] = "1";//$noPictureInfo[0];
            	$imageInfo['height'] = "1";//$noPictureInfo[1];
                $imageInfo['img'] = "";
            }
            else
            {
                $imageInfo['show_that_file_is_not_uploaded'] = '1';
            	$imageInfo['src'] = "";
            	$imageInfo['width'] = "1";
            	$imageInfo['height'] = "1";
                $imageInfo['img'] = "";
            }

            $images[$image] = $imageInfo;
        }

        $retval = array (
            "ID"             => $cid
           ,"Name"           => $result[0]['name'] //prepareHTMLDisplay($result[0]['name'])
           ,"Status"         => $result[0]['status']
           ,"ShowProductsRecursivelyStatus" => $result[0]['show_prod_recurs']
           ,"Description"    => $result[0]['descr']
           ,"LargeImage"     => $images['largeImage']['img']
           ,"LargeImageSrc"  => $images['largeImage']['src']
           ,"LargeImageWidth"=> $images['largeImage']['width']
           ,"LargeImageHeight"=> $images['largeImage']['height']
           ,"LargeImageShowThatFileIsNotUploaded" => $images['largeImage']['show_that_file_is_not_uploaded']
           ,"SmallImage"     => $images['smallImage']['img']
           ,"SmallImageSrc"  => $images['smallImage']['src']
           ,"SmallImageWidth"=> $images['smallImage']['width']
           ,"SmallImageHeight"=> $images['smallImage']['height']
           ,"SmallImageShowThatFileIsNotUploaded" => $images['smallImage']['show_that_file_is_not_uploaded']
           ,"ImageAltText"   => $result[0]['image_descr'] //prepareHTMLDisplay($result[0]['image_descr'])
           ,"PageTitle"      => $result[0]['page_title'] //prepareHTMLDisplay($result[0]['page_title'])
           ,"MetaKeywords"   => $result[0]['meta_keywords'] //prepareHTMLDisplay($result[0]['meta_keywords'])
           ,"MetaDescription"=> $result[0]['meta_descr'] //prepareHTMLDisplay($result[0]['meta_descr'])
           ,"ProductsNumber" => $prods_number
           ,"ProductsNumberRecursively" => $prods_number_recurse
           ,"SubcategoriesNumber" => $sub_number == 0 ? "0" : $sub_number
           ,"left"           => $result[0]['left1']
           ,"right"          => $result[0]['right1']
           ,"level"          => $result[0]['level']
           ,"ParentID"       => $parent
           ,"SEO_URL_prefix" => $result[0]['seo_url_prefix']
        );

        $retval["Link"] = $info_link;
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End, Category Info returned');
        return $retval;
    }


    /**
     * Checks if the module has been installed.
     *
     * @todo The method is deprecated
     * @return bool TRUE if the module has been installed, FALSE otherwise
     */
    function isInstalled()
    {
    }

    /**
     * Installs the  module.
     *
     * The install() method is called statically.
     * To call other methods of this class from this method,
     * the static call is used, for example,
     * Catalog::getTables() instead of $this->getTables()
     */
    function install()
    {
        _use(dirname(__FILE__).'/includes/xml_install.inc');
    }

    /**
     * Uninstalls the module.
     * It deletes all module tables.
     *
     * The uninstall() method is called statically.
     * To call other methods of this class from this method,
     * the static call is used, for example,
     * Catalog::getTables() instead of $this->getTables()
     */
    function uninstall()
    {
        $query = &new DB_Table_Delete(Catalog::getTables());
        global $application;
        $application->db->getDB_Result($query);
    }

    /**
     * Sets the current category ID.
     * If the category ID is not correct, the current id remains unchanged.
     */
    function setCurrentCategoryID($cid)
    {
        global $application;

        if ($this->isCorrectCategoryId($cid))
        {
            //                 storefront -          ,                  Offline:
            if (modApiFunc('Users', 'getZone') == "CustomerZone")
            {
                $cat = &new CCategoryInfo($cid);

                if( $cat->getCategoryTagValue('RecursiveStatus') == CATEGORY_STATUS_ONLINE)
                {
                    log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Current Category ID Changed From '.$this->currentCategoryID.' To '.$cid);
                    $this->currentCategoryID = $cid;
                }
                else
                {
                    log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Current Category ID is not changed because new Category "'.$cid.'" is offline, redirecting to SITE_URL');
                    $query = new Request($application->getAppIni('SITE_URL'));
                    $application->redirect($query);
                }
            }
            else
            {
                log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Current Category ID Changed From '.$this->currentCategoryID.' To '.$cid);
                $this->currentCategoryID = $cid;
            }
        }
        else
        {
            log_warn('['.__CLASS__.'::'.__FUNCTION__.'] Category ID ('.$cid.') is Incorrect');
        }
    }

    /**
     * Gets the current catalog category ID.
     *
     * @return integer Current category ID.
     */
    function getCurrentCategoryID()
    {
        return $this->currentCategoryID;
    }

    /**
     * Sets the edited category ID.
     * If the category ID is not correct, the current id remains unchanged.
     *
     * @param integer $cid category id
     */
    function setEditableCategoryID($ecid)
    {
        if ($this->isCorrectCategoryId($ecid))
        {
            $this->editableCategoryID = $ecid;
        }
    }

    /**
     * Gets the edited category ID.
     *
     * @return integer Editable category ID.
     */
    function getEditableCategoryID()
    {
        return $this->editableCategoryID;
    }

    /**
     * Sets "move to" category ID.
     * If the category ID is not correct, the current id remains unchanged.
     *
     * @param integer $cid category id
     */
    function setMoveToCategoryID($mcid)
    {
        if ($this->isCorrectCategoryId($mcid))
        {
            $this->moveToCategoryID = $mcid;
        }
    }

    /**
     * Gets "move to" category ID.
     *
     * @return integer "move to" category ID
     */
    function getMoveToCategoryID()
    {
        return $this->moveToCategoryID;
    }

    /**
     * Unsets the edited catalog category ID.
     */
    function unsetEditableCategoryID()
    {
        $this->editableCategoryID=NULL;
    }

    /**
     * Unsets "move to" catalog category ID.
     */
    function unsetMoveToCategoryID()
    {
        $this->moveToCategoryID=$this->currentCategoryID;
    }

    /**
     * Gets the current product ID.
     *
     * @return integer Current product ID.
     */
    function getCurrentProductID()
    {
        return $this->currentProductID;
    }

    /**
     * Sets the current product ID.
     * If the product ID is not correct, the current id remains unchanged.
     *
     * @param integer $pid product id
     */
    function setCurrentProductID($pid)
    {
        global $application;
        if ($this->isCorrectProductId($pid))
        {
            //                 storefront -          ,                  Offline:
            if (modApiFunc('Users', 'getZone') == "CustomerZone")
            {
                $prod = &new CProductInfo($pid);

                if($prod->haveOnlineCategory())
                {
                    log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Current Product ID Changed From '.$this->currentProductID.' To '.$pid);
                    $this->currentProductID = $pid;
                }
                else
                {
                    log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Current Product ID is not changed because new Product '.$pid.' is offline, redirecting to SITE_URL');
                    $query = new Request($application->getAppIni('SITE_URL'));
                    $application->redirect($query);
                }
            }
            else
            {
                $this->currentProductID = $pid;
            }
        }
        else
        {
            log_warn('['.__CLASS__.'::'.__FUNCTION__.'] Product ID ('.$pid.') is Incorrect');
        }
    }

    /**
     * Gets the current product type ID.
     *
     * @return integer Current product type ID.
     */
    function getCurrentProductTypeID()
    {
        return $this->currentProductTypeID;
    }

    /**
     * Sets the current product type ID.
     * If the product type ID is not correct, the current id remains unchanged.
     *
     * @param integer $ptid Product type id.
     */
    function setCurrentProductTypeID($ptid)
    {
        if ($this->isCorrectProductTypeId($ptid))
        {
            $this->currentProductTypeID = $ptid;
        }
    }

    /**
     * Gets the edited product array.
     *
     * @todo should it be saved in the module state?
     * @return array the edited product array.
     */
    function getEditableProductsID()
    {
        return $this->editableProductsID;
    }

    /**
     * Sets the edited product array.
     * Each id is checked on validity. If the invalid id is found, it should be
     * deleted from the array.
     */
    function setEditableProductsID($prodId_array)
    {
        $this->editableProductsID = array();
        foreach ($prodId_array as $prodId)
        {
            if ($this->isCorrectProductId($prodId))
            {
                $this->editableProductsID[] = $prodId;
            }
        }
    }

    /**
     * Unsets hte edited product ID.
     */
    function unsetEditableProductsID()
    {
        $this->editableProductsID=array();
    }

    /**
     * Registers all custom attributes, specified in the system for any
     * product type.
     */
    function registerAttributes()
    {
        global $application;

        $tables = $this->getTables();
        $a     = $tables['attributes']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($a['view_tag'], 'view_tag');
        $query->WhereValue($a['type'], DB_EQ, 'custom');
        $result = $application->db->getDB_Result($query);

        for ($i=0; $i<sizeof($result); $i++)
        {
            $application->registerAttributes(array('Product'.$result[$i]['view_tag'].'Custom'));
            log_debug('['.__CLASS__.'::'.__FUNCTION__.'] '.'Product'.$result[$i]['view_tag'].'Custom()'.' registered');
        }
    }

    /**
     * Gets the category full path.
     * The structure of the returned array:
     * <code>
     * array (
     *     '0' => array (
     *         'id'   => the category id.
     *         'name' => the category name.
     *         ),
     *     '1' => ...
     * )
     * </code>
     *
     * @param integer $cid the category id.
     * @return array Array of categories representing current category tree path.
     */
    function getCategoryFullPath($cid)
    {
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Start, Category ID = '.$cid);
        global $application;

        $categories = array();   #the product list in the current category
        $category = array();     #one product info

        $result = array();
        if ( $CategoryInfo = $this->fetchCategoryInfo($cid) )
        {
            $tables = $this->getTables();
            $c     = $tables['categories']['columns'];
            $cd    = $tables['categories_descr']['columns'];

            $query = &new DB_Select();
            $query->addSelectField($c['id'], 'id');
            $query->addSelectField($c['left'], 'left1');
            $query->addSelectField($c['right'], 'right1');
            $query->addSelectField($c['level'], 'level');

            $query->addSelectField($cd['name'], 'category_name');

            $query->WhereValue($c['left'], DB_LTE, $CategoryInfo['left']);
            $query->WhereAnd();
            $query->WhereValue($c['right'], DB_GTE, $CategoryInfo['right']);
            $query->WhereAnd();
            $query->WhereField($c['id'], DB_EQ, $cd['id']);
            $query->SelectOrder($c['left']);
            $result = $application->db->getDB_Result($query);
        }
        $retval = array();
        foreach ($result as $key => $value)
        {
            array_push($retval, array(
                "id"    => $value["id"],
                "name"  => $value["category_name"]
            ));
        }
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End, Result returned.');
        return $retval;
    }

    /**
     * Gets the current category full path.
     *
     * @return array Array of categories representing current category tree path
     * or NULL if current category is not set.
     * @see Catalog::getCategoryFullPath()
     */
    function getCurrentCategoryFullPath()
    {
        global $application;

        if ($this->currentCategoryID == NULL)
        {
            $retval = NULL;
        }
        else
        {
            $retval = $this->getCategoryFullPath($this->currentCategoryID);
        }

        return $retval;
    }


    /**
     * Gets the category list in current category.
     *
     * @return array Array of CCategoryInfo objects or NULL if the current category
     *         is unavailable.
     * @see Catalog::getDirectSubcategoriesListFull()
     */
    function getDirectSubcategoriesListInCurrentCategory()
    {
        if ($this->currentCategoryID == NULL)
        {
            $retval = NULL;
        }
        else
        {
            $retval = $this->getDirectSubcategoriesListFull($this->currentCategoryID);
        }

        return $retval;
    }


    /**
     * Gets the category list in current category.
     * Each element in the array is the CCategoryInfo object.
     *
     * @return array Array of CCategoryInfo objects or NULL if the current category
     *         is unavailable.
     */
    function getDirectSubcategoriesListFull($cid, $b_online_only = false)
    {
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Start, Category ID = '.$cid.', Online Only = '.($b_online_only ? 'True' : 'False') );
        global $application;

        $result = array();
        if ($CategoryInfo = $this->fetchCategoryInfo($cid))
        {
            $tables = $this->getTables();
            $field = $tables['categories']['columns'];
            $cd    = $tables['categories_descr']['columns'];

            $query = &new DB_Select();
            $query->addSelectField($field['id'], 'id');
            $query->addSelectField($field['left'], 'left1');
            $query->addSelectField($field['right'], 'right1');
            $query->addSelectField($field['level'], 'level');

            $query->addSelectField($cd['name'], 'category_name');
            $query->addSelectField($cd['image_small_file'], 'image_small_file');
            $query->addSelectField($cd['image_descr'], 'image_descr');

            $query->WhereValue($field['left'], DB_BETWEEN, $CategoryInfo['left']);
            $query->WhereAnd();
            $query->WhereValue('', '', $CategoryInfo['right']);
            $query->WhereAnd();
            $query->WhereValue($field['level'], DB_EQ, $CategoryInfo['level'] + 1);
            $query->WhereAnd();
            $query->WhereField($field['id'], DB_EQ, $cd['id']);
            if($b_online_only === true)
            {
                $query->WhereAnd();
                $query->WhereField($field['status'], DB_EQ, CATEGORY_STATUS_ONLINE);
            }


            $query->SelectOrder($field['left']);

            $result = $application->db->getDB_Result($query);
        }

        $retval = array();
        foreach ($result as $key => $value)
        {
            $c = &new CCategoryInfo($value['id']);
            array_push($retval, $c);
        }
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End, Result Returned');
        return $retval;
    }

    /**
     * Gets a list of all children subcategories in the selected category.
     *
     * @author Alexander Girin
     * @todo check if this function is needed
     * @param integer $cid the selected category id.
     * @return
     */
    function getSubcategoriesFullListWithParent($cid, $use_peginator=true, $with_descr=true)
    {
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Start, Category ID = '.$cid.', Use Paginator = '.($use_peginator ? 'True' : 'False') ) ;
        global $application;

        $result = array();
        if ($CategoryInfo = $this->fetchCategoryInfo($cid))
        {
            $tables = $this->getTables();
            $c  = $tables['categories']['columns'];
            $cd = $tables['categories_descr']['columns'];

            $query = &new DB_Select();
            $query->addSelectField($c['id'], 'id');
            $query->addSelectField($c['left'], 'c_left');
            $query->addSelectField($c['right'], 'c_right');
            $query->addSelectField($c['level'], 'level');
            if($with_descr)
            {
                $query->addSelectField($cd['name'], 'name');
                $query->addLeftJoin('categories_descr', $cd['id'], DB_EQ, $c['id']);
            };

            $query->WhereValue($c['left'], DB_GTE, $CategoryInfo['left']);
            $query->WhereAnd();
            $query->WhereValue($c['left'], DB_LT, $CategoryInfo['right']);
            $query->SelectOrder($c['left']);

            if ($use_peginator)
            {
                $query = modApiFunc('paginator', 'setQuery', $query);
            }
            $result = $application->db->getDB_Result($query);
        }
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End, Result Returned');
        return $result;
    }

    /**
     * Set a new value for the 'left' key in this category.
     * So update the parameters 'left' and 'right' of all subcategories
     * and the 'right' key of the category itself.
     * The action is similar to the moving of the subtree, whose root
     * is the specified category.
     */
    /*function updateTreeIndices($root_cid, $root_left, $root_right, $delta)
    {
        //NOTE: $root_cid is redundant
        global $application;
        $tables = $this->getTables();
        $columns = $tables['categories']['columns'];
        $query = &new DB_Update('categories');
        $query->addUpdateExpression($columns['left'], $columns['left'] . "+(" . $delta .")");
        $query->addUpdateExpression($columns['right'], $columns['right'] . "+(" . $delta .")");
        $query->WhereValue($columns['left'], DB_GTE, $root_left);
        $query->WhereAnd();
        $query->WhereValue($columns['right'], DB_LTE, $root_right);
        $application->db->getDB_Result($query);
    }*/

    /**
     * Saves the sort of subcategories to the categories in the DB.
     *
     * @author Alexander Girin
     * @param array $catSortOrderArray the array of categories id, whose order
     *               defines sort_order of the categories.
     * @return
     */
    function setCategorySortOrder($catSortOrderArray)
    {
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Start');
        global $application;
        /**
         * <pre>
         * BEFORE
         *
         * |
         * +-\ root (L_root, R_root)
         * | |                         L_1 = L_root + 1
         * | |-- subnode1 (L_1, R_1)
         * | |-- subnode2 (L_2, R_2)
         * | |-- subnode3 (L_3, R_3)
         * |                           R_3 = R_root - 1
         * |
         *
         * AFTER (new order: subnode2, subnode3, subnode1
         *
         * |
         * +-\ root (L_root, R_root)
         * | |              * = L_root + 1
         * | |-- subnode2 ( * , * + (R_2 - L_2))
         * | |-- subnode3 ( * , * + (R_3 - L_3))
         * | |-- subnode1 ( * , * + (R_1 - L_1))
         * |                   (* + (R_1 - L_1)) = R_root - 1
         * |
         * </pre>
         */

        /**
         * To make changes just update the indexes 'left' and 'right' only
         * into the subtree, whose root is the parent category for all sorted ones.
         * It can be used for optimization. The problem: how to change them all
         * together at once? If update the categories separately, then the errors
         * occur in the tree structure.
         */


        /**
         * In the current realization (February 2006) is used the fact, that
         * adding a subcategory, it is putted right from the other categories
         * of the same category. Move all the subcategories to the temporary
         * category, then return them one by one in the required order.
         */

        /**
         * The subcategories can be sorted only in the category, which has
         * at least one subcategory.
         * Take the first one in the list and determine its ( and other
         * subcategories') parent.
         */
        $arbitrary_subcategory_id   = $catSortOrderArray[0];

        $query = &new DB_MYSQL_Lock_Tables();
        /**
         * To find out the list of tables, which should be locked temporarily,
         * it is necesary to view all the invoked functions:
         * getCategoryInfo
         * addCategory
         * moveCategory.
         */

        $query->addTableToLock('categories', DB_LOCK_MODE_WRITE);
        $query->addTableToLock('categories_descr', DB_LOCK_MODE_WRITE);
        $query->addTableToLock('products', DB_LOCK_MODE_WRITE);
        $query->addTableToLock('store_settings', DB_LOCK_MODE_WRITE);
        $query->addTableToLock('products_to_categories', DB_LOCK_MODE_WRITE);
        $query->addTableToLock('log_info', DB_LOCK_MODE_WRITE);
        $query->addTableToLock('log_messages', DB_LOCK_MODE_WRITE);
        $application->db->PrepareSQL($query);
        $application->db->DB_Exec($query);

        $arbitrary_subcategory_info = $this->getCategoryInfo($arbitrary_subcategory_id);

        ksort($catSortOrderArray);
        //FIXME: magic number
        $root_cat_id = 1;
        $temp_cat_id =  $this->addCategory(
                $root_cat_id
               ,"temp_cat_name"
               ,CATEGORY_STATUS_ONLINE
               ,"temp_cat_description"
               ,"temp_cat_image_filename"
               ,"temp_cat_image_small_filename"
               ,"temp_cat_image_description"
               ,"temp_cat_page_title"
               ,"temp_cat_meta_keywords"
               ,"temp_cat_meta_description"
               ,CATEGORY_DONOTSHOW_PRODUCTS_RECURSIVELY
               ,"temp_cat_seo_url_prefix");
        $parent_cat_id = $arbitrary_subcategory_info['ParentID'];
        foreach ($catSortOrderArray as $catSort => $catId)
        {
            $this->moveCategory($temp_cat_id, $catId);
        }

        foreach ($catSortOrderArray as $catSort => $catId)
        {
            $this->moveCategory($parent_cat_id, $catId);
        }
        //Delete the temporary category.
        $this->deleteCategoriesArray(array($temp_cat_id), $temp_cat_id);

        $query = &new DB_MYSQL_Unlock_Tables();
        $application->db->PrepareSQL($query);
        $application->db->DB_Exec($query);

        modApiFunc('CatalogCacheManager', 'eventCategorySubcategoriesContentChanged', $parent_cat_id);
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End');
    }

    /**
     * Saves the sort of products to the categories in the DB.
     *
     * @author Alexander Girin
     * @param array $catSortOrderArray the array of products id, whose order
     * defines sort_order of the products.
     * @return
     */
    function setProductsSortOrder($prodSortOrderArray,$category_id)
    {
        global $application;

        foreach ($prodSortOrderArray as $prodSort => $prodId)
        {
            $tables = $this->getTables();
            $columns = $tables['products_to_categories']['columns'];
            $query = &new DB_Update('products_to_categories');
            $query->addUpdateValue($columns['sort_order'], $prodSort);
            $query->WhereValue($columns['product_id'], DB_EQ, $prodId);
            $query->WhereAND();
            $query->WhereValue($columns['category_id'], DB_EQ, $category_id);
            $application->db->getDB_Result($query);
        }

        modApiFunc('CatalogCacheManager','eventCategoryProductsContentChanged', $category_id);
    }

    /**
     * Generates product sort order in the category with the ID = $cid to append
     * to the end.
     *
     * @author Alexander Girin
     * @param integer $cid - the category ID, where the product is added
     * @return integer max sort order + 1
     */
    function generateProductSortOrder($cid)
    {
        global $application;
        $tables = $this->getTables();
        $columns = $tables['products']['columns'];
        $query = &new DB_Select();
        $query->addSelectField($query->fMax($columns['sort_order']), 'max');
        $query->WhereValue($columns['c_id'], DB_EQ, $cid);
        $result = $application->db->getDB_Result($query);
        return ($result[0]['max']+1);
    }

    /**
     * Gets the products list in category.
     * It gets the products list in category, sorted by the field
     * product_sort_order.It is created a list of the CProductInfo objects, so
     * that in each object is installed an additional tag 'N', which has a
     * serial number of each object, including paginator. It can be a sequence
     * from 1 to 10, if the first peginator page is mapped, or from 11 to 20,
     * if the second one is mapped. The additional tag is used in the AZ, in
     * the product list.
     *
     * @author Alexander Girin
     * @param integer $cid the category id
     * @param boolean $use_peginator true is by default, to use paginator, i.e.
     * the product list will be limited by the current page number and record
     * count outputted on the page. If pass false, the whole product list in the
     * selected category will be returned.
     * @return array Array of products list or NULL if current category is
     * unavailable.
     */
    function getProdsListInCategory($cid, $use_peginator=true, $return_as_pids_array = false)
    {
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Start, Category ID = '.$cid.', Use Paginator = '.($use_peginator ? 'True' : 'False').', '.($return_as_pids_array ? 'Return As IDs List' : 'Return As Product Objects List') );
        global $application;
        $products_listing = array();   #  the poduct list in the current category
        $product_info = array();    #  info about one product

        if ($use_peginator)
        {
            /*
            Variables $paginator_page_number and $paginator_products_per_page are
            used to create the cache id.
            */
            $paginator = &$application->getInstance('paginator');
            $paginator_page_number = $paginator->getPaginatorPage($paginator->getCurrentPaginatorName());
            $paginator_products_per_page = $paginator->getPaginatorRowsPerPage($paginator->getCurrentPaginatorName());
        }
        else
        {
           /*
            If paginator is not used, cast the avriables to the const value,
            i.e. given cache will be identified by the category number.
            */
            $paginator_page_number = 0;
            $paginator_products_per_page = 0;
        }

        # Check cache
        $cache_id = array($cid, $paginator_page_number, $paginator_products_per_page, $use_peginator);
        $cache = &$application->getInstance('CatalogCacheManager');
        $cachedata = $cache->CacheProductListInCategories->get($cache_id);
	    if ( !($cachedata === null) )
	    {
           /*
            Not only the database selection result but the paginator value are
            saved in cache.
            */
	        if ($use_peginator)
            {
    	        $paginator->Offset = $cachedata['PAGINATOR_OFFSET'];
	            $paginator->CurrentPaginatorTotalRows = $cachedata['PAGINATOR_CURRENTPAGINATORTOTALROWS'];
            }
	        $result = $cachedata['PRODUCTS_LISTING'];
	        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Read From Cache');
	    }
        else # no cache
        {
            if (!$this->isCorrectCategoryId($cid))
            {
                log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End, Category ID is incorrect, Returned Empty Result');
                return $products_listing;
            }

            $tables = $this->getTables();

            $t_products                = $tables['products']['columns'];
            $t_product_type_attributes = $tables['product_type_attributes']['columns'];
            $t_product_attributes      = $tables['product_attributes']['columns'];
            $t_categories              = $tables['categories']['columns'];
            $p2c                       = $tables['products_to_categories']['columns'];

            $query = &new DB_Select();
            $query->addSelectField($p2c['product_id'], 'product_id');
            if (modApiFunc('Users', 'getZone') == "AdminZone")
            {
                $query->WhereValue($p2c['category_id'], DB_EQ, $cid);
            }

            # add the filter to the query on the Available attribute.
            if (modApiFunc('Users', 'getZone') == "CustomerZone")
            {
                //          ,                                                                                
                $objCurrCategory = &new CCategoryInfo($cid);
                if ($objCurrCategory->getShowProductsRecursivelyStatus() == CATEGORY_SHOW_PRODUCTS_RECURSIVELY)
                {
                    $online_subcat_ids = $objCurrCategory->_getOnlineSubCategories();
                    $query->WhereField($p2c['category_id'], DB_IN," ('" . implode("','", $online_subcat_ids) . "') ");
                }
                else /* CATEGORY_DONOTSHOW_PRODUCTS_RECURSIVELY */
                {
                    $query->WhereValue($p2c['category_id'], DB_EQ, $cid);
                }

                $query->WhereAND();
                $query->Where($t_products['id'], DB_EQ, $p2c['product_id']);
                CCategoryInfo::_applyProductsFilter($query);
            }

            $query->SelectOrder($p2c['sort_order']);

            if ($use_peginator)
            {
                $query = modApiFunc('paginator', 'setQuery', $query);
            }

            $result = $application->db->getDB_Result($query);

            # create cache
            if ($use_peginator)
            {
               /*
                 Save into cach:
                - a paginator offset,
                - a product number on the ProductList page
                - a database selection result.
                */
                $cache_data = array('PAGINATOR_OFFSET'                    => $paginator->Offset,
                                    'PAGINATOR_CURRENTPAGINATORTOTALROWS' => $paginator->CurrentPaginatorTotalRows,
                                    'PRODUCTS_LISTING'                    => $result );
            }
            else
            {
                $cache_data = array('PAGINATOR_OFFSET'                    => 0,
                                    'PAGINATOR_CURRENTPAGINATORTOTALROWS' => 0,
                                    'PRODUCTS_LISTING'                    => $result );
            }
            log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Cache Record Created');
            $cache->CacheProductListInCategories->save($cache_id, $cache_data);
        }

        if($return_as_pids_array)
        {
            log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End, Result Returned As IDs List');
            return $result;
        };

        for ($i=0; $i<sizeof($result); $i++)
        {
            $offset=0;
            if ($use_peginator)
            {
                $offset = modApiFunc('paginator', 'getCurrentPaginatorOffset');
            }
            if (!is_numeric($offset))
            {
                $offset=0;
            }
            $product_info = &new CProductInfo($result[$i]['product_id']);
            $product_info->setAdditionalProductTag('N', ($i+1+$offset) );
            array_push($products_listing, $product_info);
        }
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End, Result Returned As Product Objects List');
        return $products_listing;
    }

    /**
     * Returns the list of product ids by specified categories.
     *
     * The parameter $ptypeId determines product type ids. The product ids
     * of only specified value will be returned. If it is necessary to get
     * the products of all types, then as the parameter should be passed in a
     * string 'ALL_PRODUCT_TYPES'.
     *
     * The parameter $catId determines, from which catagories to select products.
     * If it is necessary to select from all categories, then as that parameter
     * should be passed in a string 'ALL_CATEGORIES'.
     *
     * The parameter $isRecurs determines, whether to select products from
     * category recursively, i.e. from subcategories. For that the value
     * 'RECURSIVELY' should be passed, otherwise it won't be selected recursively.
     *
     */
    function getProdsIDsListEx($ptypeId='ALL_PRODUCT_TYPES', $catId='ALL_CATEGORIES', $isRecurs='RECURSIVELY')
    {
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Start, Product Type ID = '.$ptypeId.', Category ID = '.$catId.', Recursively = '.($isRecurs ? 'True' : 'False') );
        global $application;
        $tables = Catalog::getTables();

        // is correct $ptypeId and $catId
        if ($catId != 'ALL_CATEGORIES' && !$this->isCorrectCategoryId($catId))
        {
            log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End, Category Id is incorrect, Returned emoty result');
            return array();
        }
/*
        if ($ptypeId != 'ALL_PRODUCT_TYPES' && !$this->isCorrectProductTypeId($ptypeId))
            return array();
*/
        if ($catId=='ALL_CATEGORIES')
        {
            $catId = 1;
            $isRecurs = 'RECURSIVELY';
        }

        //$t_products   = $tables['products']['columns'];
        $t_categories = $tables['categories']['columns'];
        $p2c = $tables['products_to_categories']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($p2c['product_id'], 'product_id');

        if ($isRecurs == 'RECURSIVELY')
        {
            $cObj = &new CCategoryInfo($catId);
            $query->WhereField($t_categories['id'], DB_EQ, $p2c['category_id'] );
            $query->WhereAND();
            $query->WhereValue($t_categories['left'], DB_GTE, $cObj->getCategoryTagValue('left') );
            $query->WhereAND();
            $query->WhereValue($t_categories['right'], DB_LTE, $cObj->getCategoryTagValue('right') );
        }
        else
        {
            $query->WhereValue($p2c['category_id'], DB_EQ, $catId );
        }

        $query->SelectGroup($p2c['product_id']);
/*
        if ($ptypeId != 'ALL_PRODUCT_TYPES')
        {
            $query->WhereAND();
            $query->WhereValue($t_products['pt_id'], DB_EQ, $ptypeId );
        }
*/
        $db_result = $application->db->getDB_Result($query);

        $result = array();
        foreach($db_result as $db_item)
        {
            $result[] = $db_item['product_id'];
        }

        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End, Result Returned');
        return $result;
    }

    function _getPtypesAttrsByPIDs($pids)
    {
        global $application;
        $tables = $this->getTables();
        $ptable = $tables['products']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($ptable['pt_id'], 'pt_id');
        $query->Where($ptable['id'], DB_IN, "('".implode("','",$pids)."')");
        $query->SelectGroup('pt_id');
        $res = $application->db->getDB_Result($query);

        $attrs = array();
        for($i=0;$i<count($res);$i++)
        {
            $_ptattrs = $this->getProductTypeAttributes($res[$i]['pt_id']);
            foreach($_ptattrs as $_key => $_ptainf)
                if($_ptainf['visible'] != 1)
                    unset($_ptattrs[$_key]);
            $attrs = array_merge($attrs,$_ptattrs);
        };

        return $attrs;
    }

    function _getPTypeIDByProductID($pid)
    {
        global $application;
        $tables = $this->getTables();
        $ptable = $tables['products']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($ptable['pt_id'],'pt_id');
        $query->WhereValue($ptable['id'], DB_EQ, $pid);
        $res = $application->db->getDB_Result($query);

        return $res[0]['pt_id'];
    }

    /**
     * Gets the products list in category.
     *
     * Gets the products list in category, sorted by the field product_sort_order.
     *
     * @author Alexander Girin
     * @param boolean $use_peginator true is by default, use paginator, i.e.
     * the product list will be limited by the current page number and record
     * count outputted on the page. If pass false, the whole product list in the
     * selected category will be returned.
     * @return mixed Array of products list or NULL if current category is unavailable.
     */
    function getProdsListInCurrentCategory($use_peginator=true, $return_as_pids_array = false)
    {
       //Query to the database, get the product list in the current category
        global $application;

        if ($this->currentCategoryID == NULL)
        {
            $retval = NULL;
        }
        else
        {
            $retval = $this->getProdsListInCategory($this->currentCategoryID, $use_peginator, $return_as_pids_array);
        }

        return $retval;
    }

    /**
     * Deletes all products.
     *
     */
    function deleteAllProducts()
    {
        $tables = $this->getTables();
        $t_products = $tables['products']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($t_products['id'], 'id');

        global $application;

        $result = $application->db->getDB_Result($query);

        $product_ids = array();
        for ($i = 0; $i < sizeof($result); $i++)
        {
            $id = $result[$i]['id'];
            $product_ids[] = $id;
        }

        $this->deleteProductsArray($product_ids);
    }


    /**
     * Deletes the product array by the array of products id.
     *
     * @author Alexander Girin
     * @param array $ProdsId the array of products id
     * @return
     */
    function deleteProductsArray($ProdsId)
    {
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Start, Products = '."('" . implode("', '", $ProdsId) . "')");
        global $application;

        $tables = $this->getTables();
        $pa = $tables['product_attributes']['columns']; # product_attributes is a database table
        $pi = $tables['product_images']['columns'];
        $query = &new DB_Select();
        $query->addSelectField($pi['id'], 'id');
        $query->addSelectField($pi['name'], 'name');
        $query->addLeftJoin('product_attributes', $pa['id'], DB_EQ, $pi['pa_id']);
        $query->WhereField($pa['p_id'], DB_IN,"('" . implode("','", $ProdsId) . "')");
        $query->WhereAnd();
        $query->addWhereOpenSection();
        $query->WhereField($pa['a_id'], DB_EQ, "9");
        $query->WhereOr();
        $query->WhereField($pa['a_id'], DB_EQ, "10");
        $query->addWhereCloseSection();
        $result = $application->db->getDB_Result($query);
        $imagesId = array();
        foreach ($result as $imageInfo)
        {
            @unlink($application->getAppIni('PATH_IMAGES_DIR').$imageInfo["name"]);
            $imagesId[] = $imageInfo["id"];
            log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Image Deleted: '.$imageInfo["name"]);
        }
        $query = &new DB_Delete('product_images');
        $query->WhereField($pi['id'], DB_IN,"('" . implode("','", $imagesId) . "')");
        $application->db->getDB_Result($query);

        $query = &new DB_Delete('product_attributes');
        $query->WhereField($pa['p_id'], DB_IN,"('" . implode("','", $ProdsId) . "')");
        $application->db->getDB_Result($query);

        $p = $tables['products']['columns'];
        $p2c = $tables['products_to_categories']['columns'];

        // Select a category list, to which these products are referred.
        $query = &new DB_Select();
        $query->addSelectField($p2c['category_id'], 'c_id');
        $query->WhereField($p2c['product_id'], DB_IN,"('" . implode("','", $ProdsId) . "')");
        $categories_list = $application->db->getDB_Result($query);

        $query = &new DB_Delete('products');
        $query->WhereField($p['id'], DB_IN, "('" . implode("','", $ProdsId) . "')");
        $application->db->getDB_Result($query);

        $query = &new DB_Delete('products_to_categories');
        $query->Where($p2c['product_id'], DB_IN, "('" . implode("','", $ProdsId) . "')");
        $application->db->getDB_Result($query);

        #  delete products form cache
        foreach ($ProdsId as $id)
        {
        	modApiFunc('CatalogCacheManager','eventProductInfoChanged',$id);
        }
        foreach ($categories_list as $cid)
        {
            modApiFunc('CatalogCacheManager', 'eventCategoryProductsContentChanged', $cid['c_id']);
        }
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End');
    }

     /**
     * Deletes the category array by the array of categories id.
     *
     * @author Alexander Girin
     * @param array $CatsId the array of categories id
     * @param integer $ParentCatId parent category id
     * @return
     */
    function deleteCategoriesArray($CatsId, $ParentCatId)
    {
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Start, Categories = '."('" . implode("', '", $CatsId) . "'), Parent Category ID = ".$ParentCatId);
        global $application;

        if ($CategoryInfo = $this->fetchCategoryInfo($ParentCatId))
        {
            $tables = $this->getTables();
            $cd = $tables['categories_descr']['columns']; #categories_descr is a database table
            $query = &new DB_Select();
            $query->addSelectField($cd['image_file'], 'image_large');
            $query->addSelectField($cd['image_small_file'], 'image_small');
            $query->WhereField($cd['id'], DB_IN,"('" . implode("','", $CatsId) . "')");
            $result = $application->db->getDB_Result($query);
            foreach ($result as $images)
            {
                @unlink($application->getAppIni('PATH_IMAGES_DIR').$images['image_large']);
                @unlink($application->getAppIni('PATH_IMAGES_DIR').$images['image_small']);
                log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Images Deleted: Large - '.$images['image_large'].', Small - '.$images['image_small']);
            }

            $query = &new DB_Delete('categories_descr');
            $query->WhereField($cd['id'], DB_IN,"('" . implode("','", $CatsId) . "')");
            $application->db->getDB_Result($query);
            $c = $tables['categories']['columns']; #categories is a database table

            $query = &new DB_Delete('categories');
            $query->WhereValue($c['left'], DB_GTE, $CategoryInfo['left']);
            $query->WhereAnd();
            $query->WhereValue($c['left'], DB_LT, $CategoryInfo['right']);
            $application->db->getDB_Result($query);

            $deltaId = ($CategoryInfo['right'] - $CategoryInfo['left'])+1;
            $query = &new DB_Update('categories');
            $query->addUpdateExpression($c["left"], $query->fIf($c["left"] . DB_GT . $CategoryInfo['left'], $c['left'] . "- ".$deltaId , $c['left']));
            $query->addUpdateExpression($c['right'], $query->fIf($c['right'] . DB_GT . $CategoryInfo['left'], $c['right'] . "- ".$deltaId , $c['right']));
            $query->WhereValue($c['right'], DB_GT, $CategoryInfo['right']);
            $application->db->getDB_Result($query);

            //del links
            $this->delAllProductLinks('category_id',$CatsId);

            #  delete all categories from cache
            modApiFunc('CatalogCacheManager', 'eventCategorySubcategoriesContentChanged', $ParentCatId);
            foreach ($CatsId as $cid)
            {
                modApiFunc('CatalogCacheManager', 'eventCategoryInfoChanged', $cid['c_id']);
            }
        }
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End');
    }

    /**
     * Moves the category.
     *
     * @author Alexander Girin
     * @todo comment the code. The large code of the function is not commented.
     * @todo update cache
     * @param integer $newParentCatId the new category id,
     *        to which the category is moved
     * @param integer $cid the moved category id
     * @return
     */
    function moveCategory($newParentCatId, $cid)
    {
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Start, New Parent Category ID = '.$newParentCatId.', Moved Category ID = '.$cid);
        global $application;

        $NewParentCatInfo = $this->fetchCategoryInfo($newParentCatId);
        $CatInfo = $this->fetchCategoryInfo($cid);

        $catobj = &new CCategoryInfo($cid);
        $CurrentParentCatInfo = $catobj->getCategoryTagValue('parentid');

        if ($NewParentCatInfo == FALSE || $CatInfo == FALSE )
        {
            log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End');
            return;
        }

        $tables = $this->getTables();
        $c = $tables['categories']['columns']; # categories is a database table

        $deltaLevel = -($CatInfo['level']-1)+$NewParentCatInfo['level'];

        if ($NewParentCatInfo['left']  < $CatInfo['left'] &&
            $NewParentCatInfo['right'] > $CatInfo['right'] &&
            $NewParentCatInfo['level'] < $CatInfo['level'] - 1)
        {
            $query = &new DB_Update('categories');
            $query->addUpdateExpression($c['level'],
                                        $query->fIf($c['left'].' '.DB_BETWEEN.' '.$CatInfo['left']
                                                              .' '.DB_AND    .' '.$CatInfo['right'],
                                                    $c['level'] . sprintf('%+d', $deltaLevel),
                                                    $c['level']
                                                   )
                                       );
            $query->addUpdateExpression($c['right'],
                                        $query->fIf($c['right'].' '.DB_BETWEEN.' '.($CatInfo['right']+1)
                                                               .' '.DB_AND    .' '.($NewParentCatInfo['right']-1),
                                                    $c['right'].'-'.($CatInfo['right']-$CatInfo['left']+1),
                                                    $query->fIf($c['left'].' '.DB_BETWEEN.' '.$CatInfo['left']
                                                                          .' '.DB_AND    .' '.$CatInfo['right'],
                                                                $c['right'].'+'.((($NewParentCatInfo['right']-
                                                                                   $CatInfo['right']-
                                                                                   $CatInfo['level']+
                                                                                   $NewParentCatInfo['level'])/2)*2+
                                                                                   $CatInfo['level']-
                                                                                   $NewParentCatInfo['level']-1),
                                                                $c['right']
                                                               )
                                                   )
                                       );
            $query->addUpdateExpression($c['left'],
                                        $query->fIf($c['left'].' '.DB_BETWEEN.' '.($CatInfo['right']+1)
                                                               .' '.DB_AND   .' '.($NewParentCatInfo['right']-1),
                                                    $c['left'].'-'.($CatInfo['right']-$CatInfo['left']+1),
                                                    $query->fIf($c['left'].' '.DB_BETWEEN.' '.$CatInfo['left']
                                                                          .' '.DB_AND    .' '.$CatInfo['right'],
                                                                $c['left'].'+'.((($NewParentCatInfo['right']-
                                                                                  $CatInfo['right']-
                                                                                  $CatInfo['level']+
                                                                                  $NewParentCatInfo['level'])/2)*2+
                                                                                  $CatInfo['level']-
                                                                                  $NewParentCatInfo['level']-1),
                                                                $c['left']
                                                               )
                                                   )
                                       );
            $query->WhereField($c['left'], DB_BETWEEN,"'". ($NewParentCatInfo['left']+1)."'"
                                          .' '.DB_AND." '".($NewParentCatInfo['right']-1)."'"
                              );
            $application->db->getDB_Result($query);
        }
        elseif ($NewParentCatInfo['left'] < $CatInfo['left'])
        {
            $query = &new DB_Update('categories');
            $query->addUpdateExpression($c['level'],
                                        $query->fIf($c['left'].' '.DB_BETWEEN.' '.$CatInfo['left']
                                                              .' '.DB_AND    .' '.$CatInfo['right'],
                                                    $c['level'] . sprintf('%+d', $deltaLevel),
                                                    $c['level']
                                                   )
                                       );
            $query->addUpdateExpression($c['left'],
                                        $query->fIf($c['left'].' '.DB_BETWEEN.' '.$NewParentCatInfo['right']
                                                              .' '.DB_AND    .' '.($CatInfo['left']-1),
                                                    $c['left'].'+'.($CatInfo['right']-$CatInfo['left']+1),
                                                    $query->fIf($c['left'].' '.DB_BETWEEN.' '.$CatInfo['left']
                                                                          .' '.DB_AND    .' '.$CatInfo['right'],
                                                                $c['left'].'-'.($CatInfo['left']-
                                                                                $NewParentCatInfo['right']),
                                                                $c['left']
                                                               )
                                                   )
                                       );
            $query->addUpdateExpression($c['right'],
                                        $query->fIf($c['right'].' '.DB_BETWEEN.' '.$NewParentCatInfo['right']
                                                               .' '.DB_AND    .' '.$CatInfo['left'],
                                                    $c['right'].'+'.($CatInfo['right']-$CatInfo['left']+1),
                                                    $query->fIf($c['right'].' '.DB_BETWEEN.' '.$CatInfo['left']
                                                                           .' '.DB_AND    .' '.$CatInfo['right'],
                                                                $c['right'].'-'.($CatInfo['left']-
                                                                                $NewParentCatInfo['right']),
                                                                $c['right']
                                                               )
                                                   )
                                       );
            $query->WhereField($c['left'], DB_BETWEEN,"'". $NewParentCatInfo['left']."'"
                                          .' '.DB_AND." '".$CatInfo['right']."'"
                              );
            $query->WhereOR();
            $query->WhereField($c['right'], DB_BETWEEN,"'". $NewParentCatInfo['left']."'"
                                           .' '.DB_AND." '".$CatInfo['right']."'"
                              );
            $application->db->getDB_Result($query);
        }
        else
        {
            $query = &new DB_Update('categories');
            $query->addUpdateExpression($c['level'],
                                        $query->fIf($c['left'].' '.DB_BETWEEN.' '.$CatInfo['left']
                                                              .' '.DB_AND    .' '.$CatInfo['right'],
                                                    $c['level'] . sprintf('%+d', $deltaLevel),
                                                    $c['level']
                                                   )
                                       );
            $query->addUpdateExpression($c['left'],
                                        $query->fIf($c['left'].' '.DB_BETWEEN.' '.$CatInfo['right']
                                                              .' '.DB_AND    .' '.$NewParentCatInfo['right'],
                                                    $c['left'].'-'.($CatInfo['right']-$CatInfo['left']+1),
                                                    $query->fIf($c['left'].' '.DB_BETWEEN.' '.$CatInfo['left']
                                                                          .' '.DB_AND    .' '.$CatInfo['right'],
                                                                $c['left'].'+'.($NewParentCatInfo['right']-
                                                                                $CatInfo['right']-1),
                                                                $c['left']
                                                               )
                                                   )
                                       );
            $query->addUpdateExpression($c['right'],
                                        $query->fIf($c['right'].' '.DB_BETWEEN.' '.($CatInfo['right']+1)
                                                               .' '.DB_AND    .' '.($NewParentCatInfo['right']-1),
                                                    $c['right'].'-'.($CatInfo['right']-$CatInfo['left']+1),
                                                    $query->fIf($c['right'].' '.DB_BETWEEN.' '.$CatInfo['left']
                                                                           .' '.DB_AND    .' '.$CatInfo['right'],
                                                                $c['right'].'+'.($NewParentCatInfo['right']-
                                                                                 $CatInfo['right']-1),
                                                                $c['right']
                                                               )
                                                   )
                                       );
            $query->WhereField($c['left'], DB_BETWEEN,"'". $CatInfo['left']."'"
                                          .' '.DB_AND." '".$NewParentCatInfo['right']."'"
                              );
            $query->WhereOR();
            $query->WhereField($c['right'], DB_BETWEEN,"'". $CatInfo['left']."'"
                                           .' '.DB_AND." '".$NewParentCatInfo['right']."'"
                              );
            $application->db->getDB_Result($query);
        }
        /*$query = &new DB_Update('categories');
        $query->addUpdateValue($c['sort_order'], $this->generateCategorySortOrder($newParentCatId));
        $query->WhereValue($c['id'], DB_EQ, $cid);
        $application->db->getDB_Result($query);*/
        #   delete all categories from cache
        modApiFunc('CatalogCacheManager', 'eventCategorySubcategoriesContentChanged', $newParentCatId);
        modApiFunc('CatalogCacheManager', 'eventCategorySubcategoriesContentChanged', $CurrentParentCatInfo);
        modApiFunc('CatalogCacheManager', 'eventCategoryInfoChanged', $CurrentParentCatInfo);
        modApiFunc('CatalogCacheManager', 'eventCategoryInfoChanged', $newParentCatId);
        modApiFunc('CatalogCacheManager', 'eventCategoryInfoChanged', $cid);
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End');
    }

    /**
     * Moves products.
     *
     * @author Alexander Girin
     * @param integer $newParentCatId the new category id,
     *        to which the category is moved
     * @param integer $productsID the moved category id
     * @return
     */
    function moveProducts($oldParentCatId, $newParentCatId, $productsID)
    {
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Start, Old Parent Category ID = '.$oldParentCatId.', New Parent Category ID = '.$newParentCatId.', Products = ('.implode(', ',$productsID).')');

        if(!is_array($productsID) or empty($productsID))
        {
            log_warn('['.__CLASS__.'::'.__FUNCTION__.'] Nothing to move, Empty Products List');
            return;
        }

        $this->delProductLinksFromCategory($oldParentCatId,$productsID);
        foreach($productsID as $pid)
        {
            $prodObj = &new CProductInfo($pid);
            if(!in_array($newParentCatId,$prodObj->getCategoriesIDs()))
                $this->addProductLinkToCategory($pid, $newParentCatId);
        }

        #   delete all categories from cache
        modApiFunc('CatalogCacheManager', 'eventCategoryProductsContentChanged', $newParentCatId);
        modApiFunc('CatalogCacheManager', 'eventCategoryInfoChanged', $newParentCatId);
        modApiFunc('CatalogCacheManager', 'eventCategoryProductsContentChanged', $oldParentCatId);
        modApiFunc('CatalogCacheManager', 'eventCategoryInfoChanged', $oldParentCatId);
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End');
    }

    /**
     * Copies products.
     *
     * @author Alexander Girin
     * @param integer $newParentCatId the new category id, to which products
     *         are copied
     * @param array $productsID the array of copied products
     * @return
     */
    function copyProducts($newParentCatId, $productsID)
    {
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Start, New Parent Category ID = '.$newParentCatId.', Products = ('.implode(', ',$productsID).')');
        // copy results
        $_copy_result=array();

        $parentCategoriesList = array();
        foreach ($productsID as $pId)
        {
            $prd = &new CProductInfo($pId);
            $new_pId=$prd->clone_to_category($newParentCatId);
            $parentCategoriesList = array_merge($parentCategoriesList,$prd->getCategoriesIDs());//$prd->getProductTagValue('categoryid');
            $prd = null;

            // adding copy result
            $_copy_result[$pId]=$new_pId;
        }

        // saving copy results to session
        modApiFunc("Session","set","CopyProductsResult",$_copy_result);

        # delete from cache
        modApiFunc('CatalogCacheManager', 'eventCategoryProductsContentChanged', $newParentCatId);
        modApiFunc('CatalogCacheManager', 'eventCategoryInfoChanged', $newParentCatId);
        foreach ($parentCategoriesList as $cid)
        {
            /*
                NOTE:                                                                      
                              .

                1.         HTML           NavigationBar,                                      
                                                         ,                 .

                2.                              ,                                             
                         .
            */
            modApiFunc('CatalogCacheManager', 'eventCategoryProductsContentChanged', $cid);
            modApiFunc('CatalogCacheManager', 'eventCategoryInfoChanged', $cid);
        }
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End');
    }

    /**
     * Gets a product name or product name list by the product Id (products ID).
     *
     * @todo check if this function is needed. getProductInfo
     * @author Alexander Girin
     * @param mixed $prodId product id (products ID)
     * @return mixed product name (products name)
     */
    function getProductNameById($prodId, $use_peginator=true)
    {
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Start');
        global $application;

        $tables = $this->getTables();
        $p = $tables['products']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($p['id'], 'id');
        $query->addSelectField($p['name'], 'name');
        if (is_array($prodId))
        {
            $query->WhereField($p['id'], DB_IN,"('" . implode("','", $prodId) . "')");
            if ($use_peginator)
            {
                $query = modApiFunc('paginator', 'setQuery', $query);
            }
        }
        else
        {
            $query->WhereValue($p['id'], DB_EQ, $prodId);
        }
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End');
        return $application->db->getDB_Result($query);
    }

    /**
     * Returns complete attribute info for the specified product type.
     *
     * @param integer $type_id product type id.
     * @param string the attribute name.
     * @todo change it for more optimized version
     */
    function getAttributeInfo($type_id, $name)
    {
        $attributes = $this->getProductTypeAttributes($type_id);
        if (!array_key_exists($name, $attributes))
        {
        	return '';
        }

        $attr = $attributes[$name];
        $result = '';
        if ($attr['input_type_name'] == 'select')
        {
        	$result = 'test';
        }
        else
        {
            $result = $attr;
        }
        return $result;
    }

    /**
     * Gets complete product information with the id and returned as
     * associative array.<br>
     * It is used caching.
     * The array structure:
     * <code>
     * array(
     *   'attribute' => 'value' // TODO describe all attributes
     *  ,'attributes' => array (
     *     'attribute' => array (
     *      ,'id'       => // the attribute id
     *      ,'name'     => // the attribute name
     *      ,'size'     => // the input field size
     *      ,'min'      => // the min number of inputted characters
     *      ,'max'      => // the max number of inputted characters
     *      ,'view_tag' => // the attribute tag to use in the templates
     *      ,'group'    => array ( // info about the group, to which the
                                      attribute refers
     *         'id'   => // the id
     *        ,'name' => // the name
     *        ,'sort' => // the sorting number
     *       )
     *      ,'required' => // the flag the required input
     *      ,'visible'  => // the flag of the product visibility
     *      ,'default'  => // the default value on inputting
     *      ,'sort'     => // the sorting serial number
     *      ,'input_type_id'     => // the type id of input field
     *      ,'input_type_name'   => // the type name of input field
     *      ,'input_type_values' => array ( // ???
     *         'key' => 'value'
     *       )
     *      ,'unit_type_id'     => // the unit id
     *      ,'unit_type_name'   => // the unit name
     *      ,'unit_type_value'  => // the unit value
     *      ,'unit_type_values' => array ( // ???
     *         'key' => 'value'
     *       )
     *      ,'pa_id'    => // the attribute id for the specified product
     *      ,'value'    => // the attribute value for the specified product
     *     )
     *   )
     * )
     * </code>
     * @param integer $prod_id
     * @return array the array containing product info.
     * @see Catalog::getProductType()
     * @see Catalog::getProductTypeAttributes()
     */
    function getProductInfo($prod_id, $localized=true)
    {
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Start, Product ID = '.$prod_id.', Localized = '.($localized ? 'True' : 'False'));
        global $application;

        if (!$this->isCorrectProductId($prod_id))
        {
            log_warn('['.__CLASS__.'::'.__FUNCTION__.'] Incorrect Product ID, Returned NULL');
            return null;
        }


        $tables = $this->getTables();

        $p = $tables['products']['columns'];
        $pt  = $tables['product_types']['columns'];

        // Get general product attributes.
        $query = &new DB_Select();
        $query->addSelectField($p['id'], 'id');
        $query->addSelectField($p['name'], 'name');
        //$query->addSelectField($p['c_id'], 'catalog_id');
        $query->addSelectField($p['date_updated'], 'date_updated');
        $query->addSelectField($p['date_added'], 'date_added');
        $query->addSelectField($pt['id'], 'type_id');
        $query->addSelectField($pt['name'], 'type_name');
        $query->Where($p['id'], DB_EQ, $prod_id);
        $query->WhereAND();
        $query->WhereField($pt['id'], DB_EQ, $p['pt_id']);
        //$query->SelectOrder($p['sort_order']);
        $result = $application->db->getDB_Result($query);

        if (sizeof($result)<=0)
        {
            return null;
        }

        /**
         * Get real attribute values for the given product,
         * results are in $product_attributes.
         */
        $pa  = $tables['product_attributes']['columns'];
        $pi  = $tables['product_images']['columns'];
        $a   = $tables['attributes']['columns'];
        $query = &new DB_Select();
        $query->addSelectField($pa['id'],         'id');
        $query->addSelectField($pa['attr_value'], 'value');
        $query->addSelectField($a['view_tag'], 'view_tag');
        //                       ,             
        $query->addSelectField($pi['id'],         'image_id');
        $query->addSelectField($pi['name'],       'image_name');
        $query->addSelectField($pi['width'],      'image_width');
        $query->addSelectField($pi['height'],     'image_height');
        $query->addLeftJoin('product_images', $pa['id'], DB_EQ, $pi['pa_id']);
        $query->WhereValue($pa['p_id'], DB_EQ, $prod_id);
        $query->WhereAND();
        $query->WhereField($a['id'], DB_EQ, $pa['a_id']);
        $attr_result = $application->db->getDB_Result($query);

        $product_attributes = array();
        for ($j=0; $j<sizeof($attr_result); $j++)
        {
            $view_tag = $attr_result[$j]['view_tag'];
            $product_attributes[$view_tag] = $attr_result[$j];
        }

        //          SalePrice,                           
        //                                TaxClass.
        $tax_class_id = NULL;
        foreach ($product_attributes as $index => $attribute)
        {
            if (strtolower($index) == 'taxclass')
            {
                $tax_class_id = $product_attributes[$index]['value'];
                break;
            }
        }

        $price_including_included_taxes_if_any = NULL;
        foreach ($product_attributes as $index => $attribute)
        {
            if (strtolower($index) == 'saleprice')
            {
                $price_including_included_taxes_if_any = $product_attributes[$index]['value'];

                $product_attributes['salepriceexcludingtaxes'] = $product_attributes[$index];
                $product_attributes['salepriceexcludingtaxes']['id'] = NULL;
                $product_attributes['salepriceexcludingtaxes']['view_tag'] = NULL;

                //     TaxClass             -        ,                     .
                if($tax_class_id !== NULL)
                {
                    //                                         .
                    $price_excluding_taxes = modApiFunc("Catalog", "computePriceExcludingTaxes", $product_attributes[$index]['value'], $tax_class_id);
                    $product_attributes[$index]['value'] = $price_excluding_taxes;

                    //              _   _                         'salepriceexcludingtaxes':
                    $product_attributes['salepriceexcludingtaxes']['value'] = $price_excluding_taxes;
                }
                break;
            }
        }

        //          SalePrice,                               "          "       .
        //     TaxClass             -        ,                     .
        $display_product_price_including_taxes = modApiFunc('Configuration', 'getValue', SYSCONFIG_STORE_DISPLAY_TOTALS_INCLUDING_TAXES);

        foreach ($product_attributes as $index => $attribute)
        {
            if (strtolower($index) == 'saleprice')
            {
                $product_attributes['salepriceincludingtaxes'] = $product_attributes[$index];
                $product_attributes['salepriceincludingtaxes']['id'] = NULL;
                $product_attributes['salepriceincludingtaxes']['view_tag'] = NULL;

                if($tax_class_id !== NULL)
                {
                    //                                         .
                    $price_including_taxes = $price_including_included_taxes_if_any;
                    //                                             ,                         
                    //        ,             "          "
                    //modApiFunc("Catalog", "computePriceIncludingTaxes", $product_attributes[$index]['value'], $tax_class_id);

                    //              _c_                          'salepriceincludingtaxes':
                    $product_attributes['salepriceincludingtaxes']['value'] = $price_including_taxes;

                    if($display_product_price_including_taxes === true)
                    {
                        $product_attributes[$index]['value'] = $price_including_taxes;
                    }
                }
                break;
            }
        }

        $product = array(); //the array of tags and their values
        $product['attributes'] = array();

        // add info with the artificial attributes
        $product['Updated'] = modApiFunc("Localization", "SQL_date_format", $result[0]['date_updated']);
        $product['Added'] = modApiFunc("Localization", "SQL_date_format", $result[0]['date_added']);
        $product['TypeID'] = $result[0]['type_id'];
        $product['TypeName'] = $result[0]['type_name'];
        //$product['Catalog_ID'] = $result[0]['catalog_id'];

        // define the attribute list or the given product type
        $product_type = $this->getProductType($result[0]['type_id']);

        // copy attributes to the product description
        foreach ($product_type['attr'] as $view_tag => $attr)
        {
            $product[$view_tag] = ''; // initialize the attribute value.
            $product['attributes'][$view_tag] = array();
            $product_attribut = &$product['attributes'][$view_tag];
            $product_attribut['id'] = $attr['id'];
            $product_attribut['name'] = $attr['name'];
            $product_attribut['size'] = $attr['size'];
            $product_attribut['min'] = $attr['min'];
            $product_attribut['max'] = $attr['max'];
            $product_attribut['view_tag'] = $attr['view_tag'];
            $product_attribut['group'] = $attr['group'];
            $product_attribut['required'] = $attr['required'];
            $product_attribut['visible'] = $attr['visible'];
            $product_attribut['default'] = $attr['default'];
            $product_attribut['sort'] = $attr['sort'];
            $product_attribut['allow_html'] = $attr['allow_html'];
            $product_attribut['patt'] = $attr['patt'];
            $product_attribut['patt_type'] = $attr['patt_type'];
            $product_attribut['input_type_id'] = $attr['input_type_id'];
            $product_attribut['input_type_name'] = $attr['input_type_name'];
            $product_attribut['input_type_values'] = isset($attr['input_type_values']) ? $attr['input_type_values'] : null;
            $product_attribut['unit_type_value'] = $attr['unit_type_value'];
            $product_attribut['unit_type_values'] = isset($attr['unit_type_values']) ? $attr['unit_type_values'] : null;
            $product_attribut['unit_type_values_pattern'] = isset($attr['unit_type_values_pattern']) ? $attr['unit_type_values_pattern'] : null;
            $product_attribut['pa_id'] = null;
            $product_attribut['value'] = null;
            $product_attribut['additional_link'] = isset($attr['additional_link'])? $attr['additional_link']:"";
            $product_attribut['additional_link_text'] = isset($attr['additional_link_text'])? $attr['additional_link_text']:"";
            // add real attribute values for the given product
            if (!array_key_exists($view_tag, $product_attributes))
            {
                $imagesUrl = $application->getAppIni('URL_IMAGES_DIR');
                if ($application->getCurrentProtocol() == "https" && $application->getAppIni('HTTPS_URL_IMAGES_DIR'))
                {
                    $imagesUrl = $application->getAppIni('HTTPS_URL_IMAGES_DIR');
                }
                if ($product_attribut['input_type_name'] == 'image')
                {
                	$product_attribut['value'] = array(
                	   'url' => ''
                	  ,'exists' => false
                	  ,'width' => 1
                	  ,'height' => 1
                	   );
                	// The image is not found. Output a standard one, if necessary.
                	// FIXME Should not be used
                	if (/* $application->getAppIni('SHOW_NO_PICTURE_IMAGE') == 'ENABLED' */ false && file_exists($application->getAppIni('PATH_NO_PICTURE_IMAGE')))
                	{
                	    $noPictureInfo = @getimagesize($application->getAppIni('PATH_NO_PICTURE_IMAGE'));
                	    $product_attribut['value']['url'] = $application->getAppIni('NO_PICTURE_IMAGE');
                	    $product_attribut['value']['width'] = $noPictureInfo[0];
                	    $product_attribut['value']['height'] = $noPictureInfo[1];
                	}
                    $product[$view_tag] = '';
                    $product[$view_tag.'Src'] = ''; //$imagesUrl.$product_attribut['value']['url'];
                    $product[$view_tag.'Width'] = '1'; //$product_attribut['value']['width'];
                    $product[$view_tag.'Height'] = '1'; //$product_attribut['value']['height'];
                }
            }
            else
            {
                //  the id is in the product_attributes table
                $product_attribut['pa_id'] = $product_attributes[$view_tag]['id'];
                // the attribute value
                $product_attribut['value'] = $product_attributes[$view_tag]['value'];
                if (!$product_attribut['allow_html'] && $localized)
                {
                    // Correct, because the HTML code is unavailable.
                    $product_attribut['value'] = prepareHTMLDisplay($product_attributes[$view_tag]['value']);
                }
                if ($localized)
                {
                    $product[$view_tag] = modApiFunc("Localization", "format", $product_attribut['value'], $product_attribut['patt_type']);
                }
                else
                {
                    $product[$view_tag] = $product_attribut['value'];
                }
                // determine the URL of the image TODO remove it later
                if ($product_attribut['input_type_name'] == 'image') //LargeImage, SmallImage for products, as for 2005 oct.
                {
                    $imagesUrl = $application->getAppIni('URL_IMAGES_DIR');
                    if ($application->getCurrentProtocol() == "https" && $application->getAppIni('HTTPS_URL_IMAGES_DIR'))
                    {
                        $imagesUrl = $application->getAppIni('HTTPS_URL_IMAGES_DIR');
                    }
                    if ($product_attributes[$view_tag]['image_name'] != null && $application->isImageFileValid($product_attributes[$view_tag]['image_name']))
                    {
                    	$product_attribut['value'] = array(
                    	   'url' => $product_attributes[$view_tag]['image_name']
                    	  ,'exists' => true
                    	  ,'width' => $product_attributes[$view_tag]['image_width']
                    	  ,'height' => $product_attributes[$view_tag]['image_height']
                    	   );
                    }
                    else
                    {
                    	$product_attribut['value'] = array(
                    	   'url' => ''
                    	  ,'exists' => false
                    	  ,'width' => 1
                    	  ,'height' => 1
                    	   );
                    	// The image is not found. Output a standard one, if necessary.
                        // It should not be used.
                    	if (/* $application->getAppIni('SHOW_NO_PICTURE_IMAGE') == 'ENABLED' */ false && file_exists($application->getAppIni('PATH_NO_PICTURE_IMAGE')))
                    	{
                    	    $noPictureInfo = @getimagesize($application->getAppIni('PATH_NO_PICTURE_IMAGE'));
                    	    $product_attribut['value']['url'] = $application->getAppIni('NO_PICTURE_IMAGE');
                    	    $product_attribut['value']['width'] = $noPictureInfo[0];
                    	    $product_attribut['value']['height'] = $noPictureInfo[1];
                    	}
                    }
                    // generate the values for the derived image attributes
                    if ($product_attribut['value']['exists'])
                    {
                        $product[$view_tag] = '<img src="'.$imagesUrl.$product_attribut['value']['url'].'" width="'.$product_attribut['value']['width'].'" height="'.$product_attribut['value']['height'].'" alt="'.$product_attributes['ImageAltText']['value'].'" border="0">';

                        $product[$view_tag.'Src'] = $imagesUrl.$product_attribut['value']['url'];
                        $product[$view_tag.'Width'] = $product_attribut['value']['width'];
                        $product[$view_tag.'Height'] = $product_attribut['value']['height'];
                    }
                    else
                    {
                        $product[$view_tag] = '';

                        $product[$view_tag.'Src'] = '';
                        $product[$view_tag.'Width'] = '1';
                        $product[$view_tag.'Height'] = '1';
                    }

                }

                // add the Unit value, if it exists
                if ($product_attribut['patt_type']!='currency' && $product_attribut['unit_type_value'])
                {
                    $product[$view_tag] .= ' '.$product_attribut['unit_type_value'];
                }
                // Correct the attribute value, if it is specified input_type for it.
                if (isset($product_attribut['input_type_values']) && $product_attribut['value'] != '')
                {
                    $product[$view_tag] = $product_attribut['input_type_values'][$product_attributes[$view_tag]['value']];
                }
            }
            // if the attribute is not visible, delete values for CZ.
            if (!$attr['visible'])
            {
            	$product[$view_tag] = '';
            }
        }

        $product['ID'] = $result[0]['id'];
        $product['attributes']['ID']['value'] = $result[0]['id'];
        $product['Name'] = $localized ? prepareHTMLDisplay($result[0]['name']):$result[0]['name'];
        $product['attributes']['Name']['value'] = $localized ? prepareHTMLDisplay($result[0]['name']):$result[0]['name'];

        $product['salepriceincludingtaxes'] = $localized ? modApiFunc("Localization", "format", $product_attributes['salepriceincludingtaxes']['value'], 'currency'):$product_attributes['salepriceincludingtaxes']['value'];
        //                         .     SalePrice.
        $product['attributes']['salepriceincludingtaxes']['value'] = $product_attributes['salepriceincludingtaxes']['value'];
        $product['salepriceexcludingtaxes'] = $localized ? modApiFunc("Localization", "format", $product_attributes['salepriceexcludingtaxes']['value'], 'currency'):$product_attributes['salepriceexcludingtaxes']['value'];
        //                         .     SalePrice.
        $product['attributes']['salepriceexcludingtaxes']['value'] = $product_attributes['salepriceexcludingtaxes']['value'];
        #  create required references
        ### InfoLink
        $request=&new Request();
        $request->setView  ( 'ProductInfo' );
        $request->setAction( 'SetCurrentProduct' );
        $request->setKey   ( 'prod_id', $product['ID']);
        //$request->setCategoryID($product['Catalog_ID']);
        $request->setProductID($product['ID']);
        $product['InfoLink'] =  $request->getURL();

        ### BuyLink
        $request=&new Request();
        modApiFunc("Configuration", "getValue", "store_show_cart")? $request->setView('CartContent'):$request->setView(CURRENT_REQUEST_URL);
        $request->setAction( 'AddToCart' );
        $request->setKey   ( 'prod_id', $product['ID']);
        //$request->setCategoryID($product['Catalog_ID']);
        $request->setProductID($product['ID']);
        $product['BuyLink'] = $request->getURL();

        ### CategoryLink.
        /*
        $request=&new Request();
        $request->setView('ProductList');
        $request->setAction( "SetCurrCat" );
        $request->setKey   ( "category_id", $product['Catalog_ID']);
        $request->setCategoryID($product['Catalog_ID']);
        $request->setProductID($product['ID']);
        $product['CategoryLink'] = $request->getURL();
		*/

        #                  .                       view                                                          .
        $prodObj = &new CProductInfo($product['ID']);
        $product['CategoryLink'] = $prodObj->getProductTagValue('CategoryLink');
        $product['CategoryID'] = $prodObj->getProductTagValue('CategoryID');
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End');
        return $product;
    }

    /**
     * It is similar to getProductInfo().
     * It is outputted standard or only custom attributes info, which is
     * specified by the in-parameter.
     * Including info about artificial attributes, such as Name,
     * is specified by the in-parameter.
     * The attribute values are not processed fro inputting (for example,
     * by the function prepareHTMLDisplay).
     * </code>
     *
     * @param integer $prod_id
     * @return array the array containing product info.
     * @see Catalog::getProductType()
     * @see Catalog::getProductTypeAttributes()
     */
    function getProductInfoRaw($prod_id, $attributes_type, /* standard/custom */ $include_artificial_attributes = true)
    {
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Start, Product ID = '.$prod_id);
        global $application;
        if (!$this->isCorrectProductId($prod_id))
        {
            log_warn('['.__CLASS__.'::'.__FUNCTION__.'] Incorrect Product ID, Returned NULL');
            return null;
        }

        $tables = $this->getTables();

        $p = $tables['products']['columns'];
        $pt  = $tables['product_types']['columns'];

        // Get general product attributes.
        $query = &new DB_Select();
        $query->addSelectField($p['id'], 'id');
        $query->addSelectField($pt['id'], 'type_id');
        $query->addSelectField($p['name'], 'name');
        //$query->addSelectField($pt['id'], 'type_id');
        $query->Where($p['id'], DB_EQ, $prod_id);
        $query->WhereAND();
        $query->WhereField($pt['id'], DB_EQ, $p['pt_id']);
        //$query->SelectOrder($p['sort_order']);
        $result = $application->db->getDB_Result($query);

        if (sizeof($result)<=0)
        {
            return null;
        }

        /**
         * Get real attribute values for the given product,
         * results are $product_attributes.
         */
        $pa  = $tables['product_attributes']['columns'];
        $pi  = $tables['product_images']['columns'];
        $a   = $tables['attributes']['columns'];
        $query = &new DB_Select();
        $query->addSelectField($pa['id'],         'id');
        $query->addSelectField($pa['attr_value'], 'value');
        $query->addSelectField($a['view_tag'], 'view_tag');
        $query->addSelectField($a['name'], 'name');
        // info about images, if it exists
        $query->addSelectField($pi['id'],         'image_id');
        $query->addSelectField($pi['name'],       'image_name');
        $query->addLeftJoin('product_images', $pa['id'], DB_EQ, $pi['pa_id']);
        $query->WhereValue($pa['p_id'], DB_EQ, $prod_id);
        $query->WhereAND();
        $query->WhereField($a['id'], DB_EQ, $pa['a_id']);
        $query->WhereAND();
        $query->WhereValue($a['type'], DB_EQ, $attributes_type);
        $attr_result = $application->db->getDB_Result($query);
        $product_attributes = array();

        for ($j=0; $j<sizeof($attr_result); $j++)
        {
            $view_tag = $attr_result[$j]['view_tag'];
            $product_attributes[$view_tag] = $attr_result[$j];
        }

         $product = array(); //the array of tags and their values
        $product['attributes'] = array();

        // define the attribute list or the given product type
        $product_type = $this->getProductType($result[0]['type_id']);

        // copy attributes to the product description
        foreach ($product_type['attr'] as $view_tag => $attr)
        {
            // don't pass artificial attributes
            //if(!empty($product_attributes[$view_tag]['value']))
            if($attr['visible']==1 and isset($product_attributes[$view_tag]))
            {
                $product[$view_tag] = $product_attributes[$view_tag]['value']; //  initialize the attribute value.
                $product['attributes'][$view_tag] = array();
                $product_attribut = &$product['attributes'][$view_tag];

                $product_attribut['id'] = $attr['id'];
                $product_attribut['view_tag'] = $attr['view_tag'];
                $product_attribut['name'] = $attr['name'];
                //                   
                //  the attributes value
                $product_attribut['value'] = $product_attributes[$view_tag]['value'];
            }
        }

        if($include_artificial_attributes)
        {
            //add artificial attributes
            $product['ID'] = $result[0]['id'];
            $product['attributes']['ID']['value'] = $result[0]['id'];
            $product['Name'] = $result[0]['name'];
            $product['attributes']['Name']['value'] = $result[0]['name'];
            $product['TypeID'] = $result[0]['type_id'];
            $product['attributes']['TypeID']['value'] = $result[0]['type_id'];
        }
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End');
        return $product;
    }

    /**
     * Saves product info after editing.
     * It also checks whether each attribute for the given product exists.
     * If some attribute is not defined, a new record is created for it.
     *
     * @param $product_id integer the product id
     * @param $product_type_id integer hte product type id
     * @param $product_info array the associative array containing product info.
     * @todo         $product_info
     * @see Catalog::getProductType()
     */
    function updateProductInfo($product_id, $product_type_id, $product_info)
    {
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Start, Product ID = '.$product_id);
        global $application;
        $tables = $this->getTables();

        $product_type = $this->getProductType($product_type_id);

        $p = $tables['products']['columns'];
        $db_update = new DB_Update('products');
        $db_update->addUpdateValue($p['name'], $product_info['Name']);
        $db_update->addUpdateValue($p['date_updated'], date('Y-m-d H:i:s', time()));//getServerTime()));
        $db_update->WhereValue($p['id'], DB_EQ, $product_id);
    	$application->db->PrepareSQL($db_update);
    	$application->db->DB_Exec();

        $pa  = $tables['product_attributes']['columns'];
        $pi  = $tables['product_images']['columns'];
    	foreach ($product_type['attr'] as $view_tag => $attr)
    	{
    	    // skip artificial and invisible attributes
    	    if ($view_tag == 'ID' || $view_tag == 'Name' || !$attr['visible'])
    	    {
    	    	continue;
    	    }

            // prepare SEO URL tag
            if($view_tag == 'SEOPrefix' and isset($product_info[$view_tag]))
            {
                $product_info[$view_tag] = $this->_prepareSEOURL($product_info[$view_tag]);
            }

    	    // define if the value for the given attributes exists in the DB.
    	    $db_select = new DB_Select('product_attributes');
    	    $db_select->addSelectField($pa['id'], 'id');
            $db_select->addSelectField($pa['attr_value'], 'attr_value');
    	    $db_select->WhereValue($pa['a_id'], DB_EQ, $attr['id']);
    	    $db_select->WhereAND();
    	    $db_select->WhereValue($pa['p_id'], DB_EQ, $product_id);
    	    $result = $application->db->getDB_Result($db_select);
    	    $pa_id = 0;

            //                                               (basename).                  product_attributes
            //        :                                                          ,                     - http
            //       .                                   ,                                     .       
            //     product_attributes                  ,                          .
/*            if ($attr['input_type_name'] == 'image' && $product_info[$view_tag])
            {
                $product_info[$view_tag] = basename($product_info[$view_tag]);
            }
            */

    	    if ($result)
    	    {
    	        // The attribute exists already. Update.
    	        $pa_id = $result[0]['id'];
            	$db_update = new DB_Update('product_attributes');
        		$db_update->addUpdateValue($pa['attr_value'], $product_info[$view_tag]);
            	$db_update->WhereValue($pa['p_id'], DB_EQ, $product_id);
            	$db_update->WhereAND();
            	$db_update->WhereValue($pa['a_id'], DB_EQ, $attr['id']);
            	$application->db->PrepareSQL($db_update);
            	$application->db->DB_Exec();
    	    }
    	    else
    	    {
    	        // The attribute doesn't exist. Add.
    	        $db_insert = new DB_Insert('product_attributes');
    	        $db_insert->addInsertValue($product_id, $pa['p_id']);
    	        $db_insert->addInsertValue($attr['id'], $pa['a_id']);
    	        $db_insert->addInsertValue($product_info[$view_tag], $pa['attr_value']);
            	$application->db->PrepareSQL($db_insert);
            	$application->db->DB_Exec();
    	    }
    	    // If it is an image, save it to the separate table
    	    if ($attr['input_type_name'] == 'image')
    	    {
    	        $file_name = basename($product_info[$view_tag]);

                //                                             :
                if(!$result)
                {
                    $b_file_changed = true;
                }
                else
                {
                    $old_file_name = basename($result[0]['attr_value']);
                    $b_file_changed = ($old_file_name != $file_name);
                }

                //!$file_name -                ?
                if ($b_file_changed || !$file_name)
                {
                    $query = &new DB_Select();
                    $query->addSelectField($pi['name'], 'name');
                    $query->WhereField($pi['pa_id'], DB_EQ, $pa_id);
                    $result = $application->db->getDB_Result($query);
                    foreach ($result as $image)
                    {
                        @unlink($application->getAppIni('PATH_IMAGES_DIR').$image['name']);
                    }
                    // delete all the images for this attribute
                    $db_delete = &new DB_Delete('product_images');
                    $db_delete->WhereField($pi['pa_id'], DB_EQ, $pa_id);
                	$application->db->PrepareSQL($db_delete);
                	$application->db->DB_Exec();

                    //                   ,                                                      .
                    if ($b_file_changed && $file_name)
                    {
                        // if the image has been uploaded, save it to the folder and to the base.
                        $image_info = $application->saveUploadImage("", $file_name);
                        $db_insert = new DB_Insert('product_images');
                        $db_insert->addInsertValue($pa_id, $pi['pa_id']);
                        $db_insert->addInsertValue($image_info['name'], $pi['name']);
                        $db_insert->addInsertValue($image_info['width'], $pi['width']);
                        $db_insert->addInsertValue($image_info['height'], $pi['height']);
                        $application->db->PrepareSQL($db_insert);
                        $application->db->DB_Exec();
                    }
                }
    	    }

    	}
    	# delete the product from cache
    	modApiFunc('CatalogCacheManager', 'eventProductInfoChanged', $product_id);

    	$prodObj = &$application->getInstance('CProductInfo',$product_id);
        $cats_ids = $prodObj->getCategoriesIDs();
        foreach($cats_ids as $category_id)
            modApiFunc('CatalogCacheManager', 'eventCategoryProductsContentChanged', $category_id);

        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End');
    }

    function _updateProduct($product_id, $product_info)
    {
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Start');
        global $application;
        $tables = $this->getTables();

        $prod_table = $tables['products']['columns'];

        $db_update = new DB_Update('products');
        if(isset($product_info['Name']))
            $db_update->addUpdateValue($prod_table['name'], $product_info['Name']);
        $db_update->addUpdateValue($prod_table['date_updated'], date('Y-m-d H:i:s', time()));//getServerTime()));
        $db_update->WhereValue($prod_table['id'], DB_EQ, $product_id);
        $application->db->PrepareSQL($db_update);
        $application->db->DB_Exec();

        $pattr_table = $tables['product_attributes']['columns'];

        foreach($product_info as $attr_name => $attr_value)
        {
            if($attr_name == 'Name')
                continue;

            $attr_info = $this->_getAttrInfoByViewTag($attr_name);
            if($attr_info == null)
                continue;

            if($attr_name == 'SEOPrefix')
                $attr_value = $this->_prepareSEOURL($attr_value);

            $attr_id = $attr_info['attribute_id'];

            $query = &new DB_Update('product_attributes');
            $query->addUpdateValue($pattr_table['attr_value'], $attr_value);
            $query->WhereValue($pattr_table['p_id'], DB_EQ, $product_id);
            $query->WhereAND();
            $query->WhereValue($pattr_table['a_id'], DB_EQ, $attr_id);
            $application->db->PrepareSQL($query);
            $application->db->DB_Exec();
        };

        #                        
        modApiFunc('CatalogCacheManager', 'eventProductInfoChanged', $product_id);

        $prodObj = &new CProductInfo($product_id);
        $cats_ids = $prodObj->getCategoriesIDs();
        foreach($cats_ids as $cid)
            modApiFunc('CatalogCacheManager', 'eventCategoryProductsContentChanged', $cid);

        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End');
    }

    function _attachImageToProduct($product_id, $image_path, $image_tag_name)
    {
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Start');
        if($image_path == '')
            return;

        global $application;
        $tables = $this->getTables();

        $attr_info = $this->_getAttrInfoByViewTag($image_tag_name);
        if($attr_info == null)
            return;
        $attr_id = $attr_info['attribute_id'];

        $pattr_table = $tables['product_attributes']['columns'];
        $pimages_table = $tables['product_images']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($pattr_table['id']);
        $query->addSelectField($pattr_table['attr_value']);
        $query->WhereValue($pattr_table['p_id'], DB_EQ, $product_id);
        $query->WhereAND();
        $query->WhereValue($pattr_table['a_id'], DB_EQ, $attr_id);
        $res = $application->db->getDB_Result($query);

        if(!empty($res))
        {
            $query = &new DB_Select();
            $query->addSelectField($pimages_table['name']);
            $query->WhereValue($pimages_table['pa_id'], DB_EQ, $res[0]['product_attr_id']);
            $res_img = $application->db->getDB_Result($query);
            if(!empty($res_img))
            {
                $filename = $application->getAppIni('PATH_IMAGES_DIR').$res_img[0]['image_name'];
                @unlink($filename);
                $query = &new DB_Delete('product_images');
                $query->WhereValue($pimages_table['pa_id'], DB_EQ, $res[0]['product_attr_id']);
                $application->db->PrepareSQL($query);
                $application->db->DB_Exec();
            };
            $query = &new DB_Delete('product_attributes');
            $query->WhereValue($pattr_table['id'], DB_EQ, $res[0]['product_attr_id']);
            $application->db->PrepareSQL($query);
            $application->db->DB_Exec();
        };

        $image_info = getimagesize($image_path);
        $image_info['ext'] = str_replace("image/",".",$image_info['mime']);
        $image_info['width'] = $image_info[0];
        $image_info['height'] = $image_info[1];

        do
        {
            $image_file_path = $application->getUploadImageName($image_path);
            $image_file_name = basename($image_file_path);
        }
        while(file_exists($image_file_path));

        if(@copy($image_path, $image_file_path))
        {
            $query = &new DB_Insert('product_attributes');
            $query->addInsertValue($product_id, $pattr_table['p_id']);
            $query->addInsertValue($attr_id, $pattr_table['a_id']);
            $query->addInsertValue($application->getAppIni('URL_IMAGES_DIR').$image_file_name, $pattr_table['attr_value']);
            $application->db->PrepareSQL($query);
            $application->db->DB_Exec();

            $aid = $application->db->DB_Insert_Id();

            $query = &new DB_Insert('product_images');
            $query->addInsertValue($aid, $pimages_table['pa_id']);
            $query->addInsertValue($image_file_name, $pimages_table['name']);
            $query->addInsertValue($image_info['width'], $pimages_table['width']);
            $query->addInsertValue($image_info['height'], $pimages_table['height']);
            $application->db->PrepareSQL($query);
            $application->db->DB_Exec();
        };
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End');
    }

    function _getAttrInfoByViewTag($tag,$full=false)
    {
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Start');
        global $application;
        $tables = $this->getTables();

        $attr_table = $tables['attributes']['columns'];

        $query = &new DB_Select();
        $query->addSelectTable('attributes');
        $query->addSelectField('*');
        $query->WhereValue($attr_table['view_tag'], DB_EQ, $tag);
        $res = $application->db->getDB_Result($query);

        if(empty($res) or count($res)>1)
            return null;

        $info = array_shift($res);

        if($full)
        {
            $query = &new DB_Select();
            $query->addSelectField($tables['input_types']['columns']['name'],'input_type_name');
            $query->WhereValue($tables['input_types']['columns']['id'], DB_EQ, $info['input_type_id']);
            $res = $application->db->getDB_Result($query);
            $info['input_type_name'] = strtolower($res[0]['input_type_name']);
        };
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End');
        return $info;
    }

    /**
     * Saves new product info.
     *
     * @param $product_type_id integer the product type id
     * @param $category_id array the array containing product info.
     * @param $product_info array the array containing product info.
     * @return int $product_id - new product ID
     * @todo describe $product_info
     * @see Catalog::getProductType()
     */
    function addProductInfo($product_type_id, $category_id, $product_info)
    {
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Start');
        global $application;
        $tables = $this->getTables();

        $product_type = $this->getProductType($product_type_id);

        $p = $tables['products']['columns'];
        $db_insert = new DB_Insert('products');
        //$db_insert->addInsertValue($category_id, $p['c_id']);
        $db_insert->addInsertValue($product_type_id, $p['pt_id']);
        $db_insert->addInsertValue($product_info['Name'], $p['name']);
        $db_insert->addInsertValue(date('Y-m-d H:i:s', time()/*getServerTime()*/), $p['date_added']);
        $db_insert->addInsertValue(date('Y-m-d H:i:s', time()/*getServerTime()*/), $p['date_updated']);
        //$last_product_sort_order = $this->generateProductSortOrder($category_id);
        //$db_insert->addInsertValue($last_product_sort_order, $p['sort_order']);
    	$application->db->PrepareSQL($db_insert);
    	$application->db->DB_Exec();
    	$product_id = $application->db->DB_Insert_Id();

        //<add_link>
        $this->addProductLinkToCategory($product_id,$category_id);
        //</add_link>

        $pa  = $tables['product_attributes']['columns'];
        $pi  = $tables['product_images']['columns'];
    	foreach ($product_type['attr'] as $view_tag => $attr)
    	{
    	    // skip artificial attributes
    	    if ($view_tag == 'ID' || $view_tag == 'Name')
    	    {
    	    	continue;
    	    }

            // prepare SEO URL tag
            if($view_tag == 'SEOPrefix' and isset($product_info[$view_tag]))
            {
                $product_info[$view_tag] = $this->_prepareSEOURL($product_info[$view_tag]);
            }

	        $db_insert = new DB_Insert('product_attributes');
	        $db_insert->addInsertValue($product_id, $pa['p_id']);
	        $db_insert->addInsertValue($attr['id'], $pa['a_id']);
	        $db_insert->addInsertValue((isset($product_info[$view_tag])? $product_info[$view_tag]:""), $pa['attr_value']);
        	$application->db->PrepareSQL($db_insert);
        	$application->db->DB_Exec();
    	    $pa_id = $application->db->DB_Insert_Id();

    	    //  If it is an image, save it to the separate table.
    	    if ($attr['input_type_name'] == 'image')
    	    {

//            echo("-- Image 1");
                if ($product_info[$view_tag])
                {
//            echo("-- Image 2");
            	    // todo define if the value for the given attributes exists in the DB.
            	    $tmp_name = basename($product_info[$view_tag]);
//           echo(" !tmp_name=$tmp_name! ");




                    $image_info = $application->saveUploadImage("", $tmp_name);
        	        $db_insert = new DB_Insert('product_images');
        	        $db_insert->addInsertValue($pa_id, $pi['pa_id']);
        	        $db_insert->addInsertValue($image_info['name'], $pi['name']);
        	        $db_insert->addInsertValue($image_info['width'], $pi['width']);
        	        $db_insert->addInsertValue($image_info['height'], $pi['height']);
                	$application->db->PrepareSQL($db_insert);
                	$application->db->DB_Exec();
                }
//            echo("<br>");
    	    }
    	}

    	modApiFunc('CatalogCacheManager','eventCategoryProductsContentChanged', $category_id);

        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End');

        //return the new product ID to use later on in differnt hooks
        return $product_id;
    }

    /**
     * Adds a new product type.
     */
    function addProductType($product_type)
    {
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Start');
        global $application;
        $tables = $this->getTables();

        $pt = $tables['product_types']['columns'];
        $pta = $tables['product_type_attributes']['columns'];
        $a = $tables['attributes']['columns'];

        $db_insert = new DB_Insert('product_types');
        $db_insert->addInsertValue($product_type['TypeName'], $pt['name']);
        $db_insert->addInsertValue($product_type['TypeDescr'], $pt['descr']);
    	$application->db->PrepareSQL($db_insert);
    	$application->db->DB_Exec();
    	$product_type_id = $application->db->DB_Insert_Id();

    	foreach ($product_type as $view_tag => $attr)
    	{

			if (!is_array($attr))
				continue;

    		if ($attr['type'] == 'custom')
    		{
    			// create a new custom attribute
    			$db_insert = new DB_Insert('attributes');
    			$db_insert->addInsertValue($attr['group']['id'], $a['ag_id']);
    			$db_insert->addInsertValue($attr['input_type_id'], $a['it_id']);
    			$db_insert->addInsertValue($attr['unit_type_value'], $a['ut']);
    			$db_insert->addInsertValue($attr['view_tag'], $a['view_tag']);
    			$db_insert->addInsertValue($attr['name'], $a['name']);
    			$db_insert->addInsertValue($attr['descr'], $a['descr']);
    			$db_insert->addInsertValue($attr['type'], $a['type']);
    			$db_insert->addInsertValue($attr['min'], $a['min']);
    			$db_insert->addInsertValue($attr['max'], $a['max']);
    			$db_insert->addInsertValue($attr['size'], $a['size']);
    			$db_insert->addInsertValue($attr['sort'], $a['sort_order']);
            	$application->db->PrepareSQL($db_insert);
            	$application->db->DB_Exec();
            	$attribute_id = $application->db->DB_Insert_Id();

            	// associate with the given type
            	$db_insert = new DB_Insert('product_type_attributes');
    			$db_insert->addInsertValue($product_type_id, $pta['pt_id']);
    			$db_insert->addInsertValue($attribute_id, $pta['a_id']);
    			$db_insert->addInsertValue($attr['visible'], $pta['type_attr_visible']);
    			$db_insert->addInsertValue($attr['required'], $pta['type_attr_required']);
    			$db_insert->addInsertValue($attr['default'], $pta['type_attr_def_val']);
            	$application->db->PrepareSQL($db_insert);
            	$application->db->DB_Exec();
    		}
    		elseif ($attr['type'] == 'standard')
    		{
                $db_insert = new DB_Insert('product_type_attributes');
        		$db_insert->addInsertValue($product_type_id, $pta['pt_id']);
        		$db_insert->addInsertValue($attr['id'], $pta['a_id']);
        		$db_insert->addInsertValue($attr['visible'], $pta['type_attr_visible']);
        		$db_insert->addInsertValue($attr['required'], $pta['type_attr_required']);
        		$db_insert->addInsertValue($attr['default'], $pta['type_attr_def_val']);
            	$application->db->PrepareSQL($db_insert);
            	$application->db->DB_Exec();
    		}
    	}
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End');
    	return $product_type_id;
    }


    function getCustomAttrID($attr_tag_name)
    {
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Attribute Tag Name = '.$attr_tag_name);
        global $application;

        $tables = $this->getTables();
        $a      = $tables['attributes']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($a['id'], 'id');
        $query->WhereValue($a['type'], DB_EQ, 'custom');
        $query->WhereAND();
        $query->WhereValue($a['view_tag'], DB_EQ, $attr_tag_name);

        $result = $application->db->getDB_Result($query);

        if (is_array($result))
        {
        	if (array_key_exists(0, $result))
        	{
        		if (array_key_exists('id', $result[0]))
        		{
        			log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End');
                    return $result[0]['id'];
        		}
        	}
        }
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Returned 0 ');
        return 0;
	}


    /**
     * Adds a new generated product type.
     */
    function addGeneratedProductType($product_type)
    {
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Start ');
        global $application;
        $tables = $this->getTables();

        $pt = $tables['product_types']['columns'];
        $pta = $tables['product_type_attributes']['columns'];
        $a = $tables['attributes']['columns'];

        $db_insert = new DB_Insert('product_types');
        $db_insert->addInsertValue($product_type['TypeName'], $pt['name']);
        $db_insert->addInsertValue($product_type['TypeDescr'], $pt['descr']);
    	$application->db->PrepareSQL($db_insert);
    	$application->db->DB_Exec();
    	$product_type_id = $application->db->DB_Insert_Id();

    	foreach ($product_type as $view_tag => $attr)
    	{

    		if ($attr['type'] == 'custom')
    		{

    			// search, if the attribute with the same name exists already in the database
    			$attribute_id = $this->getCustomAttrID($attr['view_tag']);

    			// if such attribute doesn't exist,then create it
    			if (!$attribute_id)
    			{
	    			// create a new custom attribute
	    			$db_insert = new DB_Insert('attributes');
	    			$db_insert->addInsertValue($attr['group']['id'], $a['ag_id']);
	    			$db_insert->addInsertValue($attr['input_type_id'], $a['it_id']);
	    			$db_insert->addInsertValue($attr['unit_type_value'], $a['ut']);
	    			$db_insert->addInsertValue($attr['view_tag'], $a['view_tag']);
	    			$db_insert->addInsertValue($attr['name'], $a['name']);
	    			$db_insert->addInsertValue($attr['descr'], $a['descr']);
	    			$db_insert->addInsertValue($attr['type'], $a['type']);
	    			$db_insert->addInsertValue($attr['min'], $a['min']);
	    			$db_insert->addInsertValue($attr['max'], $a['max']);
	    			$db_insert->addInsertValue($attr['size'], $a['size']);
	    			$db_insert->addInsertValue($attr['sort'], $a['sort_order']);
	            	$application->db->PrepareSQL($db_insert);
	            	$application->db->DB_Exec();
	            	$attribute_id = $application->db->DB_Insert_Id();
    			}

                // associate with the given type
            	$db_insert = new DB_Insert('product_type_attributes');
    			$db_insert->addInsertValue($product_type_id, $pta['pt_id']);
    			$db_insert->addInsertValue($attribute_id, $pta['a_id']);
    			$db_insert->addInsertValue($attr['visible'], $pta['type_attr_visible']);
    			$db_insert->addInsertValue($attr['required'], $pta['type_attr_required']);
    			$db_insert->addInsertValue($attr['default'], $pta['type_attr_def_val']);
            	$application->db->PrepareSQL($db_insert);
            	$application->db->DB_Exec();
    		}
    		elseif ($attr['type'] == 'standard')
    		{
                $db_insert = new DB_Insert('product_type_attributes');
        		$db_insert->addInsertValue($product_type_id, $pta['pt_id']);
        		$db_insert->addInsertValue($attr['id'], $pta['a_id']);
        		$db_insert->addInsertValue($attr['visible'], $pta['type_attr_visible']);
        		$db_insert->addInsertValue($attr['required'], $pta['type_attr_required']);
        		$db_insert->addInsertValue($attr['default'], $pta['type_attr_def_val']);
            	$application->db->PrepareSQL($db_insert);
            	$application->db->DB_Exec();
    		}
    	}
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End ');
    	return $product_type_id;
    }


    function createCustomAttr($attr_name)
    {
    	log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Start, Attribute Name = '.$attr_name);
        $attr = array(
                'id'                => null
               ,'pta_id'            => null
               ,'name'              => prepareHTMLDisplay($attr_name)
               ,'descr'             => prepareHTMLDisplay($attr_name." attribute has been generated automatically from CSV file.")
               ,'size'              => 70
               ,'min'               => 2
               ,'max'               => 255
               ,'view_tag'          => prepareHTMLDisplay($attr_name)
               ,'group'             => array('id'=>6, 'name'=>'Custom Attributes', 'sort'=>6) // @fixme hard-coded
               ,'required'          => false
               ,'visible'           => true
               ,'default'           => ''
               ,'sort'              => 1
               ,'type'              => 'custom'
               ,'patt'              => null
               ,'patt_type'         => null
               ,'input_type_id'     => 1
               ,'input_type_name'   => 'text'
               ,'unit_type_value'   => null
               ,'vanishing'         => true
               ,'allow_html'        => 0
    		   );
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End ');
    	return $attr;
    }

    function createProductTypeInfo($base_product_type, $attr_names)
    {

		$names = array();
		$is_custom = array();

		foreach ($attr_names as $attr_name)
		{
			$names[] = $attr_name['name'];
			$is_custom[$attr_name['name']] = $attr_name['isCustom'];
		}

		$attr_in_base = array();
        foreach ($base_product_type as $view_tag => $attr)
        {
    		$attr_in_type = false;
        	if (in_array($view_tag, $names))
        	{
				$attr_in_type = true;
        		$attr_in_base[] = $view_tag;
        	}
        	else
        	{

                // The name of Custom attributes are distorted: XXX turns to Xxx. To find them you should ignore the register
        		if ($attr['type'] == 'custom')
        		{
        			foreach ($names as $name)
        			{
        				if (strcasecmp($view_tag, $name) == 0)
        				{
        					$attr_in_base[] = $name;
        					$attr_in_type = true;
        					break;
        				}
        			}
        		}
        	}

        	if ($attr_in_type)
        	{
        		$base_product_type[$view_tag]['visible'] = 1;
        	}
        	else
        	{
        		$base_product_type[$view_tag]['visible'] = 0;
        	}
        }

		// Create those attributes that doesn't exist in the base type
		foreach ($names as $name)
		{
			// search those attributes, that doesn't exist in the base type
			if (!in_array($name, $attr_in_base))
			{
        		$new_custom_attr = $this->createCustomAttr($name);
        		$base_product_type[$name] = $new_custom_attr;
			}
		}
		return $base_product_type;
    }

    /**
	 * Creates a new product type based on the list of atribute names.
     */

    function createProductType($attr_names, $new_product_type_name)
    {
    		// Gets an attribute list of base type
		$product_type = modApiFunc('Catalog', 'getProductType', 1);
		$new_product_type = $this->createProductTypeInfo($product_type['attr'], $attr_names);

    	$new_product_type['TypeName'] = $new_product_type_name;
    	$new_product_type['TypeDescr'] = 'This type has been generated automatically during import from CSV file';

    	$product_type_id = modApiFunc('Catalog', 'addGeneratedProductType', $new_product_type);
    	// return the type id
    	return $product_type_id;
    }

    function extendProductType($product_type_id, $attr_names)
   	{
    	// Get a list of base type attributes
    	$product_type = modApiFunc('Catalog', 'getProductType', $product_type_id);
		$extended_product_type = $this->createProductTypeInfo($product_type['attr'], $attr_names);

    	$extended_product_type['TypeName'] = $product_type['name'];
		$extended_product_type['TypeDescr'] = $product_type['description'];
        modApiFunc('Catalog', 'updateProductType', $product_type_id, $extended_product_type);
   	}

    /**
     * Updates the existing product type.
     */
    function updateProductType($product_type_id, $product_type)
    {
        global $application;
        $tables = $this->getTables();

        $pt = $tables['product_types']['columns'];
        $pta = $tables['product_type_attributes']['columns'];
        $a = $tables['attributes']['columns'];
        $pa = $tables['product_attributes']['columns'];

        // find the current custom attributes for this type
        $db_select = new DB_Select();
        $db_select->addSelectField($a['id'], 'a_id');
        $db_select->addSelectField($pta['id'], 'pta_id');
        $db_select->WhereValue($a['type'], DB_EQ, 'custom');
        $db_select->WhereAND();
        $db_select->WhereField($pta['a_id'], DB_EQ, $a['id']);
        $db_select->WhereAND();
        $db_select->WhereValue($pta['pt_id'], DB_EQ, $product_type_id);
        $result = $application->db->getDB_Result($db_select);
        $current_custom_attributes = array();
        if (sizeof($result) > 0)
        {
            for ($j=0; $j<sizeof($result); $j++)
            {
                $current_custom_attributes[] = $result[$j]['a_id'];
            }
	    }

        // update the name and product type description
        $db_update = new DB_Update('product_types');
        $db_update->addUpdateValue($pt['name'], $product_type['TypeName']);
        $db_update->addUpdateValue($pt['descr'], $product_type['TypeDescr']);
        $db_update->WhereValue($pt['id'], DB_EQ, $product_type_id);
    	$application->db->PrepareSQL($db_update);
    	$application->db->DB_Exec();

        $custom_attributes = array();
        $attributes = array();
    	// save current attributes
    	foreach ($product_type as $view_tag => $attr)
    	{

    		if (!is_array($attr))
    			continue;

    		if ($attr['type'] == 'custom')
    		{
    		    if ($attr['id'] == null)
    		    {
        			//                custom attribute
        			//  create a new custom attribute
        			$db_insert = new DB_Insert('attributes');
        			$db_insert->addInsertValue($attr['group']['id'], $a['ag_id']);
        			$db_insert->addInsertValue($attr['input_type_id'], $a['it_id']);
        			$db_insert->addInsertValue($attr['unit_type_value'], $a['ut']);
        			$db_insert->addInsertValue($attr['view_tag'], $a['view_tag']);
        			$db_insert->addInsertValue($attr['name'], $a['name']);
        			$db_insert->addInsertValue($attr['descr'], $a['descr']);
        			$db_insert->addInsertValue($attr['type'], $a['type']);
        			$db_insert->addInsertValue($attr['min'], $a['min']);
        			$db_insert->addInsertValue($attr['max'], $a['max']);
        			$db_insert->addInsertValue($attr['size'], $a['size']);
        			$db_insert->addInsertValue($attr['sort'], $a['sort_order']);
                	$application->db->PrepareSQL($db_insert);
                	$application->db->DB_Exec();
                	$attribute_id = $application->db->DB_Insert_Id();
                	$custom_attributes[] = $attribute_id;

                	//  associate with the given type
                	$db_insert = new DB_Insert('product_type_attributes');
        			$db_insert->addInsertValue($product_type_id, $pta['pt_id']);
        			$db_insert->addInsertValue($attribute_id, $pta['a_id']);
        			$db_insert->addInsertValue($attr['visible'], $pta['type_attr_visible']);
        			$db_insert->addInsertValue($attr['required'], $pta['type_attr_required']);
        			$db_insert->addInsertValue($attr['default'], $pta['type_attr_def_val']);
                	$application->db->PrepareSQL($db_insert);
                	$application->db->DB_Exec();
    		    }
    		    else
    		    {
        			// update existing custom attribute
        			$db_update = new DB_Update('attributes');
        			$db_update->addUpdateValue($a['it_id'], $attr['input_type_id']);
        			$db_update->addUpdateValue($a['ut'], $attr['unit_type_value']);
        			$db_update->addUpdateValue($a['view_tag'], $attr['view_tag']);
        			$db_update->addUpdateValue($a['name'], $attr['name']);
        			$db_update->addUpdateValue($a['descr'], $attr['descr']);
        			$db_update->addUpdateValue($a['type'], $attr['type']);
        			$db_update->addUpdateValue($a['min'], $attr['min']);
        			$db_update->addUpdateValue($a['max'], $attr['max']);
        			$db_update->addUpdateValue($a['size'], $attr['size']);
        			$db_update->addUpdateValue($a['sort_order'], $attr['sort']);
        			$db_update->WhereValue($a['id'], DB_EQ, $attr['id']);
                	$application->db->PrepareSQL($db_update);
                	$application->db->DB_Exec();
                	$custom_attributes[] = $attr['id'];

                    $db_update = new DB_Update('product_type_attributes');
                    $db_update->addUpdateValue($pta['type_attr_visible'], $attr['visible']);
                    $db_update->addUpdateValue($pta['type_attr_def_val'], $attr['default']);
                    $db_update->WhereValue($pta['id'], DB_EQ, $attr['pta_id']);
                	$application->db->PrepareSQL($db_update);
                	$application->db->DB_Exec();
    		    }
    		}
    		elseif ($attr['type'] == 'standard')
    		{
                $db_update = new DB_Update('product_type_attributes');
                $db_update->addUpdateValue($pta['type_attr_visible'], $attr['visible']);
                $db_update->addUpdateValue($pta['type_attr_def_val'], $attr['default']);
                $db_update->WhereValue($pta['id'], DB_EQ, $attr['pta_id']);
            	$application->db->PrepareSQL($db_update);
            	$application->db->DB_Exec();
            	$attributes[] = $attr['id'];
    		}
    	}

	    //determine deleted attributes
	    $diff = array_diff($current_custom_attributes, $custom_attributes);
        if (sizeof($diff) > 0)
        {
            // delete unused attributes from the attributes table
            $db_delete = new DB_Delete('attributes');
            $db_delete->WhereField($a['id'], DB_IN, "('" . implode("','", $diff) . "')");
            $application->db->getDB_Result($db_delete);

            //delete real values of unused attributes from the product_attributes table
            $db_delete = new DB_Delete('product_attributes');
            $db_delete->WhereField($pa['a_id'], DB_IN, "('" . implode("','", $diff) . "')");
            $application->db->getDB_Result($db_delete);

            /* delete unused attributes from the product_type_attributes table
               !!! for this product type only */
            $db_delete = new DB_Delete('product_type_attributes');
            $db_delete->WhereField($pta['a_id'], DB_IN, "('" . implode("','", $diff) . "')");
            $db_delete->WhereAND();
            $db_delete->WhereField($pta['pt_id'], DB_EQ, $product_type_id);
            $application->db->getDB_Result($db_delete);
        }

    	# delete cache
    	modApiFunc('CatalogCacheManager', 'eventProductTypeChanged', $product_type_id);
    }

    /**
     * Returns a list of all required attributes for the product type with the
     * specified id.
     */
    function getProductTypeAttributes($product_type_id)
    {
        global $application;
        $obj = &$application->getInstance('MessageResources');
        $tables = $this->getTables();

        $pta = $tables['product_type_attributes']['columns'];
        $a   = $tables['attributes']['columns'];
        $ag  = $tables['attribute_groups']['columns'];
        $it  = $tables['input_types']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($pta['id'], 'pta_id');
        $query->addSelectField($pta['type_attr_required'], 'required');
        $query->addSelectField($pta['type_attr_visible'], 'visible');
        $query->addSelectField($pta['type_attr_def_val'], 'default_value');
        // attribute info.
        $query->addSelectField($a['id'],          'id');
//New field
        $query->addSelectField($a['ut'],          'ut');
        $query->addSelectField($a['view_tag'],    'view_tag');
        $query->addSelectField($a['name'],        'name');
        $query->addSelectField($a['descr'],       'descr');
        $query->addSelectField($a['size'],        'size');
        $query->addSelectField($a['min'],         'min');
        $query->addSelectField($a['max'],         'max');
        $query->addSelectField($a['sort_order'],  'sort');
        $query->addSelectField($a['type'],        'type');
        $query->addSelectField($a['allow_html'],  'allow_html');
        // info about the group containing this attribute.
        $query->addSelectField($ag['id'],         'group_id');
        $query->addSelectField($ag['name'],       'group_name');
        $query->addSelectField($ag['sort_order'], 'group_sort');
        // info about control type for inputting the attribute.
        $query->addSelectField($it['id'],         'input_type_id');
        $query->addSelectField($it['name'],       'input_type_name');

        $query->addLeftJoin('input_types', $it['id'], DB_EQ, $a['it_id']);

        $query->WhereValue($pta['pt_id'], DB_EQ, ($product_type_id==0? 1:$product_type_id));
        $query->WhereAnd();
        $query->WhereField($a['id'], DB_EQ, $pta['a_id']);
        $query->WhereAnd();
        $query->WhereField($ag['id'], DB_EQ, $a['ag_id']);
        $attr_result = $application->db->getDB_Result($query);

        $product_type = array();
        for ($j=0; $j<sizeof($attr_result); $j++)
        {
            $view_tag = $attr_result[$j]['view_tag'];
            $product_type[$view_tag] = array();
            $product_type[$view_tag]['id'] = $attr_result[$j]['id'];
            $product_type[$view_tag]['pta_id'] = $attr_result[$j]['pta_id'];
            $product_type[$view_tag]['name'] = $attr_result[$j]['name'];
            $product_type[$view_tag]['descr'] = $attr_result[$j]['descr'];
            if ($attr_result[$j]['type']=="standard")
            {
                $product_type[$view_tag]['name'] = $obj->getMessage( new ActionMessage($product_type[$view_tag]['name']) );
                $product_type[$view_tag]['descr'] = $obj->getMessage( new ActionMessage($product_type[$view_tag]['descr']) );
            }
            $product_type[$view_tag]['size'] = $attr_result[$j]['size'];
            $product_type[$view_tag]['min'] = $attr_result[$j]['min'];
            $product_type[$view_tag]['max'] = $attr_result[$j]['max'];
            $product_type[$view_tag]['view_tag'] = $view_tag;
            $product_type[$view_tag]['group'] = array (
                 'id' => $attr_result[$j]['group_id']
                ,'name' => $obj->getMessage($attr_result[$j]['group_name'])
                ,'sort' => $attr_result[$j]['group_sort']
            );
            $product_type[$view_tag]['required'] = $attr_result[$j]['required'];
            $product_type[$view_tag]['visible'] = $attr_result[$j]['visible'];
            $product_type[$view_tag]['default'] = ($product_type_id==0? "":prepareHTMLDisplay($attr_result[$j]['default_value']));
            if ($view_tag == 'ShortDescription' || $view_tag == 'DetailedDescription')
            {
                $product_type[$view_tag]['default'] = $product_type_id==0? "":$attr_result[$j]['default_value'];
            }
            $product_type[$view_tag]['sort'] = $attr_result[$j]['sort'];
            $product_type[$view_tag]['type'] = $attr_result[$j]['type'];
            $product_type[$view_tag]['allow_html'] = $attr_result[$j]['allow_html'];
            $product_type[$view_tag]['patt_type'] = $attr_result[$j]['ut'];
            $product_type[$view_tag]['input_type_id'] = $attr_result[$j]['input_type_id'];
            $product_type[$view_tag]['input_type_name'] = strtolower($attr_result[$j]['input_type_name']);
            $product_type[$view_tag]['unit_type_values_pattern'] = modApiFunc("Localization", "getPattern", $attr_result[$j]['ut']);
            $product_type[$view_tag]['patt'] = $product_type[$view_tag]['unit_type_values_pattern']['patt_value'];
            $product_type[$view_tag]['unit_type_value'] = modApiFunc("Localization", "getUnitTypeValue", $attr_result[$j]['ut']);
        }

        // add artificial attributes
        $view_tag = 'ID';
        $product_type[$view_tag] = array(
            'id'                => null
           ,'pta_id'            => null
           ,'name'              => $obj->getMessage( new ActionMessage('PRD_ID_NAME'))
           ,'descr'             => $obj->getMessage( new ActionMessage('PRD_ID_DESCR'))
           ,'size'              => 70
           ,'min'               => 2
           ,'max'               => 255
           ,'view_tag'          => $view_tag
           ,'group'             => array('id'=>1, 'name'=>'Key Product Details', 'sort'=>1) // @fixme hard-coded
           ,'required'          => true
           ,'visible'           => true
           ,'default'           => null
           ,'sort'              => -2
           ,'type'              => 'artificial'
           ,'allow_html'        => 0
           ,'patt'              => null
           ,'patt_type'         => 'integer'
           ,'input_type_id'     => null
           ,'input_type_name'   => 'read-only'
           ,'unit_type_value'  => null
           ,'unit_type_values_pattern' => null
        );
        $view_tag = 'Name';
        $pattern = modApiFunc("Localization", "getPattern", 'string256');
        $product_type[$view_tag] = array(
            'id'                => null
           ,'pta_id'            => null
           ,'name'              => $obj->getMessage( new ActionMessage('PRD_NAME_NAME'))
           ,'descr'             => $obj->getMessage( new ActionMessage('PRD_NAME_DESCR'))
           ,'size'              => 70
           ,'min'               => 2 //FIXME: NOT USED?
           ,'max'               => 256
           ,'view_tag'          => $view_tag
           ,'group'             => array('id'=>1, 'name'=>'Key Product Details', 'sort'=>1) // @fixme hard-coded
           ,'required'          => true
           ,'visible'           => true
           ,'default'           => null
           ,'sort'              => -1
           ,'type'              => 'artificial'
           ,'allow_html'        => 0
           ,'patt_type'         => 'string256'
           ,'input_type_id'     => 1 //WHY null?
           ,'input_type_name'   => 'text'
           ,'unit_type_value'  => null
           ,'unit_type_values_pattern' => $pattern
           ,'patt'              => $pattern['patt_value']
        );

        // define values for attributes of the select type (input_type_values)
        $itv  = $tables['input_type_values']['columns'];
        $cache = &$this->_internal_cache['input_type_values'];
        foreach ($product_type as $attr_id => $attr)
        {
            // if it is not select, then skip it
//            if ($attr['input_type_name'] != 'select')
//            {
//            	continue;
//            }
            $input_type_id = $attr['input_type_id'];
            // if this type hasn't been extracted from the DB yet, then make a query.
            if (!array_key_exists($input_type_id, $cache))
            {
                if ($input_type_id == 6)
                {
                    $input_type_result = modApiFunc("Taxes", "getProductTaxClasses");
                    $cache[$input_type_id] = $input_type_result;

                    $request=&new Request();
                    $request->setView  ( 'ProductTaxClassSettings' );
                    $request->setAction( 'AddFromCatalog' );
                                                                         //Commented by Girin according the task ASC-128
                    $product_type[$attr_id]['additional_link'] = "";     //$application->href($request);
                    $product_type[$attr_id]['additional_link_text'] = "";//$obj->getMessage( new ActionMessage('PRD_TAX_CLS_ADD_LINK'));
                }
                else
                {
                    $query = &new DB_Select();
                    $query->addSelectField($itv['id'], 'id');
                    $query->addSelectField($itv['value'], 'value');
                    $query->WhereValue($itv['it_id'], DB_EQ, $input_type_id);
                    $input_type_result = $application->db->getDB_Result($query);
                    //                     
                    $cache[$input_type_id] = $input_type_result;
                }
            }
            else
            {
            	// This type has already been called from the base. Take it from the table.
            	$input_type_result = $cache[$input_type_id];
            }
            if (sizeof($input_type_result)<=0)
            {
                continue;
            }
            $product_type[$attr_id]['input_type_values'] = array();
            foreach ($input_type_result as $value)
            {
                $product_type[$attr_id]['input_type_values'][$value['id']] = $value['value'];
            }
        }
        return $product_type;
    }

    /**
     * Returns the product type list.
     */
    function getProductTypes()
    {
        global $application;
        $tables = $this->getTables();

        $pt = $tables['product_types']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($pt['id'], 'id');
        $query->addSelectField($pt['name'], 'name');
        $query->addSelectField($pt['descr'], 'description');
        $attr_result = $application->db->getDB_Result($query);

        $product_types = array();
        for ($j=0; $j<sizeof($attr_result); $j++)
        {
            $type = array();
            $type['id'] = $attr_result[$j]['id'];
            $type['name'] = prepareHTMLDisplay($attr_result[$j]['name']);
            $type['description'] = prepareHTMLDisplay($attr_result[$j]['description']);
            $product_types[] = $type;
        }
        return $product_types;
    }

    /**
     * Returns the product type description.
     */
    function getProductType($product_type_id)
    {
        global $application;
        $tables = $this->getTables();

        $pt = $tables['product_types']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($pt['id'], 'id');
        $query->addSelectField($pt['name'], 'name');
        $query->addSelectField($pt['descr'], 'description');
        $query->WhereValue($pt['id'], DB_EQ, ($product_type_id==0? 1:$product_type_id));
        $result = $application->db->getDB_Result($query);

        if (sizeof($result) <= 0)
        {
        	return null;
        }
        $product_type = array();
        $product_type['id'] = $result[0]['id'];
        $product_type['name'] = prepareHTMLDisplay($result[0]['name']);
        $product_type['description'] = prepareHTMLDisplay($result[0]['description']);
        $product_type['attr'] = $this->getProductTypeAttributes($product_type_id);

        return $product_type;
    }


    /**
     *                     ,                                    .
     */
    function getProductTypeFilename($product_type_id)
    {
        global $application;
        $tables = $this->getTables();

        $pt = $tables['product_types']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($pt['name'], 'name');
        $query->WhereValue($pt['id'], DB_EQ, ($product_type_id==0? 1:$product_type_id));
        $result = $application->db->getDB_Result($query);

        if (sizeof($result) <= 0)
        {
        	return null;
        }
        $product_type_name = $result[0]['name'];

        $filename = ereg_replace("[^a-zA-Z0-9]", "_", $product_type_name);

        return $filename;
    }

    /**
     * Saves info about custom attribute to the temporary table.
     */
    function addTempCustomAttribute($form_id, $info)
    {
        global $application;
        $tables = $this->getTables();

        $ct = $tables['catalog_temp']['columns'];

        // find a record in the temporary table for form_id
        $query = &new DB_Select();
        $query->addSelectField($ct['value'], 'value');
        $query->WhereValue($ct['form_id'], DB_EQ, $form_id);
    	$application->db->PrepareSQL($query);
        $application->db->DB_Exec();
        $application->db->DB_Result(QUERY_RESULT_ASSOC);
        $result = $application->db->ResultArray;

        if (sizeof($result) <= 0)
        {
            $arr = array();
            $arr[$info['view_tag']] = $info;
        	$db_insert = new DB_Insert('catalog_temp');
        	$db_insert->addInsertValue(serialize($arr), $ct['value']);
        	$db_insert->addInsertValue($form_id, $ct['form_id']);
        	$application->db->PrepareSQL($db_insert);
        	$application->db->DB_Exec();
        }
        else
        {
        	$arr = unserialize($result[0]['value']);
        	// determine sort_order
        	$sort_order = 0;
        	foreach ($arr as $tag => $attr)
        	{
        		if ($arr[$tag]['sort'] > $sort_order)
        		{
        			$sort_order = $arr[$tag]['sort'];
        		}
        	}
        	$info['sort'] = $sort_order + 1;
        	$arr[$info['view_tag']] = $info;
        	$db_update = new DB_Update('catalog_temp');
        	$db_update->addUpdateValue($ct['value'], serialize($arr));
        	$db_update->WhereValue($ct['form_id'], DB_EQ, $form_id);
        	$application->db->PrepareSQL($db_update);
        	$application->db->DB_Exec();
        }
    }

    /**
     * Updates info about custom attribute in the temporary table.
     */
    function updateTempCustomAttribute($form_id, $info)
    {
        global $application;
        $tables = $this->getTables();

        $ct = $tables['catalog_temp']['columns'];

        $attr = $this->getTempCustomAttributes($form_id);
        $attr[$info['view_tag']] = $info;
    	$db_update = new DB_Update('catalog_temp');
    	$db_update->addUpdateValue($ct['value'], serialize($attr));
    	$db_update->WhereValue($ct['form_id'], DB_EQ, $form_id);
    	$application->db->PrepareSQL($db_update);
    	$application->db->DB_Exec();
    }

    /**
     * Deletes info about specified attribute from the temporary table.
     */
    function deleteTempCustomAttribute($form_id, $view_tag)
    {
        global $application;
        $tables = $this->getTables();

        $ct = $tables['catalog_temp']['columns'];

        $attributes = $this->getTempCustomAttributes($form_id);
        unset($attributes[$view_tag]);

    	$db_update = new DB_Update('catalog_temp');
    	$db_update->addUpdateValue($ct['value'], serialize($attributes));
    	$db_update->WhereValue($ct['form_id'], DB_EQ, $form_id);
    	$application->db->PrepareSQL($db_update);
    	$application->db->DB_Exec();
    }
    /**
     * Returns the custom attributes list, associated with the given form.
     */
    function getTempCustomAttributes($form_id)
    {
        global $application;
        $tables = $this->getTables();

        $ct = $tables['catalog_temp']['columns'];

        // find a record in the temporary table for form_id
        $query = &new DB_Select();
        $query->addSelectField($ct['value'], 'value');
        $query->WhereValue($ct['form_id'], DB_EQ, $form_id);
    	$application->db->PrepareSQL($query);
        $application->db->DB_Exec();
        $application->db->DB_Result(QUERY_RESULT_ASSOC);
        $result = $application->db->ResultArray;

        if (sizeof($result) > 0)
        {
            return unserialize($result[0]['value']);
        }
        return null;
    }

    /**
     * Deletes all custom attributes for the given form.
     */
    function removeTempCustomAttributes($form_id)
    {
        global $application;
        $tables = $this->getTables();

        $ct = $tables['catalog_temp']['columns'];

         // find a record in the temporary table for form_id
        $query = &new DB_Delete('catalog_temp');
        $query->WhereValue($ct['form_id'], DB_EQ, $form_id);
    	$application->db->PrepareSQL($query);
        $application->db->DB_Exec();

        return null;
    }

    function updateCategory($cat_id, $cat_name, $cat_status, $description, $image_filename, $image_small_filename, $image_description, $page_title, $meta_keywords, $meta_description, $show_products_recursively, $seo_url_prefix)
    {
        global $application;

        $tables =  $this->getTables();

        $table = 'categories_descr';
        $columns = $tables[$table]['columns'];

        $image_files_array = array('image_file' => $image_filename, 'image_small_file' => $image_small_filename);
        foreach ($image_files_array as $field_name => $image_name)
        {
            $file_name = basename($image_name);

                $query = &new DB_Select();
                $query->addSelectField($columns[$field_name], 'name');
                $query->WhereValue($columns['id'], DB_EQ, $cat_id);
                $result = $application->db->getDB_Result($query);

            if(!empty($result))
            {
                $old_image_filename = basename($result[0]['name']);

                //                                     -               .
                if($old_image_filename != $file_name)
                {
                    @unlink($application->getAppIni('PATH_IMAGES_DIR').$old_image_filename);
                    if(!empty($file_name))
                {
                        //                        -                           .
                    // if the image has been uploaded, save it to the folder and to the base.
                        $image_info = $application->saveUploadImage("", $file_name);
                        $image_files_array[$field_name] = $file_name;
                    }
                }
            }
        }

        $query = &new DB_Update($table);
        $query->addUpdateValue($columns['name'],             $cat_name);
        $query->addUpdateValue($columns['descr'],            $description);
        $query->addUpdateValue($columns['image_file'],       $image_files_array['image_file']);
        $query->addUpdateValue($columns['image_small_file'], $image_files_array['image_small_file']);
        $query->addUpdateValue($columns['image_descr'],      $image_description);
        $query->addUpdateValue($columns['page_title'],       $page_title);
        $query->addUpdateValue($columns['meta_keywords'],    $meta_keywords);
        $query->addUpdateValue($columns['meta_descr'],       $meta_description);
        $query->addUpdateValue($columns['show_prod_recurs'], $show_products_recursively);
        $query->addUpdateValue($columns['seo_url_prefix'],   $this->_prepareSEOURL($seo_url_prefix));

        $query->WhereValue($columns['id'], DB_EQ, $cat_id);
        $application->db->getDB_Result($query);

        $table = 'categories';
        $columns = $tables[$table]['columns'];
        $query = &new DB_Update($table);
        $query->addUpdateValue($columns['status'],             $cat_status);
        $query->WhereValue($columns['id'], DB_EQ, $cat_id);
        $application->db->getDB_Result($query);

    	#delete category from cache
    	modApiFunc('CatalogCacheManager', 'clearAllCache');

    }

    function addCategory($parent_cat_id, $cat_name, $cat_status, $description, $image_filename, $image_small_filename, $image_description, $page_title, $meta_keywords, $meta_description, $show_products_recursively, $seo_url_prefix)
    {
        global $application;

        $tables =  $this->getTables();

        $CategoryInfo = $this->fetchCategoryInfo($parent_cat_id);

        if ($CategoryInfo == FALSE)
        {
            return $inserted_id = 1;
        }

        $table = 'categories';
        $columns = $tables[$table]['columns'];
        $query = &new DB_Update($table);
        $query->addUpdateExpression($columns["left"], $query->fIf($columns["left"] . DB_GT . $CategoryInfo["right"], $columns["left"] . "+ 2" , $columns["left"]));
        $query->addUpdateExpression($columns["right"], $query->fIf($columns["right"] . DB_GTE . $CategoryInfo["right"], $columns["right"] . "+ 2" , $columns["right"]));
        $query->WhereValue($columns["right"], DB_GTE, $CategoryInfo["right"]);

        $application->db->getDB_Result($query);

        # Dump data for table 'attribute_groups'
        $table = 'categories';
        $columns = $tables[$table]['columns'];

        $query = &new DB_Insert($table);
        $query->addInsertValue($CategoryInfo["right"], $columns['left']);
        $query->addInsertValue($CategoryInfo["right"] + 1, $columns['right']);
        $query->addInsertValue($CategoryInfo["level"] + 1, $columns['level']);
        $query->addInsertValue($cat_status,                $columns['status']);
        $application->db->getDB_Result($query);

        $inserted_id = $application->db->DB_Insert_Id();

        $tmp_small_image_name = basename($image_small_filename);
        $tmp_large_image_name = basename($image_filename);
        $image_small_info = $application->saveUploadImage("", $tmp_small_image_name);
        $image_large_info = $application->saveUploadImage("", $tmp_large_image_name);
        $table = 'categories_descr';
        $columns = $tables[$table]['columns'];
        $query = &new DB_Insert($table);
        $query->addInsertValue($inserted_id,                  $columns['id']);
        $query->addInsertValue(($cat_name),                   $columns['name']);
        $query->addInsertValue(($description),                $columns['descr']);
        $query->addInsertValue($image_large_info["name"],     $columns['image_file']);
        $query->addInsertValue($image_small_info["name"],     $columns['image_small_file']);
        $query->addInsertValue(($image_description),          $columns['image_descr']);
        $query->addInsertValue(($page_title),                 $columns['page_title']);
        $query->addInsertValue(($meta_keywords),              $columns['meta_keywords']);
        $query->addInsertValue(($meta_description),           $columns['meta_descr']);
        $query->addInsertValue(($show_products_recursively),  $columns['show_prod_recurs']);
        $query->addInsertValue($this->_prepareSEOURL($seo_url_prefix), $columns['seo_url_prefix']);

        $application->db->getDB_Result($query);

    	#     delete all categories from cache
    	modApiFunc('CatalogCacheManager', 'eventCategorySubcategoriesContentChanged', $parent_cat_id);

        return $inserted_id;
    }

    /**
     * Sets up an option which means that product info is outputted while
     * deleting the product.
     *
     * @todo deprecated. Remove the fucntion?
     */
    function setDisplayDeleteInfo()
    {
        $this->DisplayDeleteInfo = true;
    }

    /**
     * Gets a product info output mode.
     */
    function getDisplayDeleteInfo()
    {
        return $this->DisplayDeleteInfo;
    }

    /**
     * Checks if the product id is valid.
     *
     * @param integer $product_id
     * @return boolean
     * @todo caching
     */
    function isCorrectProductId($product_id)
    {
        global $application;

    	// The id can't be a string or have a negative value.
    	if (empty($product_id) || !is_numeric($product_id) || $product_id <= 0)
    	{
    		return false;
    	}
        $tables = $this->getTables();

        $p = $tables['products']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($query->fCount($p['id']), 'count_p_id');
        $query->WhereValue($p['id'], DB_EQ, $product_id);
        $result = $application->db->getDB_Result($query);

        // There should be only one product with such an id.
        if ($result[0]['count_p_id'] != 1)
        {
            return false;
        }

        return true;
    }

    /**
     * Checks if the category id is valid.
     *
     * @param integer $category_id
     * @return boolean
     * @todo caching
     */
    function isCorrectCategoryId($category_id)
    {
        global $application;

    	//  The id can't be a string or have a negative value.
    	if (empty($category_id) || !is_numeric($category_id) || $category_id <= 0)
    	{
    		return false;
    	}

    	$tables = $this->getTables();

        $c = $tables['categories']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($query->fCount($c['id']), 'count_c_id');
        $query->WhereValue($c['id'], DB_EQ, $category_id);
        $result = $application->db->getDB_Result($query);

        // There should be only one category with such an id.
        if ($result[0]['count_c_id'] != 1)
        {
            return false;
        }

        return true;
    }

    /**
     * Checks if the product type id is valid.
     *
     * @param integer $product_type_id
     * @return boolean
     * @todo caching
     */
    function isCorrectProductTypeId($product_type_id)
    {
        global $application;

    	//  The id can't be a string or have a negative value.
    	if (empty($product_type_id) || !is_numeric($product_type_id) || $product_type_id <= 0)
    	{
    		return false;
    	}

    	$tables = $this->getTables();

        $pt = $tables['product_types']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($query->fCount($pt['id']), 'count_pt_id');
        $query->WhereValue($pt['id'], DB_EQ, $product_type_id);
        $result = $application->db->getDB_Result($query);

        // There should be only one product type with such an id.
        if ($result[0]['count_pt_id'] != 1)
        {
            return false;
        }

        return true;
    }

    /**
     * Returns the value "Product Type" by default. It is used, for example,
     * at first outputting "Product Type", when the user himself hasn't selected
     * the type of the added product, but it is necesary to display the input
     * field.
     *
     * @return integer
     */
    function getDefaultProductTypeID()
    {
        return 0;
    }

    /**
     * Returns the value Home "Root category".
     *
     * @return integer
     */
    function getHomeCategoryID()
    {
        return 1;
    }

    /**
     * Gets a product quantity depending on the type.
     *
     * @return bool true if a product quantity is less than the one specified
     *              in the license key, false otherwise
     */
    function getProductsQuantityByType($typeId)
    {
        global $application;
        $tables = $this->getTables();
        $p = $tables['products']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($query->fCount($p['id']), 'count_p_id');
        $query->WhereValue($p['pt_id'], DB_EQ, $typeId);
        $result = $application->db->getDB_Result($query);
        $prods_number = $result[0]['count_p_id'];
        return $prods_number;
    }

    /**
     * Gets a product quantity in each of the category.
     *
     * @return
     */
    function getProductsQuantityByCategories()
    {
        global $application;
        $tables = $this->getTables();
        $p2c = $tables['products_to_categories']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($query->fCount($p2c['product_id']), 'count_p_id');
        $query->addSelectField($p2c['category_id'], 'c_id');
        $query->SelectGroup($p2c["category_id"]);
        $result = $application->db->getDB_Result($query);
        return $result;
    }


    /**
     * Deletes the product type.
     *
     * @author Alexandr Girin
     * @todo
     * @param integer $typeId - product type ID
     * @return
     */
    function deleteProductType($typeId)
    {
        global $application;

        $tables = $this->getTables();
        $pta = $tables['product_type_attributes']['columns']; #           - product_type_attributes
        $pta = $tables['product_type_attributes']['columns']; # product_type_attributes is a database table
        $query = &new DB_Delete('product_type_attributes');
        $query->WhereValue($pta['pt_id'], DB_EQ, $typeId);
        $application->db->getDB_Result($query);

        $pt = $tables['product_types']['columns']; # products is a database table
        $query = &new DB_Delete('product_types');
        $query->WhereField($pt['id'], DB_EQ, $typeId);
        $application->db->getDB_Result($query);

    	# delete the type from cache
    	modApiFunc('CatalogCacheManager', 'eventProductTypeChanged', $typeId);
    }

    function getImagesDir()
    {
        global $application;
        return $application->getAppIni("PATH_IMAGES_DIR");
    }

    /**
     * Checks if it is possible to write to the image folder.
     *
     * @author Vadim Lyalikov
     * @todo
     * @return
     */
    function isImageFolderNotWritable()
    {
        global $application;
        $dir_fs_name = $application->getAppIni("PATH_IMAGES_DIR");
        return !is_dir_writable($dir_fs_name);
    }

    /**
     * Changes the product attribute "TaxClass" for "Non-Taxable" when deleting
     * TaxClass on the tax setting page.
     */
    function changeAttributeValueForAllProducts($attr_id, $attr_old_value, $attr_new_value)
    {
        global $application;

        $tables =  $this->getTables();

        $pa  = $tables['product_attributes']['columns'];

        $query = &new DB_Update('product_attributes');
        $query->addUpdateValue($pa['attr_value'], $attr_new_value);
        $query->WhereValue($pa['a_id'], DB_EQ, $attr_id);
        $query->WhereAnd();
        $query->WhereValue($pa['attr_value'], DB_EQ, $attr_old_value);
        $application->db->getDB_Result($query);

        modApiFunc('CatalogCacheManager', 'clearAllCache');
    }

    /**
     * Updates the product quantity in the stock after the order confirmation.
     */
    function updateProductsQuantity($cartContent)
    {
        global $application;
        $tables =  $this->getTables();
        $pa  = $tables['product_attributes']['columns'];

        foreach ($cartContent as $prodInfo)
        {
            //TODO: the attribute is used by the id, it needs to be rewritten
            //check if the attribute of the QuantityInStock product is visible
            if (!modApiFunc("Catalog", "isProductAttributeVisible", $prodInfo['ID'], 3))
            {
                continue;
            }

            $query = &new DB_Update('product_attributes');
            $query->addUpdateExpression($pa['attr_value'], $pa['attr_value']." - ".$prodInfo['Quantity_In_Cart']);
            $query->WhereValue($pa['a_id'], DB_EQ, 3);
            $query->WhereAnd();
            $query->WhereValue($pa['p_id'], DB_EQ, $prodInfo['ID']);
            $application->db->getDB_Result($query);
            modApiFunc('CatalogCacheManager', 'eventProductInfoChanged', $prodInfo['ID']);

            $prodObj = &$application->getInstance('CProductInfo',$prodInfo['ID']);
            $cats_ids = $prodObj->getCategoriesIDs();
            foreach($cats_ids as $category_id)
            {
                modApiFunc('CatalogCacheManager', 'eventCategoryProductsContentChanged', $category_id);
                modAPIFunc('paginator', 'resetPaginator', "Catalog_ProdsList_".$category_id);
            };
        }
    }

    /**
     * Checks if the product attribute is visible.
     *
     * @param integer $pid - the product id
     * @param integer $aid - the attribute id
     * @return boolean
     */
    function isProductAttributeVisible($pid, $aid)
    {
        global $application;
        $tables =  $this->getTables();

        $p  = $tables['products']['columns'];
        $pta  = $tables['product_type_attributes']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($pta['type_attr_visible'], 'visibility');
        $query->WhereField($p['pt_id'], DB_EQ, $pta['pt_id']);
        $query->WhereAnd();
        $query->WhereValue($pta['a_id'], DB_EQ, $aid);
        $query->WhereAnd();
        $query->WhereValue($p['id'], DB_EQ, $pid);
        $result = $application->db->getDB_Result($query);
        if (sizeof($result) && $result[0]['visibility'] == '1')
        {
            return true;
        }
        return;
    }

	/**
	 *
	 */
	 function getNextProductId($type_id, $last_id)
	 {

	 	global $application;

        $tables = $this->getTables();
        $p = $tables['products']['columns'];
        $query = &new DB_Select();
        $query->addSelectField($p['id'], 'id');
        $query->WhereValue($p['pt_id'], DB_EQ, $type_id);
        $query->WhereAnd();
		$query->WhereField($p['id'], DB_GT, $last_id);
	    $query->SelectOrder($p['id']);
        $query->SelectLimit(0, 1);

        $ids = $application->db->getDB_Result($query);

        if ($ids === null)
       	{
       		return null;
       	}
       	/*
		echo("<pre>");
		print_r($ids);
		echo("</pre>");
		*/
		if (count($ids) === 0)
		{
			return null;
		}
        return $ids[0]['id'];
	 }


	 function getProductAttrs($product_id)
	 {

	 	global $application;

        $tables = $this->getTables();
        $pa = $tables['product_attributes']['columns'];
        $query = &new DB_Select();
        $query->addSelectField($pa['a_id'], 'a_id');
        $query->addSelectField($pa['attr_value'], 'attr_value');
		$query->WhereField($pa['p_id'], DB_EQ, $product_id);
        $attrs = $application->db->getDB_Result($query);

        if ($attrs === null)
       	{
       		return null;
       	}

		$result = array();
		foreach ($attrs as $attr)
		{
			$result[$attr['a_id']] = $attr['attr_value'];
		}



		return $result;
	 }


	function getProductNameByProductID($product_id)
	{

		global $application;

        $tables = $this->getTables();
        $p = $tables['products']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($p['name'], 'name');

		$query->WhereValue($p['id'], DB_EQ, $product_id);
		$name = $application->db->getDB_Result($query);


		return $name[0]['name'];
	}
/*
	function getProductCategoryByProductID($product_id)
	{
		global $application;

        $tables = $this->getTables();
        $p = $tables['products']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($p['c_id'], 'c_id');

		$query->WhereValue($p['id'], DB_EQ, $product_id);
		$result = $application->db->getDB_Result($query);

        # Generate the string, which is the full category path for this product
        $categoryPath = modApiFunc('Catalog','getCategoryFullPath',$result[0]['c_id']);
        $categoryStringPath = '';

        foreach ($categoryPath as $category)
        {
            if ($category['id'] == 1)
            {
            	continue;
            }
            $categoryStringPath .= $category['name'].'/';
        }

        return $categoryStringPath;
	}
*/

	function getProductSmallImageByProductID($product_id)
	{

		global $application;

        $tables = $this->getTables();
        $pa = $tables['product_attributes']['columns'];
        $query = &new DB_Select();
        $query->addSelectField($pa['id'], 'id');
		$query->WhereField($pa['p_id'], DB_EQ, $product_id);
      	$query->WhereAnd();
		$query->WhereField($pa['a_id'], DB_EQ, 10); // 10 - small image
        $attr_id = $application->db->getDB_Result($query);

        $id = $attr_id[0]['id'];


        $tables = $this->getTables();
        $pi = $tables['product_images']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($pi['name'], 'name');
		$query->WhereValue($pi['pa_id'], DB_EQ, $id);
		$result = $application->db->getDB_Result($query);

		if (count($result) === 0)
		{
			$filename = "";
		}
		else
		{
			$filename = $result[0]['name'];
		}

		return $filename;
	}


	function getProductLargeImageByProductID($product_id)
	{
		global $application;

        $tables = $this->getTables();
        $pa = $tables['product_attributes']['columns'];
        $query = &new DB_Select();
        $query->addSelectField($pa['id'], 'id');
		$query->WhereField($pa['p_id'], DB_EQ, $product_id);
      	$query->WhereAnd();
		$query->WhereField($pa['a_id'], DB_EQ, 9); // 9 - large image
        $attr_id = $application->db->getDB_Result($query);

        $id = $attr_id[0]['id'];


        $tables = $this->getTables();
        $pi = $tables['product_images']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($pi['name'], 'name');
		$query->WhereValue($pi['pa_id'], DB_EQ, $id);
		$result = $application->db->getDB_Result($query);

		if (count($result) === 0)
		{
			$filename = "";
		}
		else
		{
			$filename = $result[0]['name'];
		}

		return $filename;
	}


	function getNumberProductsToExport($type_id)
	{
		global $application;

        $tables = $this->getTables();

        $p = $tables['products']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($query->fCount($p['id']), 'count_p_id');
        $query->WhereValue($p['pt_id'], DB_EQ, $type_id);
        $result = $application->db->getDB_Result($query);

        return $result[0]['count_p_id'];
	}


	function getProductTagValuesHash($product_id)
	{

		$ProductObj = &new CProductInfo($product_id);
		$tagsHash = $ProductObj->getProductTagValuesHash(PRODUCTINFO_NOT_LOCALIZED_DATA,
														 PRODUCTINFO_LONG_TAG_NAMES,
														 PRODUCTINFO_NO_TAG_TYPES);

        return $tagsHash;
    }


    /**
     *                                         
     */
    function addProductLinkToCategory($product_id,$category_id)
    {
        global $application;
        $tables = $this->getTables();
        $p2c = $tables['products_to_categories']['columns'];

        $query = &new DB_Insert('products_to_categories');
        $query->addInsertValue($product_id,$p2c['product_id']);
        $query->addInsertValue($category_id,$p2c['category_id']);
        $application->db->PrepareSQL($query);
        $application->db->DB_Exec();
        $record_id = $application->db->DB_Insert_Id();

        $query = &new DB_Update('products_to_categories');
        $query->addUpdateValue($p2c['sort_order'],$record_id);
        $query->WhereValue($p2c['record_id'], DB_EQ, $record_id);
        $application->db->PrepareSQL($query);
        $application->db->DB_Exec();

        modApiFunc('CatalogCacheManager','eventCategoryProductsContentChanged', $category_id);

        return $record_id;
    }

    /**
     *                                                          
     */
    function delProductLinksFromCategory($category_id,$products_ids)
    {
        if(!is_array($products_ids) or empty($products_ids))
            return false;

        global $application;
        $tables = $this->getTables();
        $p2c = $tables['products_to_categories']['columns'];

        $query = &new DB_Delete('products_to_categories');
        $query->WhereValue($p2c['category_id'], DB_EQ, $category_id);
        $query->WhereAND();
        $query->Where($p2c['product_id'], DB_IN, "('".implode("','",$products_ids)."')");
        $application->db->PrepareSQL($query);
        $application->db->DB_Exec();

        modApiFunc('CatalogCacheManager','eventCategoryProductsContentChanged', $category_id);

        return true;
    }

    /**
     *                                                        
     */
    function delProductLinksFromCategories($product_id,$categories_ids)
    {
        if(!is_array($categories_ids) or empty($categories_ids))
            return false;

        global $application;
        $tables = $this->getTables();
        $p2c = $tables['products_to_categories']['columns'];

        $query = &new DB_Delete('products_to_categories');
        $query->WhereValue($p2c['product_id'], DB_EQ, $product_id);
        $query->WhereAND();
        $query->Where($p2c['category_id'], DB_IN, "('".implode("','",$categories_ids)."')");
        $application->db->PrepareSQL($query);
        $application->db->DB_Exec();

        foreach($categories_ids as $category_id)
            modApiFunc('CatalogCacheManager','eventCategoryProductsContentChanged', $category_id);

        return true;
    }

    /**
     *         
     *                                        
     *                 ,                                    
     *
     * @param string $by_id = enum('product_id','category_id')
     * @param array $ids
     */
    function delAllProductLinks($by_id,$ids)
    {
        if(!is_array($ids) or empty($ids))
            return false;
        if(!in_array($by_id,array('product_id','category_id')))
            return false;


        global $application;
        $tables = $this->getTables();
        $p2c = $tables['products_to_categories']['columns'];

        $query = &new DB_Delete('products_to_categories');
        $query->Where($p2c[$by_id], DB_IN, "('".implode("','",$ids)."')");
        $application->db->PrepareSQL($query);
        $application->db->DB_Exec();

        if($by_id == 'category_id')
            foreach($ids as $category_id)
                modApiFunc('CatalogCacheManager','eventCategoryProductsContentChanged', $category_id);

        return true;
    }

    function doesProductHaveLinks($product_id)
    {
        global $application;
        $tables = $this->getTables();
        $p2c = $tables['products_to_categories']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($query->fCount($p2c['record_id']),'count_rids');
        $query->WhereValue($p2c['product_id'], DB_EQ, $product_id);
        $res = $application->db->getDB_Result($query);
        return ($res[0]['count_rids'] > 0);
    }

    function getRealProductsCount()
    {
        global $application;
        $tables = $this->getTables();
        $p2c = $tables['products_to_categories']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($p2c['product_id'],'product_id');
        $query->SelectGroup($p2c['product_id']);
        $res = $application->db->getDB_Result($query);
        return count($res);
    }

    function getCategoryFullPathAsCategoriesIDs($category_id,$include_self=true)
    {
        $cats = $this->getCategoryFullPath($category_id);

        if($include_self)
            $retarr = array($category_id);
        else
            $retarr = array();

        for($i=0;$i<count($cats);$i++)
        {
            if($cats[$i]["id"]!=$category_id)
                array_unshift($retarr,$cats[$i]["id"]);
        };

        return $retarr;
    }

    function _prepareSEOURL($seo_string)
    {
        return preg_replace("/[^a-z0-9_\-]/i","_",$seo_string);
    }

    /**
     *                          ,                                 ,                                         
     * (                         ),                                     ,                     .
     */
    function computePriceExcludingTaxes($price_including_taxes, $tax_class_id, $b_force_AZ = false)
    {
        //  AZ          :
        //FIMXE:                   Tax Calculator
        if (modApiFunc('Users', 'getZone') == "AdminZone" &&
            $b_force_AZ === false)
        {
            return $price_including_taxes;
        }

        //TODO:          ,                 TaxClass               :                       .
        $products = array();
        $products[] = array("CartItemSalePrice" => $price_including_taxes,
                            "CartItemSalePriceExcludingTaxes" => NULL, //                    
                            "Quantity_In_Cart" => 1,
                            "ShippingPrice" => 0.0,
                            "TaxClass" => $tax_class_id
                            );

        $country_id = modApiFunc('Configuration', 'getValue', SYSCONFIG_STORE_OWNER_COUNTRY);
        $state = modApiFunc('Configuration', 'getValue', SYSCONFIG_STORE_OWNER_STATE);
        //            ,                                       ,         ,    ProductInfo,
        //                      .
        //                             -                                       .                   
        //                          $price_including_taxes
        if(!is_numeric($country_id) ||
           ($country_id < 1) ||
           !is_numeric($state) ||
           ($state < 1))
        {
            ////                :
            //_fatal(array( "CODE" => "CORE_057"), __CLASS__, __FUNCTION__);
            if(!is_numeric($country_id))
            {
                $country_id = NULL;
            }
            if(!is_numeric($state))
            {
                $state = NULL;
            }
            //             ,                      ,                          ,
            //                                      . TODO:              
            //                       -             .

            $addressesArray = array
            (
                 "Default"  => array(
                                     "CountryId" => $country_id,
                                     "StateId" => $state
                                    ),
                 "Shipping" => array(
                                     "CountryId" => $country_id,
                                     "StateId" => $state
                                    ),
                 "Billing"  => array(
                                     "CountryId" => $country_id,
                                     "StateId" => $state
                                    ),
                 "StoreOwner" => array(
                                     "CountryId" => $country_id,
                                     "StateId" => $state
                                    )
            );
        }
        else
        {
            $addressesArray = array
            (
                 "Default"  => array(
                                     "CountryId" => $country_id,
                                     "StateId" => $state
                                    ),
                 "Shipping" => array(
                                     "CountryId" => $country_id,
                                     "StateId" => $state
                                    ),
                 "Billing"  => array(
                                     "CountryId" => $country_id,
                                     "StateId" => $state
                                    ),
                 "StoreOwner" => array(
                                     "CountryId" => $country_id,
                                     "StateId" => $state
                                    )
            );
        }

        $sm_uuid = modApiFunc("Checkout", "getAllInactiveModuleId", "shipping");
        modApiFunc('Taxes',
                   'setTaxDebug',
                   $products,
                   0.0,
                   $sm_uuid,
                   $addressesArray
                  );

        $TaxAmounts = modApiFunc("Taxes", "getTax", true, false, true); //debug, not trace, symbolic

        if($TaxAmounts == "fatal")
        {
            //                                   -                   .
            //                       .         $price_including_taxes
            return  $price_including_taxes;
        }
        //                                                :
        //  T1 = k1 x + b1
        //  T2 = k2 x + b2
        //  T3 = k3 x + b3
        // ,    x -                          .
        // . .
        // price_including_taxes = x + T1 + T2 + T3
        // =>                     _  _       _        x:
        // x = (price_including_taxes - (b1 + b2 + b3)) / (1 + k1 + k2 + k3)
        // sum_b = b1 + b2 + b3
        // sum_k = k1 + k2 + k3
        $sum_k = 0.0;
        $sum_b = 0.0;

        $TaxNames = modApiFunc("Taxes", "getTaxNames");

        if (empty($TaxAmounts['products']))
        {
            //                   .
            //                      .         $price_including_taxes
            return  $price_including_taxes;
        }

        foreach ($TaxAmounts['products'][1] as $tax_id => $tax_amount)
        {
            //                        .                      .
            if(!is_numeric($tax_id))
                continue;

            //                        -                          ,
            //                              .
            //               $price_including_taxes

            //          -               PRICE_N_A -                 ,           .

            //          -                                          SalePrice -
            //                              ,
            //               $price_including_taxes

            //           "            " -               
            if($TaxNames[$tax_id]["included_into_price"] == "false")
                continue;

            if (is_array($tax_amount))
            {
                //                                                            .
                if(array_key_exists("k", $tax_amount) &&
                   array_key_exists("b", $tax_amount))
                {
                    $sum_k += $tax_amount["k"];
                    $sum_b += $tax_amount["b"];
                }
                else
                {
                    continue;
                }
            }
            elseif ($tax_amount == TAX_NOT_LINEAR)
            {
                //_fatal(array( "CODE" => "CORE_058"), __CLASS__, __FUNCTION__);
                //FIXME:
                ////TODO: report error,   CZ                      .                     .
                //                                              /              
                //                          .
                return  $price_including_taxes;
            }
            elseif ($tax_amount == PRICE_N_A)
            {
                //                                  .  . .         b = 0, k = 0.
                //                  ,                                      .
                continue;
            }
            else
            {
                //                          .                                .
                continue;
            }
        }

        $price_excluding_taxes = ($price_including_taxes - $sum_b) / (1 + $sum_k);
        return $price_excluding_taxes;
    }

    /**#@-*/

//------------------------------------------------
//              PRIVATE DECLARATION
//------------------------------------------------

    /**#@+
     * @access private
     */

    /**
     * The list of Catalog views.
     */
    var $ViewsList;

    /**
     * Current Category ID.
     */
    var $currentCategoryID = NULL;
    /**
     * Editable Category ID.
     */
    var $editableCategoryID = NULL;
    /**
     * Current Category Additional (besides ID) parameters
     * concerned with category tree storing method
     */
    var $currentCategoryLeft  = NULL;
    var $currentCategoryRight = NULL;
    var $currentCategoryLevel = NULL;

    /**
     * Current Product ID.
     */
    var $currentProductID;

    var $moveToCategoryID = null;

    /**
     * Current Product Type ID.
     */
    var $currentProductTypeID;

    /**
     * List of action handlers.
     */
    var $ActionHandlersList;

    /**
     * It is used to define the product info output when deleting or viewing
     * its attributes.
     */
    var $DisplayDeleteInfo = false;

    var $_internal_cache = array(
        'input_type_values' => array()
       ,'unit_type_values' => array()
    );
    /**#@-*/

}
?>