<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2007, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

include_once(dirname(__FILE__) . '/../abstract/product_info.php');

/**
 * Catalog->ProductInfo View.
 * Views product info for the CustomerZone.
 *
 * @package Catalog
 * @access  public
 * @author Alexey Kolesnikov
 */
class ProductInfo extends Catalog_ProdInfo_Base
{
//------------------------------------------------
//               PUBLIC DECLARATION
//------------------------------------------------

    /**#@+
     * @access public
     */

    /**
     * Gets a template format for the given view.
     */
    function getTemplateFormat()
    {
    	$format = array(
    	    'layout-file'        => 'product-info-config.ini'
    	   ,'files' => array(
    	        'Item'           => TEMPLATE_FILE_PRODUCT_TYPE
               ,'ItemOutOfStock' => TEMPLATE_FILE_PRODUCT_TYPE
    	    )
    	   ,'options' => array(
    	    )
    	);
    	return $format;
    }

    function ProductInfo()
    {
        global $application;

        #check if fatal errors exist in the block tag
        $this->NoView = false;
        if ($application->issetBlockTagFatalErrors("ProductInfo"))
        {
            $this->NoView = true;
        }
    }

    /**
     * Returns the product info.
     *
     * @return string.
     */
    function output()
    {
        global $application;

        #Define whether to output the view or not
        if ($this->NoView)
        {
            $application->outputTagErrors(true, "ProductInfo", "Errors");
            return "";
        }
        else
        {
            $application->outputTagErrors(true, "ProductInfo", "Warnings");
        }


        $catalog = &$application->getInstance('Catalog');
        $this->prod_id = @func_get_arg(0);
        if ($this->prod_id === false)
        {
            $this->prod_id = $catalog->getCurrentProductID();
        }
        if (!$this->prod_id)
        {
        	return;
        }

        # Get current selected product info.
        $product_info = modApiFunc('Catalog', 'getProductInfo', $this->prod_id);

        # save info
        $this->_ProductInfo = $product_info;

        #                                                      
        $application->registerAttributes(array(
                    'Local_HiddenFields'=>''
                   ,'Local_JSfuncProductFormSubmit'=>''
                   ,'ProductOptionsForm'=>''
                   ,'Local_ProductFormStart' => ''
                   ,'Local_ProductFormEnd' => ''
                   ,'Local_ProductAddToCart' => ''
                   ,'Local_FormQuantityFieldName' => ''
                   ,'Local_ProductQuantityOptions' => ''
                  ));

        $templateFiller = new TemplateFiller();
        # define the template for the given view.
        $template = $application->getBlockTemplate('ProductInfo');
        $templateFiller->setTemplate($template);

        //             OutOfStock -                             .
        //                                   OutOfStock -                      .
        $store_show_absent = modApiFunc("Configuration", "getValue", "store_show_absent");
        $qty_in_stock = $this->_ProductInfo['attributes']['QuantityInStock']['value'];
        $low_stock_level = $this->_ProductInfo['attributes']['LowStockLevel']['value'];

        $b_out_of_stock = ($qty_in_stock !== "" && $qty_in_stock <1);

        switch($store_show_absent)
        {
            case STORE_SHOW_ABSENT_SHOW_BUY:
                //                                  ,                     -
                //                       
                $template = "Item";
                break;
            case STORE_SHOW_ABSENT_SHOW_NOT_BUY:
                //                        ,                       
                //            LowStock -            Low Stock! Buy Now! 
                if($b_out_of_stock === true)
                {
                    $template = "ItemOutOfStock";
                }
                else
                {
                    $template = "Item";
                }
                break;
            case STORE_SHOW_ABSENT_NOT_SHOW_NOT_BUY:
                //                           ,                       
                if($b_out_of_stock === true)
                {
                    return "";
                }
                else
                {
                    $template = "Item";
                }
                break;
            default:
                //STORE_SHOW_ABSENT_SHOW_BUY
                $template = "Item";
                break;
        }
        $result = $templateFiller->fill($template, $product_info['TypeID']);

        return $result;
    }

    /**
     * Processes tags in the templates for the given view.
     *
     * @return string tag value, if tag has been processed. NULL, otherwise.
     */
    function getTag($tag)
    {
        global $application;
        $value = null;
        switch ($tag)
        {
            case 'Local_JSfuncProductFormSubmit':
                $value = "<script language=\"JavaScript\">".
                         "function ProductFormSubmit_".$this->_ProductInfo['ID']."()".
                         "{".
                         " document.forms['ProductForm_".$this->_ProductInfo['ID']."'].submit();".
                         "};".
                         "</script>";
                break;

            case 'Local_ProductFormStart':
                $value = '<FORM action="'.$this->_ProductInfo['BuyLink'].'" name="ProductForm_'.$this->_ProductInfo['ID'].'" id="ProductForm_'.$this->_ProductInfo['ID'].'" method="post">
                        <script language="JavaScript">
                        function ProductFormSubmit_'.$this->_ProductInfo['ID'].'()
                        {
                            document.forms[\'ProductForm_'.$this->_ProductInfo['ID'].'\'].submit();
                        };
                        </script>';
                break;

            case 'Local_ProductFormEnd':
                $value = '</FORM>';
                break;

            case 'Local_ProductAddToCart':
                $value = 'javascript: ProductFormSubmit_'.$this->_ProductInfo['ID'].'();';
                break;

            case 'ProductOptionsForm':
                $value = '';
                break;

            case 'Local_FormQuantityFieldName':
                $value = 'quantity_in_cart';
                break;

            case 'Local_ProductQuantityOptions':
                $qty_in_cart = modApiFunc("Cart", "getProductQuantity", $this->_ProductInfo['ID']);
                $value =modApiFunc("Cart", "getProductQuantityOptions", $qty_in_cart, $this->_ProductInfo['ID']);
                break;

    	    default:
    	        list($entity, $tag) = getTagName($tag);
        	    if ($entity == 'product')
        	    {
        	        $value = getKeyIgnoreCase($tag, $this->_ProductInfo);
        	    }
        		break;
        }
        return $value;
    }
    /**#@-*/

//------------------------------------------------
//              PRIVATE DECLARATION
//------------------------------------------------

    /**#@+
     * @access private
     */

    /**
     * Current produst info.
     *
     * @var array
     */
    var $_ProductInfo;

    /**#@-*/
}
?>