<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2007, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php
/**
 * Checkout Payment Modules List view.
 *getPaymentModuleGroup
 * @package Checkout
 * @author Vadim Lyalikov
 */

class CheckoutPaymentModulesList
{
//------------------------------------------------
//               PUBLIC DECLARATION
//------------------------------------------------

    /**#@+
     * @access public
     */

    /**
     *  CheckoutPaymentModulesList constructor.
     */
    function CheckoutPaymentModulesList()
    {
        global $application;

        #check if fatal errors exist in the block tag
        $this->NoView = false;
        if ($application->issetBlockTagFatalErrors("CheckoutPaymentModulesList"))
        {
            $this->NoView = true;
        }
    }

    /**
     * Outputs a list of all payment modules existing in the system, i.e. modules
     * loaded to AVACTIS with 'groups' and that have the PaymentModule group.
     */
    function getInstalledModulesListData($groups = NULL)
    {
        global $application;
        if($groups == null)
        {
            $groups = array();
        }
        $groups[] = "PaymentModule";
        $pm_list = modApiFunc("Modules_Manager", "getActiveModules", $groups);
        return $pm_list;
    }

    /**
     * Prepares data for the View "Modules List".
     */
    function getModulesList($groups = null)
    {
        global $application;

        $SelectedModules = modApiFunc("Checkout", "getSelectedModules", "payment");

        $pm_list = $this->getInstalledModulesListData($groups);

        $items = array();
        $new_selected_module_sort_order = 0;
        foreach ($pm_list as $pm_item)
        {
            //FIXME create/use some mm (ModuleManager) function to convert class names.
            $name = strtolower($pm_item->name);

            $payment_group = modApiFunc("Checkout", "getPaymentModuleGroup", $pm_item);
            /* Don't show the "All Inactive" payment module */
            if($payment_group == "")
            {
                continue;
            }

            $pmInfo = modApiFunc($name, "getInfo");

            if(!array_key_exists($pmInfo['GlobalUniquePaymentModuleID'], $SelectedModules))
                continue;

            $this->_Current_Module = $pmInfo;

            $request=&new Request();
            $request->setView( 'CheckoutPaymentModuleSettings' );
            $request->setAction( "SetCurrentPaymentModuleSettingsViewName" );
            $request->setKey   ( "pm_viewclassname", $this->_Current_Module["PreferencesAZViewClassName"]);
            $pmInfo['InfoLink'] = $request->getURL();

            //product without small image or small image file is corrupted or absent.
            $this->_Current_Module['PaymentModuleStatusMessage'] = $pmInfo["StatusMessage"];
            $this->_Current_Module['PaymentModuleName'] = prepareHTMLDisplay($pmInfo["Name"]);
            $this->_Current_Module['PaymentModuleID'] = $pmInfo["GlobalUniquePaymentModuleID"];

            $module_info = modApiFunc('Modules_Manager','getModuleInfoByName',$pmInfo["APIClassName"]);

            $MessageResources = &$application->getInstance('MessageResources', modApiFunc("Modules_Manager","getResFileByShortName", $module_info->shortName), 'AdminZone');

            $module_descr = "MODULE_DESCR";

            $description = $MessageResources->getMessage($module_descr);
            $this->_Current_Module['PaymentModuleDescription'] = $description;
//            $this->_Current_Module['PaymentModuleDescription'] = $module_descr;

            $this->_Current_Module['InfoLink'] =  $pmInfo['InfoLink'];

            $application->registerAttributes($this->_Current_Module);

            //If sort id is not defined then assign the highest possible sort id to this item: 0, -1, -2 ...
            $sort_id = empty($SelectedModules[$pmInfo["GlobalUniquePaymentModuleID"]]["sort_order"]) ? $new_selected_module_sort_order-- : $SelectedModules[$pmInfo["GlobalUniquePaymentModuleID"]]["sort_order"];

            $items[$sort_id] = modApiFunc('TmplFiller', 'fill', "checkout/payment_modules_list/", "list_item.tpl.html", array());
        }
        //Sort items by sort id and implode them.
        ksort($items, SORT_NUMERIC);
        $value = implode("", $items);
        return $value;
    }

    /**
     * Otputs the view.
     *
     * @todo $request->setView  ( '' ) - define the view name
     */
    function output()
    {
        global $application;

        #Define whether to output the view or not
        if ($this->NoView)
        {
            $application->outputTagErrors(true, "CheckoutPaymentModulesList", "Errors");
            return "";
        }

        $application->registerAttributes(array('Items'                                      => "",
                                               'PaymentModuleGroupList'                     => "",

                                               'PaymentModuleGroupList_Offline'             => "",
                                               'HiddenAvailable_Offline'                    => "",
                                               'PaymentModuleGroupList_OnlineCC'            => "",
                                               'HiddenAvailable_OnlineCC'                   => "",
                                               'PaymentModuleGroupList_OnlineECheck'        => "",
                                               'HiddenAvailable_OnlineECheck'               => "",
                                               'PaymentModuleGroupList_OnlinePaymentSystem' => "",
                                               'HiddenAvailable_OnlinePaymentSystem'        => "",
                                               'List_SelectedModules'                       => "",
                                               'HiddenSelectedModules'                      => "",

                                               'SaveSelectedPaymentModulesListHref'         => "",
                                               'HiddenFieldAction'                          => "",
                                               'HiddenArrayViewState'                       => "",
                                               'getPaymentModuleInfoItemsJS'                => "",
                                               'getPaymentModuleGroupsItemsJS'              => "",
                                               'PaymentModulesLink'                         => ""));

/*        "AvailableSelect_Offline",
        "AvailableSelect_OnlineCC",
        "AvailableSelect_OnlineECheck",
        "AvailableSelect_OnlinePaymentSystem" */

        $this->mTmplFiller = &$application->getInstance('TmplFiller');

        $this->MessageResources = &$application->getInstance('MessageResources');
        //FIXME: correct error codes
        $output = modApiFunc('TmplFiller', 'fill',
                              './../../js/','validate.msgs.js.tpl',
                              array(
                                    "STRING1024"=> $this->MessageResources->getMessage( new ActionMessage(array('CATADD_001')) ),
                                    "STRING128"=> $this->MessageResources->getMessage( new ActionMessage(array('CATADD_002')) ),
                                    "STRING256"=> $this->MessageResources->getMessage( new ActionMessage(array('CATADD_003')) ),
                                    "STRING512"=> $this->MessageResources->getMessage( new ActionMessage(array('CATADD_004')) )
                                   )
                            );

        return $output.$this->mTmplFiller->fill("checkout/payment_modules_list/", "list.tpl.html",array());
    }

    function getTag($tag)
    {
        global $application;
        $value = null;
        switch ($tag)
        {
    		case 'Items':
    			$value = $this->getModulesList();
    			break;

            case 'PaymentModulesList':
                $value = "Payment Modules List";
                break;
/*            case 'SubmitedCheckoutStoreBlocksListItemName':
                $value = "SubmitedCheckoutStoreBlocksList[shipping-method-list-input]";
                break;*/
            case 'List_SelectedModules':
            {
                $items = array();
                $SelectedModules = modApiFunc("Checkout", "getSelectedModules", "payment");
                $new_selected_module_sort_order = 0;

                $modules = $this->getInstalledModulesListData();
                foreach($modules as $module)
                {
                    $name = strtolower($module->name);
                    $payment_group = modApiFunc("Checkout", "getPaymentModuleGroup", $module);

                    /* Don't show "All Inactive" payment module */
                    if($payment_group != "")
                    {
                        $pmInfo = modApiFunc($name, "getInfo");
                        ////FIXME INCORRECT method to get group name! Remove group name from id at all.
                        //$groups_array = split(',', $module->groups);

                        if(array_key_exists($pmInfo['GlobalUniquePaymentModuleID'], $SelectedModules))
                        {
                            $PaymentModulesGroupsInfo = modApiFunc("Checkout", "getPaymentModulesGroupsInfo");
                            //If sort id is not defined then assign the highest possible sort id to this item: 0, -1, -2 ...
                            $sort_id = empty($SelectedModules[$pmInfo["GlobalUniquePaymentModuleID"]]["sort_order"]) ? $new_selected_module_sort_order-- : $SelectedModules[$pmInfo["GlobalUniquePaymentModuleID"]]["sort_order"];

                            $items[$sort_id] = "<option value=\"" . $pmInfo['GlobalUniquePaymentModuleID'] . "\" id=\"" . $pmInfo['GlobalUniquePaymentModuleID'] . "\">" . "[" . $PaymentModulesGroupsInfo[$payment_group]['short_name'] . "]" .  prepareHTMLDisplay($pmInfo["Name"]) . "</option>";
                        }
                    }
                }

                //Sort items by sort id and implode them.
                ksort($items, SORT_NUMERIC);
                $value = implode("", $items);
                break;
            }
            case 'HiddenSelectedModules':
                //Hidden field to store selected ("Selected Payment Modules") select state
                $value = "";

                $SelectedModules = modApiFunc("Checkout", "getSelectedModules", "payment");
                $new_selected_module_sort_order = 0;

                $modules = $this->getInstalledModulesListData();
                foreach($modules as $module)
                {
                    $name = strtolower($module->name);
                    $payment_group = modApiFunc("Checkout", "getPaymentModuleGroup", $module);

                    /* Don't show "All Inactive" payment module */
                    if($payment_group != "")
                    {
                        $pmInfo = modApiFunc($name, "getInfo");
                        ////FIXME INCORRECT method to get group name! Remove group name from id at all.
                        //$groups_array = split(',', $module->groups);

                        if(array_key_exists($pmInfo['GlobalUniquePaymentModuleID'], $SelectedModules))
                        {
                            $PaymentModulesGroupsInfo = modApiFunc("Checkout", "getPaymentModulesGroupsInfo");
                            //If sort id is not defined then assign the highest possible sort id to this item: 0, -1, -2 ...
                            $sort_id = empty($SelectedModules[$pmInfo["GlobalUniquePaymentModuleID"]]["sort_order"]) ? $new_selected_module_sort_order-- : $SelectedModules[$pmInfo["GlobalUniquePaymentModuleID"]]["sort_order"];

                            $value .= (($value == "") ? $pmInfo['GlobalUniquePaymentModuleID'] :
                                                  "|" . $pmInfo['GlobalUniquePaymentModuleID']);
                        }
                    }
                }
                break;

            case 'SaveSelectedPaymentModulesListHref':
                $value = $application->getPagenameByViewname("CheckoutPaymentModulesList",-1,-1,'AdminZone');
////                $request=&new Request();
////                $request->setView  ('CheckoutPaymentModulesList');
////                $request->setAction('SaveSelectedPaymentModulesList');
////                $value = modApiFunc('application', 'href', $request);
                break;
            case 'HiddenFieldAction':
                loadCoreFile('html_form.php');
                $HtmlForm = new HtmlForm();
                $value = $HtmlForm->genHiddenField('asc_action', 'SaveSelectedPaymentModulesList');
                break;
            case 'HiddenArrayViewState':
                break;

            case 'getPaymentModuleInfoItemsJS':
                $value = "";

	            $modules = $this->getInstalledModulesListData();
                foreach($modules as $module)
                {
                    $name = strtolower($module->name);
                    $pmInfo = modApiFunc($name, "getInfo");
                    $payment_group = modApiFunc("Checkout", "getPaymentModuleGroup", $module);
                    if($payment_group != "")
                    {
                        $PaymentModulesGroupsInfo = modApiFunc("Checkout", "getPaymentModulesGroupsInfo");
                        $value .= "case '" .$pmInfo['GlobalUniquePaymentModuleID']. "': switch(key){case 'PaymentModulesGroupID': value = '" . $payment_group . "'; break; case 'ShortName': value ='" . $pmInfo["Name"] . "'; break; case 'FullName': value ='[" . $PaymentModulesGroupsInfo[$payment_group]['short_name'] . "]" .$pmInfo["Name"]. "'; break;}; break;";
                        //e.g. PaymentModulesGroupID = OnlineCC
                        //     ShortName             = Paypal
                        //     FullName              = [Online CC]Paypal
                        //Notice whitespace in "[Online CC]"
                    }
                }
                break;

            case 'getPaymentModuleGroupsItemsJS':
                    $value = "";
                    $PaymentModulesGroupsInfo = modApiFunc("Checkout", "getPaymentModulesGroupsInfo");
                    $bFirstItem = true;
                    foreach($PaymentModulesGroupsInfo as $PaymentModulesGroupInfo)
                    {
                        if($bFirstItem)
                        {
                            $value .= "'" . $PaymentModulesGroupInfo['group_id'] . "'";
                            $bFirstItem = false;
                        }
                        else
                        {
                            $value .= ", '" . $PaymentModulesGroupInfo['group_id'] . "'";
                        }
                    }
                break;

            case 'PaymentModulesLink':
                $request = &new Request();
                $request->setView(CURRENT_REQUEST_URL);
                $value = $request->getURL();
                break;

            default:
                $value = "";
                $pos = strpos($tag, "_");
                if($pos != FALSE)
                {
                    $prefix = substr($tag, 0, $pos);
                    switch($prefix)
                    {
                        case "PaymentModuleGroupList":
                            //Options for "Offline" "Online CC" "Online eCheck" and "Online Payment" <select> control.

                            $SelectedModules = modApiFunc("Checkout", "getSelectedModules", "payment");

                            $group_name = substr($tag, strlen($prefix)+1);
                            $modules = $this->getInstalledModulesListData(array($group_name));
                            foreach($modules as $module)
                            {
                                $name = strtolower($module->name);
                                $pmInfo = modApiFunc($name, "getInfo");

                                if(!array_key_exists($pmInfo['GlobalUniquePaymentModuleID'], $SelectedModules))
                                {
                                    $value .= "<option value=\"" . $pmInfo['GlobalUniquePaymentModuleID'] . "\">" . prepareHTMLDisplay($pmInfo["Name"]) . "</option>";
                                }
                            }
                            break;
                        case "HiddenAvailable":
                            break;
                        default:
                            //////FIXME can it be Current Module (Modules List Item) details?
                            ////_fatal(__CLASS__ .'::'. __FUNCTION__. ': prefix = ' . $prefix);
                            break;
                    }
                }
                else
                {
                    //Current Module (Modules List Item) details
                    $value = getKeyIgnoreCase($tag, $this->_Current_Module);
                }
                break;
        }
        return $value;
    }

    /**#@-*/

//------------------------------------------------
//              PRIVATE DECLARATION
//------------------------------------------------

    /**#@+
     * @access private
     */

    /**
     * A reference to the object TemplateFiller.
     *
     * @var TemplateFiller
     */
    var $templateFiller;

    /**
     * The current selected template.
     *
     * @var array
     */
    var $template;

    /**
     * Current selected module info. It is used for the internal processing.
     *
     * @var array
     */
    var $_Current_Module;

    /**#@-*/
}
?>