<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2007, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

/**
 * @package CustomerAccount
 * @author Egor V. Derevyankin
 *
 */

class CCustomerInfo
{
    function CCustomerInfo($account='')
    {
        if(!modApiFunc('Customer_Account','doesAccountExists',$account))
        {
            $this->account = null;
            $this->base_info = null;
            $this->full_info = null;
        }
        else
        {
            $this->account = $account;
            $this->__loadBaseInfo();
        };

        $this->orders_filter = null;
        $this->orders_ids = null;
        $this->orders_info = array();
        $this->orders_summary = null;
        $this->person_info_attrs_type = PERSON_INFO_GROUP_ATTR_ALL;
    }

    function getDisplayAccountName()
    {
        if($this->account == null)
            return '';

        $account_name = $this->account;

        if($this->getPersonInfo('Status') == 'B')
        {
            $account_name = preg_replace('/'.PSEUDO_CUSTOMER_SUFFIX.'$/i','',$account_name);
            if(preg_match('/^'.PSEUDO_NA_CUSTOMER_PERFIX.'(\d)+/i',$account_name))
            {
                $account_name = 'N/A';
            };
        };

        return $account_name;
    }

    function changePassword($new_password)
    {
        if($this->account == null)
            return false;

        global $application;
        $ca_tables = modApiStaticFunc('Customer_Account','getTables');
        $customers_table = $ca_tables['ca_customers']['columns'];

        $query = &new DB_Update('ca_customers');
        $query->addUpdateValue($customers_table['customer_password'], md5($new_password));
        $query->WhereValue($customers_table['customer_account'], DB_EQ, $this->account);
        $application->db->PrepareSQL($query);
        return $application->db->DB_Exec();
    }

    function SignIn()
    {
        if($this->account == null) return;

        modApiFunc('Session','set','SignedCustomer',$this->account);
        $this->refreshCheckoutPrerequisites();
    }

    function SignOut()
    {
        if($this->account == null) return;

        if($this->isSigned())
            modApiFunc('Session','un_set','SignedCustomer');
    }

    function isSigned()
    {
        if($this->account == null) return false;

        return (modApiFunc('Session','is_set','SignedCustomer')
            and modApiFunc('Session','get','SignedCustomer') == $this->account);
    }

    function getPersonInfo($attr, $group='base')
    {
        if($this->account == null)
            return null;

        if(strtolower($attr) == 'account')
        {
            return $this->account;
        };

        if(array_key_exists(strtolower($group), $this->_groups_names))
            $group = $this->_groups_names[strtolower($group)];

        if(array_key_exists(strtolower($attr), $this->_attrs_names))
            $attr = $this->_attrs_names[strtolower($attr)];

        switch($group)
        {
            case 'base':
                if(array_key_exists($attr, $this->base_info))
                {
                    return $this->base_info[$attr];
                };
                break;
            default:
                if($this->full_info == null)
                {
                    $this->__loadFullInfo();
                };
                if(array_key_exists($group, $this->full_info))
                {
                    if(array_key_exists($attr, $this->full_info[$group]['attrs']))
                    {
                        return $this->full_info[$group]['attrs'][$attr]['attr_value'];
                    };
                };
                break;
        };

        return null;
    }

    function getPersonInfoGroupsNames()
    {
        if($this->account == null)
            return array();

        if($this->full_info == null)
        {
            $this->__loadFullInfo();
        };

        return array_keys($this->full_info);
    }

    function getPersonInfoGroupAttrsNames($group_name)
    {
        if($this->account == null)
            return array();

        if($this->full_info == null)
        {
            $this->__loadFullInfo();
        };

        if(array_key_exists($group_name, $this->full_info))
        {
            return array_keys($this->full_info[$group_name]['attrs']);
        };

        return array();
    }

    function getPersonInfoGroupInfoByName($group_name)
    {
        if($this->account == null)
            return array();

        if($this->full_info == null)
        {
            $this->__loadFullInfo();
        };

        if(array_key_exists($group_name, $this->full_info))
        {
            return array(
                'id' => $this->full_info[$group_name]['group_id']
               ,'lang_code' => $this->full_info[$group_name]['lang_code']
            );
        };

        return array();
    }

    function getPersonInfoAttrInfoByName($attr_name, $group_name)
    {
        if($this->account == null)
            return array();

        if($this->full_info == null)
        {
            $this->__loadFullInfo();
        };

        if(array_key_exists($group_name, $this->full_info))
        {
            if(array_key_exists($attr_name, $this->full_info[$group_name]['attrs']))
            {
                return $this->full_info[$group_name]['attrs'][$attr_name];
            };
        };

        return array();
    }

    function setPersonInfo($attr, $value, $group='base')
    {
        if($this->account == null)
            return false;

        switch($group)
        {
            case 'base':
                if($attr != 'Status')
                {
                    return false;
                };
                return modApiFunc('Customer_Account','setAccountStatus', $this->account, $value);
                break;
            default:
                if($this->full_info == null)
                {
                    $this->__loadFullInfo();
                };
                if(array_key_exists($group, $this->full_info))
                {
                    if(array_key_exists($attr, $this->full_info[$group]['attrs']))
                    {
                        if($this->__updatePersonInfoAttrInDB(
                            $this->full_info[$group]['group_id']
                           ,$this->full_info[$group]['attrs'][$attr]['attr_id']
                           ,$value
                         ))
                        {
                            $this->full_info[$group]['attrs'][$attr]['attr_value'] = $value;
                            return true;
                        };
                    };
                };
                break;
        };

        return false;
    }

    function setPersonInfoAttrsType($type = PERSON_INFO_GROUP_ATTR_ALL)
    {
        if(in_array($type,array(
            PERSON_INFO_GROUP_ATTR_ALL
           ,PERSON_INFO_GROUP_ATTR_VISIBLE
           ,PERSON_INFO_GROUP_ATTR_HIDDEN
          )))
        {
            if($this->person_info_attrs_type != $type)
            {
                $this->full_info = null;
            };

            $this->person_info_attrs_type = $type;
        };
    }

    function refreshCheckoutPrerequisites()
    {
        if($this->account == null)
            return;

        $groups = $this->getPersonInfoGroupsNames();

        if(empty($groups))
            return;

        $new_validation_results = modApiFunc('Checkout','getPrerequisitesValidationResults');

        foreach($groups as $group_name)
        {
            if($group_name == 'Customer')
                continue;

            $co_gn = strtolower($group_name).'Info';
            $pvr = modApiFunc("Checkout", "getPrerequisiteValidationResults", $co_gn);

            foreach($pvr['validatedData'] as $tag_name => $tag_info)
            {
                if($tag_name == 'Statetext')
                {
                    $attr_name = 'state';
                }
                else
                {
                    $attr_name = modApiFunc('Customer_Account','getPersonInfoAttrNameByCOAttrID',$tag_info['id']);
                };

                $attr_value = $tag_info['value'];

                $normal_attr_name = modApiFunc('Customer_Account','getPersionInfoAttrNameByLowcaseName', $attr_name);
                $attr_value = $this->getPersonInfo($normal_attr_name, $group_name);

                switch($tag_name)
                {
                    case 'Statemenu':
                        $attr_value = is_int($attr_value) ? $attr_value : 0;
                        break;
                    case 'Statetext':
                        $attr_value = is_string($attr_value) ? $attr_value : '';
                        break;
                };

                $pvr['validatedData'][$tag_name]['value'] = $attr_value;
            };

            $new_validation_results[$co_gn] = $pvr;
        };

        modApiFunc('Checkout','setPrerequisitesValidationResults',$new_validation_results);
    }

    function loadPersonInfoFromCheckout()
    {
        global $application;

        if($this->account == null)
            return;

        $groups = $this->getPersonInfoGroupsNames();

        if(empty($groups))
            return;

        foreach($groups as $group_name)
        {
            if($group_name == 'Customer')
                continue;

            $co_gn = strtolower($group_name).'Info';
            $pvr = modApiFunc("Checkout", "getPrerequisiteValidationResults", $co_gn);

            $customer_country = null;
            $customer_state_id = null;
            $customer_state_text = null;

            $validator = &$application->getInstance('CAValidator');

            foreach($pvr['validatedData'] as $tag_name => $tag_info)
            {
                switch($tag_name)
                {
                    case 'Country':
                        $customer_country = $tag_info['value'];
                        break;
                    case 'Statemenu':
                        $customer_state_id = $tag_info['value'];
                        break;
                    case 'Statetext':
                        $customer_state_text = $tag_info['value'];
                        break;
                };
                if(in_array($tag_name,array('Country','Statemenu','Statetext')))
                {
                    continue;
                };

                $attr_name = modApiFunc('Customer_Account','getPersonInfoAttrNameByCOAttrID',$tag_info['id']);

                $attr_value = $tag_info['value'];
                $normal_attr_name = modApiFunc('Customer_Account','getPersionInfoAttrNameByLowcaseName', $attr_name);

                $this->setPersonInfo($normal_attr_name, $attr_value, $group_name);
            };

            if($customer_country != null)
            {
                $this->setPersonInfo('Country', $customer_country, $group_name);
                if($customer_state_id != null or $customer_state_text != null)
                {
                    $tmp_arr = array(
                        'country_id' => $customer_country
                       ,'state_id' => $customer_state_id
                       ,'state_text' =>  $customer_state_text
                     );
                    if($validator->isValid('country_state', $tmp_arr))
                    {
                        $this->setPersonInfo('State', $tmp_arr['state_id'], $group_name);
                    };
                };
            };
        };

        return;
    }

    /**
     * @param array $filter
     */
    function setOrdersHistoryFilter($filter)
    {
        //TODO: check filter;

        if($this->orders_filter != $filter)
        {
            $this->orders_filter = $filter;
            $this->search_completed = false;
            $this->orders_summary = null;
        };
    }

    function getOrdersCount()
    {
        return count($this->getOrdersIDs());
    }

    function getOrdersIDs()
    {
        if($this->account == null or $this->orders_filter == null)
        {
            return array();
        };

        if(!$this->search_completed)
        {
            $this->__searchOrders();
        };

        return $this->orders_ids;
    }

    function getBaseOrderInfo($order_id)
    {
        if($this->account == null) return array();

        $order_id = intval($order_id);

        if(isset($this->orders_info[$order_id]['base']))
        {
            return $this->orders_info[$order_id]['base'];
        };

        global $application;
        $co_tables = modApiStaticFunc('Checkout','getTables');
        $orders_table = $co_tables['orders']['columns'];

        $query = &new DB_Select();
        $query->addSelectTable('orders');
        $query->addSelectField('*');
        $query->WhereValue($orders_table['id'], DB_EQ, $order_id);

        $res = $application->db->getDB_Result($query);

        if(count($res) != 1)
        {
            return array();
        };

        $this->orders_info[$order_id]['base'] = $res[0];

        return $this->orders_info[$order_id]['base'];
    }

    function getFullOrderInfo($order_id)
    {
    }

    function getOrdersMinDate()
    {
        return $this->__getOrdersSummaryTag('min_date');
    }

    function getOrdersMaxDate()
    {
        return $this->__getOrdersSummaryTag('max_date');
    }

    function getOrdersAmount()
    {
        return $this->__getOrdersSummaryTag('amount');
    }

    function getOrdersFullyPaidAmount()
    {
        return $this->__getOrdersSummaryTag('fully_paid_amount');
    }

    function copyPersonInfo($from_group, $to_group, $exclude=array())
    {
        if($this->full_info == null)
        {
            $this->__loadFullInfo();
        };

        if(!array_key_exists($from_group, $this->full_info) or !array_key_exists($to_group, $this->full_info))
            return;

        foreach($this->full_info[$from_group]['attrs'] as $attr_name => $attr_info)
        {
            if(array_key_exists($attr_name, $this->full_info[$to_group]['attrs']) and !in_array($attr_name,$exclude))
            {
                $this->setPersonInfo($attr_name, $attr_info['attr_value'], $to_group);
            };
        };
    }

    function __loadBaseInfo()
    {
        if($this->account === null)
            return;

        global $application;
        $tables = Customer_Account::getTables();
        $customers_table = $tables['ca_customers']['columns'];

        $query = &new DB_Select();
        $query->addSelectTable('ca_customers');
        $query->addSelectField('*');
        $query->WhereValue($customers_table['customer_account'], DB_EQ, $this->account);

        $res = $application->db->getDB_Result($query);

        $this->base_info = array(
            'ID'     => $res[0]['customer_id']
           ,'Account'  => $res[0]['customer_account']
           ,'Status' => $res[0]['customer_status']
        );
    }

    function __loadFullInfo()
    {
        if($this->account === null)
            return;

        global $application;
        $tables = Customer_Account::getTables();
        $atg_table = $tables['ca_attrs_to_groups']['columns'];
        $data_table = $tables['ca_person_info_data']['columns'];

        $_groups = modApiFunc('Customer_Account','getPersonInfoGroupsList');

        $pi_groups = array();

        for($i=0; $i<count($_groups); $i++)
        {
            $group_name = $_groups[$i]['group_name'];
            unset($_groups[$i]['group_name']);
            $pi_groups[$group_name] = $_groups[$i];

            $_attrs = modApiFunc('Customer_Account','getPersonInfoGroupAttrs', $pi_groups[$group_name]['group_id'], $this->person_info_attrs_type);
            $group_attrs = array();

            for($j=0; $j<count($_attrs); $j++)
            {
                $attr_name = $_attrs[$j]['attr_name'];
                unset($_attrs[$j]['attr_name']);
                $group_attrs[$attr_name] = $_attrs[$j];
            };

            $pi_groups[$group_name]['attrs'] = $group_attrs;
        };

        if(!empty($pi_groups))
        {
            foreach($pi_groups as $group_name => $group_info)
            {
                $group_id = $group_info['group_id'];
                $group_attrs = $group_info['attrs'];

                if(!empty($group_attrs))
                {
                    foreach($group_attrs as $attr_name => $attr_info)
                    {
                        $pi_groups[$group_name]['attrs'][$attr_name]['attr_value'] = '';

                        $query = &new DB_Select();
                        $query->addSelectField($data_table['data_value'], 'value');
                        $query->addSelectTable('ca_attrs_to_groups');
                        $query->addSelectTable('ca_person_info_data');
                        $query->Where($atg_table['group_id'], DB_EQ, $group_id);
                        $query->WhereAND();
                        $query->Where($atg_table['attr_id'], DB_EQ, $attr_info['attr_id']);
                        $query->WhereAND();
                        $query->Where($data_table['ag_id'], DB_EQ, $atg_table['ag_id']);
                        $query->WhereAND();
                        $query->Where($data_table['customer_id'], DB_EQ, $this->base_info['ID']);

                        $res = $application->db->getDB_Result($query);

                        if(count($res) == 1)
                        {
                            $pi_groups[$group_name]['attrs'][$attr_name]['attr_value'] = $res[0]['value'];
                        };
                    };
                };
            };
        };

        $this->full_info = $pi_groups;
    }

    function __searchOrders()
    {
        global $application;
        $tables = modApiStaticFunc('Checkout','getTables');
        $orders_table = $tables['orders']['columns'];

        $query = &new DB_Select();
        $query->addSelectTable('orders');
        $query->addSelectField($orders_table['id'], 'order_id');
        $query->WhereValue($orders_table['person_id'], DB_EQ, $this->base_info['ID']);

        if($this->orders_filter['type'] == 'id')
        {
            $query->WhereAND();
            $query->WhereValue($orders_table['id'], DB_EQ, $this->orders_filter['order_id']);
        };

        $query->SelectGroup($orders_table['id']);
        $query->SelectOrder($orders_table['id'], 'DESC');

        $oids_wo_filter = array();

        $res = $application->db->getDB_Result($query);
        for($i=0;$i<count($res);$i++)
        {
            $oids_wo_filter[] = $res[$i]['order_id'];
        };

        if($this->orders_filter['type'] != 'custom'
            and ($this->orders_filter['order_status'] == ORDER_STATUS_ALL or empty($oids_wo_filter)))
        {
            $this->__setOrdersIDs($oids_wo_filter);
            return;
        };

        $query = &new DB_Select();
        $query->addSelectTable('orders');
        $query->addSelectField($orders_table['id'], 'order_id');

        if($this->orders_filter['type'] == 'quick')
        {
            $query->WhereValue($orders_table['status_id'], DB_EQ, $this->orders_filter['order_status']);
        };

        if($this->orders_filter['type'] == 'custom')
        {
            $from_date = implode("-",array(
                $this->orders_filter['year_from']
               ,$this->orders_filter['month_from']
               ,$this->orders_filter['day_from']
            ));

            $to_date = implode("-",array(
                $this->orders_filter['year_to']
               ,$this->orders_filter['month_to']
               ,$this->orders_filter['day_to']
            ));

            $query->WhereValue($orders_table['date'], DB_GTE, $from_date);
            $query->WhereAND();
            $query->WhereValue($orders_table['date'], DB_LTE, $to_date);

            if($this->orders_filter['order_status'] != ORDER_STATUS_ALL)
            {
                $query->WhereAND();
                $query->WhereValue($orders_table['status_id'], DB_EQ, $this->orders_filter['order_status']);
            };

            if($this->orders_filter['order_payment_status'] != ORDER_PAYMENT_STATUS_ALL)
            {
                $query->WhereAND();
                $query->WhereValue($orders_table['payment_status_id'], DB_EQ, $this->orders_filter['order_payment_status']);
            };
        };

        $query->WhereAND();
        $query->Where($orders_table['id'], DB_IN, "('".implode("','",$oids_wo_filter)."')");

        $oids_with_filter = array();

        $res = $application->db->getDB_Result($query);
        for($i=0;$i<count($res);$i++)
        {
            $oids_with_filter[] = $res[$i]['order_id'];
        };

        $this->__setOrdersIDs($oids_with_filter);
    }

    function __loadOrdersSummary()
    {
        if(!$this->search_completed or empty($this->orders_ids))
        {
            return;
        };

        global $application;
        $co_tables = modApiStaticFunc('Checkout','getTables');
        $orders_table = $co_tables['orders']['columns'];

        $query = &new DB_Select();
        $query->addSelectTable('orders');
        $query->addSelectField($query->fMax($orders_table['date']), 'max_date');
        $query->addSelectField($query->fMin($orders_table['date']), 'min_date');
        $query->addSelectField($query->fSum($orders_table['total']), 'amount');
        $query->Where($orders_table['id'], DB_IN, "('".implode("','",$this->orders_ids)."')");

        $res = $application->db->getDB_Result($query);

        $this->orders_summary = $res[0];

        //load fully paid
        $query = &new DB_Select();
        $query->addSelectTable('orders');
        $query->addSelectField($query->fSum($orders_table['total']), 'amount');
        $query->Where($orders_table['id'], DB_IN, "('".implode("','",$this->orders_ids)."')");
        $query->WhereAND();
        $query->WhereValue($orders_table['payment_status_id'], DB_EQ, ORDER_PAYMENT_STATUS_FULLY_PAID);

        $res = $application->db->getDB_Result($query);

        $this->orders_summary['fully_paid_amount'] = $res[0]['amount'];
    }

    function __getOrdersSummaryTag($tag)
    {
        if($this->orders_summary == null)
        {
            $this->__loadOrdersSummary();
        };

        return $this->orders_summary[$tag];
    }

    function __setOrdersIDs($oids)
    {
        $this->orders_ids = $oids;
        $this->search_completed = true;
        $this->orders_summary = null;
    }

    function __updatePersonInfoAttrInDB($group_id, $attr_id, $attr_value)
    {
        global $application;
        $tables = Customer_Account::getTables();
        $atg_table = $tables['ca_attrs_to_groups']['columns'];
        $data_table = $tables['ca_person_info_data']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($atg_table['ag_id']);
        $query->Where($atg_table['group_id'], DB_EQ, $group_id);
        $query->WhereAND();
        $query->Where($atg_table['attr_id'], DB_EQ, $attr_id);
        $res = $application->db->getDB_Result($query);

        $ag_id = $res[0]['ag_id'];

        $query = &new DB_Replace('ca_person_info_data');
        $query->addReplaceValue($this->base_info['ID'], $data_table['customer_id']);
        $query->addReplaceValue($ag_id, $data_table['ag_id']);
        $query->addReplaceValue($attr_value, $data_table['data_value']);

        $application->db->PrepareSQL($query);

        return $application->db->DB_Exec();
    }

    var $account;
    var $base_info;
    var $full_info;
    var $orders_filter;
    var $orders_ids;
    var $search_completed;
    var $orders_info;
    var $orders_summary;
    var $person_info_attrs_type;

    var $_attrs_names = array(
        'firstname'   => 'FirstName'
       ,'lastname'    => 'LastName'
       ,'email'       => 'Email'
       ,'phone'       => 'Phone'
       ,'country'     => 'Country'
       ,'state'       => 'State'
       ,'zipcode'     => 'ZipCode'
       ,'postcode'    => 'ZipCode'
       ,'city'        => 'City'
       ,'streetline1' => 'Streetline1'
       ,'streetline2' => 'Streetline2'
    );

    var $_groups_names = array(
        'customer'  => 'Customer'
       ,'billing'   => 'Billing'
       ,'shipping'  => 'Shipping'
    );
};

?>