<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2007, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

/**
 * @package CustomerAccount
 * @author Egor V. Derevyankin
 *
 */

class Customer_Account
{
    function Customer_Account()
    {
        $this->_co_attrs_ids = array();
        $this->_co_pi_types_ids = array();
        $this->_lowcase_names = array();
        $this->_customers_search_filter = null;
        $this->_customers = null;
    }

    function install()
    {
        $query = &new DB_Table_Create(Customer_Account::getTables());
        loadClass('Customer_Account_Installer');
        $installer = &new Customer_Account_Installer();
        $installer->doInstall();
    }

    function uninstall()
    {
        $query = &new DB_Table_Delete(Customer_Account::getTables());
    }

    function getTables()
    {
        static $tables;

        if (is_array($tables))
        {
            return $tables;
        }

        $tables = array();

        $table = 'ca_customers';
        $tables[$table] = array(
            'columns' => array(
                'customer_id'       => $table.'.customer_id'
               ,'customer_account'  => $table.'.customer_account'
               ,'customer_password' => $table.'.customer_password'
               ,'customer_status'   => $table.'.customer_status'
             )
           ,'types'   => array(
                'customer_id'       => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'customer_account'  => DBQUERY_FIELD_TYPE_CHAR255.' NOT NULL'
               ,'customer_password' => DBQUERY_FIELD_TYPE_CHAR32.' NOT NULL'
               ,'customer_status'   => "ENUM ('A','N','B','R') default 'N'"
             )
           ,'primary' => array(
                'customer_id'
             )
           ,'indexes' => array(
                'UNIQUE KEY UNQ_account' => 'customer_account'
             )
        );

        $table = 'ca_person_info_groups';
        $tables[$table] = array(
            'columns' => array(
                'group_id'      => $table.'.group_id'
               ,'group_name'    => $table.'.group_name'
               ,'lang_code'     => $table.'.lang_code'
               ,'sort_order'    => $table.'.sort_order'
             )
           ,'types'   => array(
                'group_id'      => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'group_name'    => DBQUERY_FIELD_TYPE_CHAR255.' NOT NULL'
               ,'lang_code'     => DBQUERY_FIELD_TYPE_CHAR255.' NOT NULL'
               ,'sort_order'    => DBQUERY_FIELD_TYPE_INT .' NOT NULL'
             )
           ,'primary' => array(
                'group_id'
             )
           ,'indexes' => array(
                'UNIQUE KEY UNQ_gn' => 'group_name'
             )
        );

        $table = 'ca_person_info_attrs';
        $tables[$table] = array(
            'columns' => array(
                'attr_id'       => $table.'.attr_id'
               ,'attr_name'     => $table.'.attr_name'
               ,'lang_code'     => $table.'.lang_code'
             )
           ,'types'   => array(
                'attr_id'       => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'attr_name'     => DBQUERY_FIELD_TYPE_CHAR255.' NOT NULL'
               ,'lang_code'     => DBQUERY_FIELD_TYPE_CHAR255.' NOT NULL'
             )
           ,'primary' => array(
                'attr_id'
             )
           ,'indexes' => array(
                'UNIQUE KEY UNQ_an' => 'attr_name'
             )
        );

        $table = 'ca_attrs_to_groups';
        $tables[$table] = array(
            'columns' => array(
                'ag_id'         => $table.'.ag_id'
               ,'group_id'      => $table.'.group_id'
               ,'attr_id'       => $table.'.attr_id'
               ,'sort_order'    => $table.'.sort_order'
               ,'visible_name'  => $table.'.visible_name'
               ,'is_visible'    => $table.'.is_visible'
               ,'is_required'   => $table.'.is_required'
             )
           ,'types'   => array(
                'ag_id'         => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'group_id'      => DBQUERY_FIELD_TYPE_INT .' NOT NULL'
               ,'attr_id'       => DBQUERY_FIELD_TYPE_INT .' NOT NULL'
               ,'sort_order'    => DBQUERY_FIELD_TYPE_INT .' NOT NULL'
               ,'visible_name'  => DBQUERY_FIELD_TYPE_CHAR255.' NOT NULL'
               ,'is_visible'    => "ENUM ('Y','N') DEFAULT 'Y'"
               ,'is_required'   => "ENUM ('Y','N') DEFAULT 'Y'"
             )
           ,'primary' => array(
                'ag_id'
             )
           ,'indexes' => array(
                'UNIQUE KEY UNQ_gid_aid' => 'group_id,attr_id'
             )
        );

        $table = 'ca_person_info_data';
        $tables[$table] = array(
            'columns' => array(
                'data_id'       => $table.'.data_id'
               ,'customer_id'   => $table.'.customer_id'
               ,'ag_id'         => $table.'.ag_id'
               ,'data_value'    => $table.'.data_value'
             )
           ,'types'   => array(
                'data_id'       => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'customer_id'   => DBQUERY_FIELD_TYPE_INT .' NOT NULL'
               ,'ag_id'         => DBQUERY_FIELD_TYPE_INT .' NOT NULL'
               ,'data_value'    => DBQUERY_FIELD_TYPE_CHAR255.' NOT NULL'
             )
           ,'primary' => array(
                'data_id'
             )
           ,'indexes' => array(
                'UNIQUE KEY UNQ_cid_agid' => 'customer_id,ag_id'
             )
        );

        $table = 'ca_activation_keys';
        $tables[$table] = array(
            'columns' => array(
                'key_id'         => $table.'.key_id'
               ,'key_value'      => $table.'.key_value'
               ,'customer_account' => $table.'.customer_account'
             )
           ,'types'   => array(
                'key_id'         => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'key_value'      => DBQUERY_FIELD_TYPE_CHAR32.' NOT NULL'
               ,'customer_account' => DBQUERY_FIELD_TYPE_CHAR255.' NOT NULL'
             )
           ,'primary' => array(
                'key_id'
             )
           ,'indexes' => array(
                'UNIQUE KEY UNQ_account' => 'customer_account'
             )
        );

        $table = 'ca_settings';
        $tables[$table] = array(
            'columns' => array(
                'setting_id'    => $table.'.setting_id'
               ,'setting_key'   => $table.'.setting_key'
               ,'setting_value' => $table.'.setting_value'
             )
           ,'types'   => array(
                'setting_id'    => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'setting_key'   => DBQUERY_FIELD_TYPE_CHAR255.' NOT NULL'
               ,'setting_value' => DBQUERY_FIELD_TYPE_CHAR255.' NOT NULL'
             )
           ,'primary' => array(
                'setting_id'
             )
        );

        global $application;
        return $application->addTablePrefix($tables);
    }

    function getSettings()
    {
        global $application;
        $tables=$this->getTables();

        $query = &new DB_Select();
        $query->addSelectTable('ca_settings');
        $query->addSelectField('*');
        $res=$application->db->getDB_Result($query);

        $settings=array();

        foreach($res as $k => $sval)
            $settings[$sval['setting_key']]=$sval['setting_value'];

        return $settings;
    }

    function updateSettings($settings)
    {
        global $application;
        $tables=$this->getTables();
        $stable=$tables['ca_settings']['columns'];

        foreach($settings as $skey => $sval)
        {
            $query = &new DB_Update('ca_settings');
            $query->addUpdateValue($stable['setting_value'],$sval);
            $query->WhereValue($stable['setting_key'], DB_EQ, $skey);
            $application->db->PrepareSQL($query);
            $application->db->DB_Exec();
        };

        return;
    }

    function addAccount($account, $password)
    {
        global $application;
        $tables = $this->getTables();
        $customers_table = $tables['ca_customers']['columns'];

        $query = &new DB_Insert('ca_customers');
        $query->addInsertValue($account, $customers_table['customer_account']);
        $query->addInsertValue(md5($password), $customers_table['customer_password']);
        $application->db->PrepareSQL($query);
        return $application->db->DB_Exec();
    }

    function setAccountStatus($account, $status)
    {
        if(!in_array($status, array('A','N','B','R')))
            return false;

        global $application;
        $tables = $this->getTables();
        $customers_table = $tables['ca_customers']['columns'];

        $query = &new DB_Select();
        $query->addSelectTable('ca_customers');
        $query->addSelectField($customers_table['customer_status'], 'status');
        $query->WhereValue($customers_table['customer_account'], DB_EQ, $account);
        $res = $application->db->getDB_Result($query);

        if(count($res) != 1)
        {
            return false;
        }
        else
        {
            $old_status = $res[0]['status'];
        };

        $query = &new DB_Update('ca_customers');
        $query->addUpdateValue($customers_table['customer_status'], $status);
        $query->WhereValue($customers_table['customer_account'], DB_EQ, $account);
        $application->db->PrepareSQL($query);
        $application->db->DB_Exec();

        return true;
    }

    function getAccountStatus($account)
    {
        global $application;
        $tables =  $this->getTables();
        $customers_table = $tables['ca_customers']['columns'];

        $query = &new DB_Select();
        $query->addSelectTable('ca_customers');
        $query->addSelectField($customers_table['customer_status']);
        $query->WhereValue($customers_table['customer_account'], DB_EQ, $account);

        $res = $application->db->getDB_Result($query);
        if(count($res) != 1)
        {
            return null;
        };

        return $res[0]['customer_status'];
    }

    function doesAccountExists($account)
    {
        global $application;
        $tables =  $this->getTables();
        $customers_table = $tables['ca_customers']['columns'];

        $query = &new DB_Select();
        $query->addSelectTable('ca_customers');
        $query->addSelectField($customers_table['customer_status']);
        $query->WhereValue($customers_table['customer_account'], DB_EQ, $account);

        $res = $application->db->getDB_Result($query);
        return (count($res) == 1);
    }

    function registerAccount($account, $password, $register_info = null)
    {
        global $application;

        if($this->addAccount($account, $password) and $this->replaceActivationKey($account))
        {
            if($register_info != null)
            {
                $settings = $this->getSettings();
                $obj = &$application->getInstance('CCustomerInfo',$account);

                $target_groups = $obj->getPersonInfoGroupsNames();

                foreach($register_info as $attr_name => $attr_value)
                {
                    reset($target_groups);
                    foreach($target_groups as $group_name)
                    {
                        $obj->setPersonInfo($attr_name, $attr_value, $group_name);
                    };
                };

                if($settings['ACCOUNT_ACTIVATION_SCHEME'] == ACCOUNT_ACTIVATION_SCHEME_NONE)
                {
                    $obj->setPersonInfo('Status','A');
                };
            };

            return true;
        };

        return false;
    }

    function replaceActivationKey($account)
    {
        global $application;
        $tables = $this->getTables();
        $keys_table = $tables['ca_activation_keys']['columns'];

        $query = &new DB_Replace('ca_activation_keys');
        $query->addReplaceValue($this->__genActivationKey(), $keys_table['key_value']);
        $query->addReplaceValue($account, $keys_table['customer_account']);

        $application->db->PrepareSQL($query);
        return $application->db->DB_Exec();
    }

    function getActivationKey($account)
    {
        global $application;
        $tables = $this->getTables();
        $keys_table = $tables['ca_activation_keys']['columns'];

        $query = &new DB_Select();
        $query->addSelectTable('ca_activation_keys');
        $query->addSelectField($keys_table['key_value']);
        $query->WhereValue($keys_table['customer_account'], DB_EQ, $account);

        $res = $application->db->getDB_Result($query);

        if(count($res) != 1)
        {
            return null;
        }
        else
        {
            return $res[0]['key_value'];
        };
    }

    function getAccountByActivationKey($key)
    {
        if(!is_string($key))
        {
            return null;
        };

        global $application;
        $tables = $this->getTables();
        $keys_table = $tables['ca_activation_keys']['columns'];

        $query = &new DB_Select();
        $query->addSelectTable('ca_activation_keys');
        $query->addSelectField($keys_table['customer_account']);
        $query->WhereValue($keys_table['key_value'], DB_EQ, $key);

        $res = $application->db->getDB_Result($query);

        if(count($res) != 1)
        {
            return null;
        }
        else
        {
            return $res[0]['customer_account'];
        };
    }

    function dropActivationKey($val, $type='key_value')
    {
        if(!in_array($type, array('key_value','customer_account')) or !is_string($val))
        {
            return false;
        };

        global $application;
        $tables = $this->getTables();
        $keys_table = $tables['ca_activation_keys']['columns'];

        $query = &new DB_Delete('ca_activation_keys');
        $query->WhereValue($keys_table[$type], DB_EQ, $val);

        $application->db->PrepareSQL($query);

        return $application->db->DB_Exec();
    }

    function isCorrectAccountAndPasswd($account, $passwd)
    {
        if(!is_string($account) or !is_string($passwd))
        {
            return false;
        };

        global $application;
        $tables = $this->getTables();
        $customers_table = $tables['ca_customers']['columns'];

        $query = &new DB_Select();
        $query->addSelectTable('ca_customers');
        $query->addSelectField($customers_table['customer_id']);
        $query->WhereValue($customers_table['customer_account'], DB_EQ, $account);
        $query->WhereAND();
        $query->WhereValue($customers_table['customer_password'], DB_EQ, md5($passwd));

        $res = $application->db->getDB_Result($query);

        return (count($res) == 1);
    }

    function getCurrentSignedCustomer()
    {
        if(modApiFunc('Session','is_set','SignedCustomer'))
        {
            $signed_customer = modApiFunc('Session','get','SignedCustomer');
            if($this->doesAccountExists($signed_customer))
            {
                return $signed_customer;
            }
            else
            {
                return null;
            };
        }
        else
        {
            return null;
        };
    }

    function detectCOAttrID($tag)
    {
        if(empty($this->_co_attrs_ids))
        {
            $this->__loadCOAttrsIDs();
        };

        if(isset($this->_co_attrs_ids[strtolower($tag)]))
        {
            return $this->_co_attrs_ids[strtolower($tag)];
        }
        else
        {
            return null;
        };
    }

    function getPersonInfoAttrNameByCOAttrID($co_attr_id)
    {
        if(empty($this->_co_attrs_ids))
        {
            $this->__loadCOAttrsIDs();
        };

        return array_search($co_attr_id, $this->_co_attrs_ids);
    }

    function detectCOPITypeID($tag)
    {
        if(empty($this->_co_pi_types_ids))
        {
            $this->__loadCOPITypesIDs();
        };

        if(isset($this->_co_pi_types_ids[strtolower($tag)]))
        {
            return $this->_co_pi_types_ids[strtolower($tag)];
        }
        else
        {
            return null;
        };
    }

    function getPersonInfoGroupsList()
    {
        global $application;
        $tables = $this->getTables();
        $pig_table = $tables['ca_person_info_groups']['columns'];

        $query = &new DB_Select();
        $query->addSelectTable('ca_person_info_groups');
        $query->addSelectField('*');
        $query->SelectOrder($pig_table['sort_order']);
        $res = $application->db->getDB_Result($query);

        return $res;
    }

    function getPersonInfoGroupInfoByName($group_name)
    {
        global $application;
        $tables = $this->getTables();
        $pig_table = $tables['ca_person_info_groups']['columns'];

        $query = &new DB_Select();
        $query->addSelectTable('ca_person_info_groups');
        $query->addSelectField('*');
        $query->WhereValue($pig_table['group_name'], DB_EQ, $group_name);
        $res = $application->db->getDB_Result($query);

        if(count($res) == 1)
        {
            return $res[0];
        }
        else
        {
            return null;
        };
    }

    function getPersonInfoGroupNameByID($group_id)
    {
        global $application;
        $tables = $this->getTables();
        $pig_table = $tables['ca_person_info_groups']['columns'];

        $query = &new DB_Select();
        $query->addSelectTable('ca_person_info_groups');
        $query->addSelectField($pig_table['group_name'], 'group_name');
        $query->WhereValue($pig_table['group_id'], DB_EQ, $group_id);
        $res = $application->db->getDB_Result($query);

        if(count($res) == 1)
        {
            return $res[0]['group_name'];
        }
        else
        {
            return null;
        };
    }

    function getPersonInfoGroupAttrs($group_id, $attr_vis = PERSON_INFO_GROUP_ATTR_ALL)
    {
        global $application;
        $tables = $this->getTables();
        $pia_table = $tables['ca_person_info_attrs']['columns'];
        $atg_table = $tables['ca_attrs_to_groups']['columns'];

        $query = &new DB_Select();
        $query->addSelectTable('ca_person_info_attrs');
        $query->addSelectTable('ca_attrs_to_groups');
        $query->addSelectField($pia_table['attr_id'], 'attr_id');
        $query->addSelectField($pia_table['attr_name'], 'attr_name');
        $query->addSelectField($pia_table['lang_code'], 'lang_code');
        $query->addSelectField($atg_table['visible_name'], 'visible_name');
        $query->addSelectField($atg_table['is_visible'], 'is_visible');
        $query->addSelectField($atg_table['is_required'], 'is_required');
        $query->Where($atg_table['group_id'], DB_EQ, $group_id);
        $query->WhereAND();
        $query->Where($pia_table['attr_id'], DB_EQ, $atg_table['attr_id']);

        if($attr_vis == PERSON_INFO_GROUP_ATTR_VISIBLE)
        {
            $query->WhereAND();
            $query->WhereValue($atg_table['is_visible'], DB_EQ, 'Y');
        }
        elseif($attr_vis == PERSON_INFO_GROUP_ATTR_HIDDEN)
        {
            $query->WhereAND();
            $query->WhereValue($atg_table['is_visible'], DB_EQ, 'N');
        };

        $query->SelectOrder($atg_table['sort_order']);

        $res = $application->db->getDB_Result($query);

        $group_name = $this->getPersonInfoGroupNameByID($group_id);

        if($group_name == 'Customer')
        {
            $attrs = $res;
            return $attrs;
        };

        foreach($res as $k => $attr_info)
        {
            if($attr_vis == PERSON_INFO_GROUP_ATTR_ALL)
            {
                $attrs[] = $attr_info;
            };
            if($attr_vis == PERSON_INFO_GROUP_ATTR_VISIBLE)
            {
                if($this->__isCOAttrVisible($this->detectCOPITypeID($group_name), $this->detectCOAttrID($attr_info['attr_name'])))
                {
                    $attrs[] = $attr_info;
                };
            };
            if($attr_vis == PERSON_INFO_GROUP_ATTR_HIDDEN)
            {
                if($this->__isCOAttrHidden($this->detectCOPITypeID($group_name), $this->detectCOAttrID($attr_info['attr_name'])))
                {
                    $attrs[] = $attr_info;
                };
            };
        };

        return $attrs;
    }

    function getPersionInfoAttrNameByLowcaseName($lowcase_name)
    {
        if(isset($this->_lowcase_names[$lowcase_name]))
        {
            return $this->_lowcase_names[$lowcase_name];
        };

        global $application;
        $tables = $this->getTables();
        $attrs_table = $tables['ca_person_info_attrs']['columns'];

        $query = &new DB_Select();
        $query->addSelectTable('ca_person_info_attrs');
        $query->addSelectField($attrs_table['attr_name'], 'attr_name');
        $query->WhereValue("LOWER(attr_name)", DB_EQ, $lowcase_name);

        $res = $application->db->getDB_Result($query);

        if(count($res) != 1)
        {
            return null;
        }
        else
        {
            $this->_lowcase_names[$lowcase_name] = $res[0]['attr_name'];
            return $res[0]['attr_name'];
        };
    }

    function updateGroupAttrsInfo($group_name, $attrs)
    {
        global $application;
        $tables = $this->getTables();
        $atg_table = $tables['ca_attrs_to_groups']['columns'];

        $group_info = $this->getPersonInfoGroupInfoByName($group_name);
        $group_attrs = $this->getPersonInfoGroupAttrs($group_info['group_id']);

        foreach($group_attrs as $group_attr_info)
        {
            if(array_key_exists($group_attr_info['attr_name'], $attrs))
            {
                $a_info = $attrs[$group_attr_info['attr_name']];

                if(array_diff(array_keys($a_info), array_keys($atg_table)) == array_keys($a_info))
                    continue;

                $query = &new DB_Update('ca_attrs_to_groups');
                foreach($a_info as $prop => $val)
                {
                    $query->addUpdateValue($atg_table[$prop], $val);
                };
                $query->WhereValue($atg_table['group_id'], DB_EQ, $group_info['group_id']);
                $query->WhereAND();
                $query->WhereValue($atg_table['attr_id'], DB_EQ, $group_attr_info['attr_id']);

                $application->db->PrepareSQL($query);
                $application->db->DB_Exec();
            };
        };

        return;
    }

    function updateGroupAttrsSortOrder($group_name, $attrs_sort_order)
    {
        global $application;
        $tables = $this->getTables();
        $atg_table = $tables['ca_attrs_to_groups']['columns'];

        $group_info = $this->getPersonInfoGroupInfoByName($group_name);

        for($i=0; $i<count($attrs_sort_order); $i++)
        {
            $query = &new DB_Update('ca_attrs_to_groups');
            $query->addUpdateValue($atg_table['sort_order'], $i);
            $query->WhereValue($atg_table['group_id'], DB_EQ, $group_info['group_id']);
            $query->WhereAND();
            $query->WhereValue($atg_table['attr_id'], DB_EQ, $attrs_sort_order[$i]);

            $application->db->PrepareSQL($query);
            $application->db->DB_Exec();
        };

        return;
    }

    function isPersionInfoGroupActive($group_name)
    {
        if($group_name == 'Customer')
        {
            return true;
        };

        global $application;
        $co_tables = modApiStaticFunc('Checkout','getTables');
        $pit_table = $co_tables['person_info_types']['columns'];

        $query = &new DB_Select();
        $query->addSelectTable('person_info_types');
        $query->addSelectField($pit_table['active'], 'active');
        $query->WhereValue($pit_table['tag'], DB_EQ, strtolower($group_name).'Info');
        $res = $application->db->getDB_Result($query);

        if(count($res) != 1)
        {
            return false;
        }
        else
        {
            return ($res[0]['active'] == 'true');
        };
    }

    function OnOrderCreated($order_id)
    {
        global $application;

        $account = $this->getCurrentSignedCustomer();

        if($account != null)
        {
            $obj = &$application->getInstance('CCustomerInfo',$account);
        }
        else
        {
            $groups = array('billinginfo','shippinginfo');
            $email = null;
            reset($groups);
            foreach($groups as $group_name)
            {
                $em = modApiFunc('Checkout','getCustomerAttributeValue',$order_id,$group_name,'default','Email');
                if($em != '')
                {
                    $email = $em;
                    break;
                };
            };

            $settings = $this->getSettings();

            if($settings['CHECKOUT_TYPE'] == CHECKOUT_TYPE_QUICK or ($settings['CHECKOUT_TYPE'] == CHECKOUT_TYPE_AUTOACCOUNT and $settings['AUTO_CREATE_ACCOUNT'] == 'N'))
            {
                $account_name = (($email != null) ? $email : PSEUDO_NA_CUSTOMER_PERFIX.$order_id) . PSEUDO_CUSTOMER_SUFFIX;
                if($this->doesAccountExists($account_name))
                {
                    $obj = &$application->getInstance('CCustomerInfo',$account_name);
                }
                else
                {
                    $this->addAccount($account_name, time());
                    $this->setAccountStatus($account_name, 'B');
                    $obj = &$application->getInstance('CCustomerInfo',$account_name);
                };

                $obj->loadPersonInfoFromCheckout();
                if($email != null)
                    $obj->setPersonInfo('Email',$email,'Customer');

                if($this->isPersionInfoGroupActive('Billing'))
                    $obj->copyPersonInfo('Billing','Customer',array('Email'));
                elseif($this->isPersionInfoGroupActive('Shipping'))
                    $obj->copyPersonInfo('Shipping','Customer',array('Email'));
            }
            else
            {
                $account_name = $this->__genAutoAccountName();
                $this->addAccount($account_name, time());
                $this->setAccountStatus($account_name, 'R');
                $this->replaceActivationKey($account_name);

                $obj = &$application->getInstance('CCustomerInfo',$account_name);
                $obj->setPersonInfo('AccountName',$account_name,'Customer');
                $obj->loadPersonInfoFromCheckout();
                if($email != null)
                    $obj->setPersonInfo('Email',$email,'Customer');

                if($this->isPersionInfoGroupActive('Billing'))
                    $obj->copyPersonInfo('Billing','Customer',array('Email'));
                elseif($this->isPersionInfoGroupActive('Shipping'))
                    $obj->copyPersonInfo('Shipping','Customer',array('Email'));

                modApiFunc('EventsManager','throwEvent','AccountWasAutoCreated',$account_name);
            }
        };

        $ca_tables = modApiStaticFunc('Checkout','getTables');
        $orders_table = $ca_tables['orders']['columns'];

        $query = &new DB_Update('orders');
        $query->addUpdateValue($orders_table['person_id'], $obj->getPersonInfo('ID'));
        $query->WhereValue($orders_table['id'], DB_EQ, $order_id);

        $application->db->PrepareSQL($query);
        $application->db->DB_Exec();

        return;
    }

    function OnCheckoutPersonInfoFieldUpdated($field_info)
    {
        $co_variant_id = $field_info['variant_id'];
        $co_attr_id = $field_info['attribute_id'];
        $visible_name = $field_info['name'];
        $is_visible = $field_info['visible'] == 1 ? 'Y' : 'N';
        $is_required = $field_info['required'] == 1 ? 'Y' : 'N';

        if($is_required == 'Y')
            $is_visible = 'Y';

        $varinat_info = modApiFunc('Checkout','getPersonVariantInfo',$co_variant_id);

        if($varinat_info == false)
            return;

        $type_id = $varinat_info['type_id'];

        if(empty($this->_co_pi_types_ids))
        {
            $this->__loadCOPITypesIDs();
        };

        $group_name = array_search($type_id, $this->_co_pi_types_ids);

        if($group_name === false)
            return;

        $group_name = ucfirst(strtolower($group_name));

        if($this->getPersonInfoGroupInfoByName($group_name) == null)
            return;

        if(empty($this->_co_attrs_ids))
        {
            $this->__loadCOAttrsIDs();
        };

        $attr_name = array_search($co_attr_id, $this->_co_attrs_ids);

        if($attr_name === false)
            return;

        $attr_name = $this->getPersionInfoAttrNameByLowcaseName(strtolower($attr_name));

        $arr_for_update = array(
            $attr_name => array(
                    'visible_name' => $visible_name
                   ,'is_visible' => $is_visible
                   ,'is_required' => $is_required
                )
        );

        $this->updateGroupAttrsInfo($group_name, $arr_for_update);
    }

    function setCustomersSearchFilter($filter)
    {
        if($this->_customers_search_filter !== $filter)
        {
            $this->_customers_search_filter = $filter;
            $this->_customers = null;
        };
    }

    function getSearchCustomersResult()
    {
        if($this->_customers == null)
        {
            $this->__searchCustomers();
        };

        return $this->_customers;
    }

    function getCustomerAccountNameByCustomerID($customer_id)
    {
        global $application;
        $tables = $this->getTables();
        $customers_table = $tables['ca_customers']['columns'];

        $query = &new DB_Select();
        $query->addSelectTable('ca_customers');
        $query->addSelectField($customers_table['customer_account'], 'customer_account');
        $query->WhereValue($customers_table['customer_id'], DB_EQ, $customer_id);

        $res = $application->db->getDB_Result($query);

        if(count($res) != 1)
        {
            return null;
        }
        else
        {
            return $res[0]['customer_account'];
        };
    }

    function deleteCustomers($customers_ids)
    {
        if(!is_array($customers_ids) or empty($customers_ids))
        {
            return false;
        };

        global $application;
        $ca_tables = $this->getTables();
        $co_tables = modApiStaticFunc('Checkout','getTables');

        $query = &new DB_Select();
        $query->addSelectTable('orders');
        $query->addSelectField($co_tables['orders']['columns']['id'], 'order_id');
        $query->Where($co_tables['orders']['columns']['person_id'], DB_IN, "(".implode(", ",$customers_ids).")");
        $res = $application->db->getDB_Result($query);

        $orders_ids = array();
        for($i=0; $i<count($res); $i++)
        {
            $orders_ids[] = $res[$i]['order_id'];
        };

        if(!empty($orders_ids))
        {
            modApiFunc('Checkout','DeleteOrders',$orders_ids);
        };

        $accounts = array();

        $query = &new DB_Select();
        $query->addSelectField($ca_tables['ca_customers']['columns']['customer_account'], 'customer_account');
        $query->addSelectTable('ca_customers');
        $query->Where($ca_tables['ca_customers']['columns']['customer_id'], DB_IN, "(".implode(", ",$customers_ids).")");
        $res = $application->db->getDB_Result($query);

        for($i=0; $i<count($res); $i++)
        {
            $accounts[] = $res[$i]['customer_account'];
        };

        if(!empty($accounts))
        {
            $query = &new DB_Delete('ca_activation_keys');
            $query->Where($ca_tables['ca_activation_keys']['columns']['customer_account'], DB_IN, "('".implode("','",$accounts)."')");
            $application->db->PrepareSQL($query);
            $application->db->DB_Exec();
        };

        $query = &new DB_Delete('ca_person_info_data');
        $query->Where($ca_tables['ca_person_info_data']['columns']['customer_id'], DB_IN, "(".implode(", ",$customers_ids).")");
        $application->db->PrepareSQL($query);
        $application->db->DB_Exec();

        $query = &new DB_Delete('ca_customers');
        $query->Where($ca_tables['ca_customers']['columns']['customer_id'], DB_IN, "(".implode(", ",$customers_ids).")");
        $application->db->PrepareSQL($query);
        $application->db->DB_Exec();

        return true;
    }

    function dropCustomerPassword($var, $var_type='customer_account')
    {
        if(!in_array($var_type, array('customer_account','customer_id')))
        {
            return false;
        };

        global $application;
        $tables = $this->getTables();
        $ca_table = $tables['ca_customers']['columns'];

        $query = &new DB_Select();
        $query->addSelectTable('ca_customers');
        $query->addSelectField($ca_table['customer_id'],'customer_id');
        $query->addSelectField($ca_table['customer_account'],'customer_account');
        $query->WhereValue($ca_table[$var_type], DB_EQ, $var);
        $res = $application->db->getDB_Result($query);

        if(count($res) != 1)
        {
            return false;
        }
        else
        {
            $customer_id = $res[0]['customer_id'];
            $customer_account = $res[0]['customer_account'];
        };

        $query = &new DB_Update('ca_customers');
        $query->addUpdateValue($ca_table['customer_password'], $this->__genPseudoPasswd());
        $query->addUpdateValue($ca_table['customer_status'], 'R');
        $query->WhereValue($ca_table['customer_id'], DB_EQ, $customer_id);
        $application->db->PrepareSQL($query);

        return ($application->db->DB_Exec() and $this->replaceActivationKey($customer_account));
    }

    function getAllCustomerEmails()
    {
        global $application;
        $tables = $this->getTables();
        $attrs_table = $tables['ca_person_info_attrs']['columns'];
        $atg_table = $tables['ca_attrs_to_groups']['columns'];
        $data_table = $tables['ca_person_info_data']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($data_table['data_value'], 'value');
        $query->addSelectTable('ca_person_info_attrs');
        $query->addSelectTable('ca_attrs_to_groups');
        $query->addSelectTable('ca_person_info_data');
        $query->WhereValue($attrs_table['attr_name'], DB_EQ, 'Email');
        $query->WhereAND();
        $query->Where($atg_table['attr_id'], DB_EQ, $attrs_table['attr_id']);
        $query->WhereAND();
        $query->Where($data_table['ag_id'], DB_EQ, $atg_table['ag_id']);
        $query->WhereAND();
        $query->WhereValue($data_table['data_value'], DB_NEQ, '');
        $query->SelectGroup('value');

        $res = $application->db->getDB_Result($query);
        $emails = array();
        for($i=0; $i<count($res); $i++)
        {
            $emails[] = $res[$i]['value'];
        };

        return $emails;
    }

    function __genActivationKey()
    {
        return strrev(md5(mt_rand(0,1000).time().uniqid(mt_rand(435,971))));
    }

    function __genPseudoPasswd()
    {
        return strrev(md5(mt_rand(271,3141).time().uniqid(mt_rand(314,2718))));
    }

    function __genAutoAccountName()
    {
        do{
            $account_name = substr(preg_replace('/[^0-9]/i','',md5(microtime_float())),0,AUTOACCOUNT_LENGTH);
        }while($this->doesAccountExists($account_name));

        return $account_name;
    }

    function __loadCOAttrsIDs()
    {
        global $application;
        loadClass('Checkout');
        $co_tables = Checkout::getTables();
        $pa_table = $co_tables['person_attributes']['columns'];

        $query = &new DB_Select();
        $query->addSelectTable('person_attributes');
        $query->addSelectField($pa_table['id'], 'attr_id');
        $query->addSelectField($pa_table['tag'], 'attr_tag');

        $res = $application->db->getDB_Result($query);

        foreach($res as $k => $pa_info)
        {
            if(strtolower($pa_info['attr_tag']) == 'postcode')
                $pa_info['attr_tag'] = 'zipcode';

            $this->_co_attrs_ids[strtolower($pa_info['attr_tag'])] = $pa_info['attr_id'];
        };
    }

    function __loadCOPITypesIDs()
    {
        global $application;
        loadClass('Checkout');
        $co_tables = Checkout::getTables();
        $pa_table = $co_tables['person_info_types']['columns'];

        $query = &new DB_Select();
        $query->addSelectTable('person_info_types');
        $query->addSelectField($pa_table['id'], 'type_id');
        $query->addSelectField($pa_table['tag'], 'type_tag');

        $res = $application->db->getDB_Result($query);

        foreach($res as $k => $pa_info)
        {
            $this->_co_pi_types_ids[strtolower(str_replace('Info','',$pa_info['type_tag']))] = $pa_info['type_id'];
        };
    }

    function __isCOAttrVisible($type_id, $attr_id)
    {
        global $application;
        loadClass('Checkout');
        $co_tables = Checkout::getTables();
        $piv_table = $co_tables['person_info_variants']['columns'];
        $piva_table = $co_tables['person_info_variants_to_attributes']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($piva_table['visible'], 'visible');
        $query->addSelectTable('person_info_variants');
        $query->addSelectTable('person_info_variants_to_attributes');
        $query->Where($piv_table['type_id'], DB_EQ, $type_id);
        $query->WhereAND();
        $query->Where($piva_table['variant_id'], DB_EQ, $piv_table['id']);
        $query->WhereAND();
        $query->Where($piva_table['attribute_id'], DB_EQ, $attr_id);

        $res = $application->db->getDB_Result($query);

        if(count($res) == 1)
        {
            return ($res[0]['visible'] == 1);
        }
        else
        {
            return false;
        };
    }

    function __isCOAttrHidden($type_id, $attr_id)
    {
        global $application;
        loadClass('Checkout');
        $co_tables = Checkout::getTables();
        $piv_table = $co_tables['person_info_variants']['columns'];
        $piva_table = $co_tables['person_info_variants_to_attributes']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($piva_table['visible'], 'visible');
        $query->addSelectTable('person_info_variants');
        $query->addSelectTable('person_info_variants_to_attributes');
        $query->Where($piv_table['type_id'], DB_EQ, $type_id);
        $query->WhereAND();
        $query->Where($piva_table['variant_id'], DB_EQ, $piv_table['id']);
        $query->WhereAND();
        $query->Where($piva_table['attribute_id'], DB_EQ, $attr_id);

        $res = $application->db->getDB_Result($query);

        if(count($res) == 1)
        {
            return ($res[0]['visible'] == 0);
        }
        else
        {
            return true;
        };
    }

    function __searchCustomers()
    {
        if($this->_customers_search_filter === null)
        {
            $this->_customers = array();
            return;
        };

        global $application;
        $tables = $this->getTables();

        $customers_table = $tables['ca_customers']['columns'];

        $query = &new DB_Select();
        $query->addSelectTable('ca_customers');
        $query->addSelectField($customers_table['customer_id'], 'customer_id');
        $query->addSelectField($customers_table['customer_account'], 'customer_account');
        $query->addSelectField($customers_table['customer_status'], 'customer_status');

        //                                        
        //                          (ASC-1201)
        if($this->_customers_search_filter['type'] == 'letter' and $this->_customers_search_filter['search_string'] != '')
        {
            if($this->_customers_search_filter['letter_filter_by'] == 'customer_name')
            {
                $attrs_table = $tables['ca_person_info_attrs']['columns'];
                $atg_table = $tables['ca_attrs_to_groups']['columns'];
                $pid_table = $tables['ca_person_info_data']['columns'];

                $query->addSelectTable('ca_person_info_attrs');
                $query->addSelectTable('ca_attrs_to_groups');
                $query->addSelectTable('ca_person_info_data');

                $query->addWhereOpenSection();
                    $query->WhereValue($attrs_table['attr_name'], DB_EQ, 'LastName');
                    $query->WhereAND();
                    $query->Where($atg_table['attr_id'], DB_EQ, $attrs_table['attr_id']);
                    $query->WhereAND();
                    $query->Where($pid_table['ag_id'], DB_EQ, $atg_table['ag_id']);
                    $query->WhereAND();
                    $query->WhereValue($pid_table['data_value'], DB_LIKE, $this->_customers_search_filter['search_string'].'%');
                    $query->WhereAND();
                    $query->Where($pid_table['customer_id'], DB_EQ, $customers_table['customer_id']);
                $query->addWhereCloseSection();
                $query->SelectGroup($customers_table['customer_id']);
            }
            else
            {
                $like_exp_a = $this->_customers_search_filter['search_string'].'%';
                $like_exp_b = $this->_customers_search_filter['search_string'].'%'.PSEUDO_CUSTOMER_SUFFIX;

                $query->WhereValue($customers_table['customer_account'], DB_LIKE, $like_exp_a);
                $query->WhereOR();
                $query->WhereValue($customers_table['customer_account'], DB_LIKE, $like_exp_b);
            };
        };

        //                              , ID
        //          ,            ,     Email,     Phone (ASC-1207)
        if($this->_customers_search_filter['type'] == 'custom')
        {
            if(preg_match("/^[0-9]+$/",$this->_customers_search_filter['search_string']))
            {
                $query->Where($customers_table['customer_id'], DB_EQ, intval($this->_customers_search_filter['search_string']));
                $query->WhereOR();
                $query->WhereValue($customers_table['customer_account'], DB_LIKE, '%'.$this->_customers_search_filter['search_string'].'%');
                $query->WhereOR();
                $query->WhereValue($customers_table['customer_account'], DB_LIKE, '%'.$this->_customers_search_filter['search_string'].'%'.PSEUDO_CUSTOMER_SUFFIX);
            }
            else
            {
                $attrs_table = $tables['ca_person_info_attrs']['columns'];
                $atg_table = $tables['ca_attrs_to_groups']['columns'];
                $pid_table = $tables['ca_person_info_data']['columns'];

                $attrs_for_search = array('FirstName','LastName','Email','Phone');

                $query->addSelectTable('ca_person_info_attrs');
                $query->addSelectTable('ca_attrs_to_groups');
                $query->addSelectTable('ca_person_info_data');

                $query->WhereValue($customers_table['customer_account'], DB_LIKE, '%'.$this->_customers_search_filter['search_string'].'%');
                $query->WhereOR();
                $query->WhereValue($customers_table['customer_account'], DB_LIKE, '%'.$this->_customers_search_filter['search_string'].'%'.PSEUDO_CUSTOMER_SUFFIX);
                $query->WhereOR();
                $query->addWhereOpenSection();
                    $query->addWhereOpenSection();
                    for($i=0; $i<count($attrs_for_search); $i++)
                    {
                        if($i != 0) $query->WhereOR();
                        $query->WhereValue($attrs_table['attr_name'], DB_EQ, $attrs_for_search[$i]);
                    };
                    $query->addWhereCloseSection();
                    $query->WhereAND();
                    $query->Where($atg_table['attr_id'], DB_EQ, $attrs_table['attr_id']);
                    $query->WhereAND();
                    $query->Where($pid_table['ag_id'], DB_EQ, $atg_table['ag_id']);
                    $query->WhereAND();
                    $query->WhereValue($pid_table['data_value'], DB_LIKE, '%'.$this->_customers_search_filter['search_string'].'%');
                    $query->WhereAND();
                    $query->Where($pid_table['customer_id'], DB_EQ, $customers_table['customer_id']);
                $query->addWhereCloseSection();
            };

            $query->SelectGroup($customers_table['customer_id']);
        };

        $query->SelectOrder($customers_table['customer_id'], 'ASC');

        $res01 = $application->db->getDB_Result($query);

        if($this->_customers_search_filter['use_paginator'])
        {
            $res01 = modApiFunc('Paginator','setQuery',$res01,count($res01));
        };

        $this->_customers = $res01;

        return;
    }



    var $_co_attrs_ids;
    var $_co_pi_types_ids;
    var $_lowcase_names;
    var $_customers_search_filter;
    var $_customers;
};

?>