<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2007, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

/**
 * Module EventsManager
 *
 * @package EventsManager
 * @author Alexey Florinsky
 */
class EventsManager
{
    function EventsManager()
    {
    }

    function install()
    {
        $tables = EventsManager::getTables();
        $query = &new DB_Table_Create($tables);
    }

    function getTables()
    {
        static $tables;

        if (is_array($tables))
        {
            return $tables;
        }

        $tables = array ();

        $tbl_events = 'events_manager';
        $tables[$tbl_events] = array();
        $tables[$tbl_events]['columns'] = array
            (
                'event_id'             => $tbl_events.'.event_id',
                'event_name'           => $tbl_events.'.event_name',
                'handler_class'        => $tbl_events.'.handler_class',
                'handler_method'       => $tbl_events.'.handler_method',
                'handler_include_path' => $tbl_events.'.handler_include_path',
                'handler_order'        => $tbl_events.'.handler_order',
            );
        $tables[$tbl_events]['types'] = array
            (
                'event_id'             => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment',
                'event_name'           => DBQUERY_FIELD_TYPE_CHAR255 .' NOT NULL',
                'handler_class'        => DBQUERY_FIELD_TYPE_CHAR255 .' NOT NULL',
                'handler_method'       => DBQUERY_FIELD_TYPE_CHAR255 .' NOT NULL',
                'handler_include_path' => DBQUERY_FIELD_TYPE_TEXT . ' default \'\'',
                'handler_order'        => DBQUERY_FIELD_TYPE_INT . ' default 100',
            );
        $tables[$tbl_events]['primary'] = array
            (
                'event_id'
            );
        $tables[$tbl_events]['indexes'] = array
            (
                'IDX_event_name' => 'event_name',
                'IDX_handler_order' => 'handler_order',
            );

        global $application;
        return $application->addTablePrefix($tables);
    }

    function uninstall()
    {
        $query = &new DB_Table_Delete(EventsManager::getTables());
        global $application;
        $application->db->getDB_Result($query);
    }

    function addEventHandler($event_name, $handler_class, $handler_method,  $include_path=null, $sort_order=null)
    {
        $tables = EventsManager::getTables();
        $tbl_events = $tables['events_manager']['columns'];

        $db_insert = new DB_Insert('events_manager');
        $db_insert->addInsertValue($event_name,     $tbl_events['event_name']);
        $db_insert->addInsertValue($handler_class,  $tbl_events['handler_class']);
        $db_insert->addInsertValue($handler_method, $tbl_events['handler_method']);

        if ($sort_order != null)
            $db_insert->addInsertValue($sort_order, $tbl_events['handler_order']);

        if ($include_path != null)
            $db_insert->addInsertValue($include_path,$tbl_events['handler_include_path']);

        global $application;
        $application->db->PrepareSQL($db_insert);
        $application->db->DB_Exec();
    }

    function removeEventHandler($event_name, $handler_class, $handler_method)
    {
        global $application;
        $tables = EventsManager::getTables();
        $tbl_events = $tables['events_manager']['columns'];

        $query = &new DB_Delete('events_manager');
        $query->WhereValue($tbl_events['event_name'], DB_EQ, $event_name);
        $query->WhereAND();
        $query->WhereValue($tbl_events['handler_class'], DB_EQ, $handler_class);
        $query->WhereAND();
        $query->WhereValue($tbl_events['handler_method'], DB_EQ, $handler_method);
        $application->db->getDB_Result($query);
    }

    function getEventHandlers($event_name)
    {
        global $application;
        $tables = EventsManager::getTables();
        $tbl_events = $tables['events_manager']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($tbl_events['event_id'], 'event_id');
        $query->addSelectField($tbl_events['handler_class'], 'handler_class');
        $query->addSelectField($tbl_events['handler_method'], 'handler_method');
        $query->addSelectField($tbl_events['handler_include_path'], 'handler_include_path');
        $query->WhereValue($tbl_events['event_name'], DB_EQ, $event_name);
        $query->SelectOrder($tbl_events['handler_order']);
        return $application->db->getDB_Result($query);
    }

    function executeEventHandler($handler, $param1=null , $param2=null)
    {
        if ( isset($handler['handler_class']       ) &&
             isset($handler['handler_method']      ) )
        {
            if ( $this->_tryInclude($handler['handler_class'], $handler['handler_include_path']) &&
                 is_callable($handler['handler_class'], $handler['handler_method']))
            {
                global $application;
                $obj = &$application->getInstance($handler['handler_class']);
                $method = $handler['handler_method'];
                if ($param1 != null && $param2 != null)
                    $res = $obj->$method($param1, $param2);
                elseif ($param1 != null)
                    $res = $obj->$method($param1);
                else
                    $res = $obj->$method();
                return $res;
            }
            else
            {
                return null;
            }
        }
        else
        {
            return null;
        }
    }

    function &prepareEventHandler($handler)
    {
        if ( isset($handler['handler_class']       ) &&
             isset($handler['handler_method']      ) )
        {
            if ( $this->_tryInclude($handler['handler_class'], $handler['handler_include_path']) &&
                 is_callable($handler['handler_class'], $handler['handler_method']))
            {
                global $application;
                $obj = &$application->getInstance($handler['handler_class']);
                return $obj;
            }
            else
            {
                return null;
            }
        }
        else
        {
            return null;
        }
    }


    function throwEvent($event_name, $param1=null , $param2=null)
    {
        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Starting, Event "'.$event_name.'"');

        $eHandlers = $this->getEventHandlers($event_name);
        foreach($eHandlers as $handler)
        {
            $this->executeEventHandler($handler, $param1, $param2);
            log_debug('['.__CLASS__.'::'.__FUNCTION__.'] Handler "'.$handler['handler_class'].'::'.$handler['handler_method'].'" Done');
        }

        log_debug('['.__CLASS__.'::'.__FUNCTION__.'] End');
    }

    function processEvent($event_name, $param1=null , $param2=null)
    {
        $eResults = array();
        $eHandlers = $this->getEventHandlers($event_name);
        foreach($eHandlers as $handler)
            $eResults[] = $this->executeEventHandler($handler, $param1, $param2);
        return $eResults;
    }

    function _tryInclude($handler_class, $handler_include_path)
    {
        if (class_exists($handler_class))
            return true;

        if (file_exists($handler_include_path) && is_file($handler_include_path) && is_readable($handler_include_path))
        {
            include_once($handler_include_path);
            if (class_exists($handler_class))
                return true;
        }

        global $application;
        $objMM = &$application->getInstance('Modules_Manager');
        $objMM->includeAPIFileOnce($handler_class);

        if (class_exists($handler_class))
            return true;
        else
            return false;
    }
}

?>