<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2007, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

/**
 * Localization module.
 *
 * @package Localization
 * @author Alexey Florinsky, Alexander Girin
 */
class Localization
{
//------------------------------------------------
//               PUBLIC DECLARATION
//------------------------------------------------

    /**#@+
     * @access public
     */

    /**
     * Localization constructor.
     */
    function Localization()
    {
        $this->getSettings();
    }

    /**
     * Reads out module settings from the database.
     *
     * @return
     */
    function getSettings()
    {
        global $application;

        $tables = $this->getTables();

        $s = $tables['localization_settings']['columns'];
        $query = &new DB_Select();
        $query->addSelectField($s["key"], "setting_key");
        $query->addSelectField($s["val"], "setting_val");
        $result = $application->db->getDB_Result($query);

        foreach ($result as $value)
        {
            $this->$value["setting_key"] = $value["setting_val"];
        }
    }

    /**
     * Saves setting values (date format, time format etc.) in the database.
     *
     * @return
     */
    function setValue($key, $val)
    {
        global $application;

        $tables = $this->getTables();

        $table = 'localization_settings';
        $columns = $tables[$table]['columns'];

        $query = &new DB_Update($table);
        $query->addUpdateValue($columns['val'], $val);
        $query->WhereValue($columns['key'], DB_EQ, $key);
        $application->db->getDB_Result($query);
    }

    /**
     * Sets the symbol for the current selected currency.
     *
     * @param integer $currency_id - the currency ID
     * param string - the currency symbol
     * @return
     */
    function setCurrencySign($currency_id, $sign)
    {
        global $application;

        $tables = $this->getTables();

        $table = 'currencies';
        $columns = $tables[$table]['columns'];

        $query = &new DB_Update($table);
        $query->addUpdateValue($columns['sign'], $sign);
        $query->WhereValue($columns['id'], DB_EQ, $currency_id);
        $application->db->getDB_Result($query);
    }

    /**
     * Gets the value of the Localization setting by the key:
     *      DATE_FORMAT - date format
     *      TIME_FORMAT - time format
     *      NUMBER_FORMAT - number format
     *      NEGATIVE_FORMAT - negative numbers format
     *      CURRENCY - current selected currency
     *      CURRENCY_FORMAT - currency amount format
     *      CURRENCY_POSITIVE_FORMAT - positive currency amount format
     *      CURRENCY_NEGATIVE_FORMAT - negative currency amount format
     *      WEIGHT_UNIT -  symbol for unit of weight
     *      WEIGHT_COEFF - coefficient of casting the custom unit of weight to 1 kg
     *      ITEM_UNIT - symbol for numeric parameter unit
     *
     * @param integer $currency_id - the currency ID
     * param string - the currency symbol
     * @return string - the setting value
     */
    function getValue($key)
    {
        return $this->$key;
    }

    /**
     * Gets the currency symbol.
     *
     * @return
     */
    function getCurrencySign()
    {
        $sign = explode("|", $this->CURRENCY);
        //patch to allow single unicode character as currency symbol
        $value = preg_match("/^\&\#[0-9]{1,10};$/i", $sign[1]) ? $sign[1] : prepareHTMLDisplay($sign[1]);
        return $value;
    }

    /**
     * Returns the current currency code.
     *
     */
    function getCurrencyCode()
    {
        global $application;

        $sign = explode("|", $this->CURRENCY);
        $currencyId = $sign[0]; // returns the selected currency id

        $tables = $this->getTables();
        $cols = $tables['currencies']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($cols['code'], 'code');
        $query->Where($cols['id'], DB_EQ, $currencyId);
        $dbResult = $application->db->getDB_Result($query);

        if (sizeof($dbResult) == 1)
        {
        	$value = $dbResult[0]['code'];
        }
        else
        {
            //
            // TODO: to output the error
            //
        }
        return $value;
   }

    /**
     * Installs the specified module in the system.
     *
     * The install() method is called statically.
     * To call other methods of this class from this method,
     * the static call is used, for example,
     * Configuration::getTables() instead of $this->getTables()
     */
    function install()
    {
        global $application;
        $obj = &$application->getInstance('MessageResources');

        $tables = Localization::getTables();
        $query = &new DB_Table_Create($tables);

        $table = 'localization_settings';
        $columns = $tables[$table]['columns'];

        $query = &new DB_Insert($table);
        $query->addInsertValue(1, $columns['id']);
        $query->addInsertValue("DATE_FORMAT", $columns['key']);
        $query->addInsertValue("d-m-Y", $columns['val']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(2, $columns['id']);
        $query->addInsertValue("TIME_FORMAT", $columns['key']);
        $query->addInsertValue("H:i:s", $columns['val']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(3, $columns['id']);
        $query->addInsertValue("NUMBER_FORMAT", $columns['key']);
        $query->addInsertValue("2|.|,|", $columns['val']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(4, $columns['id']);
        $query->addInsertValue("NEGATIVE_FORMAT", $columns['key']);
        $query->addInsertValue("- %s", $columns['val']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(5, $columns['id']);
        $query->addInsertValue("CURRENCY", $columns['key']);
        $query->addInsertValue("289|$", $columns['val']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(6, $columns['id']);
        $query->addInsertValue("CURRENCY_FORMAT", $columns['key']);
        $query->addInsertValue("2|.|,|", $columns['val']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(7, $columns['id']);
        $query->addInsertValue("CURRENCY_POSITIVE_FORMAT", $columns['key']);
        $query->addInsertValue("{s}{v}", $columns['val']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(8, $columns['id']);
        $query->addInsertValue("CURRENCY_NEGATIVE_FORMAT", $columns['key']);
        $query->addInsertValue("-{s}{v}", $columns['val']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(9, $columns['id']);
        $query->addInsertValue("WEIGHT_UNIT", $columns['key']);
        $query->addInsertValue($obj->getMessage('WEIGHT_UNIT'), $columns['val']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(10, $columns['id']);
        $query->addInsertValue("WEIGHT_COEFF", $columns['key']);
        $query->addInsertValue("0.45359", $columns['val']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(11, $columns['id']);
        $query->addInsertValue("ITEM_UNIT", $columns['key']);
        $query->addInsertValue($obj->getMessage('ITEM_UNIT'), $columns['val']);
        $application->db->getDB_Result($query);

        $table = 'date_time_formats';           #the name of the filled table
        $columns = $tables[$table]['columns'];  #the array of field names of the table

        $query = &new DB_Insert($table);
        $query->addInsertValue(1, $columns['id']);
        $query->addInsertValue("date", $columns['d_t']);
        $query->addInsertValue("d-m-Y", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(2, $columns['id']);
        $query->addInsertValue("date", $columns['d_t']);
        $query->addInsertValue("d/m/Y", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(3, $columns['id']);
        $query->addInsertValue("date", $columns['d_t']);
        $query->addInsertValue("d.m.Y", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(4, $columns['id']);
        $query->addInsertValue("date", $columns['d_t']);
        $query->addInsertValue("m-d-Y", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(5, $columns['id']);
        $query->addInsertValue("date", $columns['d_t']);
        $query->addInsertValue("m/d/Y", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(6, $columns['id']);
        $query->addInsertValue("date", $columns['d_t']);
        $query->addInsertValue("m.d.Y", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(7, $columns['id']);
        $query->addInsertValue("date", $columns['d_t']);
        $query->addInsertValue("Y-m-d", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(8, $columns['id']);
        $query->addInsertValue("date", $columns['d_t']);
        $query->addInsertValue("Y/m/d", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(9, $columns['id']);
        $query->addInsertValue("date", $columns['d_t']);
        $query->addInsertValue("Y.m.d", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(10, $columns['id']);
        $query->addInsertValue("date", $columns['d_t']);
        $query->addInsertValue("F d, Y", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(11, $columns['id']);
        $query->addInsertValue("date", $columns['d_t']);
        $query->addInsertValue("M d, Y", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(12, $columns['id']);
        $query->addInsertValue("time", $columns['d_t']);
        $query->addInsertValue("H:i:s", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(13, $columns['id']);
        $query->addInsertValue("time", $columns['d_t']);
        $query->addInsertValue("H:i", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(14, $columns['id']);
        $query->addInsertValue("time", $columns['d_t']);
        $query->addInsertValue("h:i:s A", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(15, $columns['id']);
        $query->addInsertValue("time", $columns['d_t']);
        $query->addInsertValue("h:i A", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(16, $columns['id']);
        $query->addInsertValue("time", $columns['d_t']);
        $query->addInsertValue("h:i:s a", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(17, $columns['id']);
        $query->addInsertValue("time", $columns['d_t']);
        $query->addInsertValue("h:i a", $columns['format']);
        $application->db->getDB_Result($query);

        $table = 'negative_formats';            #the name of the filled table
        $columns = $tables[$table]['columns'];  #the array of field names of the table

        $query = &new DB_Insert($table);
        $query->addInsertValue(1, $columns['id']);
        $query->addInsertValue("(%s)", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(2, $columns['id']);
        $query->addInsertValue("-%s", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(3, $columns['id']);
        $query->addInsertValue("- %s", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(4, $columns['id']);
        $query->addInsertValue("%s-", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(5, $columns['id']);
        $query->addInsertValue("%s -", $columns['format']);
        $application->db->getDB_Result($query);

        $table = 'currencies';           #the name of the filled table
        $columns = $tables[$table]['columns'];  #  the array of field names of the table

        for ($i=1; $i<=305; $i++)
        {
            $query = &new DB_Insert($table);
            $query->addInsertValue($i, $columns['id']);
            $query->addInsertValue($obj->getMessage(sprintf('CURRENCY_NAME_%03d',$i)), $columns['name']);
            $query->addInsertValue($obj->getMessage(sprintf('CURRENCY_CODE_%03d',$i)), $columns['code']);
            switch ($i)
            {
                case 4:
                case 289:
                    $query->addInsertValue("$", $columns['sign']);
                    break;
                default:
                    $query->addInsertValue($obj->getMessage(sprintf('CURRENCY_CODE_%03d',$i)), $columns['sign']);
                    break;
            }
            $application->db->getDB_Result($query);
        }

        $table = 'positive_currency_formats';   #the name of the filled table
        $columns = $tables[$table]['columns'];  #  the array of field names of the table

        $query = &new DB_Insert($table);
        $query->addInsertValue(1, $columns['id']);
        $query->addInsertValue("{s}{v}", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(2, $columns['id']);
        $query->addInsertValue("{s} {v}", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(3, $columns['id']);
        $query->addInsertValue("{v}{s}", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(4, $columns['id']);
        $query->addInsertValue("{v} {s}", $columns['format']);
        $application->db->getDB_Result($query);

        $table = 'negative_currency_formats';   #the name of the filled table
        $columns = $tables[$table]['columns'];  #the array of field names of the table

        $query = &new DB_Insert($table);
        $query->addInsertValue(1, $columns['id']);
        $query->addInsertValue("({s}{v})", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(2, $columns['id']);
        $query->addInsertValue("({s} {v})", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(3, $columns['id']);
        $query->addInsertValue("({v}{s})", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(4, $columns['id']);
        $query->addInsertValue("({v} {s})", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(5, $columns['id']);
        $query->addInsertValue("{s}-{v}", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(6, $columns['id']);
        $query->addInsertValue("{s} -{v}", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(7, $columns['id']);
        $query->addInsertValue("-{s}{v}", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(8, $columns['id']);
        $query->addInsertValue("- {s}{v}", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(9, $columns['id']);
        $query->addInsertValue("-{s} {v}", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(10, $columns['id']);
        $query->addInsertValue("- {s} {v}", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(11, $columns['id']);
        $query->addInsertValue("-{v}{s}", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(12, $columns['id']);
        $query->addInsertValue("- {v}{s}", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(13, $columns['id']);
        $query->addInsertValue("-{v} {s}", $columns['format']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(14, $columns['id']);
        $query->addInsertValue("- {v} {s}", $columns['format']);
        $application->db->getDB_Result($query);

        # Dump data for table 'patterns'
        $table = 'patterns';                    #     the name of the filled table
        $columns = $tables[$table]['columns'];  #     the array of field names of the table

        $query = &new DB_Insert($table);
        $query->addInsertValue(1, $columns['id']);
        $query->addInsertValue('Integer', $columns['name']);
        $query->addInsertValue('All integer numbers.', $columns['descr']);
        $query->addInsertValue('/^\d*$/', $columns['value']);
        $query->addInsertValue('integer', $columns['type']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(2, $columns['id']);
        $query->addInsertValue('Float', $columns['name']);
        $query->addInsertValue('A numbers with floating point.', $columns['descr']);
        $query->addInsertValue('/^\d*\.\d*$/', $columns['value']);
        $query->addInsertValue('float', $columns['type']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(3, $columns['id']);
        $query->addInsertValue('String', $columns['name']);
        $query->addInsertValue('Simple string without restrictions.', $columns['descr']);
        $query->addInsertValue('/^.*$/', $columns['value']);
        $query->addInsertValue('string', $columns['type']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(4, $columns['id']);
        $query->addInsertValue('Multiline string', $columns['name']);
        $query->addInsertValue('String with multiline support.', $columns['descr']);
        $query->addInsertValue('/^.*$/m', $columns['value']);
        $query->addInsertValue('mstring', $columns['type']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(5, $columns['id']);
        $query->addInsertValue('Currency', $columns['name']);
        $query->addInsertValue('Currency values.', $columns['descr']);
        $query->addInsertValue('/^\d*\.\d{2}$/', $columns['value']);
        $query->addInsertValue('currency', $columns['type']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(6, $columns['id']);
        $query->addInsertValue('String1024', $columns['name']);
        $query->addInsertValue('length(x) < 1024.', $columns['descr']);
        $query->addInsertValue('/^(.{0,1024})$/', $columns['value']);
        $query->addInsertValue('string1024', $columns['type']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(7, $columns['id']);
        $query->addInsertValue('String128', $columns['name']);
        $query->addInsertValue('length(x) < 128.', $columns['descr']);
        $query->addInsertValue('/^(.{0,128})$/', $columns['value']);
        $query->addInsertValue('string128', $columns['type']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(8, $columns['id']);
        $query->addInsertValue('String256', $columns['name']);
        $query->addInsertValue('length(x) < 256.', $columns['descr']);
        $query->addInsertValue('/^(.{0,256})$/', $columns['value']);
        $query->addInsertValue('string256', $columns['type']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(9, $columns['id']);
        $query->addInsertValue('String512', $columns['name']);
        $query->addInsertValue('length(x) < 512.', $columns['descr']);
        $query->addInsertValue('/^(.{0,512})$/', $columns['value']);
        $query->addInsertValue('string512', $columns['type']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(10, $columns['id']);
        $query->addInsertValue('Weight', $columns['name']);
        $query->addInsertValue('Weight values.', $columns['descr']);
        $query->addInsertValue('/^\d*\.?\d*$/', $columns['value']);
        $query->addInsertValue('weight', $columns['type']);
        $application->db->getDB_Result($query);

        $query = &new DB_Insert($table);
        $query->addInsertValue(11, $columns['id']);
        $query->addInsertValue('Items', $columns['name']);
        $query->addInsertValue('Items values.', $columns['descr']);
        $query->addInsertValue('/^-?\d*$/', $columns['value']);
        $query->addInsertValue('item', $columns['type']);
        $application->db->getDB_Result($query);
    }

    /**
     * Installs the specified module in the system.
     *
     * The uninstall() method is called statically.
     * To call other methods of this class from this method,
     * the static call is used, for example,
     * Configuration::getTables() instead of $this->getTables()
     */
    function uninstall()
    {
    }

    /**
     * Gets the array of meta description of module tables.
     *
     * The array structure of the meta description of the table:
     * <code>
     *      $tables = array ();
     *      $table_name = 'table_name';
     *      $tables[$table_name] = array();
     *      $tables[$table_name]['columns'] = array
     *      (
     *          'fn1'               => 'table_name.field_name_1'
     *         ,'fn2'               => 'table_name.field_name_2'
     *         ,'fn3'               => 'table_name.field_name_3'
     *         ,'fn4'               => 'table_name.field_name_4'
     *      );
     *      $tables[$table_name]['types'] = array
     *      (
     *          'fn1'               => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
     *         ,'fn2'               => DBQUERY_FIELD_TYPE_INT .' NOT NULL'
     *         ,'fn3'               => DBQUERY_FIELD_TYPE_CHAR255
     *         ,'fn4'               => DBQUERY_FIELD_TYPE_TEXT
     *      );
     *      $tables[$table_name]['primary'] = array
     *      (
     *          'fn1'       # several key fields may be used, e.g. - 'fn1', 'fn2'
     *      );
     *      $tables[$table_name]['indexes'] = array
     *      (
     *          'index_name1' => 'fn2'      # several fields can be used in one index, e.g. - 'fn2, fn3'
     *         ,'index_name2' => 'fn3'
     *      );
     * </code>
     *
     * @return array -  the meta description of module tables
     */
    function getTables()
    {
        static $tables;

        if (is_array($tables))
        {
            return $tables;
        }

        $tables = array ();

        $table = 'localization_settings';
        $tables[$table] = array();
        $tables[$table]['columns'] = array
            (
                'id'                => $table.'.localization_setting_id'
               ,'key'               => $table.'.setting_key'
               ,'val'               => $table.'.setting_value'
            );
        $tables[$table]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'key'               => DBQUERY_FIELD_TYPE_CHAR50
               ,'val'               => DBQUERY_FIELD_TYPE_CHAR50
            );
        $tables[$table]['primary'] = array
            (
                'id'
            );

        $table = 'date_time_formats';
        $tables[$table] = array();
        $tables[$table]['columns'] = array
            (
                'id'                => $table.'.date_time_format_id'
               ,'d_t'               => $table.'.date_time'
               ,'format'            => $table.'.format'
            );
        $tables[$table]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'d_t'               => DBQUERY_FIELD_TYPE_CHAR5
               ,'format'            => DBQUERY_FIELD_TYPE_CHAR20
            );
        $tables[$table]['primary'] = array
            (
                'id'
            );

        $table = 'negative_formats';
        $tables[$table] = array();
        $tables[$table]['columns'] = array
            (
                'id'                => $table.'.negative_format_id'
               ,'format'            => $table.'.format'
            );
        $tables[$table]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'format'            => DBQUERY_FIELD_TYPE_CHAR20
            );
        $tables[$table]['primary'] = array
            (
                'id'
            );

        $table = 'currencies';
        $tables[$table] = array();
        $tables[$table]['columns'] = array
            (
                'id'                => $table.'.currency_id'
               ,'name'              => $table.'.currency_name'
               ,'code'              => $table.'.currency_code'
               ,'sign'              => $table.'.currency_sign'
            );
        $tables[$table]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'name'              => DBQUERY_FIELD_TYPE_CHAR255
               ,'code'              => DBQUERY_FIELD_TYPE_CHAR5
               ,'sign'              => DBQUERY_FIELD_TYPE_CHAR5
            );
        $tables[$table]['primary'] = array
            (
                'id'
            );

        $table = 'positive_currency_formats';
        $tables[$table] = array();
        $tables[$table]['columns'] = array
            (
                'id'                => $table.'.negative_format_id'
               ,'format'            => $table.'.format'
            );
        $tables[$table]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'format'            => DBQUERY_FIELD_TYPE_CHAR20
            );
        $tables[$table]['primary'] = array
            (
                'id'
            );

        $table = 'negative_currency_formats';
        $tables[$table] = array();
        $tables[$table]['columns'] = array
            (
                'id'                => $table.'.negative_format_id'
               ,'format'            => $table.'.format'
            );
        $tables[$table]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'format'            => DBQUERY_FIELD_TYPE_CHAR20
            );
        $tables[$table]['primary'] = array
            (
                'id'
            );

        $patterns = 'patterns';
        $tables[$patterns] = array();
        $tables[$patterns]['columns'] = array
            (
                'id'                => 'patterns.pattern_id'
               ,'name'              => 'patterns.pattern_name'
               ,'descr'             => 'patterns.pattern_descr'
               ,'value'             => 'patterns.pattern_value'
               ,'type'              => 'patterns.pattern_type'
            );
        $tables[$patterns]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'name'              => DBQUERY_FIELD_TYPE_CHAR50
               ,'descr'             => DBQUERY_FIELD_TYPE_CHAR255
               ,'value'             => DBQUERY_FIELD_TYPE_CHAR255
               ,'type'              => DBQUERY_FIELD_TYPE_CHAR50
            );
        $tables[$patterns]['primary'] = array
            (
                'id'
            );


        global $application;
        return $application->addTablePrefix($tables);
    }

    /**
     * Gets the unit symbol.
     *
     * @param string $entity - the entity, the units: 'currency', 'weight', 'item'
     * @return the unit symbol
     */
    function getUnitTypeValue($entity)
    {
        $value = '';
        switch ($entity)
        {
            case 'currency':
                $value = $this->getCurrencySign();
                break;
            case 'weight':
                $value = prepareHTMLDisplay($this->getValue('WEIGHT_UNIT'));
                break;
            case 'item':
                $value = prepareHTMLDisplay($this->getValue('ITEM_UNIT'));
                break;
        }
        return $value;
    }

    /**
     * Gets the specyfied regular expression for the parameter.
     *
     * @param strign $pattern - 'integer', 'floet', 'currency', 'weight', 'item', 'string' ...
     * @return the regular expression
     */
    function getPattern($pattern)
    {
        global $application;
        $tables = $this->getTables();

        $p = $tables['patterns']['columns'];
        $query = &new DB_Select();
        $query->addSelectField($p['type'], 'patt_type');
        $query->addSelectField($p['value'], 'patt_value');
        $query->WhereValue($p['type'], DB_EQ, $pattern);
        $result = $application->db->getDB_Result($query);
        if (sizeof($result))
        {
            return $result[0];
        }
        return null;
    }

    /**
     * Sets the pattern.
     *
     * @param
     * @return
     */
    function setPattern($pattern, $entity)
    {
        global $application;
        $tables = $this->getTables();

        $p = $tables['patterns']['columns'];

        $query = &new DB_Update('patterns');
        $query->addUpdateValue($p['value'], $pattern);
        $query->WhereValue($p['type'], DB_EQ, $entity);
        $application->db->getDB_Result($query);
    }

    /**
     * Gets the list of possible formats.
     *
     * @return
     */
    function getFormatsList($entity)
    {
        global $application;

        $tables = $this->getTables();

        $query = &new DB_Select();
        switch($entity)
        {
            case 'date':
                $d  = $tables['date_time_formats']['columns'];
                $query->addSelectField($d["id"], "id");
                $query->addSelectField($d["format"], "format");
                $query->WhereValue($d["d_t"], DB_EQ, "date");
                break;
            case 'time':
                $t  = $tables['date_time_formats']['columns'];
                $query->addSelectField($t["id"], "id");
                $query->addSelectField($t["format"], "format");
                $query->WhereValue($t["d_t"], DB_EQ, "time");
                break;
            case 'negative':
                $t  = $tables['negative_formats']['columns'];
                $query->addSelectField($t["id"], "id");
                $query->addSelectField($t["format"], "format");
                break;
            case 'currency':
                $t  = $tables['currencies']['columns'];
                $query->addSelectField($t["id"], "id");
                $query->addSelectField($t["name"], "name");
                $query->addSelectField($t["code"], "code");
                $query->addSelectField($t["sign"], "sign");
                break;
            case 'positive_currency':
                $t  = $tables['positive_currency_formats']['columns'];
                $query->addSelectField($t["id"], "id");
                $query->addSelectField($t["format"], "format");
                break;
            case 'negative_currency':
                $t  = $tables['negative_currency_formats']['columns'];
                $query->addSelectField($t["id"], "id");
                $query->addSelectField($t["format"], "format");
                break;
        }
        return $application->db->getDB_Result($query);
    }

    /**
     * Gets the current, specyfied format of any entity.
     */
    function getFormat($entity)
    {
        switch($entity)
        {
            case 'date':
                return $this->DATE_FORMAT;
                break;
            case 'time':
                return $this->TIME_FORMAT;
                break;
            case 'number':
                return $this->NUMBER_FORMAT;
                break;
            case 'negative':
                return $this->NEGATIVE_FORMAT;
                break;
            case 'currency':
                return $this->CURRENCY;
                break;
            case 'currency_format':
                return $this->CURRENCY_FORMAT;
                break;
            case 'currency_positive_format':
                return $this->CURRENCY_POSITIVE_FORMAT;
                break;
            case 'currency_negative_format':
                return $this->CURRENCY_NEGATIVE_FORMAT;
                break;
        }
    }

    /**
     * Casts the parameter value to the specified format before outputting it
     * to the browser window.
     *
     * @param mixed $value - the parameter value
     * @param string $entity - 'currency', 'weight'
     * @return string - the formatted parameter value
     */
    function format($value, $entity)
    {
        switch ($entity)
        {
            case 'currency':
                $value = $this->currency_format($value);
                break;
            case 'weight':
            case 'item':
            case 'number':
                $value = $this->num_format($value);
                break;
            default:

                break;
        }
        return $value;
    }

    /**
     * Gets the settings, used in the javascript functions.
     *
     * @param string $entity - 'currency', 'weight'
     * @return string
     */
    function format_settings_for_js($entity)
    {
        switch ($entity)
        {
            case 'currency':
                $format = explode("|", $this->CURRENCY_FORMAT);
                $value = "decimals = \"".$format[0]."\" dec_point = \"".$format[1]."\"";
                break;
            case 'weight':
                $format = explode("|", $this->NUMBER_FORMAT);
                $value = "decimals = \"".$format[0]."\" dec_point = \"".$format[1]."\"";
                break;
            case 'weight_coeff':
                $format = explode("|", $this->NUMBER_FORMAT);
                $value = "decimals = \"6\" dec_point = \"".$format[1]."\"";
                break;
            default:
                $value = "";
                break;
        }
        return $value;
    }

    function getPrecision($entity)
    {
        $precision = 1;
        switch ($entity)
        {
            case "currency":
                $format = explode("|", $this->CURRENCY_FORMAT);
                $num = $format[0];
                break;
            case "weight":
                $format = explode("|", $this->NUMBER_FORMAT);
                $num = $format[0];
                break;
            case "item":
                $num = 0;
                break;
            default:

                break;
        }
        for ($i=0; $i<$num; $i++)
        {
            $precision /= 10;
        }
        return $precision;
    }

    /**
     * Converts the string with the value, with the number, as float,
     * inputting by user, before saving it to the database.
     *
     * @param mixed $value - the parameter value
     * @param string $entity - 'currency', 'weight'
     * @return - returns the value, which will be written to the database.
     */
    function FormatStrToFloat($value, $entity)
    {
        switch ($entity)
        {
            case 'currency':
                $format = explode("|", $this->CURRENCY_FORMAT);
                $value = strtr($value, $format[1], '.');
                break;
            case 'weight':
            case 'weight_coeff':
                $format = explode("|", $this->NUMBER_FORMAT);
                $value = strtr($value, $format[1], '.');
                break;
            default:
                break;
        }
        return $value;
    }

    /**
     * Converts the value from the database to the formatted string, according
     * to the current settings, before outputting to the entry field.
     *
     * @param mixed $value - the parameter value
     * @param string $entity - 'currency', 'weight'
     * @return - returns the value, which will be outputted to the entry field
     */
    function FloatToFormatStr($value, $entity)
    {
        switch ($entity)
        {
            case 'currency':
                if ($value != null && $value != "")// && $value!=0)
                {
                    $format = explode("|", $this->CURRENCY_FORMAT);
                    $value = number_format($value, $format[0], $format[1], '');
                }
                break;
            case 'weight':
                if ($value != null && $value != "")// && $value!=0)
                {
                    $format = explode("|", $this->NUMBER_FORMAT);
                    $value = number_format($value, $format[0], $format[1], '');
                }
                break;
            case 'weight_coeff':
                if ($value != null && $value != "")// && $value!=0)
                {
                    $format = explode("|", $this->NUMBER_FORMAT);
                    $value = number_format($value, 6, $format[1], '');
                }
                break;
            default:
                break;
        }
        return $value;
    }

    /**
     * Converts the number to the form, appropriate to the current number format.
     *
     * @param float $num - the number, which should be formatted
     * @return string - the formatted number of the string type
     */
    function num_format($num)
    {
        $negative = false;
        $num_format = explode("|", $this->NUMBER_FORMAT);
        $num *= 1;
        if (!is_int($num)&&!is_float($num))
        {
            return $num;
        }
        if ($num < 0)
        {
            $negative = true;
            $num *= -1;
        }
        if (!is_int($num))
        {
            $num = number_format($num, $num_format[0], $num_format[1], $num_format[2]);
        }
        else
        {
            $num = number_format($num, 0, $num_format[1], $num_format[2]);
        }
        if ($negative)
        {
            $num = sprintf($this->NEGATIVE_FORMAT, $num);
        }
        return $num;
    }

    /**
     * Converts the sum to the form, appropriate to the current currency format.
     *
     * @param float $sum - the sum, which should be formatted
     * @return string - the fomatted sum of the string type
     */
    function currency_format($sum)
    {
        global $application;
        if($sum == PRICE_N_A)
        {
            switch(modApiFunc('Users', 'getZone'))
            {
                case "CustomerZone":
                {
                    $obj = &$application->getInstance('MessageResources',"messages", "CustomerZone");
                    $value = $obj->getMessage('PRICE_N_A');
                    break;
                }
                case "AdminZone":
                {
                    $obj = &$application->getInstance('MessageResources',"system-messages", "AdminZone");
                    $value = $obj->getMessage('PRICE_N_A');
                    break;
                }
                default:
                {
                    $value = "";
                    //FIXME: TODO: call _fatal()
                }
            }
            return $value;
        }

        $negative = false;
        $cur_sign = explode("|", $this->CURRENCY);

        //patch to allow single unicode character as currency symbol
        $cur_sign = preg_match("/^\&\#[0-9]{1,10};$/i", $cur_sign[1]) ? $cur_sign[1] : prepareHTMLDisplay($cur_sign[1]);

        $cur_format = explode("|", $this->CURRENCY_FORMAT);
        $sum *= 1;
        if (!is_int($sum)&&!is_float($sum))
        {
            return $sum;
        }
        if ($sum < 0)
        {
            $negative = true;
            $sum *= -1;
        }
        $sum = number_format($sum, $cur_format[0], $cur_format[1], $cur_format[2]);
        if ($negative)
        {
            $sum = strtr($this->CURRENCY_NEGATIVE_FORMAT, array("{s}" => $cur_sign, "{v}" => $sum));
        }
        else
        {
            $sum = strtr($this->CURRENCY_POSITIVE_FORMAT, array("{s}" => $cur_sign, "{v}" => $sum));
        }
        return $sum;
    }

    /**
     * Returns the formatted date, depending on the specified time format.
     *
     * @param mixed $date - the date might have different types:
     *                       integer - timestamp;
     *                       string  - the date in the format MySQL: '0000-00-00'     '0000-00-00 00:00:00';
     *                       array(
     *                             int $year,
     *                             int $month,
     *                             int $day
     *                            )  - the date of the array type year, month, day
     * @return the formatted date
     */
    function date_format($date)
    {
        $time_shift = modApiFunc('Configuration', 'getValue', SYSCONFIG_STORE_TIME_SHIFT)*60*60;
        if (is_int($date))
        {
            return date($this->DATE_FORMAT, ($date + $time_shift));
        }
        elseif (is_string($date))
        {
            if (strlen($date)<=10)
            {
                $date = explode("-", $date);
                return date($this->DATE_FORMAT, (mktime(0, 0, 0, $date[1], $date[2], $date[0]) + $time_shift));
            }
            else
            {
                $date = explode(" ", $date);
                $date = explode("-", $date[0]);
                return date($this->DATE_FORMAT, (mktime(0, 0, 0, $date[1], $date[2], $date[0]) + $time_shift));
            }
        }
        elseif (is_array($date))
        {
            return date($this->DATE_FORMAT, (mktime(0, 0, 0, $date[1], $date[2], $date[0]) + $time_shift));
        }
        return $date;
    }

    /**
     * Casts the date to the specified format from timestamp.
     *
     * @param integer $date -  the date timestamp
     * @return string - the formatted date
     */
    function timestamp_date_format($date)
    {
        $date *=1;
        $time_shift = modApiFunc('Configuration', 'getValue', SYSCONFIG_STORE_TIME_SHIFT)*60*60;
        return date($this->DATE_FORMAT, ($date + $time_shift));
    }

    /**
     * Casts the date to the specified format from the string in the format,
     * defined as SQL - '2005-12-31' or '2005-12-31 12:34:56'
     *
     * @param integer $date -  the date timestamp
     * @return string - the formatted date
     */
    function SQL_date_format($date)
    {
        $time_shift = modApiFunc('Configuration', 'getValue', SYSCONFIG_STORE_TIME_SHIFT)*60*60;
        if (strlen($date)<=10)
        {
            $date = explode("-", $date);
            return date($this->DATE_FORMAT, (mktime(0, 0, 0, $date[1], $date[2], $date[0]) + $time_shift));
        }
        else
        {
            $date = explode(" ", $date);
            $date = explode("-", $date[0]);
            return date($this->DATE_FORMAT, (mktime(0, 0, 0, $date[1], $date[2], $date[0]) + $time_shift));
        }
    }

    /**
     * Casts the time to the specified format from timestamp.
     *
     * @param integer $date - the date timestamp
     * @return string - the formatted date
     */
    function timestamp_time_format($time)
    {
        $time *=1;
        $time_shift = modApiFunc('Configuration', 'getValue', SYSCONFIG_STORE_TIME_SHIFT)*60*60;
        return date($this->TIME_FORMAT, ($time + $time_shift));
    }

    /**
     * Casts the time to the specified format from the string in the format,
     * defined as SQL - '12:34:56' or '2005-12-31 12:34:56'
     *
     * @param integer $date -  the date timestamp
     * @return string - the formatted date
     */
    function SQL_time_format($time)
    {
        $time_shift = modApiFunc('Configuration', 'getValue', SYSCONFIG_STORE_TIME_SHIFT)*60*60;
        if (strlen($time)<=8)
        {
            $time = explode(":", $time);
            return date($this->TIME_FORMAT, (mktime($time[0], $time[1], $time[2], 2, 1, 1970) + $time_shift));
        }
        else
        {
            $time = explode(" ", $time);
            $time = explode(":", $time[1]);
            return date($this->TIME_FORMAT, (mktime($time[0], $time[1], $time[2], 2, 1, 1970) + $time_shift));
        }
    }

    /**
     * Rounds the monetary unit.
     *
     * @return float - the rounded unit
     */
    function roundMonetaryUnit($value)
    {
        return number_format($value, "2", ".", "");
    }

    /**
     *                             .
     */
    function formatFileSize($size)
    {
        $a = array('b','Kb','Mb','Gb','Tb');
        $i = 0;
        $have_mod = false;
        while($size > 1024 and $i < 4)
        {
            $have_mod = ($have_mod or (($size % 1024) > 0));
            $size = $size / 1024;
            $i++;
        };

        $str = '';
        $frm = "%d ";
        if($i)
        {
            if($have_mod)
                $str .= '~';
            $frm = "%.2f ";
        };
        $str .= sprintf($frm,round($size,2)).$a[$i];
        return $str;
    }

    /**#@-*/

//------------------------------------------------
//              PRIVATE DECLARATION
//------------------------------------------------

    /**#@+
     * @access private
     */

    /**#@-*/

}
?>