<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2007, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

/**
 * Module "Shipping Cost Calculator"
 *
 * @package ShippingCostCalculator
 * @author Egor V. Derevyankin
 */

class Shipping_Cost_Calculator
{

//------------------------------------------------
//               PUBLIC DECLARATION
//------------------------------------------------

    /**#@+
     * @access public
     */

    function Shipping_Cost_Calculator()
    {
        $this->ShippingInfo=array();
        $this->Cart=array();
        $this->PaymentModuleInfo=array();
        $this->fictitious_subtotal=0;
        $this->fictitious_weight=0;
        $this->fictitious_items_count=0;
        $this->per_item_shipping_sum=0;
        $this->per_order_shipping_fee=0;
        $this->per_item_handling_sum=0;
        $this->per_order_handling_fee=0;
        $this->is_debug=false;
        $this->is_full_free_shipping=false;
        $this->is_shipping_not_needed=false;
    }

    function getInfo()
    {
    }

    /**
     * @return structure of the tables of this module
     */
    function getTables()
    {
        static $tables;

        if (is_array($tables))
        {
            return $tables;
        };

        $tables = array ();

        $settings = 'scc_settings';
        $tables[$settings] = array();
        $tables[$settings]['columns'] = array
            (
                'id'                => $settings.'.'.$settings.'_id'
               ,'key'               => $settings.'.'.$settings.'_key'
               ,'value'             => $settings.'.'.$settings.'_value'
            );
        $tables[$settings]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'key'               => DBQUERY_FIELD_TYPE_CHAR50
               ,'value'             => DBQUERY_FIELD_TYPE_CHAR50
            );
        $tables[$settings]['primary'] = array
            (
                'id'
            );

        global $application;
        return $application->addTablePrefix($tables);

    }

    /**
     * Module installator.
     */
    function install()
    {
        _use(dirname(__FILE__)."/includes/install.inc");
    }

    /**
     * Uninstalls the module.
     */
    function uninstall()
    {
        $query = &new DB_Table_Delete(Shipping_Cost_Calculator::getTables());
        global $application;
        $application->db->getDB_Result($query);
    }

    /**
     * Gets settings.
     *
     * @return array of the settings
     */
    function getSettings()
    {
        global $application;
        $tables = $this->getTables();
        $columns=$tables['scc_settings']['columns'];

        $query = &new DB_Select();
        $query->addSelectTable('scc_settings');
        $query->addSelectField($columns['key']);
        $query->addSelectField($columns['value']);
        $result = $application->db->getDB_Result($query);
        $settings=array();
        foreach($result as $k => $v)
            $settings[$v['scc_settings_key']]=$v['scc_settings_value'];
        return $settings;
    }

    /**
     * Sets settings.
     *
     * @param $settings array of the setings
     */
    function setSettings($settings)
    {
        global $application;
        $tables = $this->getTables();
        $columns=$tables['scc_settings']['columns'];

        foreach($settings as $k => $v)
        {
            $query = &new DB_Update('scc_settings');
            $query->addUpdateValue($columns['value'],$v);
            $query->WhereValue($columns['key'], DB_EQ, $k);
            $application->db->getDB_Result($query);
        };

    }

    function setShippingInfo($s_info)
    {
        $this->ShippingInfo=$s_info;
    }

    function setCart($s_cart)
    {
        $this->Cart=$s_cart;
        $this->processCart();
    }

    function setPaymentModuleInfo($pm_info)
    {
        $this->PaymentModuleInfo = $pm_info;
    }

    function setDebugFlag($d_flag)
    {
        $this->is_debug=$d_flag;
    }

    function calculateShippingCost()
    {
        if($this->is_shipping_not_needed)
        {
            return array("Shipping_Not_Needed" =>array(
                    "methods" => array(
                        "0" => array(
                            "id" => modApiFunc("Checkout","getNotNeedShippingMethodID"),
                            "shipping_cost" => $this->calculateTotalShippingCost(0.0)
                        )
                    )
                ));
        }

        $sm_list = modApiFunc("Checkout", "getInstalledAndActiveModulesListData", "shipping");

        if(count($sm_list)==1)
        {
            $sm = array_pop(array_values($sm_list));
            if($sm->name == modApiFunc("Checkout", "getAllInactiveModuleClassAPIName", "shipping"))
                return array(modApiFunc("Checkout", "getAllInactiveModuleClassAPIName", "shipping") =>array(
                        "methods" => array(
                            "0" => array(
                                "id" => modApiFunc(modApiFunc("Checkout", "getAllInactiveModuleClassAPIName", "shipping"), "getSingleAvailableMethodId"),
                                "shipping_cost" => $this->calculateTotalShippingCost(0.0)
                            )
                        )
                    ));
        }

        // Temporary solution begin
        // TODO: put to Checkout!
        $SelectedModules = modApiFunc("Checkout", "getSelectedModules", "shipping");
        $items = array();
        $new_selected_module_sort_order = 0;
        foreach ($sm_list as $sm_item)
        {
            //FIXME create/use some mm function to convert class names.
            $name = strtolower($sm_item->name);
            $smInfo = modApiFunc($name, "getInfo");
            //If sort id is not defined then assign the highest possible sort id to this item: 0, -1, -2 ...
            $sort_id = empty($SelectedModules[$smInfo["GlobalUniqueShippingModuleID"]]["sort_order"]) ? $new_selected_module_sort_order++ : $SelectedModules[$smInfo["GlobalUniqueShippingModuleID"]]["sort_order"];
            $items[$sort_id]=$sm_item;
        }

        //Sort items by sort_id.
        ksort($items, SORT_NUMERIC);
        $sm_list=$items;
        // Temporary solution end

        $results = array();
        foreach($sm_list as $sm)
        {
            modApiFunc($sm->name,"setDebugFlag",$this->is_debug);
            $results[$sm->name]["methods"]=modApiFunc($sm->name,"getShippingMethods","AVAILABLE",true);
            if($this->is_debug)
                $results[$sm->name]["debug_info"]=modApiFunc($sm->name,"getDebugInfo");
            if(count($results[$sm->name]["methods"]))
            {
                foreach($results[$sm->name]["methods"] as $mk => $minf)
                    $results[$sm->name]["methods"][$mk]["shipping_cost"]=$this->calculateTotalShippingCost($minf["cost"]);
            }
            else
            {
                if(!$this->is_debug)
                    unset($results[$sm->name]);
            }
        };

        return $results;

    }

    function getCalculatedMethod($API_name,$method_id)
    {
        if($API_name == 'Shipping_Not_Needed')
        {
            $method = array(
                'shipping_cost' => $this->calculateTotalShippingCost(0.0)
            );
        }
        else
        {
            $method=modApiFunc($API_name,"getShippingMethodInfo",$method_id,true);
            $method["shipping_cost"]=$this->calculateTotalShippingCost($method["cost"]);
        };
        return $method;
    }

    function getShippingInfo()
    {
        return $this->ShippingInfo;
    }

    function getPackWeight()
    {
        return $this->fictitious_weight;
    }

    function getCartSubtotal()
    {
        return $this->fictitious_subtotal;
    }

    function getItemsCount()
    {
        return $this->fictitious_items_count;
    }

    /**
     * The parameter $customer_cart is an array, which returns the method
     * Cart::getCartContent.
     */
    function formatCart($customer_cart)
    {
        $formatted_cart=array("products"=>array(),'subtotal'=>0);

        /*
         * It needs to know, that any product attribute can be hidden
         * in the product type. That's why each product attribute should
         * be checked, before using its value.
         * I.e. if the attribute is hidden, then attribute value can't be used.
         * Moreover, if the attribute is public, then the attribute value
         * still should be checked, if it is null, because the attribute value
         * can be null.
         */

        for($i=0;$i<count($customer_cart);$i++)
        {
            $product=array();
            $product["qty"]=$customer_cart[$i]["Quantity_In_Cart"];

            $_cycle = array("weight"=>"Weight",
                            "cost"=>"SalePrice",
                            "ship_charge"=>"PerItemShippingCost",
                            "hand_charge"=>"PerItemHandlingCost",
                            "free_ship"=>"FreeShipping",
                            "need_ship"=>"NeedShipping");
            foreach($_cycle as $_key=>$_attribute)
            {
                if (isset($customer_cart[$i]["attributes"][$_attribute])
                    && $customer_cart[$i]["attributes"][$_attribute]["visible"] == TRUE
                    && !empty($customer_cart[$i]["attributes"][$_attribute]["value"]))
                {
                    $product[$_key] = $customer_cart[$i]["attributes"][$_attribute]["value"];
                }
                else
                {
                    switch($_key)
                    {
                        case 'free_ship': $product[$_key] = 'NO'; break;
                        case 'need_ship': $product[$_key] = 'YES'; break;
                        default: $product[$_key] = 0.0; break;
                    };
                }
            }

            $formatted_cart["products"][]=$product;
            $formatted_cart["subtotal"] += $product["cost"] * $product["qty"];
        }

        return $formatted_cart;
    }

    function isShippingNotNeeded()
    {
        return $this->is_shipping_not_needed;
    }

    /**#@-*/

//------------------------------------------------
//              PRIVATE DECLARATION
//------------------------------------------------

    /**#@+
     * @access private
     */

    function processCart()
    {
        if(count($this->Cart["products"])>0)
        {
            $fs_flag=true;
            $nns_flag=true;
        }
        else
        {
            $fs_flag=false;
            $nns_flag=false;
        };
        $this->fictitious_subtotal=0;
        $this->fictitious_weight=0;
        $this->fictitious_items_count=0;
        $this->per_item_shipping_sum=0;
        $this->per_order_shipping_fee=0;
        $this->per_item_handling_sum=0;
        $this->per_order_handling_fee=0;
        for($i=0;$i<count($this->Cart["products"]);$i++)
        {
            $product=$this->Cart["products"][$i];

            if(!isset($product["need_ship"]) or ($product["need_ship"]!="NO" and $product["need_ship"]!="2"))
                $nns_flag=false;
            else
                continue;

            if(!isset($product["free_ship"])  or $product["free_ship"]==""
               or $product["free_ship"]=="NO" or $product["free_ship"]=="2")
            {
                $fs_flag=false;
                $this->fictitious_weight+=($product["weight"]*$product["qty"]);
                $this->fictitious_subtotal+=($product["cost"]*$product["qty"]);
                $this->per_item_shipping_sum+=($product["ship_charge"]*$product["qty"]);
                $this->fictitious_items_count+=$product["qty"];
            };
            $this->per_item_handling_sum+=($product["hand_charge"]*$product["qty"]);
        };
        $this->is_full_free_shipping=$fs_flag;
        $this->is_shipping_not_needed=$nns_flag;

        $settings=$this->getSettings();

        if($settings["PO_SC_TYPE"]=="A")
            $this->per_order_shipping_fee=$settings["PO_SC"];
        elseif($settings["PO_SC_TYPE"]=="P")
            $this->per_order_shipping_fee=round(($settings["PO_SC"]/100*$this->Cart["subtotal"]),2);

        $this->per_order_handling_fee=$settings["PO_HC"];

        if($this->fictitious_subtotal<0)
            $this->fictitious_subtotal=0;
        if($this->fictitious_weight<0)
            $this->fictitious_weight=0;
        if($this->per_item_shipping_sum<0)
            $this->per_item_shipping_sum=0;
        if($this->per_item_handling_sum<0)
            $this->per_item_handling_sum=0;

    }

    /**
     * Converts the value of the monetary sum to be used _out_ ASC.
     * If the price equals PRICE_N_A, then it is changed to 0.0.
     */
    function export_PRICE_N_A($price)
    {
        return ($price == PRICE_N_A) ? 0.0 : $price;
    }

    function calculateTotalShippingCost($delivery_cost)
    {
        $settings = $this->getSettings();
        $return = array(
            // settings
            "FreeHandlingForOrdersOver" => is_numeric($settings["FH_OO"]) ? $settings["FH_OO"] : PRICE_N_A,
            "FreeShippingForOrdersOver" => is_numeric($settings["FS_OO"]) ? $settings["FS_OO"] : PRICE_N_A,
            "MinimumShippingCost"       => $settings["MIN_SC"],

            // shipping
            'PerItemShippingCostSum'=> $this->per_item_shipping_sum,
            'PerOrderShippingFee'   => $this->per_order_shipping_fee,
            "ShippingMethodCost"    => $delivery_cost,
            "TotalShippingCharge"   => ($this->per_item_shipping_sum+$this->per_order_shipping_fee+$delivery_cost),

            // handling
            'PerItemHandlingCostSum'=> $this->per_item_handling_sum,
            'PerOrderHandlingFee'   => $this->per_order_handling_fee,
            "TotalHandlingCharge"   => ($this->per_item_handling_sum+$this->per_order_handling_fee),

            //Payment
            'PerOrderPaymentModuleShippingFee' => empty($this->PaymentModuleInfo) ? PRICE_N_A : $this->PaymentModuleInfo['PerOrderPaymentModuleShippingFee'],

            // overall
            'TotalShippingAndHandlingCost' => 0 // see below
        );

        if($return["TotalShippingCharge"] < $settings["MIN_SC"])
            $return["TotalShippingCharge"] = $settings["MIN_SC"];

        if(is_numeric($settings["FS_OO"]) and ($this->fictitious_subtotal >= $settings["FS_OO"]))
            $return["TotalShippingCharge"] = 0;

        if($this->is_full_free_shipping)
            $return["TotalShippingCharge"] = 0;

        if(is_numeric($settings["FH_OO"]) and ($this->fictitious_subtotal >= $settings["FH_OO"]))
            $return["TotalHandlingCharge"] = 0;

        //         FS_OO              PerOrderPaymentModuleShippingFee.          ,
        //                          -      .
        $return['TotalShippingAndHandlingCost'] = $return['TotalShippingCharge'] +
                                                  $return['TotalHandlingCharge'] +
                                                  $this->export_PRICE_N_A($return['PerOrderPaymentModuleShippingFee']);

        if($this->is_shipping_not_needed)
            $return = array_map(array(&$this,"__set_to_zero"),$return);

        return $return;
    }

    function __set_to_zero($a)
    {
        return 0;
    }

    var $ShippingInfo;
    var $Cart;
    var $fictitious_weight;
    var $fictitious_subtotal;
    var $fictitious_items_count;
    var $per_item_shipping_sum;
    var $per_order_shipping_fee;
    var $per_item_handling_sum;
    var $per_order_handling_fee;
    var $is_debug;
    var $is_full_free_shipping;
    var $is_shipping_not_needed;
};

?>