<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2007, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php
/**
 * Checkout module.
 * Finishing the order: choosing shipping, payment methods and processing online payment.
 * Order info is saved to database and becomes available through AZ.
 *
 * 2005-11-03
 * It should be kept a compatability: store block <=> prerequisite.
 * There are several such prerequisites:
 * shipping-info, shipping-method, billing-info, payment-method,
 * customer-registratration-main-info
 * NOTE: Confirmation is NOT a prerequisite.
 *
 * Store blocks are called differently. E.g. ShippingMethodsListInput.
 * It would be better to make a compatability table. For example:
 * shipping-info                      => Address
 * shipping-method                    => ShippingMethodsListInput
 * billing-info                       => Address
 * payment-method                     => PaymentMethodsListInput
 * customer-registratration-main-info => CustomerInfoInput

 * @package Checkout
 * @author  Vadim Lyalikov
 * @access  public
*/
class Checkout
{
//------------------------------------------------
//               PUBLIC DECLARATION
//------------------------------------------------

    /**#@+
     * @access public
     */

    /**
     * Checkout constructor.
     *
     * @todo finish the functions on this page
     */
    function Checkout()
    {
        global $application;

        Checkout::updateAllInactiveModuleStatus("payment");
        Checkout::updateAllInactiveModuleStatus("shipping");

        if(!modApiFunc('Session', 'is_Set', 'PrerequisitesValidationResults'))
        {
            //FIXME: ERROR arises if not installed
            Checkout::initPrerequisitesValidationResults();
            Checkout::initAdditionalPrerequisitesValidationResults("creditCardInfo");
            Checkout::initAdditionalPrerequisitesValidationResults("bankAccountInfo");
            //                                ,                   
            // $this->initStoreOwnerInfo();
            //           LoadState
        }

        $key = Checkout::initCheckoutCZBlowfishKey();
        $this->setPerRequestVariable("CHECKOUT_CZ_BLOWFISH_KEY", $key);
    }

    function initStoreOwnerInfo()
    {
        //                                           Checkout               storeOwnerInfo.
        //                                       visible/required.
        $store_owner_country = modApiFunc('Configuration', 'getValue', SYSCONFIG_STORE_OWNER_COUNTRY);
        $store_owner_state = modApiFunc('Configuration', 'getValue', SYSCONFIG_STORE_OWNER_STATE);
        $emulated_post_data = array();
        $emulated_post_data["Country"] = $store_owner_country;
        if(is_numeric($store_owner_state))
        {
            $emulated_post_data["Statemenu"] = $store_owner_state;
        }
        else
        {
            $emulated_post_data["Statetext"] = $store_owner_state;
        }
        Checkout::validateInputForPrerequisite("storeOwnerInfo", $emulated_post_data);
    }
    /**
     * Initializes "pseudo-session" Blowfish key to enctypt data symetrically
     * into Checkout CZ.
     * It is not stored in the session and is sent only as POST or GET parameter.
     * If it was lost at one of the steps ( it wasn't passed neither by GET,
     * nor by POST), then all data, which was encrypted by the key, is deleted,
     * because it can't be decrypted without key.
     */
    /*static*/ function initCheckoutCZBlowfishKey()
    {
        global $application;
        $request = $application->getInstance('Request');
        $checkout_cz_blowfish_key = $request->getValueByKey("CHECKOUT_CZ_BLOWFISH_KEY");
        if(NULL === $checkout_cz_blowfish_key)
        {
            if(modApiFunc('Session', 'is_Set', 'CHECKOUT_CZ_BLOWFISH_KEY_IS_ALREADY_DEFINED'))
            {
                $val = modApiFunc('Session', 'get', 'CHECKOUT_CZ_BLOWFISH_KEY_IS_ALREADY_DEFINED');
                if($val === TRUE)
                {
                    //clear all encrypted data: it can't be decrypted. The key is lost.
                    Checkout::clear_all_checkout_cz_blowfish_encoded_data();
                }
            }
            $ret_value = modApiFunc("Crypto", "blowfish_gen_blowfish_key");
            /**
             * The state hasn't been loaded yet - LoadState() is processed after the construction.
             * Save it in the session.
             */
            modApiFunc('Session', 'set', 'CHECKOUT_CZ_BLOWFISH_KEY_IS_ALREADY_DEFINED', TRUE);
        }
        else
        {
            $ret_value = $checkout_cz_blowfish_key;
        }
        return $ret_value;
    }

    //Clear data,which could be encrypted by the key, that is lost now.
    /*static*/ function clear_all_checkout_cz_blowfish_encoded_data()
    {
        //Credit cards:
        Checkout::initAdditionalPrerequisitesValidationResults("creditCardInfo");
        //Other encrypted data:

        /**
         * Note: if some not cleared filled encrypted prerequisites are left, it doesn't matter:
         * the key is lost, it will be impossible to decrypt data.
         */
    }

    function encrypt_prerequisite_with_checkout_cz_blowfish_key($name)
    {
        if(isset($this->PrerequisitesValidationResults[$name]))
        {
            foreach($this->PrerequisitesValidationResults[$name]["validatedData"] as $key => $info)
            {
                $plaintext  = $info["value"];
//           die($this->getPerRequestVariable("CHECKOUT_CZ_BLOWFISH_KEY"));
                $cyphertext = modApiFunc("Crypto", "blowfish_encrypt", $plaintext, $this->getPerRequestVariable("CHECKOUT_CZ_BLOWFISH_KEY"));
                $this->PrerequisitesValidationResults[$name]["validatedData"][$key]["value"] = $cyphertext;
            }
        }
    }

    function decrypt_prerequisite_with_checkout_cz_blowfish_key($name)
    {
        if(isset($this->PrerequisitesValidationResults[$name]))
        {
            foreach($this->PrerequisitesValidationResults[$name]["validatedData"] as $key => $info)
            {
                $cyphertext = $info["value"];
                $plaintext  = modApiFunc("Crypto", "blowfish_decrypt", $cyphertext, $this->getPerRequestVariable("CHECKOUT_CZ_BLOWFISH_KEY"));
                $this->PrerequisitesValidationResults[$name]["validatedData"][$key]["value"] = $plaintext;
            }
        }
    }

    /**
     * Sets up a value of some variable for the time of processing one
     * HTTP request. Something like a flag or a configuration variable.
     */
    function setPerRequestVariable($key, $value)
    {
        if(!isset($this->PerRequestVariables))
        {
            $this->PerRequestVariables = array();
        }
        $this->PerRequestVariables[$key] = $value;
    }

    /**
     * Returns a value of some variable (for the time of processing one
     * HTTP request). See setPerRequestVariable.
     */
    function getPerRequestVariable($key)
    {
        if(!isset($this->PerRequestVariables))
        {
            $this->PerRequestVariables = array();
            return NULL;
        }
        else
        {
            if(isset($this->PerRequestVariables[$key]))
            {
                return $this->PerRequestVariables[$key];
            }
            else
            {
                return NULL;
            }
        }

    }

    /*static*/ function getAllInactiveModuleClassAPIName($ModuleClass)
    {
        $mId = Checkout::getAllInactiveModuleId($ModuleClass);
        switch ($ModuleClass)
        {
            case "payment":
                //FIXME: hardcoded value
                return "Payment_Module_All_Inactive";
                $mInfo = Checkout::getPaymentModuleInfo($mId);
                break;
            case "shipping":
                //FIXME: hardcoded value
                return "Shipping_Module_All_Inactive";
                $mInfo = Checkout::getShippingModuleInfo($mId);
                break;
            default:
                _fatal(__FILE__ . " : " . __LINE__);
                break;
        }

        if(isset($mInfo['APIClassName']))
        {
            return $mInfo['APIClassName'];
        }
        else
        {
            //FIXME: TODO: move message to resources.
            _fatal(array( "CODE" => "CORE_054"), __FILE__, __LINE__);
        }
    }

    /*static*/ function getAllInactiveModuleId($ModuleClass)
    {
        switch($ModuleClass)
        {
            case "payment":
            {
                $value = "A50F9CD5-9F45-12CB-353C-03EC75493A0A";//"Payment_Module_All_Inactive";
                break;
            }
            case "shipping":
            {
                $value = "6F82BA03-C5B1-585B-CE2E-B8422A1A19F6";//"Shipping_Module_All_Inactive";
                break;
            }
            default:
            {
                _fatal(__FILE__ . " : " . __LINE__);
                break;
            }
        }
        return $value;
    }

    /*static*/ function updateAllInactiveModuleStatus($ModuleClass)
    {
        $AllInactiveModuleClassAPIName = Checkout::getAllInactiveModuleClassAPIName($ModuleClass);
        //Inquire all Payment modules.
        //If even one Active exists, install AllInactive [payment] module status in Inactive.
        //  Otherwise install AllInactive [payment] module satus in Active.

        $modules = Checkout::getInstalledAndActiveModulesListData($ModuleClass);
        if(sizeof($modules) == 0)
        {
            //No active payment modules exist, install the module status
            //  AllInactive in Active.
            //FIXME: optimize SELECT first. If the status is valid, there is no need to do UPDATE.
            Checkout::setModuleActive(Checkout::getAllInactiveModuleId($ModuleClass), true);
        }
        else
        if(sizeof($modules) == 1)
        {
            //If the module AllInactive is active, then fine,
            //  if other module is active, then AllInactive module
            //  is inactive but this is correct too.
        }
        else
        {
            //Make AllInactive module inactive.
            // FIXME: optimize SELECT first. If the status is valid, there is no need to do UPDATE.
            Checkout::setModuleActive(Checkout::getAllInactiveModuleId($ModuleClass), false);
        }
    }

    function getPaymentModulesGroupsInfo()
    {
        global $application;
        $obj = &$application->getInstance('MessageResources');

        return array("Offline"            => array("group_id"   => "Offline",
                                                   "short_name" => $obj->getMessage('CHECKOUT_PAYMENT_GROUP_OFFLINE_SHORT_NAME'))
                    ,"OnlineCC"           => array("group_id"   => "OnlineCC",
                                                   "short_name" => $obj->getMessage('CHECKOUT_PAYMENT_GROUP_ONLINE_CC_SHORT_NAME'))
                    ,"OnlineECheck"       => array("group_id"   => "OnlineECheck",
                                                   "short_name" => $obj->getMessage('CHECKOUT_PAYMENT_GROUP_ONLINE_ECHECK_SHORT_NAME'))
//                    ,"OnlinePaymentSystem"=> array("group_id"   => "OnlinePaymentSystem",
//                                                   "short_name" => $obj->getMessage('CHECKOUT_PAYMENT_GROUP_ONLINE_PAYMENT_SYSTEM_SHORT_NAME'))
                    );
    }

    function getShippingModulesGroupsInfo()
    {
        global $application;
        $obj = &$application->getInstance('MessageResources');
        return array("Offline"            => array("group_id"   => "Offline",
                                                   "short_name" => $obj->getMessage('CHECKOUT_SHIPPING_GROUP_OFFLINE_SHORT_NAME'))
                     );
    }

    function getPaymentModuleGroup($payment_module_data)
    {
        ////FIXME INCORRECT method to get group name!
        $groups_array = split(',', $payment_module_data->groups);
        $payment_group = array_key_exists(1, $groups_array) ? $groups_array[1] : "";
        return $payment_group;
    }

    function getShippingModuleGroup($shipping_module_data)
    {
        ////FIXME INCORRECT method to get group name!
        $groups_array = split(',', $shipping_module_data->groups);
        $shipping_group = array_key_exists(1, $groups_array) ? $groups_array[1] : "";
        return $shipping_group;
    }

    /**
     * Returns a meta description of database tables, defined for storing data
     * of the Catalog module.
     *
     * @return array table meta info
     */
    /*static?*/ function getTables ()
    {
        static $tables;

        if (is_array($tables))
        {
            return $tables;
        }

        $tables = array ();

        # the main table ORDERS
        $orders = 'orders';
        $tables[$orders] = array();
        $tables[$orders]['columns'] = array
            (
                'id'                => 'orders.order_id'
               ,'date'              => 'orders.order_date'
               ,'status_id'         => 'orders.order_status_id' // reference to order_status
               ,'payment_status_id' => 'orders.order_payment_status_id' // reference to order_payment_status
               ,'payment_method'    => 'orders.order_payment_method'
               ,'payment_module_id' => 'orders.order_payment_module_id'
               ,'payment_method_detail' => 'orders.order_payment_method_detail'
               ,'payment_processor_order_id' => 'orders.order_payment_processor_order_id'
               ,'shipping_method'   => 'orders.order_shipping_method'
               ,'track_id'          => 'orders.order_track_id'
               ,'subtotal'          => 'orders.order_subtotal'
               ,'total'             => 'orders.order_total'
               ,'person_id'         => 'orders.order_person_id' // reference to persons
            );
        $tables[$orders]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'date'              => DBQUERY_FIELD_TYPE_DATE . ' NOT NULL'
               ,'status_id'         => DBQUERY_FIELD_TYPE_INT . ' NOT NULL'
               ,'payment_status_id' => DBQUERY_FIELD_TYPE_INT . ' NOT NULL'
               ,'payment_method'    => DBQUERY_FIELD_TYPE_CHAR255
               ,'payment_module_id' => DBQUERY_FIELD_TYPE_CHAR255 . ' NOT NULL'
               ,'payment_method_detail' => DBQUERY_FIELD_TYPE_CHAR255
               ,'payment_processor_order_id' => DBQUERY_FIELD_TYPE_CHAR255
               ,'shipping_method'   => DBQUERY_FIELD_TYPE_CHAR255
               ,'track_id'          => DBQUERY_FIELD_TYPE_CHAR255
               ,'subtotal'          => DBQUERY_FIELD_TYPE_FLOAT . ' NOT NULL default 0.0'
               ,'total'             => DBQUERY_FIELD_TYPE_FLOAT . ' NOT NULL default 0.0'
               ,'person_id'         => DBQUERY_FIELD_TYPE_INT
            );
        $tables[$orders]['primary'] = array
            (
                'id'
            );
        $tables[$orders]['indexes'] = array
            (
                'IDX_date' => 'date' // @todo
            );

        # a table of possible payment status
        $payment_status = 'order_payment_status';
        $tables[$payment_status] = array();
        $tables[$payment_status]['columns'] = array
            (
                'id'                => 'order_payment_status.order_payment_status_id'
               ,'name'              => 'order_payment_status.order_payment_status_name'
               ,'descr'             => 'order_payment_status.order_payment_status_description'
               ,'sort'              => 'order_payment_status.order_payment_status_sort'
               ,'type'              => 'order_payment_status.order_payment_status_type'
               ,'active'            => 'order_payment_status.order_payment_status_active'
            );
        $tables[$payment_status]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'name'              => DBQUERY_FIELD_TYPE_CHAR50 . ' NOT NULL'
               ,'descr'             => DBQUERY_FIELD_TYPE_CHAR255
               ,'sort'              => DBQUERY_FIELD_TYPE_INT
               ,'type'              => DBQUERY_FIELD_TYPE_CHAR20 . ' NOT NULL default \'custom\''
               ,'active'            => DBQUERY_FIELD_TYPE_BOOL
            );
        $tables[$payment_status]['primary'] = array
            (
                'id'
            );

        # a table of possible order status
        $order_status = 'order_status';
        $tables[$order_status] = array();
        $tables[$order_status]['columns'] = array
            (
                'id'                => 'order_status.order_status_id'
               ,'name'              => 'order_status.order_status_name'
               ,'descr'             => 'order_status.order_status_description'
               ,'sort'              => 'order_status.order_status_sort'
               ,'type'              => 'order_status.order_status_type'
               ,'active'            => 'order_status.order_status_active'
            );
        $tables[$order_status]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'name'              => DBQUERY_FIELD_TYPE_CHAR50 . ' NOT NULL'
               ,'descr'             => DBQUERY_FIELD_TYPE_CHAR255
               ,'sort'              => DBQUERY_FIELD_TYPE_INT
               ,'type'              => DBQUERY_FIELD_TYPE_CHAR20 . ' NOT NULL default \'custom\''
               ,'active'            => DBQUERY_FIELD_TYPE_BOOL
            );
        $tables[$order_status]['primary'] = array
            (
                'id'
            );

        # a table of order notes
        $order_notes = 'order_notes';
        $tables[$order_notes] = array();
        $tables[$order_notes]['columns'] = array
            (
                'order_id'          => 'order_notes.order_id'
               ,'date'              => 'order_notes.order_note_date'
               ,'content'           => 'order_notes.order_note_content'
               ,'type'              => 'order_notes.order_note_type'
            );
        $tables[$order_notes]['types'] = array
            (
                'order_id'          => DBQUERY_FIELD_TYPE_INT . ' NOT NULL'
               ,'date'              => DBQUERY_FIELD_TYPE_DATETIME . ' NOT NULL default \'0000-00-00:00:00\''
               ,'content'           => DBQUERY_FIELD_TYPE_TEXT . ' NOT NULL'
               ,'type'              => DBQUERY_FIELD_TYPE_CHAR50 . ' NOT NULL'
            );

        # a table to store info about  Customer, Billing, Shipping
        $order_person_data = 'order_person_data';
        $tables[$order_person_data] = array();
        $tables[$order_person_data]['columns'] = array
            (
                'id'                => 'order_person_data.order_person_data_id'
               ,'order_id'          => 'order_person_data.order_id' // reference to orders
               ,'variant_id'        => 'order_person_data.person_info_variant_id' // reference to person_info_type
               ,'attribute_id'      => 'order_person_data.person_attribute_id' // reference to person_attribute
               ,'name'              => 'order_person_data.order_person_data_name'
               ,'value'             => 'order_person_data.order_person_data_value'
               ,'desc'              => 'order_person_data.order_person_data_description'
               ,'b_encrypted'       => 'order_person_data.order_person_data_b_encrypted'
               ,'encrypted_secret_key'
                                    => 'order_person_data.order_person_data_encrypted_secret_key'
               ,'rsa_public_key_asc_format'
                                    => 'order_person_data.order_person_data_rsa_public_key_asc_format'
            );
        $tables[$order_person_data]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT . ' NOT NULL auto_increment'
               ,'order_id'          => DBQUERY_FIELD_TYPE_INT . ' NOT NULL'
               ,'variant_id'        => DBQUERY_FIELD_TYPE_INT . ' NOT NULL'
               ,'attribute_id'      => DBQUERY_FIELD_TYPE_INT . ' NOT NULL'
               ,'name'              => DBQUERY_FIELD_TYPE_CHAR255 . ' NOT NULL'
               ,'value'             => DBQUERY_FIELD_TYPE_CHAR255
               ,'desc'              => DBQUERY_FIELD_TYPE_CHAR255
               ,'b_encrypted'       => DBQUERY_FIELD_TYPE_INT .' NOT NULL'
               ,'encrypted_secret_key'
                                    => DBQUERY_FIELD_TYPE_TEXT //Use CHAR256 instead of CHAR255 for 2048-bit RSA key.
               ,'rsa_public_key_asc_format'
                                    => DBQUERY_FIELD_TYPE_TEXT
            );
        $tables[$order_person_data]['primary'] = array
            (
                'id'
            );
        $tables[$order_person_data]['indexes'] = array
            (
                'IDX_unique' => 'order_id, variant_id, attribute_id, b_encrypted'
            );

        # a table for storing product info in the order
        $order_product = 'order_product';
        $tables[$order_product] = array();
        $tables[$order_product]['columns'] = array
            (
                'id'                => 'order_product.order_product_id'
               ,'order_id'          => 'order_product.order_id' //  reference to orders
               ,'qty'               => 'order_product.order_product_qty'
               ,'name'              => 'order_product.order_product_name'
               ,'type'              => 'order_product.order_product_type'
               ,'store_id'          => 'order_product.order_product_store_id'
            );
        $tables[$order_product]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT . ' NOT NULL auto_increment'
               ,'order_id'          => DBQUERY_FIELD_TYPE_INT . ' NOT NULL'
               ,'qty'               => DBQUERY_FIELD_TYPE_INT . ' NOT NULL default 1'
               ,'name'              => DBQUERY_FIELD_TYPE_CHAR255 . ' NOT NULL'
               ,'type'              => DBQUERY_FIELD_TYPE_CHAR255 . ' NOT NULL'
               ,'store_id'          => DBQUERY_FIELD_TYPE_INT
            );
        $tables[$order_product]['primary'] = array
            (
                'id'
            );

        # a table for storing product attribute info in the order
        $order_product_to_attributes = 'order_product_to_attributes';
        $tables[$order_product_to_attributes] = array();
        $tables[$order_product_to_attributes]['columns'] = array
            (
                'attribute_id'      => 'order_product_to_attributes.attribute_id' // reference to attributes in Catalog
               ,'product_id'        => 'order_product_to_attributes.order_product_id' // reference to order_product
               ,'value'             => 'order_product_to_attributes.order_product_attribute_value'
            );
        $tables[$order_product_to_attributes]['types'] = array
            (
                'attribute_id'      => DBQUERY_FIELD_TYPE_INT . ' NOT NULL'
               ,'product_id'        => DBQUERY_FIELD_TYPE_INT . ' NOT NULL'
               ,'value'             => DBQUERY_FIELD_TYPE_TEXT
            );

        # a table for storing product custom attribute info in the order
        $order_product_custom_attributes = 'order_product_custom_attributes';
        $tables[$order_product_custom_attributes] = array();
        $tables[$order_product_custom_attributes]['columns'] = array
            (
                'id'                => 'order_product_custom_attributes.order_product_custom_attribute_id'
               ,'product_id'        => 'order_product_custom_attributes.order_product_id' //  reference to order_product
               ,'tag'               => 'order_product_custom_attributes.order_product_custom_attribute_tag'
               ,'name'              => 'order_product_custom_attributes.order_product_custom_attribute_name'
               ,'value'             => 'order_product_custom_attributes.order_product_custom_attribute_value'
            );
        $tables[$order_product_custom_attributes]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT . ' NOT NULL auto_increment'
               ,'product_id'        => DBQUERY_FIELD_TYPE_INT . ' NOT NULL'
               ,'tag'               => DBQUERY_FIELD_TYPE_CHAR255 . ' NOT NULL'
               ,'name'              => DBQUERY_FIELD_TYPE_CHAR255 . ' NOT NULL'
               ,'value'             => DBQUERY_FIELD_TYPE_TEXT
            );
        $tables[$order_product_custom_attributes]['primary'] = array
            (
                'id'
            );

        # a table of calculated order prices
        $order_prices = 'order_prices';
        $tables[$order_prices] = array();
        $tables[$order_prices]['columns'] = array
            (
                'order_id'                              => 'order_prices.order_id'
               ,'minimum_shipping_cost'                 => 'order_prices.minimum_shipping_cost'
               ,'free_shipping_for_orders_over'         => 'order_prices.free_shipping_for_orders_over'
               ,'free_handling_for_orders_over'         => 'order_prices.free_handling_for_orders_over'
               ,'per_item_shipping_cost_sum'            => 'order_prices.per_item_shipping_cost_sum'
               ,'per_order_shipping_fee'                => 'order_prices.per_order_shipping_fee'
               ,'shipping_method_cost'                  => 'order_prices.shipping_method_cost'
               ,'total_shipping_charge'                 => 'order_prices.total_shipping_charge'
               ,'per_item_handling_cost_sum'            => 'order_prices.per_item_handling_cost_sum'
               ,'per_order_handling_fee'                => 'order_prices.per_order_handling_fee'
               ,'total_handling_charge'                 => 'order_prices.total_handling_charge'
               ,'total_shipping_and_handling_cost'      => 'order_prices.total_shipping_and_handling_cost'
               ,'order_tax_total'                       => 'order_prices.order_tax_total'
               ,'subtotal_global_discount'              => 'order_prices.subtotal_global_discount'
               ,'subtotal_promo_code_discount'          => 'order_prices.subtotal_promo_code_discount'
               ,'quantity_discount'                     => 'order_prices.quantity_discount'
               ,'discounted_subtotal'                   => 'order_prices.discounted_subtotal'
               ,'order_not_included_tax_total'          => 'order_prices.order_not_included_tax_total'
               ,'per_order_payment_module_shipping_fee' => 'order_prices.per_order_payment_module_shipping_fee'
            );
        $tables[$order_prices]['types'] = array
            (
                'order_id'                              => DBQUERY_FIELD_TYPE_INT . ' NOT NULL'
               ,'minimum_shipping_cost'                 => DBQUERY_FIELD_TYPE_DECIMAL12_2 . ' NOT NULL'
               ,'free_shipping_for_orders_over'         => DBQUERY_FIELD_TYPE_DECIMAL12_2 . ' NOT NULL'
               ,'free_handling_for_orders_over'         => DBQUERY_FIELD_TYPE_DECIMAL12_2 . ' NOT NULL'
               ,'per_item_shipping_cost_sum'            => DBQUERY_FIELD_TYPE_DECIMAL12_2 . ' NOT NULL'
               ,'per_order_shipping_fee'                => DBQUERY_FIELD_TYPE_DECIMAL12_2 . ' NOT NULL'
               ,'shipping_method_cost'                  => DBQUERY_FIELD_TYPE_DECIMAL12_2 . ' NOT NULL'
               ,'total_shipping_charge'                 => DBQUERY_FIELD_TYPE_DECIMAL12_2 . ' NOT NULL'
               ,'per_item_handling_cost_sum'            => DBQUERY_FIELD_TYPE_DECIMAL12_2 . ' NOT NULL'
               ,'per_order_handling_fee'                => DBQUERY_FIELD_TYPE_DECIMAL12_2 . ' NOT NULL'
               ,'total_handling_charge'                 => DBQUERY_FIELD_TYPE_DECIMAL12_2 . ' NOT NULL'
               ,'total_shipping_and_handling_cost'      => DBQUERY_FIELD_TYPE_DECIMAL12_2 . ' NOT NULL'
               ,'order_tax_total'                       => DBQUERY_FIELD_TYPE_DECIMAL12_2 . ' NOT NULL'
               ,'subtotal_global_discount'              => DBQUERY_FIELD_TYPE_DECIMAL12_2 . ' NOT NULL'
               ,'subtotal_promo_code_discount'          => DBQUERY_FIELD_TYPE_DECIMAL12_2 . ' NOT NULL'
               ,'quantity_discount'                     => DBQUERY_FIELD_TYPE_DECIMAL12_2 . ' NOT NULL'
               ,'discounted_subtotal'                   => DBQUERY_FIELD_TYPE_DECIMAL12_2 . ' NOT NULL'
               ,'order_not_included_tax_total'          => DBQUERY_FIELD_TYPE_DECIMAL12_2 . ' NOT NULL'
               ,'per_order_payment_module_shipping_fee' => DBQUERY_FIELD_TYPE_DECIMAL12_2 . ' NOT NULL'
            );
        $tables[$order_prices]['indexes'] = array
            (
                'UNIQUE IDX_unique' => 'order_id'
            );

        #                                          
        $order_taxes = 'order_taxes';
        $tables[$order_taxes] = array();
        $tables[$order_taxes]['columns'] = array
            (
                'order_id'          => 'order_taxes.order_id'
               ,'type'              => 'order_taxes.order_tax_type'
               ,'value'             => 'order_taxes.order_tax_value'
            );
        $tables[$order_taxes]['types'] = array
            (
                'order_id'          => DBQUERY_FIELD_TYPE_INT . ' NOT NULL'
               ,'type'              => DBQUERY_FIELD_TYPE_CHAR50 . ' NOT NULL'
               ,'value'             => DBQUERY_FIELD_TYPE_DECIMAL12_2 . ' NOT NULL'
            );
        $tables[$order_taxes]['indexes'] = array
            (
                'IDX_type' => 'type' // @todo
            );

        # the main table PERSONS
        $persons = 'persons';
        $tables[$persons] = array();
        $tables[$persons]['columns'] = array
            (
                'id'                => 'persons.person_id'
               ,'login'             => 'persons.person_login'
               ,'password'          => 'persons.person_password'
            );
        $tables[$persons]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'login'             => DBQUERY_FIELD_TYPE_CHAR255 . ' NOT NULL'
               ,'password'          => DBQUERY_FIELD_TYPE_CHAR255 . ' NOT NULL'
            );
        $tables[$persons]['primary'] = array
            (
                'id'
            );
        $tables[$persons]['indexes'] = array
            (
                'IDX_login' => 'login'
            );

        # a table of attributes to store person info
        $person_attributes = 'person_attributes';
        $tables[$person_attributes] = array();
        $tables[$person_attributes]['columns'] = array
            (
                'id'                => 'person_attributes.person_attribute_id'
               ,'tag'               => 'person_attributes.person_attribute_view_tag'
               ,'pattern_id'        => 'person_attributes.pattern_id' // reference to patterns (Catalog)
               ,'input_type_id'     => 'person_attributes.input_type_id' //          input_types (Catalog)
               ,'input_validation_func_name'     => 'person_attributes.input_validation_func_name' // the function name, called to check a user input (Checkout)
               ,'min'               => 'person_attributes.attribute_min_value'
               ,'max'               => 'person_attributes.attribute_max_value'
               ,'size'              => 'person_attributes.attribute_html_size'
            );
        $tables[$person_attributes]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'tag'               => DBQUERY_FIELD_TYPE_CHAR255 . ' NOT NULL'
               ,'pattern_id'        => DBQUERY_FIELD_TYPE_INT
               ,'input_type_id'     => DBQUERY_FIELD_TYPE_INT
               ,'input_validation_func_name' => DBQUERY_FIELD_TYPE_CHAR255 . ' NOT NULL'
               ,'min'               => DBQUERY_FIELD_TYPE_CHAR255
               ,'max'               => DBQUERY_FIELD_TYPE_CHAR255
               ,'size'              => DBQUERY_FIELD_TYPE_CHAR255
            );
        $tables[$person_attributes]['primary'] = array
            (
                'id'
            );

        # a table of person info type
        $person_info_types = 'person_info_types';
        $tables[$person_info_types] = array();
        $tables[$person_info_types]['columns'] = array
            (
                'id'                => 'person_info_types.person_info_type_id'
               ,'tag'               => 'person_info_types.person_info_type_tag'
               ,'name'              => 'person_info_types.person_info_type_name'
               ,'descr'             => 'person_info_types.person_info_type_description'
               ,'visible_name'      => 'person_info_types.person_info_type_visible_name'
               ,'active'            => 'person_info_types.person_info_type_active'
            );
        $tables[$person_info_types]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'tag'               => DBQUERY_FIELD_TYPE_CHAR50 . ' NOT NULL'
               ,'name'              => DBQUERY_FIELD_TYPE_CHAR255 . ' NOT NULL'
               ,'descr'             => DBQUERY_FIELD_TYPE_CHAR255
               ,'visible_name'      => DBQUERY_FIELD_TYPE_CHAR255 . ' NOT NULL'
               ,'active'            => DBQUERY_FIELD_BOOLEAN_DEFAULT_TRUE
            );
        $tables[$person_info_types]['primary'] = array
            (
                'id'
            );
        $tables[$person_info_types]['indexes'] = array
            (
                'IDX_name' => 'name'
               ,'UNIQUE IDX_tag' => 'tag'
            );

        # a table of info variants for person variants
        $person_info_variants = 'person_info_variants';
        $tables[$person_info_variants] = array();
        $tables[$person_info_variants]['columns'] = array
            (
                'id'                => 'person_info_variants.person_info_variant_id'
                //fk:
               ,'type_id'           => 'person_info_variants.person_info_type_id'
               ,'tag'               => 'person_info_variants.person_info_variant_tag'
               ,'name'              => 'person_info_variants.person_info_variant_name'
               ,'descr'             => 'person_info_variants.person_info_variant_description'
                //similar to person_type:
               ,'visible_name'      => 'person_info_variants.person_info_variant_visible_name'
            );
        $tables[$person_info_variants]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'type_id'           => DBQUERY_FIELD_TYPE_INT .' NOT NULL'
               ,'tag'               => DBQUERY_FIELD_TYPE_CHAR50 . ' NOT NULL'
               ,'name'              => DBQUERY_FIELD_TYPE_CHAR255 . ' NOT NULL'
               ,'descr'             => DBQUERY_FIELD_TYPE_CHAR255
               ,'visible_name'      => DBQUERY_FIELD_TYPE_CHAR255 . ' NOT NULL'
            );
        $tables[$person_info_variants]['primary'] = array
            (
                'id'
            );
        $tables[$person_info_variants]['indexes'] = array
            (
                'IDX_name' => 'name'
               ,'IDX_type_id' => 'type_id'
               ,'UNIQUE IDX_tag_type_id' => 'type_id, tag'
            );

        # a table links attributes to person info variants
        $person_info_variants_to_attributes = 'person_info_variants_to_attributes';
        $tables[$person_info_variants_to_attributes] = array();
        $tables[$person_info_variants_to_attributes]['columns'] = array
            (
                'variant_id'        => 'person_info_variants_to_attributes.person_info_variant_id' //           person_info_variants
               ,'attribute_id'      => 'person_info_variants_to_attributes.person_attribute_id' //           person_attributes
               ,'name'              => 'person_info_variants_to_attributes.person_attribute_visible_name'
               ,'descr'             => 'person_info_variants_to_attributes.person_attribute_description'
               ,'unremovable'       => 'person_info_variants_to_attributes.person_attribute_unremovable'
               ,'visible'           => 'person_info_variants_to_attributes.person_attribute_visible'
               ,'required'          => 'person_info_variants_to_attributes.person_attribute_required'
               ,'sort'              => 'person_info_variants_to_attributes.person_attribute_sort_order'
            );
        $tables[$person_info_variants_to_attributes]['types'] = array
            (
                'variant_id'        => DBQUERY_FIELD_TYPE_INT . ' NOT NULL'
               ,'attribute_id'      => DBQUERY_FIELD_TYPE_INT . ' NOT NULL'
               ,'name'              => DBQUERY_FIELD_TYPE_CHAR255 . ' NOT NULL'
               ,'descr'             => DBQUERY_FIELD_TYPE_CHAR255
               ,'unremovable'       => DBQUERY_FIELD_TYPE_BOOL
               ,'visible'           => DBQUERY_FIELD_TYPE_BOOL
               ,'required'          => DBQUERY_FIELD_TYPE_BOOL
               ,'sort'              => DBQUERY_FIELD_TYPE_INT
            );
        $tables[$person_info_variants_to_attributes]['primary'] = array
            (
                'variant_id', 'attribute_id'
            );

        # a table of real attribute values for persons
        $persons_data = 'persons_data';
        $tables[$persons_data] = array();
        $tables[$persons_data]['columns'] = array
            (
                'variant_id'        => 'persons_data.person_info_variant_id'
               ,'attribute_id'      => 'persons_data.person_attribute_id'
               ,'person_id'         => 'persons_data.person_id'
               ,'value'             => 'persons_data.person_attribute_value'
            );
        $tables[$persons_data]['types'] = array
            (
                'variant_id'        => DBQUERY_FIELD_TYPE_INT . ' NOT NULL'
               ,'attribute_id'      => DBQUERY_FIELD_TYPE_INT . ' NOT NULL'
               ,'person_id'         => DBQUERY_FIELD_TYPE_INT . ' NOT NULL'
               ,'value'             => DBQUERY_FIELD_TYPE_TEXT . ' NOT NULL'
            );
        $tables[$persons_data]['primary'] = array
            (
                'variant_id', 'attribute_id', 'person_id'
            );

        # a table links persons, person info variants and attribute values
        $person_to_info_variants = 'person_to_info_variants';
        $tables[$person_to_info_variants] = array();
        $tables[$person_to_info_variants]['columns'] = array
            (
                'person_id'         => 'person_to_info_variants.person_id' //  reference to persons
               ,'variant_id'        => 'person_to_info_variants.person_info_variant_id' //  reference to person_info_types
            );
        $tables[$person_to_info_variants]['types'] = array
            (
                'person_id'         => DBQUERY_FIELD_TYPE_INT . ' NOT NULL'
               ,'variant_id'        => DBQUERY_FIELD_TYPE_INT . ' NOT NULL'
            );
        $tables[$person_to_info_variants]['primary'] = array
            (
                'person_id', 'variant_id'
            );

        $settings = 'checkout_pm_sm_settings';
        $tables[$settings] = array();
        $tables[$settings]['columns'] = array
            (
                'id'                         => $settings.'.checkout_pm_sm_settings_id'
               ,'module_id'                  => $settings.'.checkout_pm_sm_settings_module_id'
               ,'module_class_name'          => $settings.'.checkout_pm_sm_settings_module_class_name'
               ,'module_group'               => $settings.'.checkout_pm_sm_settings_module_group'
               ,'status_active_value_id'     => $settings.'.checkout_pm_sm_settings_status_active_value_id'
               ,'status_selected_value_id'   => $settings.'.checkout_pm_sm_settings_status_selected_value_id'
               ,'sort_order'                 => $settings.'.checkout_pm_sm_settings_sort_order'

            );
        $tables[$settings]['types'] = array
            (
                'id'                         => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'module_id'                  => DBQUERY_FIELD_TYPE_CHAR50
               ,'module_class_name'          => DBQUERY_FIELD_TYPE_CHAR255 . ' NOT NULL'
               ,'module_group'               => DBQUERY_FIELD_TYPE_CHAR50
               ,'status_active_value_id'     => DBQUERY_FIELD_TYPE_INT .' NOT NULL'
               ,'status_selected_value_id'   => DBQUERY_FIELD_TYPE_INT .' NOT NULL'
               ,'sort_order'                 => DBQUERY_FIELD_TYPE_INT .' NOT NULL'
            );
        $tables[$settings]['primary'] = array
            (
                'id'
            );

        global $application;
        return $application->addTablePrefix($tables);
    }

    function setPrerequisitesValidationResults($NewPrerequisitesValidationResults)
    {
        $this->PrerequisitesValidationResults = $NewPrerequisitesValidationResults;
    }

    function getPrerequisitesValidationResults()
    {
        return $this->PrerequisitesValidationResults;
    }

    function clearAllNotMetPrerequisitesValidationResultsData()
    {
        foreach($this->PrerequisitesValidationResults as $PrerequisiteName => $ValidatedData)
        {
            $ValidatedData = &$this->PrerequisitesValidationResults[$PrerequisiteName];

            if($ValidatedData["isMet"] == false)
            {
                $ValidatedData["error_code"] = "";
                if(!empty($ValidatedData["validatedData"]))
                {
                    foreach($ValidatedData["validatedData"] as $PersonInfoAttributeName => $data)
                    {
                        $data = &$ValidatedData["validatedData"][$PersonInfoAttributeName];
                        if(($data["error_code_full"] != "") ||
                           ($data["error_code_short"] != ""))
                        {
                            $data["value"] = "";
                            $data["error_code_full"] = "";
                            $data["error_code_short"] = "";
                        }
                        unset($data);
                    }
                }
            }

            unset($ValidatedData);
        }
    }

   /*
     If we are at step i, clear error info for i+1 step and for
     all posterior ones: i+2 , ...
     A designer's decision.
     Mainly to remove outputting the error 004, where possible.
    */
    function clearNotMetPrerequisitesValidationResultsDataForAllPosteriorSteps($step_id)
    {
        $prerequisitesForAllSteps = $this->getPrerequisitesINI();

       /* FIXME the consequence of the steps (which of them follows which,
        how many steps at all...) in the Checkout CZ is not specified. Suppose,
        that the number of prerequisites can increase during the process.
        */
        $next_step_id = $step_id + 1;

        //$step_id can be the last step
        if(isset($prerequisitesForAllSteps[$next_step_id]))
        {
            foreach($this->PrerequisitesValidationResults as $PrerequisiteName => $PrerequisiteInfo)
            {
                if(!in_array($PrerequisiteName, $prerequisitesForAllSteps[$next_step_id])
                &&  multi_array_search($PrerequisiteName, $prerequisitesForAllSteps))
                {
                    $ValidatedData = &$this->PrerequisitesValidationResults[$PrerequisiteName];
                    if($ValidatedData["isMet"] == false)
                    {
                        $ValidatedData["error_code"] = "";
                        if(!empty($ValidatedData["validatedData"]))
                        {
                            foreach($ValidatedData["validatedData"] as $PersonInfoAttributeName => $data)
                            {
//                                echo '_'.$PersonInfoAttributeName.'<br>';
                                $data = &$ValidatedData["validatedData"][$PersonInfoAttributeName];
                                if(($data["error_code_full"] != "") ||
                                   ($data["error_code_short"] != ""))
                                {
                                    $data["value"] = "";
                                    $data["error_code_full"] = "";
                                    $data["error_code_short"] = "";
                                }
                                unset($data);
                            }
                        }
                    }
                    unset($ValidatedData);
                }
            }
        }
    }

   /* At the input, there are two copies of PersonInfo in the format prerequisite
       validated data, see Checkout::getValidatedDataStructure
       Fills all attributes in $to, existed in $from.
       The fields "error_code_full" and "error_code_short" are copied too.
    */
    function mergePersonInfo($from, &$to)
    {
        foreach($from as $attribute_tag => $attribute_data)
        {
            if(array_key_exists($attribute_tag, $to))
            {
                $to[$attribute_tag] = $from[$attribute_tag];
            }
        }
    }

    function changeCurrentPersonInfoVariant($type_tag /* prerequisite name */ , $new_person_info_variant_tag)
    {
        if(isset($this->PrerequisitesValidationResults[$type_tag]) &&
                 $this->PrerequisitesValidationResults[$type_tag]["variant_tag"] != $new_person_info_variant_tag)
        {
            $from = $this->PrerequisitesValidationResults[$type_tag];
            $prerequisite_name = $type_tag;
            $to = array();
            $to["validatedData"] = $this->getValidatedDataStructure($prerequisite_name, $new_person_info_variant_tag);
            $this->mergePersonInfo($from["validatedData"], $to["validatedData"]);

            $this->setPrerequisitesValidationResultsItem($prerequisite_name
                                                        ,$new_person_info_variant_tag
                                                        , /* isMet */ false
                                                        , /* error_code */ ""
                                                        , /* error_message_parameters */ array()
                                                        , $to["validatedData"]);
        }
    }

    function insertPersonInfoVariant($person_info_type_id
                                    ,$person_info_variant_tag
                                    ,$person_info_variant_name
                                    ,$person_info_variant_description
                                    ,$person_info_variant_visible_name
                                    ,$variant_to_attributes)
    {
        global $application;
        /*
         $variant_to_attributes - the array of the array type
         array
         (
            array
            (
                "person_attribute_id"
                "person_attribute_visible_name"
                "person_attribute_description"
                "person_attribute_visible"
                "person_attribute_required"
                "person_attribute_sort_order"
            )
            array
            (
                "person_attribute_id"
                "person_attribute_visible_name"
                "person_attribute_description"
                "person_attribute_visible"
                "person_attribute_required"
                "person_attribute_sort_order"
            )
         )
        */
        //@todo Add a validation of all input data.

        $tables = Checkout::getTables();
        $table = 'person_info_variants';
        $columns = $tables[$table]['columns'];

        $query = &new DB_Insert($table);
        $query->addInsertValue($person_info_type_id, $columns['type_id']);
        $query->addInsertValue($person_info_variant_tag, $columns['tag']);
        $query->addInsertValue($person_info_variant_name, $columns['name']);
        $query->addInsertValue($person_info_variant_description, $columns['descr']);
        $query->addInsertValue($person_info_variant_visible_name, $columns['visible_name']);
        $application->db->getDB_Result($query);
        $inserted_id = $application->db->DB_Insert_Id();

        foreach($variant_to_attributes as $attribute)
        {
            $table = 'person_info_variants_to_attributes';
            $columns = $tables[$table]['columns'];

            $query = &new DB_Insert($table);
            $query->addInsertValue($inserted_id, $columns['variant_id']);
            $query->addInsertValue($attribute['person_attribute_id'], $columns['attribute_id']);
            $query->addInsertValue($attribute['person_attribute_visible_name'], $columns['name']);
            $query->addInsertValue($attribute['person_attribute_description'], $columns['descr']);
            $query->addInsertValue($attribute['person_attribute_unremovable'], $columns['unremovable']);
            $query->addInsertValue($attribute['person_attribute_visible'], $columns['visible']);
            $query->addInsertValue($attribute['person_attribute_required'], $columns['required']);
            $query->addInsertValue($attribute['person_attribute_sort_order'], $columns['sort']);
            $application->db->getDB_Result($query);
        }

        return $inserted_id;
    }


    function getValidatedDataStructure($prerequisite_name, $person_info_variant_tag = NULL)
    /* prerequisite_name <=> type_id,
       (type_id, variant_tag) - UNIQUE
    */
    {
         //Delete the suffix of CCInfo prerequisites.
//$prerequisite_name_copy = $prerequisite_name;
        loadCoreFile('UUIDUtils.php');
        $prerequisite_name = UUIDUtils::cut_uuid_suffix($prerequisite_name, "js");
       /*
         Define the prerequisite variant: default, Paypal Pro, ...
         If it's possible to get it from the session, then take it.
         Otherwise - common.
        */

        global $application;
        // Possible values of $prerequisite_name:
        //  customerInfo;
        //  billingInfo;
        //  shippingInfo;
        //  shippingModuleAndMethod;
        //  paymentMethod;
        // Structure is in this case not a structure as internal arrangement, but more as
        //     a data set or an array, as in C++
        $value = array();
        if (strpos($prerequisite_name, "Module"))
        {
            switch ($prerequisite_name)
            {
                case "shippingModuleAndMethod":
                $value["method_code"] = array( 'view_tag'   => 'Shipping_Module_and_Method_Code',
                                                   'id'         => '',
                                                   'input_type_id' => '1',
                                                   'attribute_visible_name' => 'Shipping Method Code',
                                                   'attribute_description' => '',
                                                   'attribute_required' => true,
                                                   'value'      => '', //Method code, smth like 2_11 ,
                                                                       // where 2 = shipping module id
                                                                       //      11 = its submethod id
                                                   'pattern_id' => ''
                                                   ,'input_validation_func_name' => "is_valid_shipping_method_code"
                                                   ,'error_code_full' => ''
                                                   ,'error_code_short' => ''
                                                  );
                    break;
                case "paymentModule":
                $value["method_code"] = array( 'view_tag'   => 'PaymentModuleCode',
                                                   'id'         => '',
                                                   'input_type_id' => '1',
                                                   'attribute_visible_name' => 'Payment Method Code',
                                                   'attribute_description' => '',
                                                   'attribute_required' => true,
                                                   'value'      => '', //Method code, smth like 2_11 ,
                                                                       // where 2 = shipping module id
                                                                       //      11 = its submethod id
                                                   'pattern_id' => ''
                                                   ,'input_validation_func_name' => "is_valid_payment_method_code"
                                                   ,'error_code_full' => ''
                                                   ,'error_code_short' => ''
                                                  );
                    break;
                default:
                    //FIXME: output error message
                    break;
            }
        }
        else
        {
            $tables = $this->getTables();

            //...get id of person info variant from the session
            $piv  = $tables['person_info_variants']['columns'];
            $pivta = $tables['person_info_variants_to_attributes']['columns'];
            $pa   = $tables['person_attributes']['columns'];
            $pit = $tables['person_info_types']['columns'];

            $query = &new DB_Select();
            $query->addSelectField($pa['id'], 'id');
            $query->addSelectField($pa['tag'], 'view_tag');
            $query->addSelectField($pa['pattern_id'], 'pattern_id');
            $query->addSelectField($pa['input_type_id'], 'input_type_id');
            $query->addSelectField($pa['input_validation_func_name'], 'input_validation_func_name');
            $query->addSelectField($pa['min'], 'min');
            $query->addSelectField($pa['max'], 'max');
            $query->addSelectField($pa['size'], 'size');
            $query->addSelectField($pivta['name'], 'attribute_visible_name');
            $query->addSelectField($pivta['descr'], 'attribute_description');
            $query->addSelectField($pivta['required'], 'attribute_required');
            $query->addLeftJoin('person_info_variants_to_attributes', $pivta['variant_id'], DB_EQ, $piv['id']);
            $query->addLeftJoin('person_attributes', $pa['id'], DB_EQ, $pivta['attribute_id']);
            if(!empty($person_info_variant_tag))
            {
               /* The ID of the required variant was passed as a function parameter,
                  e.g. by the payment module request: "Paypal Pro billing info"
                */
                $query->WhereValue($piv['tag'], DB_EQ, $person_info_variant_tag);
                $query->WhereAnd();
                $query->WhereValue($pit['tag'], DB_EQ, $prerequisite_name);
                $query->addLeftJoin('person_info_types', $pit['id'], DB_EQ, $piv['type_id']);
            }
            else
            if(    isset($this->PrerequisitesValidationResults[$prerequisite_name]['variant_tag'])
               && !empty($this->PrerequisitesValidationResults[$prerequisite_name]['variant_tag']))
            {
                $query->WhereValue($piv['tag'], DB_EQ, $this->PrerequisitesValidationResults[$prerequisite_name]['variant_tag']);
                $query->WhereAnd();
                $query->WhereValue($pit['tag'], DB_EQ, $prerequisite_name);
                $query->addLeftJoin('person_info_types', $pit['id'], DB_EQ, $piv['type_id']);
            }
            else
            {
                //At the first initialization. The variant is by default.
                $query->WhereValue($piv['tag'], DB_EQ, "default");
                $query->WhereAnd();
                $query->WhereValue($pit['tag'], DB_EQ, $prerequisite_name);
                $query->addLeftJoin('person_info_types', $pit['id'], DB_EQ, $piv['type_id']);
            }
            $query->WhereAnd();
            $query->WhereValue($pivta['visible'], DB_EQ, 1);
            $query->SelectOrder($pivta['sort']);
            $result = $application->db->getDB_Result($query);
            foreach ($result as $attr)
            {
                //FIXME : ONE database person_attribute "State" is translated into TWO session attributes: "StateMenu" and "StateText".
                if($attr["view_tag"] == "State")
                {
                    $state_menu_name = "Statemenu";
                    $state_text_name = "Statetext";

                    $state_menu_pattern_id = "1"; //"integer"
                    $state_text_pattern_id = "3"; //"string"

                    $state_menu_validation_func_name = "is_valid_state_menu"; //"integer"
                    $state_text_validation_func_name = "is_valid_state_text"; //"string"

                    $value[$state_menu_name] = array
                                 (
                                    'view_tag'  => $state_menu_name
                                   ,'id'         => $attr["id"] //FIXME: sharing the same id among TWO sesssion attributes
                                   ,'input_type_id' => '3'
                                   ,'attribute_visible_name' => $attr["attribute_visible_name"]
                                   ,'attribute_description' => $attr["attribute_description"]
                                   ,'attribute_required' => $attr["attribute_required"]
                                   ,'value'     => ''
                                   ,'pattern_id'=> $state_menu_pattern_id
                                   ,'input_validation_func_name' => $state_menu_validation_func_name
                                   ,'error_code_full' => ''
                                   ,'error_code_short' => ''
                                   ,'max' => ''
                                   ,'size' => ''
                                 );

                    $value[$state_text_name] = array
                                 (
                                    'view_tag'  => $state_text_name
                                   ,'id'         => '' // $attr["id"] //FIXME: sharing the same id among TWO sesssion attributes
                                   ,'input_type_id' => '1'
                                   ,'attribute_visible_name' => ''
                                   ,'attribute_description' => ''
                                   ,'attribute_required' => ''
                                   ,'value'     => ''
                                   ,'pattern_id'=> $state_text_pattern_id
                                   ,'input_validation_func_name' => $state_text_validation_func_name
                                   ,'error_code_full' => ''
                                   ,'error_code_short' => ''
                                   ,'max' => $attr["max"]
                                   ,'size' => $attr["size"]
                                 );
                }
                else
                {
                    $value[$attr["view_tag"]] = array(
                                                        'view_tag'  => $attr["view_tag"]
                                                       ,'id'         => $attr["id"]
                                                       ,'input_type_id' => $attr["input_type_id"]
                                                       ,'attribute_visible_name' => $attr["attribute_visible_name"]
                                                       ,'attribute_description' => $attr["attribute_description"]
                                                       ,'attribute_required' => $attr["attribute_required"]
                                                       ,'value'     => ''
                                                       ,'pattern_id'=> $attr["pattern_id"]
                                                       ,'input_validation_func_name' => $attr["input_validation_func_name"]
                                                       ,'max' => $attr['max']
                                                       ,'size' => $attr['size']
                                                       ,'error_code_full' => ''
                                                       ,'error_code_short' => ''
                                                     );
                }
            }
        }
        return $value;
    }

    /**
     * WARNING:
     * It is called statically only. Before LoadState is called. At this moment
     * it is called in the middle of the constructor.
     * It updates session data, which will be used then in LoadState and
     * will be data of the Checkout class.
     */
    /*static*/ function initSinglePrerequisiteValidationResults($variant_view_tag, $variant_tag, $variant_type_id, $variant_visible_name)
    {
        global $application;
        loadCoreFile('UUIDUtils.php');

        //Cut the suffix. CCInfo prerequisites.
        //FIXME TODO: may be to pass in PersonInfoVariantID to getValidatedDataStructure, not the tag.
        $variant_view_tag_for_data_structure = UUIDUtils::cut_uuid_suffix($variant_view_tag, "js");

        {
            /**
             * Call either before the construcor finishes its work,
             * or satatically.
             * Work on the sesson only.
             */
            //Check, if data is already in the session:
            if(modApiFunc('Session', 'is_Set', 'PrerequisitesValidationResults') === true)
            {
                $PrerequisitesValidationResults = modApiFunc('Session', 'get', 'PrerequisitesValidationResults');
            }
            else
            {
                //Initialize from the beginning.
                $PrerequisitesValidationResults = array();
            }
            $PrerequisitesValidationResults[$variant_view_tag] = array
                (
                    'variant_tag'   => $variant_tag,
                    'isMet'         => false,
                    'id'            => $variant_type_id,
                    'visibleName'   => $variant_visible_name,
                    'error_code'    => '',
                    'error_message_parameters' => '',
                    'validatedData' => $this->getValidatedDataStructure($variant_view_tag_for_data_structure)
                );

            //         shipping_method,                           ,  . .            
            //                                     ,                         .
            //                   CheckoutFormEditor -                              .
            if($variant_view_tag == "shippingModuleAndMethod")
            {
                $b_is_shipping_method_required = Checkout::isPrerequisiteRequired("shippingModuleAndMethod");
                if($b_is_shipping_method_required === false ||
                   Checkout::arePersonInfoTypesActive(array("shippingModuleAndMethod")) === false)
                {
                    //                             shipping method: AllInactive
                    $PrerequisitesValidationResults[$variant_view_tag]['isMet'] = true;
                    $PrerequisitesValidationResults[$variant_view_tag]['validatedData']['method_code']['value'] =
                        Checkout::getAllInactiveModuleId("shipping")
                      . "_"
                      . modApiFunc(Checkout::getAllInactiveModuleClassAPIName("shipping"), "getSingleAvailableMethodId");
                }
            }
            //         payment_method,                           ,  . .            
            //                                     ,                         .
            //                   CheckoutFormEditor -                              .
            if($variant_view_tag == "paymentModule")
            {
                $b_is_payment_method_required = Checkout::isPrerequisiteRequired("paymentModule");
                if($b_is_payment_method_required === false ||
                   Checkout::arePersonInfoTypesActive(array("paymentModule")) === false)
                {
                    //                             payment method: AllInactive
                    $PrerequisitesValidationResults[$variant_view_tag]['isMet'] = true;
                    $PrerequisitesValidationResults[$variant_view_tag]['validatedData']['method_code']['value'] =
                        Checkout::getAllInactiveModuleId("payment");
                }
            }
            modApiFunc('Session', 'set', 'PrerequisitesValidationResults', $PrerequisitesValidationResults);
        }
    }

    /* static */ function initPrerequisitesValidationResults()
    {
        global $application;
        $tables = Checkout::getTables();

        $pit  = $tables['person_info_types']['columns'];
        $piv  = $tables['person_info_variants']['columns'];
        /* a tag, which is selected by default */
        $variant_tag  = "default";

        $query = &new DB_Select();
        $query->addSelectField($piv['id'], 'variant_id');
        $query->addSelectField($piv['tag'], 'variant_tag');
        $query->addSelectField($piv['visible_name'], 'visible_name');
        $query->addSelectField($piv['tag'], 'variant_tagvisible_name');
        $query->addSelectField($pit['tag'], 'view_tag');
        $query->addSelectField($pit['id'], 'type_id');
        $query->Where($piv['type_id'], DB_EQ, $pit['id']);
        $query->WhereAnd();
        $query->Where($piv['tag'], DB_EQ, '"'. $variant_tag . '"');
        $result = $application->db->getDB_Result($query);
        foreach ($result as $variant)
        {
            Checkout::initSinglePrerequisiteValidationResults($variant["view_tag"], $variant["variant_tag"], $variant["type_id"], $variant["visible_name"]);
        }
        /**
         * Hard-coded prerequisites
         */
        Checkout::initSinglePrerequisiteValidationResults('shippingModuleAndMethod', '', '', 'Shipping Module and Method');
        Checkout::initSinglePrerequisiteValidationResults('paymentModule', '', '', 'Payment Method');
    }

    /**
     *            -   payment              Bank Account     Credit Card                  ,
     *                                       Checkout.
     * Checkout                    prerequisite,                    
     *                                               .
     */
    /*static*/ function initAdditionalPrerequisitesValidationResults($prerequisite_type)//creditCardInfo, bankAccountInfo
    {
        /* Inquire all Payment modules.
           If the method getInitPrerequisitesValidationResultsData exists - call it
           and add a corresponding prerequisite.
        */
        global $application;
        /* create the object Modules_Manager to upload all required modules */
        $mmObj = &$application->getInstance('Modules_Manager');
        $modules = Checkout::getInstalledAndActiveModulesListData("payment");
        foreach($modules as $moduleInfo)
        {
            /* Load a required file */
            $mmObj->includeAPIFileOnce($moduleInfo->name);
            /* This condition can be checked only after loading */
            if(is_callable(array($moduleInfo->name,"getAdditionalPersonInfoVariantTag")))
            {
                /* It MUST be called statically,
                 * otherwise, if modApiFunc is used, the object of this module
                 * will be created, which will lead to creation
                 * of the checkout object, so the payment/shipping module will
                 * ask the checkout object for isActive, and that will lead to
                 * the complete cycling. */
                $variant_tag = call_user_func(array($moduleInfo->name, 'getAdditionalPersonInfoVariantTag'), $prerequisite_type);
                //false                     ,                       Checkout CZ              -         
                //                                    CCInfo        : AuthorizeNet, PaypalPro.         
                //            ,                                     CCInfo,        ,                      -
                //         false.                    CCInfo                        .
                if($variant_tag !== false)
                {
                    $prerequisite_name = Checkout::getAdditionalPrerequisiteName($prerequisite_type, call_user_func(array($moduleInfo->name, "getUid")));

                    $variant_id = Checkout::getPersonInfoVariantId($prerequisite_type, $variant_tag);
                    $variant_info = Checkout::getPersonVariantInfo($variant_id);

                    $variant_view_tag = $prerequisite_name;
                    $prerequisite_data =
                    array
                    (
                        "variant_view_tag"     => $variant_view_tag
                       ,"variant_tag"          => $variant_info['variant_tag']
                       ,"variant_type_id"      => $variant_info['type_id']
                       ,"variant_visible_name" => $variant_info['visible_name']
                    );

                    Checkout::initSinglePrerequisiteValidationResults
                    (
                        $prerequisite_data["variant_view_tag"]
                       ,$prerequisite_data["variant_tag"]
                       ,$prerequisite_data["variant_type_id"]
                       ,$prerequisite_data["variant_visible_name"]
                    );
                }
            }
            else
            {
                //echo "DIE not callable: ". $moduleInfo->name . " getInitPrerequisitesValidationResultsData();" . "<br>";
            }
        }
    }

    function setChosenShippingMethod($module_id, $method_id)
    {
        if($this->areValidShippingModuleAndMethodIDs($module_id, $method_id))
        {
            $shippingModuleAndMethod = $this->getPrerequisiteValidationResults("shippingModuleAndMethod");
            $validatedData = $shippingModuleAndMethod['validatedData'];
            $validatedData["method_code"]["value"] = $module_id . '_' . $method_id;
            $this->setPrerequisitesValidationResultsItem('shippingModuleAndMethod' /*prerequisiteName*/,
                                                         '' /*variant_tag*/,
                                                         true /*isMet*/,
                                                         '' /*error_code*/,
                                                         '' /*error_message_parameters*/,
                                                         $validatedData);
        }
    }

    /**
     * Restores the module state.
     */
    function loadState()
    {
        if(modApiFunc('Session', 'is_Set', 'currentStepID'))
        {
            $this->setCurrentStepID(modApiFunc('Session', 'get', 'currentStepID'));
        }
        else
        {
            //FIXME. Is it at all correct to SET currentStepID, if its value is not even defined?
            $this->currentStepID = 1;
        }

        if(modApiFunc('Session', 'is_Set', 'currentOrderID'))
        {
            $this->currentOrderID = modApiFunc('Session', 'get', 'currentOrderID');
        }

        if(modApiFunc('Session', 'is_Set', 'lastPlacedOrderID'))
        {
            $this->lastPlacedOrderID = modApiFunc('Session', 'get', 'lastPlacedOrderID');
        }

        if(modApiFunc('Session', 'is_Set', 'currentCustomerID'))
        {
            $this->currentCustomerID = modApiFunc('Session', 'get', 'currentCustomerID');
        }

        if(modApiFunc('Session', 'is_Set', 'PrerequisitesValidationResults'))
        {
            $this->setPrerequisitesValidationResults(modApiFunc('Session', 'get', 'PrerequisitesValidationResults'));
        }
        else
        {
            $err_params = array(
                                "CODE"    => "CHECKOUT_011"
                               );
            _fatal($err_params);
            //$this->initPrerequisitesValidationResults();
        }

        if(modApiFunc('Session', 'is_Set', 'CheckoutOrderSearch'))
        {
            $this->order_search_filter = modApiFunc('Session', 'get', 'CheckoutOrderSearch');
        }
        else
        {
            $this->order_search_filter = array(
                'filter_status_id' => '1'
               ,'status_id' => null
               ,'payment_status_id' => null
               ,'from_day' => null
               ,'from_month' => null
               ,'from_year' => null
               ,'to_day' => null
               ,'to_month' => null
               ,'to_year' => null
               ,'order_id' => null
               ,'search_by' => 'status'
            );
            modApiFunc('Session', 'set', 'CheckoutOrderSearch', $this->order_search_filter);
        }

        if(modApiFunc('Session', 'is_Set', 'CheckoutCustomerSearch'))
        {
            $this->customer_search_filter = modApiFunc('Session', 'get', 'CheckoutCustomerSearch');
        }
        else
        {
            $this->customer_search_filter = array(
               'search_by' => 'letter' // 'letter' or 'field'
              ,'letter' => null
              ,'field_name' => null
              ,'field_value' => null
            );
            modApiFunc('Session', 'set', 'CheckoutCustomerSearch', $this->customer_search_filter);
        }

        if(modApiFunc('Session', 'is_Set', 'CurrentPaymentModuleSettingsViewName'))
        {
            $this->setCurrentPaymentModuleSettingsViewName(modApiFunc('Session', 'get', 'CurrentPaymentModuleSettingsViewName'));
        }
        else
        {
            $this->CurrentPaymentModuleSettingsViewName = NULL;
        }

        if(modApiFunc('Session', 'is_Set', 'CustomPaymentGatewayPageContents'))
        {
            $this->setCustomPaymentGatewayPageContents(modApiFunc('Session', 'get', 'CustomPaymentGatewayPageContents'));
        }
        else
        {
            $this->CustomPaymentGatewayPageContents = NULL;
        }


        if(modApiFunc('Session', 'is_Set', 'CurrentShippingModuleSettingsViewName'))
        {
            $this->setCurrentShippingModuleSettingsViewName(modApiFunc('Session', 'get', 'CurrentShippingModuleSettingsViewName'));
        }
        else
        {
            $this->CurrentShippingModuleSettingsViewName = NULL;
        }

        if(modApiFunc('Session', 'is_Set', 'OrdersForDelete'))
        {
            $this->ordersId = modApiFunc('Session', 'get', 'OrdersForDelete');
        }
        else
        {
            $this->ordersId = array();
        }

        if(modApiFunc('Session', 'is_Set', 'DeleteOrdersFlag'))
        {
            $this->DeleteOrdersFlag = modApiFunc('Session', 'get', 'DeleteOrdersFlag');
            modApiFunc('Session', 'un_Set', 'DeleteOrdersFlag');
        }
        else
        {
            $this->DeleteOrdersFlag = 'false';
        }


        if(modApiFunc('Session', 'is_Set', 'CHECKOUT_CZ_BLOWFISH_KEY_IS_ALREADY_DEFINED'))
        {
            $this->CHECKOUT_CZ_BLOWFISH_KEY_IS_ALREADY_DEFINED = modApiFunc('Session', 'get', 'CHECKOUT_CZ_BLOWFISH_KEY_IS_ALREADY_DEFINED');
        }

//        //FIXME:                                         ,                         ,
//        //                                          -                    .         
//        //                          .                   prerequisite.
//        $this->initStoreOwnerInfo();
    }

    /**
     * Saves the module state.
     */
    function saveState()
    {
        modApiFunc('Session', 'set', 'currentStepID', $this->currentStepID);
        modApiFunc('Session', 'set', 'currentOrderID', $this->currentOrderID);
        modApiFunc('Session', 'set', 'lastPlacedOrderID', $this->lastPlacedOrderID);
        modApiFunc('Session', 'set', 'currentCustomerID', $this->currentCustomerID);
        modApiFunc('Session', 'set', 'PrerequisitesValidationResults', $this->PrerequisitesValidationResults);
        modApiFunc('Session', 'set', 'CheckoutOrderSearch', $this->order_search_filter);
        modApiFunc('Session', 'set', 'CheckoutCustomerSearch', $this->customer_search_filter);
        modApiFunc('Session', 'set', 'CurrentPaymentModuleSettingsViewName', $this->CurrentPaymentModuleSettingsViewName);
        modApiFunc('Session', 'set', 'CustomPaymentGatewayPageContents', $this->CustomPaymentGatewayPageContents);
        modApiFunc('Session', 'set', 'CurrentShippingModuleSettingsViewName', $this->CurrentShippingModuleSettingsViewName);
        modApiFunc('Session', 'set', 'OrdersForDelete', $this->ordersId);
        modApiFunc('Session', 'set', 'DeleteOrdersFlag', $this->DeleteOrdersFlag);
        modApiFunc('Session', 'set', 'CHECKOUT_CZ_BLOWFISH_KEY_IS_ALREADY_DEFINED', $this->CHECKOUT_CZ_BLOWFISH_KEY_IS_ALREADY_DEFINED);
    }

    /**
     * Checks if module has been installed.
     *
     * @todo finish the functions on this page
     * @return bool TRUE if module has been installed, FALSE otherwise
     */
    function isInstalled()
    {
    }

    /* static */ function getActiveModules($modulesType)
    {
        global $application;
        $tables = Checkout::getTables();
        $columns = $tables['checkout_pm_sm_settings']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($columns["module_id"], "module_id");
        $query->addSelectField($columns["module_class_name"], "module_class_name");
        $query->addSelectField($columns["status_active_value_id"], "status_active_value_id");
        $query->addSelectField($columns["status_selected_value_id"], "status_selected_value_id");
        $query->addSelectField($columns["sort_order"], "sort_order");
        $query->WhereValue($columns["status_active_value_id"], DB_EQ, 1);
        $query->WhereAnd();

        $query->WhereValue($columns["module_group"], DB_LIKE, "%".$modulesType);
        $sql_result = $application->db->getDB_Result($query);

        $result = array();
        for ($i=0; $i<sizeof($sql_result); $i++)
        {
            $result[$sql_result[$i]["module_id"]] = array
            (
                "module_id"              => $sql_result[$i]["module_id"]
               ,"module_class_name"      => $sql_result[$i]["module_class_name"]
            );
        }
        return $result;
    }

    //modulesType -   lower case
    function getSelectedModules($modulesType)
    {
        global $application;
        $tables = $this->getTables();
        $columns = $tables['checkout_pm_sm_settings']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($columns["module_id"], "module_id");
        $query->addSelectField($columns["module_class_name"], "module_class_name");
        $query->addSelectField($columns["status_active_value_id"], "status_active_value_id");
        $query->addSelectField($columns["status_selected_value_id"], "status_selected_value_id");
        $query->addSelectField($columns["sort_order"], "sort_order");
        $query->WhereValue($columns["status_selected_value_id"], DB_EQ, 1);
        $query->WhereAnd();

        $query->WhereValue($columns["module_group"], DB_LIKE, "%".$modulesType);
        $sql_result = $application->db->getDB_Result($query);

        $result = array();
        for ($i=0; $i<sizeof($sql_result); $i++)
        {
            $result[$sql_result[$i]["module_id"]] = array
            (
                "module_id"              => $sql_result[$i]["module_id"]
               ,"module_class_name"      => $sql_result[$i]["module_class_name"]
               ,"module_group"           => $modulesType
               ,"b_is_active"            => $sql_result[$i]["status_active_value_id"] == 1 ? true : false
//               ,"b_is_selected"          => $result[$i]["status_selected_value_id"] == 1 ? true : false
               ,"sort_order"             => $sql_result[$i]["sort_order"]
            );
        }

        //If such modules don't exist, make all-inactive be selected and active. Save the settings.
        if(empty($result))
        {
            $all_inactive_module_id = Checkout::getAllInactiveModuleId($modulesType);
            $all_inactive_module  = array
            (
                "module_id"              => $all_inactive_module_id
               ,"module_group"           => $modulesType
//               ,"b_is_active"            => true
               ,"b_is_selected"          => true
               ,"sort_order"             => 0
            );

            $modules = array();
            $modules[$all_inactive_module_id] = $all_inactive_module;

            $this->setSelectedModules($modules, $modulesType, true);
            Checkout::setModuleActive($all_inactive_module_id, true);
            return $this->getSelectedModules($modulesType);
        }
        else
        {
            return $result;
        }
    }

    /**
     * Sets up "is_selected" in false, and also "is_active" in false in all
     * modules, except the passed in ones. It sets "is_selected" in true in all
     * passed modules,that exist in the database. It adds the modules, that don't
     * exist in the table. So that it should be specified in the inputted data
     * which data should be added as "active", and which of them shouldn't.
     *
     * Parameters have a complicated structure, because words Selected and Active
     * for modules depend: if a module is not Selected, it can't be Active.
     * And also because the status Selected/non-Selected is usually changed for
     * several modules at once. Administrator choses, which modules should be
     * Selected, which of them are non-Selected and then presses the button:
     * save the changes.
     */
    function setSelectedModules($modules, $modulesType, $b_settings_table_is_empty = false)
    {
        global $application;
        //The list of modules,which should become selected.
        $selected_modules_ids = array();
        foreach($modules as $module_id => $module_settings)
        {
            $selected_modules_ids[] = "'" . $module_id . "'";
        }

        $tables = $this->getTables();
        $columns = $tables['checkout_pm_sm_settings']['columns'];
        $YES = "1";
        $NO  = "2";

        //Sets up "is_selected" in false, and also "is_active" in false in all modules, except the passed in ones.
        $query =& new DB_Update('checkout_pm_sm_settings');
        $query->addUpdateExpression($columns['status_selected_value_id'], $NO);
        $query->addUpdateExpression($columns['status_active_value_id']  , $NO);
        $query->WhereField($columns['module_id'], DB_NIN, "(" . implode(",", $selected_modules_ids) . ")");
        $query->WhereAnd();
        $query->WhereValue($columns['module_group'], DB_EQ, $modulesType);
        $application->db->getDB_Result($query);

        //Set "is_selected" in true, and also sort_order in all passed modules that exist in the database.
        //It adds the modules, that don't exist in the table. So that it should be specified in the inputted data
        //which data should be added as "active", and which of them shouldn't.
        //First get the list of modules, entered to the database:
        if($b_settings_table_is_empty === true)
        {
            $modules_already_in_db = array();
        }
        else
        {
            $modules_already_in_db = $this->getSelectedModules($modulesType);
        }

        foreach($modules as $module_id => $module_settings)
        {
            if(!array_key_exists($module_id, $modules_already_in_db))
            {
                //The module doesn't exist in the database.
                //Add module settings to the database.

                //Get the module name from ModuleManager:
                if(!isset($mm_info))
                {
                    $mm_info = Checkout::getInstalledModulesListData($modulesType, NULL, true);
                    //Sort by UUID.
                    $mm_info_indexed_by_uuid = array();
                    foreach($mm_info as $key => $m_info)
                    {
                        $mm_info_indexed_by_uuid[$m_info->UUID] = $m_info;
                    }
                }

                $db_insert = new DB_Insert('checkout_pm_sm_settings');
                $db_insert->addInsertValue($module_settings['module_id'], $columns['module_id']);
                $db_insert->addInsertValue($mm_info_indexed_by_uuid[$module_settings['module_id']]->name, $columns['module_class_name']);
                $db_insert->addInsertValue($module_settings['module_group'], $columns['module_group']);
                $db_insert->addInsertValue(//$module_settings['b_is_active'] === true ? $YES :
                                           $NO, $columns['status_active_value_id']);
                $db_insert->addInsertValue($module_settings['b_is_selected'] === true ? $YES : $NO, $columns['status_selected_value_id']);
                $db_insert->addInsertValue($module_settings['sort_order'], $columns['sort_order']);
                $application->db->PrepareSQL($db_insert);
                $application->db->DB_Exec();
            }
            else
            {
                //The module exists in the database. Update sort_order and is_selected.
                $query =& new DB_Update('checkout_pm_sm_settings');
                $query->addUpdateExpression($columns['status_selected_value_id'], $YES);
                $query->addUpdateExpression($columns['sort_order'], $module_settings['sort_order']);
                $query->WhereValue($columns['module_id'], DB_EQ, $module_id);
                $application->db->getDB_Result($query);
            }
        }
    }

    /*static*/ function setModuleActive($module_id, $b_active)
    {
        //The condition, where a module doesn't exist in the Checkout table
        // settings, and setModuleActive is called for it, should
        //not exist. So you can get to the place only from the page of
        // module settings, or by automatic operatins into the system
        // (for example, by adding all_inactive to the table at the begining,
        //  when the table is empty).
        //In the first case to open the page of settings, the module should
        // be Selected, and has been added to the setting table before.
        //In the second case hte function setModuleActive is not called. The flag
        // Active is passed in within data for the function setSelectedModules.


        //Delete operations with the falg Active from the function setSelectedModules,
        // it may take much time. For all modules, which become non-Selected,
        // should be set the status non-Active.
        //But to simplify the code, they can be separated.

        global $application;
        $tables = Checkout::getTables();
        $columns = $tables['checkout_pm_sm_settings']['columns'];

        $YES = "1";
        $NO  = "2";
        $query =& new DB_Update('checkout_pm_sm_settings');
        $query->addUpdateExpression($columns['status_active_value_id'], $b_active === true ? $YES : $NO);
        $query->WhereValue($columns['module_id'], DB_EQ, $module_id);
        $application->db->getDB_Result($query);
        //FIXME: add the check to update only one string
        // If zero strings are updated, then an error occurred: the module
        // doesn't exist  in the table.
    }

    /**
     * Installs the module.
     *
     * The install() method is called statically.
     * To call other methods of this class from this method,
     * the static call is used, for example,
     * Checkout::getTables() instead of $this->getTables()
     *
     * @todo copy to create only necessary tables and data.
     */
    function install()
    {
        _use(dirname(__FILE__).'/includes/install.inc');
    }

    /**
     * Uninstalls the module.
     *
     * The uninstall() method is called statically.
     * To call other methods of this class from this method,
     * the static call is used, for example,
     * Checkout::getTables() instead of $this->getTables().
     *
     * @todo finish the functions on this page
     */
    function uninstall()
    {
    }

    /**
     * Sets the current checkout step ID.
     *
     * @return mixed Current step ID or NULL if category is not selected
     */
    function setCurrentStepID($step_id)
    {
        $this->currentStepID = $step_id;
    }

    /**
     * Gets the current checkout step ID.
     *
     * @return mixed Current step ID or NULL if step is not defined
     */
    function getCurrentStepID()
    {
        return $this->currentStepID;
    }

    /**
     * Returns info about current used chain Checkout.
     * For each step :
     * ID,
     * Name,
     * Link (a reference to the page, by clicking it, a user
     *       goes to the appropriate step checkout)
     */
    function getStepsInfo()
    {
        global $application;
        $prerequisitesINI = $this->getPrerequisitesINI();
        $retval = array();
        $step_id = 0;
        foreach($prerequisitesINI as $prerequisiteINI)
        {
            $step_id++;
            $request = new Request();
            $request->setAction('SetCurrStep');
            $request->setView('CheckoutView');
            $request->setKey('step_id', $step_id);
            $request = $this->appendCheckoutCZGETParameters($request);

            $retval[] = array("ID" => $step_id,
                              "Name" => $step_id,
                              "Link" => $request->getURL()
                             );
        }
        return $retval;
    }

    /**
     * Get/Set function. See the description of the variable
     * CustomPaymentGatewayPageContents.
     */
    function setCustomPaymentGatewayPageContents($NewCustomPaymentGatewayPageContents)
    {
        $this->CustomPaymentGatewayPageContents = $NewCustomPaymentGatewayPageContents;
    }

    /**
     *  Get/Set function. See the description of the variable
     * CustomPaymentGatewayPageContents.
     */
    function getCustomPaymentGatewayPageContents()
    {
        return $this->CustomPaymentGatewayPageContents;
    }

    /**
     * Sets currently selected Order ID.
     */
    function setCurrentOrderID($order_id)
    {
        $this->currentOrderID = $order_id;
    }

    /**
     * Sets last placed order Order ID.
     */
    function setLastPlacedOrderID($order_id)
    {
        $this->lastPlacedOrderID = $order_id;
    }

    /**
     * Returns currently selected Order ID.
     */
    function getCurrentOrderID()
    {
        return $this->currentOrderID;
    }

    /**
     * Returns last placed Order ID.
     */
    function getLastPlacedOrderID()
    {
        return $this->lastPlacedOrderID;
    }

    /**
     * Sets currently selected Customer ID.
     */
    function setCurrentCustomerID($customer_id)
    {
        $this->currentCustomerID = $customer_id;
    }

    /**
     * Returns currently selected Customer ID.
     */
    function getCurrentCustomerID()
    {
        return $this->currentCustomerID;
    }

    function getPrerequisitesListForStep($step_id)
    {
        $prerequisitesForAllSteps = $this->getPrerequisitesINI();
        if(!isset($prerequisitesForAllSteps[$step_id]))
        {
            $err_params = array(
                                "CODE"    => "CHECKOUT_CONFIG_INI_001",
                                "FILE"    => "checkout-config.ini",
                                "SECTION" => "CheckoutNormal",
                                "DIRECTIVE" => "Option-prerequisite" . $step_id
                                );
            _fatal($err_params, $step_id);
        }
        else
        {
            return $prerequisitesForAllSteps[$step_id];
        }
    }

    /**
     *                                 checkout-sequence,         -required.
     *       - required.
     */
    function isPrerequisiteRequired($prerequisite_name)
    {
        $prerequisites = Checkout::getPrerequisitesINI();
        foreach($prerequisites as $step_id => $names)
        {
            if(in_array($prerequisite_name, $names))
            {
                return true;
            }
        }
        return false;
    }

    function getPrerequisitesINI()
    {
        global $application;
        $prerequisites = array();
        $prerequisites_tag = "prerequisite";

        $template = $application->getBlockTemplate('CheckoutView');

        if ($template == null)
            return $prerequisites;

        //TODO Move the check to application and make it more strict.
        for($i=1 /* 2 */ ;;$i++)
        {
            $prerequisite = $application->getBlockOption($template, $prerequisites_tag . $i);
            if( $prerequisite === null)
            {
                break;
            }
            else
            {
                if(trim($prerequisite) == "")
                {   //to avoid array(0=>"") as a result
                    $prerequisites[$i] = array();
                }
                else
                {
                    $tmp = split(",", trim($prerequisite));
                    $prerequisites[$i] = array();
                    foreach($tmp as $prerequisite_name)
                    {
                        $prerequisites[$i][] = trim($prerequisite_name);
                    }
                }
                //break;
            }
        }

        return $prerequisites;
    }

    function setPrerequisitesValidationResultsItem($index, $variant_tag, $isMet, $error_code, $error_message_parameters, $validatedData)
    //"customer-info", data)
    {
        //TODO check if $this->PrerequisitesValidationResults array exists at all.
        $this->PrerequisitesValidationResults[$index]['variant_tag'] = $variant_tag;
        $this->PrerequisitesValidationResults[$index]['isMet'] = $isMet;
        $this->PrerequisitesValidationResults[$index]['error_code'] = $error_code;
        $this->PrerequisitesValidationResults[$index]['error_message_parameters'] = $error_message_parameters;
        $this->PrerequisitesValidationResults[$index]['validatedData'] = $validatedData;
        //TODO replace with $this->PrerequisitesValidationResults[$index] = array (...) ?
    }

//    function setPrerequisitesValidationResultsItemError($index, $error_code, $error_message_parameters)
//    {
//        //TODO check if $this->PrerequisitesValidationResults array exists at all.
//        $this->PrerequisitesValidationResults[$index]['error_code'] = $error_code;
//        $this->PrerequisitesValidationResults[$index]['error_message_parameters'] = $error_message_parameters;
//        //TODO replace with $this->PrerequisitesValidationResults[$index] = array (...) ?
//    }

    function is_valid_shipping_method_code($val)
    {
        //// Module and Method
        $shipping_module_and_method_pattern = "/^([0-9A-F]{8}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{12})_(\d+)$/";

        $matches = array();
        if(preg_match($shipping_module_and_method_pattern, $val, $matches))
        {
            if($this->areValidShippingModuleAndMethodIDs($matches[1], $matches[2]))
            {
                return true;
            }
            else
            {
                //Method id forge attempt.
                return array('error_code_full' => 'CHECKOUT_ERR_SHIPPING_METHOD_002',
                             'error_code_short' => 'CHECKOUT_ERR_SHIPPING_METHOD_002',
                             'error_message_parameters' => array($matches[1], $matches[2]));
            }
        }
        else
        {
            return array('error_code_full' => 'CHECKOUT_ERR_FORM_FIELD_FULL_002'

            ,'error_code_short' => 'CHECKOUT_ERR_FORM_FIELD_SHORT_002'
//            ,'error_message_parameters' => array($this->PrerequisitesValidationResults["shippingModuleAndMethod"]["validatedData"]["method_code"]["view_tag"], "shippingModuleAndMethod")
                        );
        }
    }

    function is_valid_payment_method_code($val)
    {
        //// Module (without Method)
        $shipping_module_pattern = "/^([0-9A-F]{8}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{12})$/";

        $matches = array();
        if(preg_match($shipping_module_pattern, $val, $matches))
        {
            if($this->isValidPaymentModuleID($matches[1]))
            {
                return true;
            }
            else
            {
                return array('error_code_full' => 'CHECKOUT_ERR_PAYMENT_METHOD_002',
                             'error_code_short' => 'CHECKOUT_ERR_PAYMENT_METHOD_002',
                             'error_message_parameters' => array($matches[1]));
            }
        }
        else
        {
            return array('error_code_full' => 'CHECKOUT_ERR_PAYMENT_METHOD_001'
                        ,'error_code_short' => 'CHECKOUT_ERR_PAYMENT_METHOD_001');
        }
    }

    function is_valid_email($val)
    {
        //// Email
        if(!empty($val))
        {
            if(modApiFunc("Users", "isValidEmail", $val))
            {
                return true;
            }
            else
            {
                return array('error_code_full' => 'CHECKOUT_ERR_FORM_FIELD_FULL_002'
                ,'error_code_short' => 'CHECKOUT_ERR_FORM_FIELD_SHORT_002'
                ,'error_message_parameters' => array());//array($this->PrerequisitesValidationResults["shippingModuleAndMethod"]["validatedData"]["method_code"]["view_tag"], "shippingModuleAndMethod"));
            }
        }
        else
        {
            return array('error_code_full' => 'CHECKOUT_ERR_FORM_FIELD_FULL_001'
            ,'error_code_short' => 'CHECKOUT_ERR_FORM_FIELD_SHORT_001');
        }
    }

    function is_valid_street_line_1($val)
    {
        return true;
    }

    function is_valid_street_line_2($val)
    {
        return true;
    }

    function is_valid_city($val)
    {
        return true;
    }

    function is_valid_state($val)
    {
        //                  .
        //          ,       ID                            .  . .                  ID          _POST
        //            ID/             (               )                                             .
        //FIXME:                                                                                                  ?
        return true;
    }

    function is_valid_state_menu($val, $country_id)
    //FIXME: "State Menu" attribute strongly depends on "Country" attribute
    {
        //Its necessary to know the country.
        //The country ID should be validated. The cases when the country ID in _POST
        // follows the state ID/name (both of them are correct) should be processed correctly and separately.
        //FIXME: is it possible to force the attribute validation, on which the current validated attribute depends?
        $states = modApiFunc("Location", "getStates", $country_id);
        if((!is_array($states)) || (sizeof($states) == 0))
        {
            return true;
        }
        else
        {
            if(empty($val))
            {
                return array('error_code_full' => 'CHECKOUT_ERR_DROPDOWN_FULL_002'
                            ,'error_code_short' => 'CHECKOUT_ERR_DROPDOWN_SHORT_002');
            }
            else
            {
                $sl = modApiFunc("Location", "getStates", $country_id);
                if(array_key_exists($val, $sl))
                {
                    return true;
                }
                else
                {
                    return array('error_code_full' => 'CHECKOUT_ERR_DROPDOWN_FULL_001'
                                ,'error_code_short' => 'CHECKOUT_ERR_DROPDOWN_SHORT_001');
                }
            }
        }
    }

    function is_valid_state_text($val, $country_id)
    //FIXME: "State Text" attribute strongly depends on "Country" attribute
    {
        //FIXME: force the check of "State Menu" before checking "State Text". Because, "State Menu"
        // can be empty only if "State Menu"="Manual Input"

        //If at least one state of this country exists in the database,
        // return true. It's necessary to avoid outputting an error. It can
        // occur, because at this moment (Feb. 2006) from one DB-attribute "State"
        // is created two session-attributes "State Menu" and "State Text". A customer
        // should have chosen a state from the list "State Menu".
        $states = modApiFunc("Location", "getStates", $country_id);
        if(is_array($states) && (sizeof($states) > 0))
        {
            return true;
        }
        else
        {
            $hide_state = modApiFunc('Configuration', 'getValue', SYSCONFIG_STORE_HIDE_STATE_TEXT_INPUT);
            if(empty($val) && $hide_state !== true)
            {
                return array('error_code_full' => 'CHECKOUT_ERR_FORM_FIELD_FULL_001'
                            ,'error_code_short' => 'CHECKOUT_ERR_FORM_FIELD_SHORT_001');
            }
            else
            {
                return true;
            }
        }
    }

    function is_valid_postcode($val)
    {
        return true;
    }

    //Returns the Javascript-code to synchronize the lists "Vountries" and "States"
    function getJavascriptSynchronizeCountriesAndStatesLists()
    {
        return
        "<SCRIPT LANGUAGE=\"JavaScript\">\n".
        "<!--\n".

        "function removeGroups(select_id)". "\n".
        "{". "\n".
        "". "\n".
        "   try". "\n".
        "   {". "\n".
        "      var select_list = document.getElementById(select_id);". "\n".
        "". "\n".
        "      var eachGroup = select_list.firstChild;". "\n".
        "      while (eachGroup)". "\n".
        "      {". "\n".
        "         lastGroup = eachGroup;". "\n".
        "         eachGroup = lastGroup.nextSibling;". "\n".
        "         if ('OPTGROUP' == lastGroup.tagName) "."\n".
        "         { "."\n".
        "             select_list.removeChild(lastGroup);". "\n".
        "         } "."\n".
        "      }". "\n".
        "   }". "\n".
        "   catch(er){ }". "\n".
        "}". "\n".

        "function refreshStatesList(country_list_id, state_list_id, state_input_id)". "\n".
        "{". "\n".
        "    var country_list = document.getElementById(country_list_id);". "\n".
        "    var country_id   = country_list.options[country_list.selectedIndex].value;". "\n".
        "    if (!country_id) return; //Empty option, like \"--- Select a Country ---\"". "\n".
        "". "\n".
        "    var state_list = document.getElementById(state_list_id);". "\n".
        "    if(!state_list)". "\n".
        "        return;". "\n".
        "    var state_list_tr = document.getElementById('tr_' + state_list_id);". "\n".
        "    //Save currently selected element". "\n".
//        "    var state_list_selected_id = state_list.selectedIndex;". "\n".
        "    var state_list_selected_id = state_list.value;". "\n".
        "    var state_input_text_field = document.getElementById(state_input_id);". "\n".
        "    state_list.options.length = 0;". "\n".
        "    state_list.length = 0;". "\n".
        "    removeGroups(state_list_id);". "\n".
        "". "\n".
        "    if(countryIdToStatesIdArray[country_id] == undefined)". "\n".
        "    {". "\n".
        "        //Country has no states. Activate \"Manual Input\" control.". "\n".
        "        //Hide \"State Menu\"". "\n".
////        "        state_list.style.visibility='hidden';". "\n".
        "        if(state_list)state_list.style.display='none';". "\n".
        "        if(state_list_tr)state_list_tr.style.display='none';". "\n".
//FIXME:
//  work out a variant with states, so that, when using Javascript, the values of both attributes
// state_menu and state_text are passed to the POST data.
/*   var form = document.forms['checkout'];

   var el = document.createElement("input");
   el.type = "hidden";
   el.name = "myHiddenField";
   el.id = "myHiddenField";
   el.value = "myValue";
   form.appendChild(el);        */


        "        //Unhide manual input element". "\n".
//        "        state_input_text_field.style.visibility='visible'". "\n".
        "        if(state_input_text_field)". "\n".
        "        state_input_text_field.style.display='block'". "\n".
        "". "\n".
        "    }". "\n".
        "    else". "\n".
        "    {". "\n".
        "        //Hide manual input element.". "\n".
        "        if(state_list)state_list.style.display='block';". "\n".
        "        if(state_list_tr)state_list_tr.style.display='';". "\n".
//        "        state_list.style.visibility='visible';". "\n".
        "        //Unhide \"State Menu\" element". "\n".
//        "        state_input_text_field.style.visibility='hidden'". "\n".
        "        if(state_input_text_field)". "\n".
        "        state_input_text_field.style.display='none'". "\n".
        "". "\n".
        "        var state_id_array = countryIdToStatesIdArray[country_id];". "\n".
        "        if (!state_list_selected_id)\n".
        "        {\n".
        "           state_list_selected_id = defaultStatesIdArray[country_id];\n".
        "        }\n".
        "        for(i=0; i< state_id_array.length; i+=1)". "\n".
        "        {". "\n".
        "            state_list.options[i] = new Option(statesArray[state_id_array[i]], state_id_array[i]);". "\n".
        "            if(state_id_array[i] == state_list_selected_id)". "\n".
        "            {". "\n".
        "                 state_list.options[i].selected = true;" . "\n".
        "            }". "\n".
        "        }". "\n".
        "    }". "\n".
        "}".
        "//-->\n".
        "</SCRIPT>";
    }

    //Returns the Javascript-code to synchronize two different Person Info.
    function getJavascriptCopyPersonInfo()
    {
        return
        "<SCRIPT LANGUAGE=\"JavaScript\">\n".
        "<!--\n".

        "function copyPersonInfo(form_name, person_info_prefix_from, person_info_prefix_to)" . "\n".
        "{" . "\n".
        "    var i = 0;" . "\n".
        "    while(document[form_name].elements[i])" . "\n".
        "    {   " . "\n".
        "        //Check if element is from \"from\" Customer Info set." . "\n".
        "        var regExpr = \"^\" + person_info_prefix_from + \"\\\\[([a-zA-Z0-9_]*)\\\\]$\";" . "\n".
        "        var re = new RegExp(regExpr);//  /^_form1\\[([a-zA-Z0-9_]*)\\]$/;" . "\n".
        "        if(re.test(document[form_name].elements[i].name))" . "\n".
        "        {" . "\n".
        "            var matches = document[form_name].elements[i].name.match(re);" . "\n".
        "            var attribute_name = matches[1];" . "\n".
        "            var attribute_name_from = person_info_prefix_from + \"[\" + attribute_name + \"]\";" . "\n".
        "            var attribute_name_to   = person_info_prefix_to   + \"[\" + attribute_name + \"]\";" . "\n".
        "" . "\n".
        "            if(document[form_name][attribute_name_to])" . "\n".
        "            {" . "\n".
        "                //Consider attribute html type: input text, select, ..." . "\n".
        "                if(document[form_name][attribute_name_to].type == \"text\")" . "\n".
        "                {" . "\n".
        "                    document[form_name][attribute_name_to].value = document[form_name][attribute_name_from].value;" . "\n".
        "                }" . "\n".
        "                else if(document[form_name][attribute_name_to].type == \"select-one\")" . "\n".
        "                {" . "\n".
        "                    document[form_name][attribute_name_to].selectedIndex = document[form_name][attribute_name_from].selectedIndex;" . "\n".
        "                    if(document[form_name][attribute_name_to].onchange)" . "\n".
        "                    {" . "\n".
        "                       document[form_name][attribute_name_to].onchange();" . "\n".
        "                    }" . "\n".
        "                }" . "\n".
        "            }" . "\n".
        "        }" . "\n".
        "        i++;" . "\n".
        "    }" . "\n".
        "}" . "\n".
        "//-->\n".
        "</SCRIPT>";
    }

    //Returns the HTML-code to choose a country.
    function genCountrySelectList($country_id, $with_states_only = false, $bWithAllOtherCountry = false)
    {
        global $application;
        //WARNING:
        //  Country ="" (empty string) means that user have not been prompted yet to
        //      choose a country

        $value = "";
        $countries = modApiFunc("Location", "getCountries", true, array(), $with_states_only);

        if (!$country_id)
        {
            $country_id = modApiFunc("Location", "getDefaultCountryId");
        }

        foreach($countries as $_country_id => $_country_name)
        {
            $Selected = ($country_id == $_country_id) ? "SELECTED" : "";
            $value .= "<option value='" . $_country_id . "' " . $Selected . ">" . $_country_name . "</option>";
        }
        if($bWithAllOtherCountry === true)
        {
            $Selected = ($country_id == modApiFunc("Location", "get_constant", "ALL_OTHER_COUNTRIES_COUNTRY_ID")) ? "SELECTED" : "";
            $value = "<option value='" . modApiFunc("Location", "get_constant", "ALL_OTHER_COUNTRIES_COUNTRY_ID") . "' " . $Selected . ">" . msg("COUNTRY_ALL_LABEL") . "</option>"
                    .$value;
        }
        return $value;
    }

    //Returns the HTML-code to select the type of credit card.
    //@todo to do a setting tool for this list in AZ
    function genCreditCardTypeSelectList($cc_type_id)
    {
        $options = modApiFunc("Configuration", "getCreditCardSettings");
        $value = "";
        foreach($options as $id => $info)
        {
            $Selected = ($id == $cc_type_id) ? "SELECTED" : "";
            $value .= "<option value='" . $id . "' " . $Selected . ">" . $info["name"] . "</option>";
        }
        return $value;
    }

    function getMonthNames()
    {
        global $application;
        $mr = &$application->getInstance('MessageResources');
        return array("01" => $mr->getMessage("GENERAL_MONTH_01")
                    ,"02" => $mr->getMessage("GENERAL_MONTH_02")
                    ,"03" => $mr->getMessage("GENERAL_MONTH_03")
                    ,"04" => $mr->getMessage("GENERAL_MONTH_04")
                    ,"05" => $mr->getMessage("GENERAL_MONTH_05")
                    ,"06" => $mr->getMessage("GENERAL_MONTH_06")
                    ,"07" => $mr->getMessage("GENERAL_MONTH_07")
                    ,"08" => $mr->getMessage("GENERAL_MONTH_08")
                    ,"09" => $mr->getMessage("GENERAL_MONTH_09")
                    ,"10" => $mr->getMessage("GENERAL_MONTH_10")
                    ,"11" => $mr->getMessage("GENERAL_MONTH_11")
                    ,"12" => $mr->getMessage("GENERAL_MONTH_12"));
    }

    //Returns the HTML-code to select a month.
    function genMonthSelectList($selected_id)
    {
        $options = $this->getMonthNames();
        $value = "";
        foreach($options as $id => $name)
        {
            $Selected = ($id == $selected_id) ? "SELECTED" : "";
            $value .= "<option value='" . $id . "' " . $Selected . ">" . $name . "</option>";
        }
        return $value;
    }

    function getCCYearNames()
    {
        return array("2007" => "2007"
                    ,"2008" => "2008"
                    ,"2009" => "2009"
                    ,"2010" => "2010"
                    ,"2011" => "2011"
                    ,"2012" => "2012"
                    ,"2013" => "2013"
                    ,"2014" => "2014"
                    ,"2015" => "2015"
                    ,"2016" => "2016");
    }

    function getCCValidFromYearNames()
    {
        return array("2000" => "2000"
                    ,"2001" => "2001"
                    ,"2002" => "2002"
                    ,"2003" => "2003"
                    ,"2004" => "2004"
                    ,"2005" => "2005"
                    ,"2006" => "2006"
                    ,"2007" => "2007");
    }

    //Returns the HTML-code to select a year.
    function genYearSelectList($selected_id)
    {
        $value = "";
        $options = $this->getCCYearNames();
        foreach($options as $id => $name)
        {
            $Selected = ($id == $selected_id) ? "SELECTED" : "";
            $value .= "<option value='" . $id . "' " . $Selected . ">" . $name . "</option>";
        }
        return $value;
    }

    function genValidFromYearSelectList($selected_id)
    {
        $value = "";
        $options = $this->getCCValidFromYearNames();
        foreach($options as $id => $name)
        {
            $Selected = ($id == $selected_id) ? "SELECTED" : "";
            $value .= "<option value='" . $id . "' " . $Selected . ">" . $name . "</option>";
        }
        return $value;
    }

    //Returns HTML-code to choose a country. The states are groupped by countries.
    //  For those countries, that have no specified state, it is outputted only one item:
    //  "Manual input"
    function genStateSelectList($state_id, $country_id, $bWithAllOtherState = false)
    //FIXME: 'state' "person attribute" strongly depends on 'Country' "person attribute".
    {
        global $application;
        //WARNING:
        //  State ="" (empty string) means that user have not been prompted yet to
        //      choose a state
        //  State =NULL (NULL value, MySQL, PHP) means that user already chose
        //  the Country with no states and inputed the State name manually.
        $obj = &$application->getInstance('MessageResources');
        $MANUAL_STATE_INPUT_MESSAGE = $obj->getMessage("CHECKOUT_STATE_001");

//        $value = "<select name='state'>";
        $value = "";
        $countries = modApiFunc("Location", "getCountries"); //modApiFunc("Location", "getCountries");
        $bThereIsACountryWithoutStates = false;
        $ManualStateInputSelected = false;

        if (!$country_id)
        {
            $country_id = modApiFunc("Location", "getDefaultCountryId");
        }
        if (!$state_id)
        {
            $state_id = modApiFunc("Location", "getDefaultStateId", $country_id);
        }

        foreach($countries as $_country_id => $_country_name)
        {
            $states = modApiFunc("Location", "getStates", $_country_id); //modApiFunc("Location", "getStates", $_country_id);
            if(empty($states))
            {
                $bThereIsACountryWithoutStates = true;
                $ManualStateInputSelected = ($country_id == $_country_id && $state_id === NULL) ? "SELECTED" : "";
//                $bSelected = ($country_id == $_country_id && $state_id === NULL) ? "SELECTED" : "";
//                $value .= "<option value='NULL' $bSelected>" . $MANUAL_STATE_INPUT_MESSAGE . "</option>";
            }
            else
            {
                $value .= "<optgroup label='" . $_country_name. "'>";
                foreach($states as $_state_id => $_state_name)
                {
                    $Selected = ($country_id == $_country_id && $state_id == $_state_id) ? "SELECTED" : "";

                    $value .= "<option value='" .$_state_id. "' " . $Selected . ">" .$_state_name. "</option>";
                }
                if($bWithAllOtherState === true)
                {
                    $Selected = ($country_id == $_country_id && $state_id == modApiFunc("Location", "get_constant", "ALL_OTHER_STATES_STATE_ID")) ? "SELECTED" : "";
                    $value = "<option value='" .modApiFunc("Location", "get_constant", "ALL_OTHER_STATES_STATE_ID"). "' " . $Selected . ">" .msg("STATE_ALL_LABEL"). "</option>"
                            .$value;
                }
                $value .= "</optgroup>";
            }

        }
        if($bThereIsACountryWithoutStates)
        {
            $value = "<option value='NULL' " . $ManualStateInputSelected . ">" . $MANUAL_STATE_INPUT_MESSAGE . "</option>" . $value;
        }
//        $value .= "</select>";
        return $value;
    }

    function is_valid_country($val)
    {
        //The country id is given at the input.
        //Make a query to the DB. If the country with such ID is already in the DB, return true.
        if(empty($val))
        {
            return array('error_code_full' => 'CHECKOUT_ERR_DROPDOWN_FULL_002'
                        ,'error_code_short' => 'CHECKOUT_ERR_DROPDOWN_SHORT_002');
        }
        else
        {
            $cl = modApiFunc("Location", "getCountries");
            if(array_key_exists($val, $cl))
            {
                return true;
            }
            else
            {
                return array('error_code_full' => 'CHECKOUT_ERR_DROPDOWN_FULL_001'
                            ,'error_code_short' => 'CHECKOUT_ERR_DROPDOWN_SHORT_001');
            }
        }

    }

    function is_valid_phone($val)
    {
        //If the value is an empty string, ask to input again.
        if(empty($val))
        {
            return array('error_code_full' => 'CHECKOUT_ERR_FORM_FIELD_FULL_001'
                        ,'error_code_short' => 'CHECKOUT_ERR_FORM_FIELD_SHORT_001');
        }
        else
        {
            return true;
        }
    }

    function is_valid_string1024_null($val)
    {
        /*
        //how to portably match new line character?
        $pattern = "/^(.{0,1024})$/";

        $matches = array();
        if(preg_match($pattern, $val, $matches))
        {
            return true;
        }*/
        if(strlen($val) <= 1024)
        {
            return true;
        }
        else
        {
            return array('error_code_full' => 'CHECKOUT_ERR_FORM_FIELD_FULL_002'
                        ,'error_code_short' => 'CHECKOUT_ERR_FORM_FIELD_SHORT_002'
                        );
        }
    }

    function is_valid_credit_card_type($cc_type)
    {
        $list = modApiFunc("Configuration", "getCreditCardSettings");
        if(array_key_exists($cc_type, $list))
        {
            return true;
        }
        else
        {
            return array('error_code_full' => 'CCTYPE_ERR_002'
                        ,'error_code_short' => 'CCTYPE_ERR_002'
                        );
        }
    }

    function is_valid_debit_card_number($dc_number, $dc_rules)
    {
        for ($i = 0; $i < sizeof($dc_rules); $i++)
        {
            $rule = $dc_rules[$i];
            $rule_details = explode(",", $rule);

            $hi_prefix        = $rule_details[0];
            $lo_prefix        = $rule_details[1];
            $val_length       = intval($rule_details[2]);
            $issue_length     = $rule_details[3];
            $start_date_length = isset($rule_details[4])? $rule_details[4]:"";

            $card_prefix = substr($dc_number, 0, strlen($hi_prefix));
            if ($card_prefix >= $hi_prefix && $card_prefix <= $lo_prefix && strlen($dc_number) == $val_length)
            {
                return true;
            }
        }
        return false;
    }

    function is_valid_credit_card_number($cc_number, $cc_type)
    {
        switch($cc_type)
        {
            case "Visa":
                $valid = ereg("^4[0-9]{12}([0-9]{3})?$", $cc_number);
                if (!$valid)
                {
                    return array('error_code_full' => 'CCNUM_ERR_002'
                                ,'error_code_short' => 'CCNUM_ERR_002');
                }
                break;
            case "MasterCard":
                $valid = ereg("^5[1-5][0-9]{14}$", $cc_number);
                if (!$valid)
                {
                    return array('error_code_full' => 'CCNUM_ERR_003'
                                ,'error_code_short' => 'CCNUM_ERR_003');
                }
                break;
            case "Discover":
                $valid = ereg("^(3[0-9]{4}|2131|1800)[0-9]{11}$", $cc_number);
                if (!$valid)
                {
                    return array('error_code_full' => 'CCNUM_ERR_004'
                                ,'error_code_short' => 'CCNUM_ERR_004');
                }
                break;
            case "Amex":
                $valid = ereg("^3[47][0-9]{13}$", $cc_number);
                if (!$valid)
                {
                    return array('error_code_full' => 'CCNUM_ERR_005'
                                ,'error_code_short' => 'CCNUM_ERR_005');
                }
                break;
            case "Maestro":
                $MaestroRules = array(
                "490302,490309,18,1","490335,490339,18,1","491101,491102,16,1","491174,491182,18,1","493600,493699,19,1","564182,564182,16,2","633300,633300,16,0","633301,633301,19,1","633302,633349,16,0","675900,675900,16,0","675901,675901,19,1","675902,675904,16,0","675905,675905,19,1","675906,675917,16,0","675918,675918,19,1","675919,675937,16,0","675938,675940,18,1","675941,675949,16,0","675950,675962,19,1","675963,675997,16,0","675998,675998,19,1","675999,675999,16,0"
                                     );
                $valid = $this->is_valid_debit_card_number($cc_number, $MaestroRules);
                if (!$valid)
                {
                    return array('error_code_full' => 'CCNUM_ERR_007'
                                ,'error_code_short' => 'CCNUM_ERR_007');
                }
                break;
            case "Solo":
                $SoloRules = array(
                "633450,633453,16,0","633454,633457,16,0","633458,633460,16,0","633461,633461,18,1","633462,633472,16,0","633473,633473,18,1","633474,633475,16,0","633476,633476,19,1","633477,633477,16,0","633478,633478,18,1","633479,633480,16,0","633481,633481,19,1","633482,633489,16,0","633490,633493,16,1","633494,633494,18,1","633495,633497,16,2","633498,633498,19,1","633499,633499,18,1","676700,676700,16,0","676701,676701,19,1","676702,676702,16,0","676703,676703,18,1","676704,676704,16,0","676705,676705,19,1","676706,676707,16,2","676708,676711,16,0","676712,676715,16,0","676716,676717,16,0","676718,676718,19,1","676719,676739,16,0","676740,676740,18,1","676741,676749,16,0","676750,676762,19,1","676763,676769,16,0","676770,676770,19,1","676771,676773,16,0","676774,676774,18,1","676775,676778,16,0","676779,676779,18,1","676780,676781,16,0","676782,676782,18,1","676783,676794,16,0","676795,676795,18,1","676796,676797,16,0","676798,676798,19,1","676799,676799,16,0"
                                     );
                $valid = $this->is_valid_debit_card_number($cc_number, $MaestroRules);
                if (!$valid)
                {
                    return array('error_code_full' => 'CCNUM_ERR_008'
                                ,'error_code_short' => 'CCNUM_ERR_008');
                }
                break;
            default:
                {
                    $list = modApiFunc("Configuration", "getCreditCardSettings");
                    if(array_key_exists($cc_type, $list))
                    {
                        //                                    .
                        //                       .
                        return true;
                    }
                    else
                    {
                        return array('error_code_full' => 'CCTYPE_ERR_002'
                                    ,'error_code_short' => 'CCTYPE_ERR_002');
                    }
                }
        }
        $cc_number = strrev($cc_number);
        $numSum = 0;
        for($i = 0; $i < strlen($cc_number); $i++)
        {
            $currentNum = substr($cc_number, $i, 1);
            // Double every second digit
            if($i % 2 == 1)
            {
              $currentNum *= 2;
            }
            // Add digits of 2-digit numbers together
            if($currentNum > 9)
            {
              $firstNum = $currentNum % 10;
              $secondNum = ($currentNum - $firstNum) / 10;
              $currentNum = $firstNum + $secondNum;
            }
            $numSum += $currentNum;
        }
        if (!($numSum % 10 == 0))
        {
            return array('error_code_full' => 'CCNUM_ERR_006'
                        ,'error_code_short' => 'CCNUM_ERR_006');
        }
        else
        {
            return true;
        }
    }

    function is_valid_credit_card_verification_number($cc_cvv)
    {
        if(strlen($cc_cvv) <= 10)
        {
            $valid = ereg("^[0-9a-zA-Z]*$", $cc_cvv);
            if (!$valid)
            {
                return array('error_code_full' => 'CVV2_ERR_001_FULL'
                            ,'error_code_short' => 'CVV2_ERR_002_SHORT');
            }
            else
            {
                return true;
            }
        }
        else
        {
            return array('error_code_full' => 'CVV2_ERR_001_FULL'
                        ,'error_code_short' => 'CVV2_ERR_002_SHORT');
        }
    }

    function is_valid_credit_card_issue_number($cc_issue_number)
    {
        return true;
    }

    function str2dec_int($string)
    {
       $y = ltrim($string, '0');
       $z = 0 + $y;
       return $z;
    }

    /**
     *                            .       Expiration Date               
     *             .
     */
    function is_valid_month($month, $year)
    {
        $month  = $this->str2dec_int($month);
        if($month >0 &&
           $month <13)
        {
            $cur_year_4digits = date("Y");
            if($year > $cur_year_4digits)
            {
                return true;
            }
            else if($year == $cur_year_4digits)
            {
                $cur_month_1_2_digits = date("n");
                if($month >= $cur_month_1_2_digits)
                {
                    return true;
                }
                else
                {
                    return array('error_code_full' => 'CCEXPMON_ERR_003'
                                ,'error_code_short' => 'CCEXPMON_ERR_003');
                }
            }
            else
            {
                return array('error_code_full' => 'CCEXPMON_ERR_003'
                            ,'error_code_short' => 'CCEXPMON_ERR_003');
            }
        }
        else
        {
            return array('error_code_full' => 'CCEXPMON_ERR_003'
                        ,'error_code_short' => 'CCEXPMON_ERR_003');
        }
    }

    /**
     *                            .       'valid from' Date              
     *             .
     */
    function is_valid_month_valid_from($month, $year)
    {
        $month  = $this->str2dec_int($month);
        if($month >0 &&
           $month <13)
        {
            $cur_year_4digits = date("Y");
            if($year < $cur_year_4digits)
            {
                return true;
            }
            else if($year == $cur_year_4digits)
            {
                $cur_month_1_2_digits = date("n");
                if($month <= $cur_month_1_2_digits)
                {
                    return true;
                }
                else
                {
                    return array('error_code_full' => 'CC_VALID_FROM_MON_ERR_003'
                                ,'error_code_short' => 'CC_VALID_FROM_MON_ERR_003');
                }
            }
            else
            {
                return array('error_code_full' => 'CC_VALID_FROM_MON_ERR_003'
                            ,'error_code_short' => 'CC_VALID_FROM_MON_ERR_003');
            }
        }
        else
        {
            return array('error_code_full' => 'CC_VALID_FROM_MON_ERR_003'
                        ,'error_code_short' => 'CC_VALID_FROM_MON_ERR_003');
        }
    }

    function is_valid_year($year)
    {
        $year  = $this->str2dec_int($year);
        if($year >0)
        {
            return true;
        }
        else
        {
            return array('error_code_full' => 'CCEXPYEAR_ERR_002'
                        ,'error_code_short' => 'CCEXPYEAR_ERR_002');
        }
    }

    function is_valid_year_valid_from($year)
    {
        $year  = $this->str2dec_int($year);
        $cur_year_4digits = date("Y");
        if($year <= $cur_year_4digits)
        {
            return true;
        }
        else
        {
            return array('error_code_full' => 'CC_VALID_FROM_YEAR_ERR_002'
                        ,'error_code_short' => 'CC_VALID_FROM_YEAR_ERR_002');
        }
    }


    function is_valid_first_name($val)
    {
        //If the value is an empty string, ask to input again.
        if(empty($val))
        {
            return array('error_code_full' => 'CHECKOUT_ERR_FORM_FIELD_FULL_001'
                        ,'error_code_short' => 'CHECKOUT_ERR_FORM_FIELD_SHORT_001');
        }
        else
        {
            return true;
        }
    }

    function is_valid_last_name($val)
    {
        //If the value is an empty string, ask to input again.
        if(empty($val))
        {
            return array('error_code_full' => 'CHECKOUT_ERR_FORM_FIELD_FULL_001'
                        ,'error_code_short' => 'CHECKOUT_ERR_FORM_FIELD_SHORT_001');
        }
        else
        {
            return true;
        }
    }

    function is_valid_bank_account_name($val)
    {
        return true;
    }

    function is_valid_bank_routing_number($val)
    {
        return true;
    }

    function is_valid_bank_account_number($val)
    {
        return true;
    }

    function is_valid_bank_state_branch($val)
    {
        return true;
    }

    function is_valid_bank_international_bank_account_number($val)
    {
        return true;
    }

    /** Input validation */
    /*
     * $post_data -                      .                            
     *                 .                              ,           
     *                                                   Checkout,      
     *         "    "                    -                     , required/visible, ...
     * */
    function validateInputForPrerequisite($prerequisiteName, $post_data = CHECKOUT_POST_DATA_NOT_EMULATED)
    {
        global $application;

        $validatedData = $this->getValidatedDataStructure($prerequisiteName);

        //simple stub
        if($post_data === CHECKOUT_POST_DATA_NOT_EMULATED)
        {
            $request = $application->getInstance('Request');
            $post_data = $request->getValueByKey($prerequisiteName);
        }
        $isMet = true;
        $error_code = "";
        $error_message_parameters = array();

        if(NULL === $post_data)
        {
            /* As we get into this function, then the Block-tag, for example, Billing
               Info, was however outputted. If no data array goes with it,
               then probably, all its marked as "invisible".
             */
            switch($prerequisiteName)
            /* prerequisites "shippingModuleAndMethod" and "paymentModule" are not
               adjusted through AZ and processed differently.
              They have the customized error messages.
             */
            {
                case "shippingModuleAndMethod":
                {
                    break;
                }
                case "paymentModule":
                {
                    break;
                }
                default:
                {
                    $post_data = array();
                }
            }
        }
        if(NULL !== $post_data)
        {
            //Sort the prerequisite fields: those, that require preliminary check
            //should be put to the end. E.g.: the state (requires a country),
            //the number of the credit card (requires the type of the credit card).
            $validatedData_copy_with_proper_sort_order = $validatedData;
            asc_assoc_array_move_back($validatedData, "Statemenu");
            asc_assoc_array_move_back($validatedData, "Statetext");
            asc_assoc_array_move_back($validatedData, "CreditCardNumber");
            asc_assoc_array_move_back($validatedData, "ExpirationMonth");
            asc_assoc_array_move_back($validatedData, "ValidFromMonth");

            //Get a list of all fields, which make this prerequisite:
            foreach($validatedData as $validatedDataItemKey => $validatedDataItemVal)
            {
                if(!method_exists($this, $validatedDataItemVal["input_validation_func_name"]))
                {
//--- (a branch is not tested)
                    //The definition of validation function doesn't exist. It helps to check if
                    // an inputted value is correct.
                    $isMet = false;
                    $validatedData[$validatedDataItemKey]['error_code_full'] = 'CHECKOUT_ERR_002';
                    $validatedData[$validatedDataItemKey]['error_code_short'] = 'CHECKOUT_ERR_002';
                    $validatedData[$validatedDataItemKey]['error_message_parameters'] = array($validatedDataItemVal["input_validation_func_name"], $validatedDataItemKey);
                }
                else
                {
                    //Check if data was passed by POST:
                    if(!isset($post_data[$validatedDataItemKey]))
                    {
                        if(//FIXME: work out a variant, when the fields are
                           // inserted to POST using JavaScript
                            $validatedDataItemKey != 'Statemenu' &&
                            $validatedDataItemKey != 'Statetext' &&

                            (     $validatedDataItemVal['attribute_required'] == true ||
                                  !(bool)modApiFunc("Configuration", "getValue", "store_online")
                            )
                          )
                        {
//--- (a branch is not tested)
                            //FIXME: process separartely a checkbox group which is not ticked off.
                            //Example: if admistrator can choose a list of
                            //  inputted fields for filling them and he chooses
                            //  only one field and it will be CHECKBOX list
                            //  and a buyer won't choose any of the CHECKBOX.
                            //  Then nothing will passed to the POST.

                            //Data wasn't passed (such index doesn't exist in _POST).
                            // Sush error may occur, when for example, the administrator added
                            //a field to person info, and forgot to update a customized template.

                            //It is a required attribute
                            // OR the shop is NOT Online
                            // and it's better to send an error message.
                            // Validation failed.Save the error.
                            $isMet = false;
                            $validatedData[$validatedDataItemKey]['error_code_full'] = 'CHECKOUT_ERR_003';
                            $validatedData[$validatedDataItemKey]['error_code_short'] = 'CHECKOUT_ERR_003';
                            $validatedData[$validatedDataItemKey]['error_message_parameters'] = array($this->PrerequisitesValidationResults[$prerequisiteName]["validatedData"][$validatedDataItemKey]["attribute_visible_name"], $this->PrerequisitesValidationResults[$prerequisiteName]["visibleName"]);
                            continue;
                        }
                        else
                        {
//--- (a branch is not tested)
                            //FIXME: process separartely a checkbox group which is not ticked off.

                            //Data wasn't passed (such index doesn't exist in _POST).
                            // The attribute is not required
                            // OR the shop is NOT Online
                            // and an error message should be sent.

                            //Assign a value "empty string" to the attribute
                            // and don't send an error message on this step.

                            //Pretend as if the attribute value in the request was: "empty string".
                            $post_data[$validatedDataItemKey] = "";
                        }
                    }

                    //On this step either a value has been assigned to the attribute artificially (empty
                    // string), or it has existed in the POST data. Go on to validate.

                    //Data was passed through POST.
                    //Perhaps it's true, that the passed value is an empty string,
                    //  i.e. a user, for example, didn't fill out an input field.

                    if(!($validatedDataItemKey == 'Statemenu' ||
                         $validatedDataItemKey == 'Statetext') &&
                       empty($post_data[$validatedDataItemKey])
//FIXME: What is the condition input_validation_func_name == false for?
//$validatedDataItemVal["input_validation_func_name"] == false
                      )
                    {
                        if((bool)$validatedDataItemVal['attribute_required'] == true)
                        {
//+ (a branch has been tested)
                            //If an attribute is required and
                            //    1) http data (_POST) didn't come for it
                            //    2) or the arrived value is an empty string
                            // (discussed with af on 02(feb)-01-2006)
                            // - output an error before validation fumction is called. Like
                            // "required attribute"
                            $isMet = false;
                            $validatedData[$validatedDataItemKey]['error_code_full'] = 'CHECKOUT_ERR_FORM_FIELD_FULL_001';
                            $validatedData[$validatedDataItemKey]['error_code_short'] = 'CHECKOUT_ERR_FORM_FIELD_SHORT_001';
                            $validatedData[$validatedDataItemKey]['error_message_parameters'] = array($this->PrerequisitesValidationResults[$prerequisiteName]["validatedData"][$validatedDataItemKey]["attribute_visible_name"], $this->PrerequisitesValidationResults[$prerequisiteName]["visibleName"]);
                            continue;
                        }
                        else
                        {
                            //If an attribute is not required and
                            //     1) http data (_POST) didn't come for it
                            //     2) or the arrived value is an empty string
                            // - Validation is successful. Don't call the validation function.
                            $r = true;
                        }
                    }
                    else
                    {
                        // Data was passed through the POST.
                        //The value is not an empty string (the states are separately)
                        // if it is called a validation function.

                        //Validate the inputted data.
                        //There are special cases, when not only inputted by user
                        // attribute values but optional parameters are passed, listed below:
                        // 1) a state (is selected from menu),
                        // 2) a state ( the text is inputted manually for countries, that don't
                        //    have any state )
                        // in both cases the country id should be passed.
                        //
                        // FIXME:         :                                              
                        //                    .          ,              id                 
                        //                    prerequisite,           .
                        // FIXME: a problem: the country should already be checked before the state checking.
                        //     Both the country id and the state are is in the same prerequisite block.
                        switch($validatedDataItemKey)
                        {
                            case "Statemenu":
                            case "Statetext":
                            {
//   --- (a branch is not tested)
                                $country_id = $validatedData['Country']['value'];
                                $r = call_user_func(array(&$this, $validatedDataItemVal["input_validation_func_name"]), $post_data[$validatedDataItemKey], $country_id);
                                break;
                            }
                            case "CreditCardNumber":
                            {
                                $credit_card_type = $validatedData['CreditCardType']['value'];
                                $r = call_user_func(array(&$this, $validatedDataItemVal["input_validation_func_name"]), $post_data[$validatedDataItemKey], $credit_card_type);
                                break;
                            }
                            case "ExpirationMonth":
                            {
                                $year = $validatedData['ExpirationYear']['value'];
                                $r = call_user_func(array(&$this, $validatedDataItemVal["input_validation_func_name"]), $post_data[$validatedDataItemKey], $year);
                                break;
                            }
                            case "ValidFromMonth":
                            {
                                $year = $validatedData['ValidFromYear']['value'];
                                $r = call_user_func(array(&$this, $validatedDataItemVal["input_validation_func_name"]), $post_data[$validatedDataItemKey], $year);
                                break;
                            }
                            default:
                            {
//   --- (a branch is not tested)
                                $r = call_user_func(array(&$this, $validatedDataItemVal["input_validation_func_name"]), $post_data[$validatedDataItemKey]);
                                break;
                            }
                        }
                    }

                    if(!is_array($r) && ($r == true))
                    {
//   --- (a branch is not tested)
                        //data are validated successfully.
                        //save the checked value.
                        $validatedData[$validatedDataItemKey]['value'] = $post_data[$validatedDataItemKey];
                    }
                    else
                    {
//   --- (a branch is not tested)
                        //data are not validated.
                        $validatedData[$validatedDataItemKey]['value'] = $post_data[$validatedDataItemKey];
                        $isMet = false;
                        if(is_array($r))
                        {
//       --- (a branch is not tested)
                            if(!empty($r['error_code_full']) || !empty($r['error_code_short']))
                            {
//        --- (a branch is not tested)
                                $validatedData[$validatedDataItemKey]['error_code_full'] = $r['error_code_full'];
                                $validatedData[$validatedDataItemKey]['error_code_short'] = $r['error_code_short'];
                                if(!empty($r['error_message_parameters']))
                                {
//         --- (a branch is not tested)
                                    $validatedData[$validatedDataItemKey]['error_message_parameters'] = $r['error_message_parameters'];
                                }

                                //There is a special handling of some error types. Add error
                                //  parameters, which can't be defined into the called
                                //  checking function. For example, the name of the current Person Info.
                                switch($validatedData[$validatedDataItemKey]['error_code_full'])
                                {
                                    case "CHECKOUT_ERR_FORM_FIELD_FULL_001":
                                    case "CHECKOUT_ERR_FORM_FIELD_FULL_002":
                                    case "CHECKOUT_ERR_DROPDOWN_FULL_002":
                                    case "CHECKOUT_ERR_DROPDOWN_FULL_001":
                                    case "CHECKOUT_ERR_FORM_FIELD_FULL_001":
                                    {
                                        //WARNING: The old parameter values are not saved!
                                        $form_name = $this->PrerequisitesValidationResults[$prerequisiteName]["visibleName"];
                                        switch($validatedDataItemKey)
                                        {
                                            case "Statetext":
                                            {
                                                $fakeDataItemKey = "Statemenu";
                                                $attribute_name = $this->PrerequisitesValidationResults[$prerequisiteName]["validatedData"][$fakeDataItemKey]["attribute_visible_name"];
                                                break;
                                            }
                                            default:
                                            {
                                                $attribute_name = $this->PrerequisitesValidationResults[$prerequisiteName]["validatedData"][$validatedDataItemKey]["attribute_visible_name"];
                                                break;
                                            }
                                        }

                                        $validatedData[$validatedDataItemKey]['error_message_parameters'] = array($attribute_name, $form_name);
                                        break;
                                    }
                                    default:
                                    {
                                        break;
                                    }
                                }
                            }
                        }
                    }
                }
            }
            //Restore the order of the items.
            foreach($validatedData as $validatedDataItemKey => $validatedDataItemVal)
            {
                $validatedData_copy_with_proper_sort_order[$validatedDataItemKey] = $validatedDataItemVal;
            }
            $validatedData = $validatedData_copy_with_proper_sort_order;
            //END restore the order of the items.
        }
        else
        {
//          --- (a branch is tested)
            $isMet = false;

            switch($prerequisiteName)
            //customize the error message
            {
                case "shippingModuleAndMethod":
                {
                    if($this->isShippingModulesListEmpty() == false)
                    {
                        $error_code = 'CHECKOUT_ERR_004_SHIPPING_MODULE_AND_METHOD_AVAILABLE_BUT_NONE_SELECTED';
                    }
                    else
                    {
                        $error_code = 'CHECKOUT_ERR_004_SHIPPING_MODULE_AND_METHOD_NOT_AVAILABLE_AND_NOT_SELECTED';
                    }
                    break;
                }
                case "paymentModule":
                {
                    if($this->isPaymentModulesListEmpty() == false)
                    {
                        $error_code = 'CHECKOUT_ERR_004_PAYMENT_MODULE_AVAILABLE_BUT_NONE_SELECTED';
                    }
                    else
                    {
                        $error_code = 'CHECKOUT_ERR_004_PAYMENT_MODULE_NOT_AVAILABLE_AND_NOT_SELECTED';
                    }
                    break;
                }
                default:
                {
                    $error_code = 'CHECKOUT_ERR_004_NO_POST_DATA_FOR_PREREQUISITE';
                    break;
                }
            }
            $error_message_parameters = array($this->PrerequisitesValidationResults[$prerequisiteName]["visibleName"], $prerequisiteName);

//            $this->PrerequisitesValidationResults[$prerequisiteName]["isMet"] = false;
//            $this->PrerequisitesValidationResults[$prerequisiteName]["error_code"] = 'CHECKOUT_ERR_004';
//            $this->PrerequisitesValidationResults[$prerequisiteName]["error_message_parameters"] = array($this->PrerequisitesValidationResults[$prerequisiteName]["visibleName"], $prerequisiteName);

//            $isMet = false;
///            $err_params = array(
///                                "CODE"    => "CHECKOUT_ERR_004"
///                               );
///            _fatal($err_params);
        }

/*           --- (a branch is not tested).
            The branch works when no shipping methods matching
            the shipping address exist. */
        $variant_tag = $this->PrerequisitesValidationResults[$prerequisiteName]['variant_tag'];
        //$variant_id = $this->PrerequisitesValidationResults[$prerequisiteName]['variant_id'];
        if( empty($variant_tag) &&
            (($prerequisiteName != "shippingModuleAndMethod") && ($prerequisiteName != "paymentModule")))
        {/* a fast check, with specified names */
            /*fatal error. The data are initialized in the constructor and had
             to be filled already */
            _fatal(__FILE__ . " : " . __LINE__  . " : " . $prerequisiteName);
        }
        else
        {
            $this->setPrerequisitesValidationResultsItem($prerequisiteName, $variant_tag, $isMet, $error_code, $error_message_parameters, $validatedData);
        }
        //return false;
    }

    function areValidShippingModuleAndMethodIDs($module_id, $method_id)
    {
        if($module_id == $this->getNotNeedShippingModuleID() and $method_id == $this->getNotNeedShippingMethodID())
            return true;

        $pm_list = Checkout::getInstalledAndActiveModulesListData("shipping");

        $items = array();
        $new_selected_module_sort_order = 0;
        foreach ($pm_list as $pm_item)
        {
            //FIXME create/use some mm function to convert class names.
            $name = strtolower($pm_item->name);

            $pmInfo = modApiFunc($name, "getInfo");

            if($pmInfo['GlobalUniqueShippingModuleID'] != $module_id)
            {
                continue;
            }
            else
            {
                return modApiFunc($pm_item->name, "isValidShippingMethodId", $method_id);
            }
        }
        return false;
    }

    function isValidPaymentModuleID($module_id)
    {
        $pm_list = Checkout::getInstalledAndActiveModulesListData("payment");

        $items = array();
        $new_selected_module_sort_order = 0;
        foreach ($pm_list as $pm_item)
        {
            //FIXME create/use some mm function to convert class names.
            $name = strtolower($pm_item->name);

            $pmInfo = modApiFunc($name, "getInfo");

            if($pmInfo['GlobalUniquePaymentModuleID'] != $module_id)
            {
                continue;
            }
            else
            {
                return true;
            }
        }
        return false;
    }

    function getPrerequisiteValidationResults($prerequisite_name)
    {
        //TODO check if $prerequisite_name is acceptable.
        //FIXME check the realization of the arrays in af.
        if(isset($this->PrerequisitesValidationResults[$prerequisite_name]))
        {
            return $this->PrerequisitesValidationResults[$prerequisite_name];
        }
        else
        {
            return false;
        }
    }

    function getPrerequisiteStoreBlock($prerequisite_name)
    {
        //TODO check if $prerequisite_name is acceptable.
        return $this->PrerequisiteToStoreBlockTable[$prerequisite_name];
    }

    function getPrerequisiteValidationFunction($prerequisite_name)
    {
        //TODO check if $prerequisite_name is acceptable.
        //FIXME Second question PrerequisiteValidation or maybe StoreBlockValidation ?
        return $this->PrerequisitesValidationFunctionsTable[$prerequisite_name];
    }

    function getPrerequisiteErrorToStepIDTable()
    {
        //Read out the table.
        // The step checkout => prerequisite1, prerequisite2, prerequisite3 ...
        // Then, note, that each prerequisite appears at some step and is required
        // at the next steps.
        // To define at which step to redirect the customer, we should see all
        // prerequisites, and find for each of them that initial step, where they
        // become discontent. Select a min value from all the found prerequisites.
        // The last step of the algorythm (the selection of the value from all discontent ones)
        // is not included in the function getPrerequisiteToStepIDTable().

        // FIXME: it's a little bit inconsequent. When checking errors, it is considered that,
        // if some prerequisite was required at step i, then it must be required at the next steps too.
        // But in the ini file in the table Option-prerequisite can be written a configuration that
        // doesn't correspond this idea. What should be changed in this case: the error handling
        //  or the flexibility of the table prerequisites should be reduced
        // (e.g. legalize the idea described above) is not clear.

        $prerequisites_table = $this->getPrerequisitesINI();
        // Select all available prerequisites from the list of lists. Then for each of them find the first step in the
        // checkout where this prerequisite occurs.
        $prerequisites_list = array();
        foreach($prerequisites_table as $step_id => $prerequisites_list_for_some_step)
        {
            foreach($prerequisites_list_for_some_step as $prerequisite)
            {
                if(!array_key_exists($prerequisite, $prerequisites_list))
                {
                    $prerequisites_list[$prerequisite] = $step_id - 1; //NOTE "-1" !
                    //Under assumption that if prerequisite is already required at step i,
                    //  then it should be entered at step i-1 or earlier.
                }
            }
        }

        return $prerequisites_list;
    }

    /**#@-*/

    /* remove this function after replacing it with new logic. It is not consistent
    with dynamic & flexible checkout cz nature */
    function isLastStepWithPrerequisites($step_id)
    {
        //FIXME
        //Simple search stub
        //WARNING: prerequisites may come not in alphanumeric order in .ini
        //  e.g.
        //  Option-prerequisite2 = ...
        //  Option-prerequisite1 = ...
        //  Option-prerequisite3 = ...
        $prerequisitesTable = $this->getPrerequisitesINI();
        for($i=1;;$i++)
        {
            if(!array_key_exists($i, $prerequisitesTable))
            {
                //Suppose, $i is the last step with prerequisites
                return ($i-1 == $step_id);
            }
        }
    }

    /**
     * Is the outputted view is CheckoutConfirmation?
     * It is used to replace the address in the form, for example, to send
     * data to the payment gateway.
     */
    function isCheckoutConfirmationStep($step_id)
    {
        //FIXME
        //Simple stub
        //Suppose for the time being, that "the last step (with the max number in the config file)" =>
        // "CheckoutConfirmation"
        //The one genuine checking is to find in the template a block of
        // "CheckoutConfirmation" entries.
        //As a variant, always add checkout to the last step, which is
        // CheckoutConfirmation and remove it from the configuration file.
        return $this->isLastStepWithPrerequisites($step_id);
    }

    function getPaymentModuleShippingSettings($module_id)
    {
        global $application;

        $mInfo = Checkout::getPaymentModuleInfo($module_id);
        if($mInfo === NULL)
        {
            return array("PerOrderPaymentModuleShippingFee" => PRICE_N_A);
        }
        else
        {
            if(is_callable(array($mInfo["APIClassName"],"getPerOrderPaymentModuleShippingFee")))
            {
                return array("PerOrderPaymentModuleShippingFee" => call_user_func(array($mInfo["APIClassName"], 'getPerOrderPaymentModuleShippingFee')));
            }
            else
            {
                return array("PerOrderPaymentModuleShippingFee" => PRICE_N_A);
            }
        }
    }

    /**
     * Outputs the price of the defined products, services (shipping, payment).
     * It is used to specify the names of different costs: Subtotal,
     * Total and others. It is important, when for example the shipping cost
     * depends on payment method (example: the shipping cost of some services
     * is higher if a payment method is cash on delivery). That's why it's
     * important what goes first the payment method description or the shipping
     * method.
     *
     * If the required price type is not defined, output null.
     *
     * @param $shipping_method_cost - calculate per item of
     * and total shipping cost, in case the cost of the selected method
     * is known and is shipping_method_cost. It might be used to output
     * shipping cocts on the page of shipping method selection.
     *
     * It is called by the shipping modules.
     */
    function getOrderPrice($type, $shipping_method_cost = NULL)
    {
        $value = null;

       /*
        What for is the order info used here?

        All order information is in the session and should be determined,
        untill the order is created in the database.

        Once the order is created, nothing exists in the session, all order info
        can be taken from the database.

        Here you can get info about the last order (the last order in
        this session, if a customer will start the checkout again,
        then lastPlacedOrderID is zeroed).

        Warning: to make all parameters of the price ($type) available
        after creating the order, it's necessary to save them correctly to the
        database in the method createOrderInDB and to load them correctly from
        the database in the method getOrderInfo.
        */
        if(!empty($this->lastPlacedOrderID))
        {
            $orderInfo = $this->getOrderInfo($this->lastPlacedOrderID);
        }
        else
        {
            $orderInfo = array();
        }

       /* Initialize the array $_shipping_cost_array. It should be done before
           switch, as this code is used in almost all cases.
        */
        if($this->getChosenShippingModuleIdCZ() === NULL)
        {
            //Shipping Module (shipping method) is not selected yet.
            $_shipping_cost_array = null;
        }
        else
        {
            $shipping_method_id = $this->getChosenShippingMethodIdCZ();
            $shipping_module_info = Checkout::getShippingModuleInfo($this->getChosenShippingModuleIdCZ());

            $formatted_cart = modApiFunc("Shipping_Cost_Calculator","formatCart",modApiFunc("Cart","getCartContentExt"));
            modApiFunc("Shipping_Cost_Calculator","setShippingInfo",modApiFunc("Checkout", "getPrerequisiteValidationResults", "shippingInfo"));
            $payment_module_info = Checkout::getPaymentModuleShippingSettings($this->getChosenPaymentModuleIdCZ());
            modApiFunc("Shipping_Cost_Calculator","setPaymentModuleInfo",$payment_module_info);
            modApiFunc("Shipping_Cost_Calculator","setCart",$formatted_cart);
            $_shipping_cost_array = modApiFunc("Shipping_Cost_Calculator","getCalculatedMethod",$shipping_module_info["APIClassName"],$shipping_method_id);
        }

        switch($type)
        {
            case "Subtotal":
            {
                if(!empty($orderInfo))
                {
                    $value = $orderInfo["Subtotal"];
                }
                else
                {
                    /*      SalePriceExcludingTaxes               .
                               SalePrice,                ,                   
                       Included                  Default Address,               ,
                                             ,                                       .

                                     Checkout             SalePriceExcludingTaxes.
                                                              .
                                             .              (                     ,
                                               ,               )                (  
                                         ,                             ,              
                                 ,                   ).
                                       Subtotal.                                               .
                                       Shipping.
                                       Total.   Subtotal'                                  ,
                                                        Shipping.
                    */

                    $value = modApiFunc("Cart", "getCartSubtotalExcludingTaxes");

                    $display_totals_including_taxes = modApiFunc('Configuration', 'getValue', SYSCONFIG_STORE_DISPLAY_TOTALS_INCLUDING_TAXES);
                    if($display_totals_including_taxes === true)
                    {
                        $taxes = modApiFunc("Taxes", "getTax");
                        $value += ((isset($taxes['IncludedTaxTotalAmount']) && $taxes['IncludedTaxTotalAmount'] != PRICE_N_A) ? $taxes['IncludedTaxTotalAmount'] : 0.0);
                    }
                }
                break;
            }
            case "DiscountedSubtotal":
            {
                if(!empty($orderInfo))
                {
                    $value = $orderInfo["Price"]["DiscountedSubtotal"];
                }
                else
                {
                    $order_subtotal = $this->getOrderPrice("Subtotal");
                    $o_subtotal = $order_subtotal == PRICE_N_A ? 0 : $order_subtotal;

                    $subtotal_global_discount = $this->getOrderPrice("SubtotalGlobalDiscount");
                    $o_subtotal_global_discount = $subtotal_global_discount == PRICE_N_A ? 0 : $subtotal_global_discount;

                    $subtotal_promo_code_discount = $this->getOrderPrice("SubtotalPromoCodeDiscount");
                    $o_subtotal_promo_code_discount = $subtotal_promo_code_discount == PRICE_N_A ? 0 : $subtotal_promo_code_discount;

                    $quantity_discount = $this->getOrderPrice("QuantityDiscount");
                    $o_quantity_discount = $quantity_discount == PRICE_N_A ? 0 : $quantity_discount;

                    $value = $o_subtotal -
                             $o_subtotal_global_discount -
                             $o_subtotal_promo_code_discount -
                             $o_quantity_discount;

                    if($value < 0.0)
                    {
                        $value = 0.0;
                    }
                }
                break;
            }
            case "DiscountsSum":
            {
                $value = 0.0;
                break;
            }
            case "SubtotalGlobalDiscount":
            {
    			$value = 0.0;
                break;
            }
            case "QuantityDiscount":
            {
                $value = 0.00;
                break;
            }
            case "SubtotalPromoCodeDiscount":
            {
        	    $value = 0.0;
                break;
            }
            case "ShippingMethodCost":
            {
                if(!empty($orderInfo))
                {
                    $value = $orderInfo["Price"]["ShippingMethodCost"];
                }
                else
                {
                    if ($_shipping_cost_array == null)
                        $value = PRICE_N_A;
                    else
                        $value = $_shipping_cost_array["shipping_cost"]["ShippingMethodCost"];
                }
                break;
            }
            case "PerOrderPaymentModuleShippingFee":
            {
                if(!empty($orderInfo))
                {
                    $value = $orderInfo["Price"]["PerOrderPaymentModuleShippingFee"];
                }
                else
                {
                    if ($_shipping_cost_array == null)
                        $value = PRICE_N_A;
                    else
                        $value = $_shipping_cost_array["shipping_cost"]["PerOrderPaymentModuleShippingFee"];
                }
                break;
            }
            case "PerItemShippingCostSum":
            {
                if(!empty($orderInfo))
                {
                    $value = $orderInfo["Price"]["PerItemShippingCostSum"];
                }
                else
                {
                    if ($_shipping_cost_array == null)
                        $value = PRICE_N_A;
                    else
                        $value = $_shipping_cost_array["shipping_cost"]["PerItemShippingCostSum"];
                }
                break;
            }
            case "TotalShippingAndHandlingCost":
            {
                if(!empty($orderInfo))
                {
                    $value = $orderInfo["Price"]["TotalShippingAndHandlingCost"];
                }
                else
                {
                    if ($_shipping_cost_array == null)
                        $value = PRICE_N_A;
                    else
                        $value = $_shipping_cost_array["shipping_cost"]["TotalShippingAndHandlingCost"];
                }
                break;
            }
            case "FreeHandlingForOrdersOver":
            {
                if(!empty($orderInfo))
                {
                    $value = $orderInfo["Price"]["FreeHandlingForOrdersOver"];
                }
                else
                {
                    if ($_shipping_cost_array == null)
                        $value = PRICE_N_A;
                    else
                        $value = $_shipping_cost_array["shipping_cost"]["FreeHandlingForOrdersOver"];
                }
                break;
            }
            case "FreeShippingForOrdersOver":
            {
                if(!empty($orderInfo))
                {
                    $value = $orderInfo["Price"]["FreeShippingForOrdersOver"];
                }
                else
                {
                    if ($_shipping_cost_array == null)
                        $value = PRICE_N_A;
                    else
                        $value = $_shipping_cost_array["shipping_cost"]["FreeShippingForOrdersOver"];
                }
                break;
            }
            case "MinimumShippingCost":
            {
                if(!empty($orderInfo))
                {
                    $value = $orderInfo["Price"]["MinimumShippingCost"];
                }
                else
                {
                    if ($_shipping_cost_array == null)
                        $value = PRICE_N_A;
                    else
                        $value = $_shipping_cost_array["shipping_cost"]["MinimumShippingCost"];
                }
                break;
            }
            case "PerOrderShippingFee":
            {
                if(!empty($orderInfo))
                {
                    $value = $orderInfo["Price"]["PerOrderShippingFee"];
                }
                else
                {
                    if ($_shipping_cost_array == null)
                        $value = PRICE_N_A;
                    else
                        $value = $_shipping_cost_array["shipping_cost"]["PerOrderShippingFee"];
                }
                break;
            }
            case "TotalShippingCharge":
            {
                if(!empty($orderInfo))
                {
                    $value = $orderInfo["Price"]["TotalShippingCharge"];
                }
                else
                {
                    if ($_shipping_cost_array == null)
                        $value = PRICE_N_A;
                    else
                        $value = $_shipping_cost_array["shipping_cost"]["TotalShippingCharge"];
                }
                break;
            }
            case "PerItemHandlingCostSum":
            {
                if(!empty($orderInfo))
                {
                    $value = $orderInfo["Price"]["PerItemHandlingCostSum"];
                }
                else
                {
                    if ($_shipping_cost_array == null)
                        $value = PRICE_N_A;
                    else
                        $value = $_shipping_cost_array["shipping_cost"]["PerItemHandlingCostSum"];
                }
                break;
            }
            case "PerOrderHandlingFee":
            {
                if(!empty($orderInfo))
                {
                    $value = $orderInfo["Price"]["PerOrderHandlingFee"];
                }
                else
                {
                    if ($_shipping_cost_array == null)
                        $value = PRICE_N_A;
                    else
                        $value = $_shipping_cost_array["shipping_cost"]["PerOrderHandlingFee"];
                }
                break;
            }
            case "TotalHandlingCharge":
            {
                if(!empty($orderInfo))
                {
                    $value = $orderInfo["Price"]["TotalHandlingCharge"];
                }
                else
                {
                if ($_shipping_cost_array == null)
                        $value = PRICE_N_A;
                    else
                        $value = $_shipping_cost_array["shipping_cost"]["TotalHandlingCharge"];
                }
                break;
            }
            case "IncludedTax":
            {
                //             Subtotal       .
                //                 "                                subtotal",
                //              - 0.
                if(!empty($orderInfo))
                {
                    //FIXME:           ,                 
                    //                         (                             ,  
                    //  TaxDisplayOptions)                                   
                    //                               .                       
                    //                                                   !

                    //                                                       
                    //                                 ,                        
                    //  "          ".                                               
                    //  TaxDisplayOption' ,                        -    .

                    //                                            ,              
                    //          (2007.07.30)       -       Checkout::getOrderPrice()
                    //
                    //    $order_total = $o_discounted_subtotal +
                    //                   $o_shipping +
                    //                   $o_tax;
                    //    if($display_totals_including_taxes === true)
                    //    {
                    //        $value = $value - $o_included_tax;
                    //    }
                    $order_tax = $this->getOrderPrice("Tax");
                    $o_tax = $order_tax == PRICE_N_A ? 0 : $order_tax;

                    $order_not_included_tax = $this->getOrderPrice("NotIncludedTax");
                    $o_not_included_tax = $order_not_included_tax == PRICE_N_A ? 0 : $order_not_included_tax;

                    $value = $order_tax - $order_not_included_tax;
                    if($value < 0.0)
                    {
                        $value = 0.0;
                    }
                }
                else
                {
                    $display_totals_including_taxes = modApiFunc('Configuration', 'getValue', SYSCONFIG_STORE_DISPLAY_TOTALS_INCLUDING_TAXES);
                    if($display_totals_including_taxes === true)
                    {
                        $result = modApiFunc("Taxes", "getTax");
                        $value = isset($result['IncludedTaxTotalAmount']) ? $result['IncludedTaxTotalAmount'] : 0.0;
                    }
                    else
                    {
                        $value = 0.0;
                    }
                }
                break;
            }
            case "NotIncludedTax":
            {
                //             Subtotal       .
                //                 "                                subtotal"   
                //                                   .
                if(!empty($orderInfo))
                {
                    $value = $orderInfo["Price"]["OrderNotIncludedTaxTotal"];
                }
                else
                {
                    $order_tax = $this->getOrderPrice("Tax");
                    $o_tax = $order_tax == PRICE_N_A ? 0 : $order_tax;

                    $order_included_tax = $this->getOrderPrice("IncludedTax");
                    $o_included_tax = $order_included_tax == PRICE_N_A ? 0 : $order_included_tax;

                    $value = $o_tax - $o_included_tax;
                    if($value < 0.0)
                    {
                        $value = 0.0;
                    }
                }
                break;
            }
            case "Tax":
            {
                if(!empty($orderInfo))
                {
                    $value = $orderInfo["Price"]["OrderTaxTotal"];
                }
                else
                {
                    $result = modApiFunc("Taxes", "getTax");
                    $value = isset($result['TaxTotalAmount']) ? $result['TaxTotalAmount'] : 0.0;
                }
                break;
            }
            case "Payment":
            {
                $value = 0;
                break;
            }
            case "Total":
            {
                //FIXME: Shipping may not be defined. Or tax may not be defined.
                $order_discounted_subtotal = $this->getOrderPrice("DiscountedSubtotal");
                $o_discounted_subtotal = $order_discounted_subtotal == PRICE_N_A ? 0 : $order_discounted_subtotal;

                $order_shipping = $this->getOrderPrice("TotalShippingAndHandlingCost");
                $o_shipping = $order_shipping == PRICE_N_A ? 0 : $order_shipping;

/*                $order_tax = $this->getOrderPrice("Tax");
                $o_tax = $order_tax == PRICE_N_A ? 0 : $order_tax;

                $order_included_tax = $this->getOrderPrice("IncludedTax");
                $o_included_tax = $order_included_tax == PRICE_N_A ? 0 : $order_included_tax; */

                $order_not_included_tax = $this->getOrderPrice("NotIncludedTax");
                $o_not_included_tax = $order_not_included_tax == PRICE_N_A ? 0 : $order_not_included_tax;

                $value = $o_discounted_subtotal +
                         $o_shipping +
                         $order_not_included_tax;

/*                //     included                      subtotal -                 ,                    
                //       .
                $display_totals_including_taxes = modApiFunc('Configuration', 'getValue', SYSCONFIG_STORE_DISPLAY_TOTALS_INCLUDING_TAXES);
                if($display_totals_including_taxes === true)
                {
                    $value = $value - $o_included_tax;
                } */

                if($value < 0.0)
                {
                    $value = 0.0;
                }

                break;
            }
            default:
            {
                $err_params = array(
                                    "CODE"    => "CHECKOUT_005"
                                   );
                _fatal($err_params, $type);
                break;
            }
        }
        return $value;
    }

    /**
     * Returns a total product weight.
     *
     * It is called by shipping modules.
     */
    function getOrderWeight($type)
    {
        $value = 0;
        switch($type)
        {
            case "netto":
                $value = modApiFunc("Cart", "getCartProductsWeightNetto");
                break;
            case "brutto":
                $value = $this->getOrderWeight("netto") +
                         $this->getOrderWeight("tare");
                break;
            case "tare":
                $value = 0.0;
                break;
            default:
                $err_params = array(
                                    "CODE"    => "CHECKOUT_006"
                                   );
                _fatal($err_params, $type);
                break;
        }
        return $value;
    }

    /**
     * Returns a product number in the order.
     * It is called by shipping modules.
     */
    function getOrderProductsNumber()
    {
        return modApiFunc("Cart", "getCartProductsQuantity");
    }

    /**
     * Checks if the order id is valid.
     *
     * @param integer $order_id
     * @return boolean
     * @todo do the caching
     */
    function isCorrectOrderId($order_id)
    {
        global $application;

        $order_id += 0;
        // the id can't be a string or has a negative value.
        if (empty($order_id) || !is_int($order_id) || $order_id <= 0)
        {
            return false;
        }
        $tables = $this->getTables();

        $o = $tables['orders']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($query->fCount($o['id']), 'count_id');
        $query->WhereValue($o['id'], DB_EQ, $order_id);
        $result = $application->db->getDB_Result($query);

        // there should only one order that has such id
        if ($result[0]['count_id'] != 1)
        {
            return false;
        }

        return true;
    }

    /**
     * Returns a list of domain values for the order status.
     */
    function getOrderStatusList($s_id=NULL)
    {
        global $application;
        $tables = $this->getTables();
        $s = $tables['order_status']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($s['id'], 'id');
        $query->addSelectField($s['name'], 'name');
        $query->addSelectField($s['descr'], 'descr');
        if ($s_id)
        {
            $query->WhereValue($s['id'], DB_EQ, $s_id);
        }
        // @todo sort order
        $result = $application->db->getDB_Result($query);
        $array = array();
        for ($i=0; $i<sizeof($result); $i++)
        {
            $array[$result[$i]['id']] = array(
                'id' => $result[$i]['id']
               ,'name' => $result[$i]['name']
               ,'descr' => $result[$i]['descr']
            );
        }
        return $array;
    }

    /**
     * Returns a list of domain values for the order payment status.
     */
    function getOrderPaymentStatusList($ps_id=NULL)
    {
        global $application;
        $tables = $this->getTables();
        $ps = $tables['order_payment_status']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($ps['id'], 'id');
        $query->addSelectField($ps['name'], 'name');
        $query->addSelectField($ps['descr'], 'descr');
        if ($ps_id)
        {
            $query->WhereValue($ps['id'], DB_EQ, $ps_id);
        }
        // @todo sort order
        $result = $application->db->getDB_Result($query);
        $array = array();
        for ($i=0; $i<sizeof($result); $i++)
        {
            $array[$result[$i]['id']] = array(
                'id' => $result[$i]['id']
               ,'name' => $result[$i]['name']
               ,'descr' => $result[$i]['descr']
            );
        }
        return $array;
    }

    /**
     * Returns order info with the specified number.
     */
    function getOrderInfo($order_id, $b_encrypted = false)
    {
        global $application;
        if (!$this->isCorrectOrderId($order_id))
        {
            return null;
        }
        $tables = $this->getTables();
        $catalog_tables = modApiFunc('Catalog', 'getTables');

        $o = $tables['orders']['columns'];
        $s = $tables['order_status']['columns'];
        $ps = $tables['order_payment_status']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($o['id'], 'id');
        $query->addSelectField($o['date'], 'date');
        $query->addSelectField($o['status_id'], 'status_id');
        $query->addSelectField($o['person_id'], 'person_id');
        $query->addSelectField($o['payment_status_id'], 'payment_status_id');
        $query->addSelectField($o['payment_method'], 'payment_method');
        $query->addSelectField($o['payment_module_id'], 'payment_module_id');
        $query->addSelectField($o['payment_method_detail'], 'payment_method_detail');
        $query->addSelectField($o['payment_processor_order_id'], 'payment_processor_order_id');
        $query->addSelectField($o['shipping_method'], 'shipping_method');
        $query->addSelectField($o['track_id'], 'track_id');
        $query->addSelectField($o['subtotal'], 'subtotal');
        $query->addSelectField($o['total'], 'total');
        # order status info
        $query->addSelectField($s['name'],       'status');
        $query->addLeftJoin('order_status', $o['status_id'], DB_EQ, $s['id']);
        # order payment status info
        $query->addSelectField($ps['name'],       'payment_status');
        $query->addLeftJoin('order_payment_status', $o['payment_status_id'], DB_EQ, $ps['id']);
        # select only one order
        $query->Where($o['id'], DB_EQ, $order_id);
        $result = $application->db->getDB_Result($query);

        # get order price info
        $op = $tables['order_prices']['columns'];
        $query = &new DB_Select();
        $query->addSelectField($op['minimum_shipping_cost']           , 'minimum_shipping_cost');
        $query->addSelectField($op['free_shipping_for_orders_over']   , 'free_shipping_for_orders_over');
        $query->addSelectField($op['free_handling_for_orders_over']   , 'free_handling_for_orders_over');
        $query->addSelectField($op['per_item_shipping_cost_sum']      , 'per_item_shipping_cost_sum');
        $query->addSelectField($op['per_order_shipping_fee']          , 'per_order_shipping_fee');
        $query->addSelectField($op['shipping_method_cost']            , 'shipping_method_cost');
        $query->addSelectField($op['total_shipping_charge']           , 'total_shipping_charge');
        $query->addSelectField($op['per_item_handling_cost_sum']      , 'per_item_handling_cost_sum');
        $query->addSelectField($op['per_order_handling_fee']          , 'per_order_handling_fee');
        $query->addSelectField($op['total_handling_charge']           , 'total_handling_charge');
        $query->addSelectField($op['total_shipping_and_handling_cost'], 'total_shipping_and_handling_cost');
        $query->addSelectField($op['order_tax_total']                 , 'order_tax_total');
        $query->addSelectField($op['subtotal_global_discount']        , 'subtotal_global_discount');
        $query->addSelectField($op['subtotal_promo_code_discount']    , 'subtotal_promo_code_discount');
        $query->addSelectField($op['quantity_discount']               , 'quantity_discount');
        $query->addSelectField($op['discounted_subtotal']             , 'discounted_subtotal');
        $query->addSelectField($op['order_not_included_tax_total']    , 'order_not_included_tax_total');
        $query->Where($op['order_id'], DB_EQ, $order_id);
        $_prices = $application->db->getDB_Result($query);
        $prices = array();
        for ($i=0; $i<sizeof($_prices); $i++)
        {
            //Remove artificial attributes: ShippingCostCorrector Settings
            $prices['MinimumShippingCost']          = $_prices[$i]['minimum_shipping_cost'];
            $prices['FreeShippingForOrdersOver']    = $_prices[$i]['free_shipping_for_orders_over'];
            $prices['FreeHandlingForOrdersOver']    = $_prices[$i]['free_handling_for_orders_over'];
            $prices['PerItemShippingCostSum']       = $_prices[$i]['per_item_shipping_cost_sum'];
            $prices['PerOrderShippingFee']          = $_prices[$i]['per_order_shipping_fee'];
            $prices['ShippingMethodCost']           = $_prices[$i]['shipping_method_cost'];
            $prices['TotalShippingCharge']          = $_prices[$i]['total_shipping_charge'];
            $prices['PerItemHandlingCostSum']       = $_prices[$i]['per_item_handling_cost_sum'];
            $prices['PerOrderHandlingFee']          = $_prices[$i]['per_order_handling_fee'];
            $prices['TotalHandlingCharge']          = $_prices[$i]['total_handling_charge'];
            $prices['TotalShippingAndHandlingCost'] = $_prices[$i]['total_shipping_and_handling_cost'];
            $prices['OrderTaxTotal']                = $_prices[$i]['order_tax_total'];
            $prices['SubtotalGlobalDiscount']       = $_prices[$i]['subtotal_global_discount'];
            $prices['SubtotalPromoCodeDiscount']    = $_prices[$i]['subtotal_promo_code_discount'];
            $prices['QuantityDiscount']             = $_prices[$i]['quantity_discount'];
            $prices['DiscountedSubtotal']           = $_prices[$i]['discounted_subtotal'];
            $prices['OrderNotIncludedTaxTotal']     = $_prices[$i]['order_not_included_tax_total'];
        }
        //      :
        $otx = $tables['order_taxes']['columns'];
        $query = &new DB_Select();
        $query->addSelectField($otx['type'], 'type');
        $query->addSelectField($otx['value'], 'value');
        $query->Where($otx['order_id'], DB_EQ, $order_id);
        $_taxes = $application->db->getDB_Result($query);
        for ($i=0; $i<sizeof($_taxes); $i++)
        {
            $prices["Taxes['" . $_taxes[$i]['type'] . "']"] = $_taxes[$i]['value'];
        }

        # get Customer, Shipping and Billing info
        $opd = $tables['order_person_data']['columns'];
        $pit = $tables['person_info_types']['columns'];
        $piv = $tables['person_info_variants']['columns'];
        $pa  = $tables['person_attributes']['columns'];
        $query = &new DB_Select();
        $query->addSelectField($opd['id'], 'id');
        $query->addSelectField($opd['variant_id'], 'person_info_variant_id');
        $query->addSelectField($opd['attribute_id'], 'person_attribute_id');
        $query->addSelectField($opd['name'], 'name');
        $query->addSelectField($opd['value'], 'value');
        $query->addSelectField($opd['desc'], 'descr');
        $query->addSelectField($opd['b_encrypted'], 'b_encrypted');
        $query->addSelectField($opd['encrypted_secret_key'], 'encrypted_secret_key');
        $query->addSelectField($opd['rsa_public_key_asc_format'], 'rsa_public_key_asc_format');
        # info type information
        $query->addSelectField($pit['id'], 'pit_id');
        $query->addSelectField($pit['tag'], 'pit_tag');
        $query->addSelectField($pit['name'], 'pit_name');
        $query->addSelectField($pit['descr'], 'pit_descr');
        $query->addLeftJoin('person_info_variants', $opd['variant_id'], DB_EQ, $piv['id']);
        # attribute info for the info type
        $query->addSelectField($pa['tag'], 'pa_tag');
        $query->addSelectField($pa['input_type_id'], 'pa_input_type_id');
        $query->addLeftJoin('person_attributes', $opd['attribute_id'], DB_EQ, $pa['id']);
        //Variant type is the second key in the variant table person info
        $query->addSelectField($piv['tag'], 'piv_tag');

        $query->Where($pit['id'], DB_EQ, $piv['type_id']);
        $query->WhereAnd();
        # for one order only
        $query->Where($opd['order_id'], DB_EQ, $order_id);
        $query->WhereAnd();
        # either only encrypted order info, or only not decrypted info
        $query->Where($opd['b_encrypted'], DB_EQ, $b_encrypted === true ? "1" : "0");
        $_person_data = $application->db->getDB_Result($query);

        $person_data = array();
        for ($i=0; $i<sizeof($_person_data); $i++)
        {
            if (!isset($person_data[$_person_data[$i]['pit_tag']]))
            {
                $person_data[$_person_data[$i]['pit_tag']] = array(
                    'id'    => $_person_data[$i]['pit_id']
                   ,'person_info_variant_id' => $_person_data[$i]['person_info_variant_id']
                   ,'tag'   => $_person_data[$i]['pit_tag']
                   ,'name'  => $_person_data[$i]['pit_name']
                   ,'descr' => $_person_data[$i]['pit_descr']
                   ,'attr'  => array()
                );
            }
            $info_tag = $_person_data[$i]['pit_tag'];
            $attr_tag = $_person_data[$i]['pa_tag'];
            $person_data[$info_tag]['attr'][$attr_tag] = array (
                'id'                   => $_person_data[$i]['id']
               ,'person_attribute_id'  => $_person_data[$i]['person_attribute_id']
               ,'tag'                  => $attr_tag
               ,'name'                 => $_person_data[$i]['name']
               ,'value'                => $_person_data[$i]['value']
               ,'descr'                => $_person_data[$i]['descr']
               ,'b_encrypted'          => $_person_data[$i]['b_encrypted']
               ,'encrypted_secret_key' => $_person_data[$i]['encrypted_secret_key']
               ,'rsa_public_key_asc_format' =>
                                          $_person_data[$i]['rsa_public_key_asc_format']
               ,'input_type_id'        => $_person_data[$i]['pa_input_type_id']
                );
        }

        # artificial attribute 'name'
        /*
        $person_data['customerInfo']['attr']['name'] = array(
            'id' => null
           ,'tag' => 'name'
           ,'name' => 'Customer Name'
           ,'value' => isset($person_data['customerInfo']) ? $person_data['customerInfo']['attr']['Firstname']['value'] . " " . $person_data['customerInfo']['attr']['Lastname']['value'] : ""
           ,'descr' => null
           ,'b_encrypted' => ""
           ,'encrypted_secret_key' => ""
           ,'rsa_public_key_asc_format' => ""
           ,'input_type_id' => "1" //<input type="text">
        );
        */

        # get product info
        $opr = $tables['order_product']['columns'];
        $query = &new DB_Select();
        $query->addSelectField($opr['id'], 'id');
        $query->addSelectField($opr['name'], 'name');
        $query->addSelectField($opr['type'], 'type');
        $query->addSelectField($opr['qty'], 'qty');
        $query->addSelectField($opr['store_id'], 'store_id');
        $query->Where($opr['order_id'], DB_EQ, $order_id);
        $_products = $application->db->getDB_Result($query);
        $products = array();

        for ($i=0; $i<sizeof($_products); $i++)
        {
            $product = array(
                'id' => $_products[$i]['id']
               ,'storeProductID' => $_products[$i]['store_id']
               ,'name' => $_products[$i]['name']
               ,'type' => $_products[$i]['type']
               ,'qty' => $_products[$i]['qty']
               ,'attr' => array()
            );
            # get info of all attributes this for this product
            $opta = $tables['order_product_to_attributes']['columns'];
            $a = $catalog_tables['attributes']['columns'];
            $query = &new DB_Select();
            $query->addSelectField($opta['value'], 'value');
            # real attribute info
            $query->addSelectField($a['view_tag'], 'a_tag');
            $query->addSelectField($a['name'], 'a_name');
            $query->addLeftJoin('attributes', $opta['attribute_id'], DB_EQ, $a['id']);
            $query->Where($opta['product_id'], DB_EQ, $_products[$i]['id']);
            $_attr = $application->db->getDB_Result($query);
            for ($j=0; $j<sizeof($_attr); $j++)
            {
                $a_tag = $_attr[$j]['a_tag'];
                $product['attr'][$a_tag] = array(
                    'name'  => $_attr[$j]['a_name']
                   ,'value' => $_attr[$j]['value']
                );
                $product[$a_tag] = $_attr[$j]['value'];
            }

            // get custom attributes
            $opca = $tables['order_product_custom_attributes']['columns'];
            $query = &new DB_Select();
            $query->addSelectField($opca['tag'], 'tag');
            $query->addSelectField($opca['name'], 'name');
            $query->addSelectField($opca['value'], 'value');
            $query->WhereValue($opca['product_id'], DB_EQ, $_products[$i]['id']);
            $product['custom_attributes'] = $application->db->getDB_Result($query);

            $products[] = $product;
        }

        # get comment info
        $on = $tables['order_notes']['columns'];
        $query = &new DB_Select();
        $query->addSelectField($on['type'], 'type');
        $query->addSelectField($on['date'], 'date');
        $query->addSelectField($on['content'], 'content');
        $query->Where($on['order_id'], DB_EQ, $order_id);
        $query->SelectOrder($on['date'], 'DESC');
        $_notes = $application->db->getDB_Result($query);
        $comments = array();
        $history = array();
        for ($i=0; $i<sizeof($_notes); $i++)
        {
            if ($_notes[$i]['type'] == 'comment')
            {
                $_notes[$i]['content'] = str_replace("\n", '<br/>', $_notes[$i]['content']);
                $comments[] = $_notes[$i];
            }
            elseif ($_notes[$i]['type'] == 'history')
            {
                $_notes[$i]['content'] = str_replace("\n", '<br/>', $_notes[$i]['content']);
                $history[] = $_notes[$i];
            }
        }

        $order = array();
        $order['ID'] = sprintf("%05d", $result[0]['id']);
        $order['Date'] = $result[0]['date'];
        $order['StatusId'] = $result[0]['status_id'];
        $order['Status'] = $result[0]['status'];
        $order['PaymentStatusId'] = $result[0]['payment_status_id'];
        $order['PaymentStatus'] = $result[0]['payment_status'];
        $order['PaymentMethod'] = $result[0]['payment_method'];
        $order['PaymentModuleId'] = $result[0]['payment_module_id'];
        $order['PaymentMethodDetail'] = $result[0]['payment_method_detail'];
        $order['PaymentProcessorOrderId'] = $result[0]['payment_processor_order_id'];
        $order['ShippingMethod'] = $result[0]['shipping_method'];
        $order['TrackId'] = $result[0]['track_id'];
        $order['Products'] = $products;
        $order['Subtotal'] = $result[0]['subtotal'];
        $order['Total'] = $result[0]['total'];
        $order['Price'] = $prices;
        $order['Comments'] = $comments;
        $order['History'] = $history;
        //$order['Customer'] = array_key_exists('customerInfo', $person_data) ? $person_data['customerInfo'] : array('attr' => array());
        $order['Billing'] = array_key_exists('billingInfo', $person_data) ? $person_data['billingInfo'] : array('attr' => array());
        $order['Shipping'] = array_key_exists('shippingInfo', $person_data) ? $person_data['shippingInfo'] : array('attr' => array());
        $order['CreditCard'] = array_key_exists('creditCardInfo', $person_data) ? $person_data['creditCardInfo'] : array('attr' => array());
        $order['BankAccount'] = array_key_exists('bankAccountInfo', $person_data) ? $person_data['bankAccountInfo'] : array('attr' => array());

        $order['PersonId'] = $result[0]['person_id'];
        //$order['Person'] = $this->getPersonInfo($result[0]['person_id']);

        return $order;
    }

    /**
     * Returns an order list.
     */
    function getOrderList()
    {
        global $application;

        $tables = $this->getTables();

        $o = $tables['orders']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($o['id'], 'id');
        # use a filter
        $filter = $this->order_search_filter;
        if ($filter['search_by'] == 'id' && $filter['order_id'] != null)
        {
            $query->Where($o['id'], DB_EQ, $filter['order_id']);
        }
        elseif ($filter['search_by'] == 'date')
        {
            $from_date = $filter['from_year'] . "-" . $filter['from_month'] . "-" . $filter['from_day'];
            $to_date = $filter['to_year'] . "-" . $filter['to_month'] . "-" . $filter['to_day'];
            $query->Where($o['date'], DB_GTE, "'".$from_date."'");
            $query->WhereAND();
            $query->Where($o['date'], DB_LTE, "'".$to_date."'");
            if (!empty($filter['status_id']))
            {
                $query->WhereAND();
                $query->Where($o['status_id'], DB_EQ, $filter['status_id']);
            }
            if (!empty($filter['payment_status_id']))
            {
                $query->WhereAND();
                $query->Where($o['payment_status_id'], DB_EQ, $filter['payment_status_id']);
            }
        }
        elseif ($filter['search_by'] == 'status' && $filter['filter_status_id'] != 0)
        {
            $query->Where($o['status_id'], DB_EQ, $filter['filter_status_id']);
        }
        $query->SelectOrder($o['date'], 'DESC');
        $query->SelectOrder($o['id'], 'DESC');
        $query = modApiFunc('paginator', 'setQuery', $query);
        $result = $application->db->getDB_Result($query);

        $orders = array();
        for ($i=0; $i<sizeof($result); $i++)
        {
//            $order = $this->getOrderInfo($result[$i]['id']);
//            array_push($orders, $order);
            array_push($orders, $result[$i]['id']);
        }
        return $orders;
    }

    /**
     * Calculates order count with the specified status.
     */
    function getOrderCount($status_id)
    {
        if ($status_id === null)
        {
            return 0;
        }

        global $application;

        $tables = $this->getTables();

        $o = $tables['orders']['columns'];

        $query = &new DB_Select();
        $query->addSelectField("count(".$o['id'].")", 'count');
        if ($status_id != 0)
        {
            $query->Where($o['status_id'], DB_EQ, $status_id);
        }
        $result = $application->db->getDB_Result($query);

        return $result[0]['count'];
    }

    /**
     * Returns filter array for orders.
     */
    function getOrderSearchFilter()
    {
        return $this->order_search_filter;
    }

    /**
     * Saves filter array for orders.
     */
    function setOrderSearchFilter($array)
    {
        $this->order_search_filter = $array;
    }

    /**
     * Returns filter array for customers.
     */
    function getCustomerSearchFilter()
    {
        return $this->customer_search_filter;
    }

    /**
     * Saves filter array for customers.
     */
    function setCustomerSearchFilter($array)
    {
        $this->customer_search_filter = $array;
    }

    function getCurrentPaymentModuleSettingsViewName()
    {
        return $this->CurrentPaymentModuleSettingsViewName;
    }

    /**
     * Returns the id, chosen for pament module.
     * CZ.
     * WARNING: the customer should have chosen the module by that moment!
     */
    function getChosenPaymentModuleIdCZ()
    {
        $PrerequisiteValidationResults = $this->getPrerequisiteValidationResults('paymentModule');
        $module_id = empty($PrerequisiteValidationResults['validatedData']['method_code']['value']) ? NULL : $PrerequisiteValidationResults['validatedData']['method_code']['value'];
        if(NULL == $module_id)
        {
            //Output an error message: Payment Module (payment method) is not chosen.
            //FIXME: TEST IT
            //Returns NULL, and for example, not the id of the AllInactive module, because
            //  nobody will know what is wrong: no methods were activated,
            //  but PaymentMethodInput were in the templates, or prerequisite PaymentModule
            //  was removed from checkout-config.ini, or it was left
            //  PaymentMethodOutput, but PaymentMethodInput was removed.
            //So each object that calls getChosenPaymentModuleIdCZ can processes
            //   NULL, the way it likes:
            //  for example:
            //  outputs null or AllInactive data, or an error, or something else.
            return NULL;

            ///$err_params = array(
            /// "CODE"    => "CHECKOUT_001"
            ///);
            ///_fatal($err_params);
        }
        else
        {
            return $module_id;
        }
    }

    function getChosenShippingModuleIdCZ()
    {
        $PrerequisiteValidationResults = $this->getPrerequisiteValidationResults('shippingModuleAndMethod');
        if($PrerequisiteValidationResults["isMet"] == false)
        {
            return NULL;
        }

        $module_and_method_ids = empty($PrerequisiteValidationResults['validatedData']['method_code']['value']) ? NULL : $PrerequisiteValidationResults['validatedData']['method_code']['value'];
        if(NULL == $module_and_method_ids)
        {
            return NULL;

            /// Output an error message: Shipping Module (shipping method) is not chosen.
            ////FIXME: TEST IT
            //$err_params = array(
            //      "CODE"    => "CHECKOUT_002"
            //);
            //_fatal($err_params);
        }
        else
        {
            //// Module and Method
            $shipping_module_and_method_pattern = "/^([0-9A-F]{8}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{12})_(\d+)$/";

            $matches = array();
            if(preg_match($shipping_module_and_method_pattern, $module_and_method_ids, $matches))
            {
                return $matches[1];
            }
            else
            {

                /////Output an error message: shipping module/method data format is not valid.
                /////FIXME: TEST IT
                ///$err_params = array(
                /////Must NOT be.
                ///     "CODE"    => "CHECKOUT_003"
                ///);
                ///_fatal($err_params);
                return NULL;
            }
        }
    }

    function getChosenShippingMethodIdCZ()
    {
        $PrerequisiteValidationResults = $this->getPrerequisiteValidationResults('shippingModuleAndMethod');
        $module_and_method_ids = empty($PrerequisiteValidationResults['validatedData']['method_code']['value']) ? NULL : $PrerequisiteValidationResults['validatedData']['method_code']['value'];
        if(NULL == $module_and_method_ids)
        {
            //  Output an error message: Shipping Module (shipping method) is not chosen.
            //FIXME: TEST IT
            return NULL;

            ///$err_params = array(
            ///     "CODE"    => "CHECKOUT_002"
            ///);
            ///_fatal($err_params);
        }
        else
        {
            //// Module and Method
            $shipping_module_and_method_pattern = "/^([0-9A-F]{8}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{12})_(\d+)$/";

            $matches = array();
            if(preg_match($shipping_module_and_method_pattern, $module_and_method_ids, $matches))
            {
                return $matches[2];
            }
            else
            {
                //Output an error message: shipping module/method data format is not valid.
                //FIXME: TEST IT
                //Must not be
                ///$err_params = array(
                ///     "CODE"    => "CHECKOUT_003"
                ///);
                ///_fatal($err_params);
                return NULL;
            }
        }
    }

    /* static */ function getPaymentModuleInfo($module_id = null)
    {
        global $application;

        $pm_list = Checkout::getInstalledModulesListData("payment");

        $items = array();
        foreach ($pm_list as $pm_item)
        {
            //FIXME create/use some mm function to convert class names.
            $name = strtolower($pm_item->name);

            $pmInfo = modApiFunc($name, "getInfo");

            if($pmInfo['GlobalUniquePaymentModuleID'] == $module_id)
            {
                return $pmInfo;
            }
            ////FIXME : check if function exists
            //call_user_func($smInfo['CZInputViewClassName']));
        }
        return null;
    }

    /* static */ function getShippingModuleInfo($module_id = null)
    {
        global $application;

        if($module_id == $this->getNotNeedShippingModuleID())
        {
            return array(
                'GlobalUniqueShippingModuleID' => $module_id
               ,'APIClassName' => 'Shipping_Not_Needed'
            );
        }

        $sm_list = Checkout::getInstalledModulesListData("shipping");

        $items = array();
        foreach ($sm_list as $sm_item)
        {
            //FIXME create/use some mm function to convert class names.
            $name = strtolower($sm_item->name);

            $smInfo = modApiFunc($name, "getInfo");

            if($smInfo['GlobalUniqueShippingModuleID'] == $module_id)
            {
                return $smInfo;
            }
            ////FIXME : check if function exists
            //call_user_func($smInfo['CZInputViewClassName']));
        }
        return null;
    }

    function getShippingMethodInfo($module_id = null, $method_id = null)
    {
        if($module_id == $this->getNotNeedShippingModuleID() and $method_id == $this->getNotNeedShippingMethodID())
        {
            return array(
                'id' => $method_id
               ,'method_name' => $this->getNotNeedShippingMethodName()
               ,'method_code' => 'NNS'
               ,'destination' => 'L'
               ,'available' => 'Y'
               ,'cost' => 0.0
            );
        }

        global $application;

        $sm_list = Checkout::getInstalledModulesListData("shipping");

        $items = array();
        foreach ($sm_list as $sm_item)
        {
            //FIXME create/use some mm function to convert class names.
            $name = strtolower($sm_item->name);

            $smInfo = modApiFunc($name, "getInfo");

            if($smInfo['GlobalUniqueShippingModuleID'] == $module_id)
            {
                $method_info = modApiFunc($name, "getShippingMethodInfo", $method_id);
                return $method_info;
            }
            ////FIXME : check if function exists
            //call_user_func($smInfo['CZInputViewClassName']));
        }
        return null;
    }

    function setCurrentPaymentModuleSettingsViewName($view_class_name)
    {
        $this->CurrentPaymentModuleSettingsViewName = $view_class_name;
    }

    function getCurrentShippingModuleSettingsViewName()
    {
        return $this->CurrentShippingModuleSettingsViewName;
    }

    function setCurrentShippingModuleSettingsViewName($view_class_name)
    {
        $this->CurrentShippingModuleSettingsViewName = $view_class_name;
    }

    /**
     * Gives a list of all payment modules exist in the system.
     * I.e. loaded AVACTIS modules with     * 'groups', containing PaymentModule.
     */
    /* static */ function getInstalledModulesListData($ModuleClass, $groups = NULL, $b_with_uuid = false)
    {
        global $application;
        if($groups == null)
        {
            $groups = array();
        }
        if($ModuleClass == "payment")
        {
            $groups[] = "PaymentModule";
        }
        else if($ModuleClass == "shipping")
        {
            $groups[] = "ShippingModule";
        }
        else
        {
            _fatal(__FILE__ . " : " . __LINE__);
        }
        $m_list = modApiFunc("Modules_Manager", "getActiveModules", $groups);

        //Return also UUID for each module. It's slow, but sometimes necessary.
        if($b_with_uuid === true)
        {
            foreach($m_list as $key => $m_info)
            {
                $m_info =& $m_list[$key];
                $m = &$application->getInstance( $m_info->name );
                $m_info->UUID = $m->getUid();
                unset($m_info);
            }
        }
        return $m_list;
    }

    /**
     * Gives a list of all payment/... modules exist in the system.
     * I.e. loaded AVACTIS modules with     * 'groups', containing PaymentModule/....
     * and activated by administrator.
     */
    /*static*/ function getInstalledAndActiveModulesListData($ModuleClass, $groups = NULL)
    {
        global $application;
        if($groups == null)
        {
            $groups = array();
        }
        if($ModuleClass == "payment")
        {
            $groups[] = "PaymentModule";
        }
        else if($ModuleClass == "shipping")
        {
            $groups[] = "ShippingModule";
        }
        else
        {
            _fatal(__FILE__ . " : " . __LINE__);
        }

        $m_list = modApiFunc("Modules_Manager", "getActiveModules", $groups);
        $active_modules = Checkout::getActiveModules($ModuleClass);
        //Make a name array for
        $active_modules_class_names = array();
        foreach($active_modules as $uuid => $m_info)
        {
            $active_modules_class_names[$m_info["module_class_name"]] = $m_info;
        }
        //the end of making the name list.

        foreach($m_list as $key => $module_info)
        {
            if(!array_key_exists($module_info->name, $active_modules_class_names))
            {
                unset($m_list[$key]);
            }
        }
        return $m_list;
    }

    /**
     * Returns complete person info with the specified id, which is not used
     * (Nov.2006). It wasn't tested with the person info variants.
     */
    function getPersonInfo($person_id)
    {
        global $application;
        $tables = $this->getTables();

        $pit = $tables['person_info_types']['columns'];
        $piv = $tables['person_info_variants']['columns'];
        $ptiv = $tables['person_to_info_variants']['columns'];
        $pd = $tables['persons_data']['columns'];

        # get a list of variants, associated with the person
        $query = &new DB_Select();
        #get data on person info type variant
        $query->addSelectField($piv['tag'], 'piv_tag');
        $query->addLeftJoin('person_info_variants', $ptiv['variant_id'], DB_EQ, $piv['id']);
        $query->addSelectField($pit['id'], 'type_id');
        $query->addLeftJoin('person_info_types', $pit['id'], DB_EQ, $piv['type_id']);
        $query->addSelectField($ptiv['variant_id'], 'variant_id');
        $query->Where($ptiv['person_id'], DB_EQ, $person_id);
        $result = $application->db->getDB_Result($query);

        # get all data associated with that person
        $query = &new DB_Select();
        $query->addSelectField($pd['value'], 'value');
        $query->addSelectField($pd['variant_id'], 'variant_id');
        $query->addSelectField($pd['attribute_id'], 'attribute_id');
        $query->Where($pd['person_id'], DB_EQ, $person_id);
        $data_result = $application->db->getDB_Result($query);
        # structure for the following usage
        $person_data = array();
        foreach ($data_result as $data)
        {
            if (!array_key_exists($data['variant_id'], $person_data))
            {
                $person_data[$data['variant_id']] = array();
            }
            $person_data[$data['variant_id']][$data['attribute_id']] = $data['value'];
        }

        $variants = array();
        for ($i=0; $i<sizeof($result); $i++)
        {
            $variant = $this->getPersonVariantInfo($result[$i]['variant_id']);
            # real attribute values for this type
            if (array_key_exists($variant['id'], $person_data))
            {
                $variant_value = &$person_data[$variant['id']];

                # save real values
                foreach ($variant['attr'] as $attr)
                {
                    if (!array_key_exists($attr['id'], $variant_value)) continue;
                    $variant['attr'][$attr['tag']]['value'] = $variant_value[$attr['id']];
                }
            }
            $variants[$variant['type_tag']] = $variant;
        }

        return $variants;
    }

    /**
     * Returns complete info description of the specified variant.
     * As for now it isn't used and has errors. Test and correct it
     * after defining how to work with the persons.
     */
    function getPersonVariantInfo($variant_id)
    {
        global $application;
        $tables = Checkout::getTables();

        $piv = $tables['person_info_variants']['columns'];
        $pit = $tables['person_info_types']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($piv['tag'], 'variant_tag');
        $query->addSelectField($piv['visible_name'], 'visible_name');
        $query->addSelectField($pit['tag'], 'variant_view_tag');
        $query->addSelectField($pit['id'], 'type_id');
        $query->addSelectField($pit['visible_name'], 'type_visible_name');
        $query->Where($piv['id'], DB_EQ, $variant_id);
        $query->WhereAnd();
        $query->Where($piv['type_id'], DB_EQ, $pit['id']);
        $result = $application->db->getDB_Result($query);
        if(sizeof($result) == 1)
        {
            return $result[0];
        }
        else
        {
            return false;
        }
    }

//------------------------------------------------
//              PRIVATE DECLARATION
//------------------------------------------------

    /**
     * Adds product info.
     */
    function addOrderProducts($order_id, $CartProductsInfo)
    {

        global $application;
        $tables =  $this->getTables();

        foreach($CartProductsInfo as $CartProductInfo)
        {
            $table = 'order_product';            # the name of the filled table
            $columns = $tables[$table]['columns'];  # the array of field names of the table

            $CartProductStandardInfoRaw = modApiFunc("Catalog", "getProductInfoRaw", $CartProductInfo['ID'], "standard");
            $query = &new DB_Insert($table);
            $query->addInsertValue($order_id, $columns['order_id']);
            $query->addInsertValue($CartProductInfo['Quantity_In_Cart'], $columns['qty']);
            $query->addInsertValue($CartProductStandardInfoRaw['attributes']['Name']['value'], $columns['name']);
            $query->addInsertValue($CartProductStandardInfoRaw['attributes']['TypeID']['value'], $columns['type']);
            $query->addInsertValue($CartProductStandardInfoRaw['attributes']['ID']['value'], $columns['store_id']);
            $application->db->getDB_Result($query);
            $inserted_order_product_id = $application->db->DB_Insert_Id();

            //Add standard attributes
            $ProductStandardInfoRawWithoutArtificialAttributes = modApiFunc("Catalog", "getProductInfoRaw", $CartProductInfo['ID'], "standard", false);

            foreach($ProductStandardInfoRawWithoutArtificialAttributes['attributes'] as $attr_info)
            {
                //Sale price -          .
                if($attr_info['id'] == SALE_PRICE_PRODUCT_ATTRIBUTE_ID)
                {
                    $display_totals_including_taxes = modApiFunc('Configuration', 'getValue', SYSCONFIG_STORE_DISPLAY_TOTALS_INCLUDING_TAXES);
                    if($display_totals_including_taxes === true)
                    {
                        $attr_info['value'] = $CartProductInfo['CartItemSalePriceIncludingTaxes'];
                    }
                    else
                    {
                        $attr_info['value'] = $CartProductInfo['CartItemSalePriceExcludingTaxes'];
                    }

                }
                $table = 'order_product_to_attributes';             #  the name of the filled table
                $columns = $tables[$table]['columns'];              #  the array of field names of the table

                $query = &new DB_Insert($table);
                $query->addInsertValue($attr_info['id'], $columns['attribute_id']);
                $query->addInsertValue($inserted_order_product_id, $columns['product_id']);
                $query->addInsertValue($attr_info['value'], $columns['value']);
                $application->db->getDB_Result($query);
            }

            //Add custom attributes
            $ProductInfoRawWithoutArtificialAttributes = modApiFunc("Catalog", "getProductInfoRaw", $CartProductInfo['ID'], "custom", false);

            foreach($ProductInfoRawWithoutArtificialAttributes['attributes'] as $attr_info)
            {
                $table = 'order_product_custom_attributes';             #   the name of the filled table
                $columns = $tables[$table]['columns'];              #   the array of field names of the table

                $query = &new DB_Insert($table);
                //$query->addInsertValue($attr_info['id'], $columns['attribute_id']);
                $query->addInsertValue($inserted_order_product_id, $columns['product_id']);
                $query->addInsertValue($attr_info['view_tag'], $columns['tag']);
                $query->addInsertValue($attr_info['name'], $columns['name']);
                $query->addInsertValue($attr_info['value'], $columns['value']);
                $application->db->getDB_Result($query);
            }

            modApiFunc('EventsManager','throwEvent','ProductWasSold',$CartProductInfo['ID'],$CartProductInfo['Quantity_In_Cart']);
        }
    }


    /**
     * Returns a list of Person Info variants.
     *
     * @author Oleg Vlasenko
     * @return array the array of variant ids and their visible prices
     */
    function getPersonInfoVariantList()
    {
        global $application;
        $tables = $this->getTables();
        $columns = $tables['person_info_variants']['columns'];
        $query = &new DB_Select();
        $query->addSelectField($columns['id'], 'id');
        $query->addSelectField($columns['type_id'], 'type_id');
        $query->addSelectField($columns['visible_name'], 'visible_name');

        $result = $application->db->getDB_Result($query);

        $s = $tables['person_info_types']['columns'];

        $array = array();
        for ($i = 0; $i < sizeof($result); $i++)
        {
            $variant_id = $result[$i]['id'];
            $person_info_type_id = $result[$i]['type_id'];
            $visible_name = $this->getClearVariantName($result[$i]['visible_name']);
            $array[$variant_id] = array(
                'variant_id'    => $variant_id
               ,'visible_name'  => $visible_name
               ,'type_id'       => $person_info_type_id
            );
        }

        return $array;
    }

    //                person info type   On    Off       Off    On.
    function flipPersonInfoTypeStatus($person_info_type_id)
    {
        global $application;
        $tables = $this->getTables();
        $columns = $tables['person_info_types']['columns'];
        $query = &new DB_Update('person_info_types');
        $query->addUpdateExpression($columns['active'], "IF(".$columns['active']." = '".DB_TRUE."','".DB_FALSE."','".DB_TRUE."')");
        $query->WhereValue($columns['id'], DB_EQ, $person_info_type_id);
        $application->db->getDB_Result($query);
    }

    /**
     * Returns a list of Person Info types.
     *
     * @return array the array of variant ids and their visible prices
     */
    /* static */ function getPersonInfoTypeList()
    {
        global $application;
        $tables = Checkout::getTables();
        $columns = $tables['person_info_types']['columns'];
        $query = &new DB_Select();
        $query->addSelectField($columns['id'], 'id');
        $query->addSelectField($columns['active'], 'active');
        $query->addSelectField($columns['tag'], 'tag');

        $result = $application->db->getDB_Result($query);

        $array = array();
        for ($i = 0; $i < sizeof($result); $i++)
        {
            $array[$result[$i]['id']] = array(
                'type_id'    => $result[$i]['id']
               ,'active'  => $result[$i]['active']
               ,'tag' => $result[$i]['tag']
            );
        }

        return $array;
    }

    /**
     *            true,                     PersonInfoType'         .
     * false -      .
     */
    /* static */ function arePersonInfoTypesActive($types)
    {
        $type_list = Checkout::getPersonInfoTypeList();
        //                ,                       .
        $type_list_indexed_by_tags = array();
        foreach($type_list as $type_id => $info)
        {
            $type_list_indexed_by_tags[$info['tag']] = $info;
        }

        //                                      
        foreach($types as $tag)
        {
            if(!array_key_exists($tag, $type_list_indexed_by_tags) ||
                $type_list_indexed_by_tags[$tag]['active'] == DB_FALSE)
            {
                return false;
            }
        }
        return true;
    }

    /**
     * Returns a "clear" visible name for the Person Info variant.
     *
     * @author Oleg Vlasenko
     * @param string $variant_name - vsible variant name. For default variants
     *               a visible name begins with substring "[default]"
     * @return string a visible name without substring "[default]"
     */
    function getClearVariantName($variant_name)
    {
        $pos = strrpos($variant_name, "]");
        if ($pos != false)
        {
            $tail = substr($variant_name, $pos + 1);
            return $tail;
        }
        else
        {
            return $variant_name;
        }
    }


    /**
     * Returns a visible name for the Person Info variant.
     *
     * @author Oleg Vlasenko
     * @param integer $variant_id - the Id of the attribute variant.
     * @return string a visible name
     */
    function getVariantNameById($variant_id)
    {
        global $application;
        $tables = $this->getTables();
        $columns = $tables['person_info_variants']['columns'];
        $query = &new DB_Select();
        $query->addSelectField($columns['visible_name'], 'visible_name');
        $query->WhereValue($columns['id'], DB_EQ, $variant_id);

        $result = $application->db->getDB_Result($query);
        $visible_name = $this->getClearVariantName($result[0]['visible_name']);

        return $visible_name;
    }


    /**
     * Returns a list of ids for the Person Info variant, sorted in order
     * fields sort.
     *
     * @author Oleg Vlasenko
     * @param integer $variant_id - the Id of the attribute variant.
     * @return array the array of attribute ids
     */
    function getPersonInfoAttributeIdList($variant_id)
    {
        global $application;
        $tables = $this->getTables();
        $s = $tables['person_info_variants_to_attributes']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($s['attribute_id'], 'attribute_id');
        $query->WhereValue($s['variant_id'], DB_EQ, $variant_id);
        $query->SelectOrder($s['sort']);

        $result = $application->db->getDB_Result($query);

        $ids = array();
        for ($i = 0; $i < sizeof($result); $i++)
        {
            $ids[] = $result[$i]['attribute_id'];
        }

        return $ids;
    }


    /**
     * Returns a list of fields for the Person Info attribute.
     *
     * @author Oleg Vlasenko
     * @param integer $variant_id - the Id of the attribute variant.
     * @param integer $attribute_id - the attribute Id .
     * @return array the array of values of attribute fields.
     */
    function getPersonInfoFieldsList($variant_id, $attribute_id)
    {
        global $application;
        $tables = $this->getTables();
        $s = $tables['person_info_variants_to_attributes']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($s['variant_id'], 'variant_id');
        $query->addSelectField($s['attribute_id'], 'attribute_id');
        $query->addSelectField($s['name'], 'name');
        $query->addSelectField($s['descr'], 'descr');
        $query->addSelectField($s['unremovable'], 'unremovable');
        $query->addSelectField($s['visible'], 'visible');
        $query->addSelectField($s['required'], 'required');

        $query->WhereValue($s['variant_id'], DB_EQ, $variant_id);
        $query->WhereAnd();
        $query->WhereValue($s['attribute_id'], DB_EQ, $attribute_id);

        $result = $application->db->getDB_Result($query);

        if ($result != null)
        {

            $array = array(
                    'variant_id'    => $result[0]['variant_id']
                   ,'attribute_id'  => $result[0]['attribute_id']
                   ,'name'          => $result[0]['name']
                   ,'descr'         => $result[0]['descr']
                   ,'unremovable'   => $result[0]['unremovable']
                   ,'visible'       => $result[0]['visible']
                   ,'required'      => $result[0]['required']
                );

            return $array;
        }
        else
        {
            return null;
        }
    }


    /**
     * Saves a list of fields for the Person Info attribute. If it saves fields
     * for the Country attribute and it is invisible, then the State attribute,
     * associated with it, should be invisible too.
     *
     * @author Oleg Vlasenko
     * @param integer $fields - the array of new values of the fields
     * @return void
     */
    function setPersonInfoFieldList($fields)
    {
        global $application;
        $tables = $this->getTables();
        $columns = $tables['person_info_variants_to_attributes']['columns'];

        $variant_id = $fields['variant_id'];
        $attribute_id = $fields['attribute_id'];

        $fields_original = $this->getPersonInfoFieldsList($variant_id, $attribute_id);

        $fields['unremovable'] = $fields_original['unremovable'];
        if ($fields_original['unremovable'] != 0)
        {
            $fields['visible'] = '1';
            $fields['required'] = '1';
        }
        $query = &new DB_Update('person_info_variants_to_attributes');
        $query->addUpdateValue($columns['name'], $fields['name']);
        $query->addUpdateValue($columns['descr'], $fields['descr']);
        $query->addUpdateValue($columns['unremovable'], $fields['unremovable']);
        $query->addUpdateValue($columns['visible'], $fields['visible']);
        $query->addUpdateValue($columns['required'], $fields['required']);
        $query->WhereValue($columns['variant_id'], DB_EQ, $variant_id);
        $query->WhereAnd();
        $query->WhereValue($columns['attribute_id'], DB_EQ, $attribute_id);
        $application->db->getDB_Result($query);

        // Correct the State field visibility if the Country field vsibility is changed
        if ($attribute_id == 9) // attrubute = Country
        {
            $country_visible = $fields['visible'];
            $states_attribute_id = 7;

            // get the State attribute state
            $query = &new DB_Select();

            $query->addSelectField($columns['visible'], 'visible');
            $query->WhereValue($columns['variant_id'], DB_EQ, $variant_id);
            $query->WhereAnd();
            $query->WhereValue($columns['attribute_id'], DB_EQ, $states_attribute_id); // States

            $result = $application->db->getDB_Result($query);

            $state_visible = $result[0]['visible'];

            // If State is visible and Country is invisible, then hide State
            if (!$country_visible && $state_visible)
            {
                $query = &new DB_Update('person_info_variants_to_attributes');
                $query->addUpdateValue($columns['visible'], 0);
                $query->addUpdateValue($columns['required'], 0);
                $query->WhereValue($columns['variant_id'], DB_EQ, $variant_id);
                $query->WhereAnd();
                $query->WhereValue($columns['attribute_id'], DB_EQ, $states_attribute_id);
                $application->db->getDB_Result($query);
            }
        }

        modApiFunc('EventsManager','throwEvent','CheckoutPersonInfoFieldUpdated',$fields);
    }


    /**
     * Saves the attribute sort in the variant to the DB.
     *
     * @author Oleg Vlasenko
     * @param array $attrSortOrderArray the array of attribute ids, the sequence
     *              of which is defined by sort_order of attributes.
     * @param integer $variantId the Id of the attribute variant.
     * @return void
     */
    function setAttributesSortOrder($attrSortOrderArray, $variantId)
    {
        global $application;

        $i = 1;
        foreach ($attrSortOrderArray as $attrId)
        {
            $tables = $this->getTables();
            $columns = $tables['person_info_variants_to_attributes']['columns'];
            $query = &new DB_Update('person_info_variants_to_attributes');
            $query->addUpdateValue($columns['sort'], $i);
            $query->WhereValue($columns['variant_id'], DB_EQ, $variantId);
            $query->WhereAnd();
            $query->WhereValue($columns['attribute_id'], DB_EQ, $attrId);
            $application->db->getDB_Result($query);
            $i++;
        }
    }


    /**#@+
     * @access private
     */
    function updateOrderPaymentProcessorOrderID($order_id, $new_payment_processor_order_id)
    {
        global $application;
        $tables = $this->getTables();
        $o = $tables['orders']['columns'];

        $query = &new DB_Update('orders');
        $query->addUpdateValue($o['payment_processor_order_id'], $new_payment_processor_order_id);
        $query->WhereValue($o["id"], DB_EQ, $order_id);
        $result = $application->db->getDB_Result($query);
    }

    /**#@+
     * @access private
     */
    function addOrder($data)
    {
        global $application;

        $tables =  $this->getTables();

        $table = 'orders';            #    the name of the filled table
        $columns = $tables[$table]['columns'];  #    the array of field names of the table

        $query = &new DB_Insert($table);
        $query->addInsertExpression($query->fNow(), $columns['date']);
        $query->addInsertValue($data['status_id'], $columns['status_id']);
        $query->addInsertValue($data['payment_status_id'], $columns['payment_status_id']); //Waiting
        $query->addInsertValue($data['payment_method'], $columns['payment_method']);
        $query->addInsertValue($data['payment_module_id'], $columns['payment_module_id']);
        $query->addInsertValue($data['payment_method_detail'], $columns['payment_method_detail']);
        $query->addInsertValue($data['payment_processor_order_id'], $columns['payment_processor_order_id']);
        $query->addInsertValue($data['shipping_method'], $columns['shipping_method']);
        $query->addInsertValue($data['track_id'], $columns['track_id']);
        $query->addInsertValue($data['subtotal'], $columns['subtotal']);
        $query->addInsertValue($data['total'], $columns['total']);
        $query->addInsertValue($data['person_id'], $columns['person_id']);

//        $query->addInsertValue(2, $columns['billing_method']);
//        $query->addInsertValue(1, $columns['shipping_method']);
//        $query->addInsertValue("tax info", $columns['tax_info']);
//        $query->addInsertValue("products", $columns['products']);

        $application->db->getDB_Result($query);

        $inserted_id = $application->db->DB_Insert_Id();

        return $inserted_id;
    }

    function addOrderPromoCodeHistory($order_id, $promo_code_id)
    {
    	global $application;
        $messageResources = &$application->getInstance('MessageResources');
        $history_text = $messageResources->getMessage
        (
            new ActionMessage
            (
                array('ORDERS_HISTORY_PROMO_CODE_APPLIED', $campaign_name, $promo_code, $discount_text)
            )
        );
        $this->addOrderHistory($order_id, $history_text);
    }

    /**#@+
     * @access private
     */
    function addOrderPrices($order_id, $old_format_order_prices)
    {
        global $application;
        $tables =  $this->getTables();

        $table = 'order_prices';            #     the name of the filled table
        $columns = $tables[$table]['columns'];  #     the array of field names of the table


        $table_taxes = 'order_taxes';            #                            
        $columns_taxes = $tables[$table_taxes]['columns'];  #                              

        $order_prices = array();
        foreach($old_format_order_prices as $order_price)
        {
            //$order_prices[$order_price['type']] = number_format($order_price['value'], 2, '.', ',');
            $order_prices[$order_price['type']] = $order_price['value'];
        }

        $query = &new DB_Insert($table);
        $query->addInsertValue($order_id, $columns['order_id']);
        $query->addInsertValue($order_prices['MinimumShippingCost']             , $columns['minimum_shipping_cost']);
        $query->addInsertValue($order_prices['FreeShippingForOrdersOver']       , $columns['free_shipping_for_orders_over']);
        $query->addInsertValue($order_prices['FreeHandlingForOrdersOver']       , $columns['free_handling_for_orders_over']);
        $query->addInsertValue($order_prices['PerItemShippingCostSum']          , $columns['per_item_shipping_cost_sum']);
        $query->addInsertValue($order_prices['PerOrderShippingFee']             , $columns['per_order_shipping_fee']);
        $query->addInsertValue($order_prices['ShippingMethodCost']              , $columns['shipping_method_cost']);
        $query->addInsertValue($order_prices['TotalShippingCharge']             , $columns['total_shipping_charge']);
        $query->addInsertValue($order_prices['PerItemHandlingCostSum']          , $columns['per_item_handling_cost_sum']);
        $query->addInsertValue($order_prices['PerOrderHandlingFee']             , $columns['per_order_handling_fee']);
        $query->addInsertValue($order_prices['TotalHandlingCharge']             , $columns['total_handling_charge']);
        $query->addInsertValue($order_prices['TotalShippingAndHandlingCost']    , $columns['total_shipping_and_handling_cost']);
        $query->addInsertValue($order_prices['OrderTaxTotal']                   , $columns['order_tax_total']);
        $query->addInsertValue($order_prices['SubtotalGlobalDiscount']          , $columns['subtotal_global_discount']);
        $query->addInsertValue($order_prices['SubtotalPromoCodeDiscount']       , $columns['subtotal_promo_code_discount']);
        $query->addInsertValue($order_prices['QuantityDiscount']                , $columns['quantity_discount']);
        $query->addInsertValue($order_prices['DiscountedSubtotal']              , $columns['discounted_subtotal']);
        $query->addInsertValue($order_prices['OrderNotIncludedTaxTotal']        , $columns['order_not_included_tax_total']);
        $query->addInsertValue($order_prices['PerOrderPaymentModuleShippingFee'], $columns['per_order_payment_module_shipping_fee']);


        $application->db->getDB_Result($query);

        //                       -            
        //                         :
        reset($old_format_order_prices);
        foreach($old_format_order_prices as $order_price)
        {
            $regexp = "/^Taxes\[\'(.+)\'\]$/";
            if(preg_match($regexp, $order_price['type'], $matches) == 1)
            {
                $query = &new DB_Insert($table_taxes);
                $query->addInsertValue($order_id, $columns_taxes['order_id']);
                $query->addInsertValue($matches[1], $columns_taxes['type']);
                $query->addInsertValue($order_price['value'], $columns_taxes['value']);
                $application->db->getDB_Result($query);
            }
        }
    }


    function getPersonInfoVariantId($person_info_type_tag, $person_info_variant_tag)
    {
        global $application;
        loadCoreFile('UUIDUtils.php');

        $person_info_type_tag = UUIDUtils::cut_uuid_suffix($person_info_type_tag, "js");
        $tables = Checkout::getTables();

        $piv  = $tables['person_info_variants']['columns'];
        $pit = $tables['person_info_types']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($piv['id'], 'id');
        $query->WhereValue($piv['tag'], DB_EQ, $person_info_variant_tag);
        $query->WhereAnd();
        $query->WhereValue($pit['tag'], DB_EQ, $person_info_type_tag);
        $query->addLeftJoin('person_info_types', $pit['id'], DB_EQ, $piv['type_id']);
        $result = $application->db->getDB_Result($query);
        if(sizeof($result) != 1)
        {
            //report error
            _fatal(__FILE__ . " : " . __LINE__);
        }
        else
        {
            return $result[0]['id'];
        }
    }

    /**
     *
     *
     * @todo
     * @param
     * @return
     * It isn't used. It wasn't tested. It can contain errors, associated with
     * person_info_variant.
     */
/*    function addPerson()
    {
        global $application;
        $ret_data = array(); //                           ,                            :
                             // "order_person_id"       => "",
                             // "order_shipping_method" => "",
                             // "order_billing_method"  => ""

        $tables =  $this->getTables();

        $p = $tables["persons"]['columns'];
        $ptiv = $tables["person_to_info_variants"]['columns'];
        $pd = $tables["persons_data"]['columns'];
        //                     persons
        $query = &new DB_Insert("persons");
        $query->addInsertValue("", $p['login']);    //FIXME
        $query->addInsertValue("", $p['password']); //FIXME
        $application->db->getDB_Result($query);
        $person_id = $application->db->DB_Insert_Id();
        $ret_data["order_person_id"] = $person_id;

        $data = $this->getPrerequisitesValidationResults();
        foreach ($data as $key => $info)
        {
            if (strpos($key, "Method"))
            {
                switch ($key)
                {
                    case "shippingMethod":
                        $ret_data["order_shipping_method"] = $info["value"];
                        break;
                    case "paymentMethod":
                        $ret_data["order_billing_method"] = $info["value"];
                        break;
                }
            }
            else
            {
                //                     person_to_info_variants
                //FIXME -                ,                                  
                $query = &new DB_Insert("person_to_info_variants");
                $variant_id = $this->getPersonInfoVariantId($key, $info['variant_tag']);
                $query->addInsertValue($variant_id, $ptiv["variant_id"]);
                $query->addInsertValue($person_id, $ptiv["person_id"]);
                $application->db->getDB_Result($query);
                $person_to_info_variant_id = $application->db->DB_Insert_Id();

                foreach ($info["validatedData"] as $key => $validatedData)
                {
                    //                     persons_data
                    $query = &new DB_Insert("persons_data");
                    $query->addInsertValue($validatedData["id"], $pd["attribute_id"]);
                    $query->addInsertValue($person_id, $pd["person_id"]);
                    $query->addInsertValue($person_to_info_variant_id, $pd["variant_id"]);
                    $query->addInsertValue($validatedData["value"], $pd["value"]);
                    $application->db->getDB_Result($query);
                }
            }
        }

        return $ret_data;
    }
*/

    /*static*/ function getAdditionalPrerequisiteName($prerequisite_type, $payment_module_id)
    {
        loadCoreFile('UUIDUtils.php');
        return $prerequisite_type . UUIDUtils::convert("minuses_and_capitals", "js", $payment_module_id);
    }

    /**
       2.1 Mask account numbers when displayed (the first six and last four
       digits are the maximum number of digits to be displayed).

       Visa U.S.A. Cardholder Information Security Program (CISP)
       Payment Application Best Practices
    */
    function ccnum2public($cc_num)
    {
        $len = strlen($cc_num);
        for($i = 0; $i< ($len - 4); $i++)
        {
            $cc_num[$i] = "*";
        }
        return $cc_num;
    }

    /**
       2.1 Mask account numbers when displayed (the first six and last four
       digits are the maximum number of digits to be displayed).
               CVV                      .

       Visa U.S.A. Cardholder Information Security Program (CISP)
       Payment Application Best Practices
    */
    function cvv2public($cvv)
    {
        $len = strlen($cvv);
        for($i = 0; $i< $len; $i++)
        {
            $cvv[$i] = "*";
        }
        return $cvv;
    }

    function get_public_view_of_secured_data($value, $person_attribute_id)
    {
        switch($person_attribute_id)
        {
            case 13 /* CreditCardNumber */:
                $value = $this->ccnum2public($value);
                break;

            case 14 /* CreditCardVerificationNumber */:
                $value = $this->cvv2public($value);
                break;

            default:
                break;
        }
        return $value;
    }

    function encryptOrderPersonAttribute($value, $key)
    {
        //FIXME: implement proper symmetric encryption.
        return modApiFunc("Crypto", "blowfish_encrypt", $value, $key);
    }

    function addOrderPersonAttribute($order_id,
                                     $person_info_variant_id,
                                     $attribute_id,
                                     $attribute_visible_name,
                                     $attribute_value,
                                     $attribute_description,
                                     $b_encrypted,
                                     $encrypted_secret_key,
                                     $rsa_public_key_asc_format)
    {
        global $application;
        $tables =  $this->getTables();


        $ptiv = $tables["person_to_info_variants"]['columns'];
        $opd = $tables["order_person_data"]['columns'];

        //  add to the table order_person_data
        $query = &new DB_Insert("order_person_data");
        $query->addInsertValue($order_id, $opd["order_id"]);
    //FIXME what is the id (becomes a type_id in the DB)? Into the prerequisite
        $query->addInsertValue($person_info_variant_id, $opd["variant_id"]);
        $query->addInsertValue($attribute_id, $opd["attribute_id"]);
        $query->addInsertValue($attribute_visible_name, $opd["name"]);
        $query->addInsertValue($attribute_value, $opd["value"]);
        $query->addInsertValue($attribute_description, $opd["desc"]);
        $query->addInsertValue($b_encrypted, $opd["b_encrypted"]);
        $query->addInsertValue($encrypted_secret_key, $opd["encrypted_secret_key"]);
        $query->addInsertValue($rsa_public_key_asc_format, $opd["rsa_public_key_asc_format"]);
        $application->db->getDB_Result($query);
    }

    function appendCheckoutCZGETParameters($request)
    {
        $request->setKey   ( 'CHECKOUT_CZ_BLOWFISH_KEY', modApiFunc("Checkout", "getPerRequestVariable", "CHECKOUT_CZ_BLOWFISH_KEY"));
        return $request;
    }

    function getDecryptedCreditCardInfoPrerequisiteData($payment_module_id)
    {
         $prerequisite_name = Checkout::getAdditionalPrerequisiteName("creditCardInfo", $payment_module_id);
         if(isset($this->PrerequisitesValidationResults[$prerequisite_name]))
         {
            //Decrypt data in the session
            $this->decrypt_prerequisite_with_checkout_cz_blowfish_key($prerequisite_name);
            $decrypted_data = $this->getPrerequisitesValidationResults();
            $info = $decrypted_data[$prerequisite_name];
            //Encrypt data in the session
            $this->encrypt_prerequisite_with_checkout_cz_blowfish_key($prerequisite_name);

            //$person_info_variant_id = $this->getPersonInfoVariantId($prerequisite_name, $info['variant_tag']);

            return $info["validatedData"];
         }
         else
         {
             return false;
         }
    }

    /**
     * Adds a Custoner (List Person) to this order.
     * It is used, for example, to save unregistered customers.
     * A separate record is not created in the table persons.
     *
     * @todo
     * @param
     * @return
     */
    function addOrderPerson($order_id)
    {
        global $application;
        $tables =  $this->getTables();

        $ptiv = $tables["person_to_info_variants"]['columns'];
        $opd = $tables["order_person_data"]['columns'];

        $data = $this->getPrerequisitesValidationResults();
        $payment_module_id = modApiFunc("Checkout", "getChosenPaymentModuleIdCZ");
        foreach ($data as $prerequisite_key => $info)
        {
            if (strpos($prerequisite_key, "Module"))
            {
                //"shippingModuleAndMethod","paymentModule"
            }
            else
            {
                $person_info_variant_id = $this->getPersonInfoVariantId($prerequisite_key, $info['variant_tag']);
                // add to the table order_person_data
                foreach ($info["validatedData"] as $attribute_key => $validatedData)
                {
                    if($attribute_key=="Statemenu" || $attribute_key=="Statetext")
                    {
                        //An attribute "state" from the DB matches two attributes
                        // state_menu and state_text in the session. They are mutually exclussive in meaning:
                        // state_menu is the ID of the record about the state in the DB, i.e.
                        // a number. sate_text is a state name, inputted manually by a customer.
                        // It is inputted only if the customer selected a country, which has no
                        // defined states in the DB. As for now (Dec 2005) in the DB
                        // in the field "state" is saved only one of the values, which is not empty.
                        // Either sate_menu, or state_text.
                        if($attribute_key == "Statetext")
                        {
                            continue;
                        }
                        if($attribute_key == "Statemenu")
                        {
                            $state_menu_value = $info["validatedData"]["Statemenu"]["value"];
                            $state_text_value = $info["validatedData"]["Statetext"]["value"];
                            //FIMXE: check if both values are empty.
                            $value = empty($state_menu_value) ? $state_text_value : $state_menu_value;

                            //FIXME: analyze the conversion "1 atribute" <=> "2 attributes" for
                            // "state". As for now(Dec 2005) data on DB-attribute "state"
                            // is saved to the session-attribute "Statemenu"

                            // add to the table order_person_data
                            $query = &new DB_Insert("order_person_data");
                            $query->addInsertValue($order_id, $opd["order_id"]);
//FIXME what is the id (becomes a type_id in the DB)? Into the prerequisite
                            $query->addInsertValue($person_info_variant_id, $opd["variant_id"]);

                            $query->addInsertValue($validatedData["id"], $opd["attribute_id"]);
                            $query->addInsertValue($validatedData["attribute_visible_name"], $opd["name"]);
                            //Write a state name, but not the id
                            //FIXME: depends on another attribute value: Country
                            if(is_numeric($value))
                            {
                                //FIXME: can validatedData contain a nested
                                //  structure with the same name validatedData?
                                $states = modApiFunc("Location", "getStates", $info["validatedData"]["Country"]["value"]);
                                $value = $states[$value];
                            };
                            $query->addInsertValue($value, $opd["value"]);
                            $query->addInsertValue($validatedData["attribute_description"], $opd["desc"]);
                            $application->db->getDB_Result($query);
                        }
                    }
                    else
                    {
                        //  add to the table order_person_data
                        $query = &new DB_Insert("order_person_data");
                        $query->addInsertValue($order_id, $opd["order_id"]);
//FIXME what is the id (becomes a type_id in the DB)? Into the prerequisite
                        $query->addInsertValue($person_info_variant_id, $opd["variant_id"]);
                        $query->addInsertValue($validatedData["id"], $opd["attribute_id"]);
                        $query->addInsertValue($validatedData["attribute_visible_name"], $opd["name"]);
                        //Write a name for the country rather than the id
                        if($attribute_key == "Country")
                        {
                            $countries = modApiFunc("Location", "getCountries");
                            $query->addInsertValue(empty($validatedData["value"]) ? "" : $countries[$validatedData["value"]], $opd["value"]);
                        }
                        else
                        {
                            $query->addInsertValue($validatedData["value"], $opd["value"]);
                        }
                        $query->addInsertValue($validatedData["attribute_description"], $opd["desc"]);
                        $application->db->getDB_Result($query);
                    }
                }
            }
        }
    }


    /**
     * Updates order info.
     */
    function updateOrder($data)
    {
        global $application;

        if (!is_array($data))
        {
            return;
        }
        $tables = $this->getTables();

        $o = $tables['orders']['columns'];
        $on = $tables['order_notes']['columns'];
        $opd = $tables['order_person_data']['columns'];

        $statusChanged = array("order_status" => array(), "payment_status" => array());
        foreach ($data as $order)
        {
            $order_id = $order['order_id'];
            $status_id = $order['status_id'];
            $payment_status_id = $order['payment_status_id'];
            $track_id = $order['track_id'];
            $comment = $order['comment'];
            $processor_order_id = $order['processor_order_id'];
            $payment_method = $order['payment_method'];
            $shipping_method = $order['shipping_method'];
            $customer_info = $order['customer_info'];
            $billing_info = $order['billing_info'];
            $shipping_info = $order['shipping_info'];
            $bank_account_info = $order['bank_account_info'];
            $credit_card_info = $order['credit_card_info'];
            if (!$this->isCorrectOrderId($order_id)) continue;

            # get current order info
            $order = $this->getOrderInfo($order_id);
            # check if it is necessary to update status
            $update_status_id = $status_id != $order['StatusId'];
            $update_payment_status_id = $payment_status_id != $order['PaymentStatusId'];
            $update_track_id = $track_id != null && $track_id != $order['TrackId'];
            $update_processor_order_id = $processor_order_id != null && $processor_order_id != $order['PaymentProcessorOrderId'];
            $update_payment_method = $payment_method != null && $payment_method != $order['PaymentMethod'];
            $update_shipping_method = $shipping_method != null && $shipping_method != $order['ShippingMethod'];

            if($update_status_id)
            {
                $statusChanged["order_status"][$order_id] = array("old_status" => $order['StatusId'], "new_status" => $status_id);
            }
            if($update_payment_status_id)
            {
                $statusChanged["payment_status"][$order_id] = array("old_status" => $order['PaymentStatusId'], "new_status" => $payment_status_id);
            }

            if($update_status_id || $update_payment_status_id || $update_track_id || $update_processor_order_id || $update_payment_method || $update_shipping_method)
            {
                $db_update = new DB_Update('orders');
                $db_update->addUpdateValue($o['status_id'], $status_id);
                $db_update->addUpdateValue($o['payment_status_id'], $payment_status_id);
                if ($update_track_id)
                {
                    $db_update->addUpdateValue($o['track_id'], $track_id);
                }
                if ($update_processor_order_id)
                {
                    $db_update->addUpdateValue($o['payment_processor_order_id'], $processor_order_id);
                }
                if ($update_payment_method)
                {
                    $db_update->addUpdateValue($o['payment_method'], $payment_method);
                }
                if ($update_shipping_method)
                {
                    $db_update->addUpdateValue($o['shipping_method'], $shipping_method);
                }
                $db_update->WhereValue($o['id'], DB_EQ, $order_id);
                $application->db->PrepareSQL($db_update);
                $application->db->DB_Exec();
            }

            #  check if it is necessary to update Customer Info
            $update_customer_info = false;
            if (is_array($customer_info))
            {
                foreach ($order['Customer']['attr'] as $tag => $attr)
                {
                    if (array_key_exists($tag, $customer_info) && trim($attr['value']) != trim($customer_info[$tag]))
                    {
                        $update_customer_info = true;
                        $db_update = new DB_Update('order_person_data');
                        $db_update->addUpdateValue($opd['value'], $customer_info[$tag]);
                        $db_update->WhereValue($opd['id'], DB_EQ, $attr['id']);
                        $application->db->PrepareSQL($db_update);
                        $application->db->DB_Exec();
                    }
                }
            }

            #  check if it is necessary to update Billing Info
            $update_billing_info = false;
            if (is_array($billing_info))
            {
                foreach ($order['Billing']['attr'] as $tag => $attr)
                {
                    if (array_key_exists($tag, $billing_info) && trim($attr['value']) != trim($billing_info[$tag]))
                    {
                        $update_billing_info = true;
                        $db_update = new DB_Update('order_person_data');
                        $db_update->addUpdateValue($opd['value'], $billing_info[$tag]);
                        $db_update->WhereValue($opd['id'], DB_EQ, $attr['id']);
                        $application->db->PrepareSQL($db_update);
                        $application->db->DB_Exec();
                    }
                }
            }

            #      check if it is necessary to update Shipping Info
            $update_shipping_info = false;
            if (is_array($shipping_info))
            {
                foreach ($order['Shipping']['attr'] as $tag => $attr)
                {
                    if (array_key_exists($tag, $shipping_info) && trim($attr['value']) != trim($shipping_info[$tag]))
                    {
                        $update_shipping_info = true;
                        $db_update = new DB_Update('order_person_data');
                        $db_update->addUpdateValue($opd['value'], $shipping_info[$tag]);
                        $db_update->WhereValue($opd['id'], DB_EQ, $attr['id']);
                        $application->db->PrepareSQL($db_update);
                        $application->db->DB_Exec();
                    }
                }
            }

            #      check if it is necessary to update Bank Account Info
            $update_bank_account_info = false;
            if (is_array($bank_account_info))
            {
                foreach ($order['BankAccount']['attr'] as $tag => $attr)
                {
                    if (array_key_exists($tag, $bank_account_info) && trim($attr['value']) != trim($bank_account_info[$tag]))
                    {
                        $update_bank_account_info = true;
                        $db_update = new DB_Update('order_person_data');
                        $db_update->addUpdateValue($opd['value'], $bank_account_info[$tag]);
                        $db_update->WhereValue($opd['id'], DB_EQ, $attr['id']);
                        $application->db->PrepareSQL($db_update);
                        $application->db->DB_Exec();
                    }
                }
            }

            #      check if it is necessary to update Credit Card Info
            $update_credit_card_info = false;
            if (is_array($credit_card_info))
            {
                //Credit card info is ALWAYS encrypted.
                //Prepare the encrypting keys.

                $symmetric_secret_key = modApiFunc("Crypto", "blowfish_gen_blowfish_key");
                $rsa_public_key = modApiFunc("Payment_Module_Offline_CC", "getRSAPublicKeyInCryptRSAFormat");
                $rsa_public_key_asc_format = modApiFunc("Payment_Module_Offline_CC", "getRSAPublicKeyInASCFormat");

                $rsa_obj = new Crypt_RSA;
                $encrypted_symmetric_secret_key = $rsa_obj->encrypt($symmetric_secret_key, $rsa_public_key);
                $person_info_variant_id = $order['CreditCard']['person_info_variant_id'];
                foreach ($order['CreditCard']['attr'] as $tag => $attr)
                {
                    if (array_key_exists($tag, $credit_card_info))// && trim($attr['value']) != trim($credit_card_info[$tag]))
                    //Reencrypt the whole block rather than by attributes. This is the fast way.
                    {
                        $update_credit_card_info = true;
                        //Not encrypted obfuscated value
                        $b_encrypted = false;
                        $db_update = new DB_Update('order_person_data');
                        $db_update->addUpdateValue($opd['value'], $this->get_public_view_of_secured_data($credit_card_info[$tag], $attr['person_attribute_id']));
                        $db_update->addUpdateValue($opd['encrypted_secret_key'], $encrypted_symmetric_secret_key);
                        $db_update->addUpdateValue($opd['rsa_public_key_asc_format'], $rsa_public_key_asc_format);
                        $db_update->WhereValue($opd['order_id'], DB_EQ, $order_id);
                        $db_update->WhereAND();
                        $db_update->WhereValue($opd['variant_id'], DB_EQ, $person_info_variant_id);
                        $db_update->WhereAND();
                        $db_update->WhereValue($opd['attribute_id'], DB_EQ, $attr['person_attribute_id']);
                        $db_update->WhereAND();
                        $db_update->WhereValue($opd['b_encrypted'], DB_EQ, "0");
                        $application->db->PrepareSQL($db_update);
                        $application->db->DB_Exec();

                        //  encrypted not obfuscated value
                        $b_encrypted = true;
                        $db_update = new DB_Update('order_person_data');
                        $db_update->addUpdateValue($opd['value'], base64_encode($this->encryptOrderPersonAttribute($credit_card_info[$tag], $symmetric_secret_key)));
                        $db_update->addUpdateValue($opd['encrypted_secret_key'], $encrypted_symmetric_secret_key);
                        $db_update->addUpdateValue($opd['rsa_public_key_asc_format'], $rsa_public_key_asc_format);
                        $db_update->WhereValue($opd['order_id'], DB_EQ, $order_id);
                        $db_update->WhereAND();
                        $db_update->WhereValue($opd['variant_id'], DB_EQ, $person_info_variant_id);
                        $db_update->WhereAND();
                        $db_update->WhereValue($opd['attribute_id'], DB_EQ, $attr['person_attribute_id']);
                        $db_update->WhereAND();
                        $db_update->WhereValue($opd['b_encrypted'], DB_EQ, "1");
                        $application->db->PrepareSQL($db_update);
                        $application->db->DB_Exec();
                    }
                }
            }

            # add comment, if it is defined
            if (!empty($comment))
            {
                $query = &new DB_Insert('order_notes');
                $query->addInsertValue($order_id, $on['order_id']);
                $query->addInsertValue(date('Y-m-d H:i:s', time()/*getServerTime()*/), $on['date']);
                $query->addInsertValue($comment, $on['content']);
                $query->addInsertValue('comment', $on['type']);
                $application->db->getDB_Result($query);
            }

            # generate records for the order history
            $history = array();
            $status_list = $this->getOrderStatusList();
            $payment_status_list = $this->getOrderPaymentStatusList();
            if ($update_status_id)
            {
                $history[] = new ActionMessage(array('ORDERS_HISTORY_STATUS_UPDATED', $status_list[$order['StatusId']]['name'], $status_list[$status_id]['name']));
            }
            if ($update_payment_status_id)
            {
                $history[] = new ActionMessage(array('ORDERS_HISTORY_PAYMENT_STATUS_UPDATED', $payment_status_list[$order['PaymentStatusId']]['name'], $payment_status_list[$payment_status_id]['name']));
            }
            if ($update_track_id)
            {
                $history[] = new ActionMessage(array('ORDERS_HISTORY_TRACKING_NUMBER_UPDATED', $order['TrackId'], $track_id));
            }
            if ($update_processor_order_id)
            {
                $history[] = new ActionMessage(array('ORDERS_HISTORY_PAYMENT_PROCESSOR_ORDER_ID_UPDATED', $order['PaymentProcessorOrderId'], $processor_order_id));
            }
            if ($update_payment_method)
            {
                $history[] = new ActionMessage(array('ORDERS_HISTORY_PAYMENT_METHOD_UPDATED', $order['PaymentMethod'], $payment_method));
            }
            if ($update_shipping_method)
            {
                $history[] = new ActionMessage(array('ORDERS_HISTORY_SHIPPING_METHOD_UPDATED', $order['ShippingMethod'], $shipping_method));
            }
            if ($update_customer_info)
            {
                $history[] = new ActionMessage(array('ORDERS_HISTORY_CUSTOMER_INFO_UPDATED'));
            }
            if ($update_billing_info)
            {
                $history[] = new ActionMessage(array('ORDERS_HISTORY_BILLING_INFO_UPDATED'));
            }
            if ($update_shipping_info)
            {
                $history[] = new ActionMessage(array('ORDERS_HISTORY_SHIPPING_INFO_UPDATED'));
            }
            if ($update_bank_account_info)
            {
                $history[] = new ActionMessage(array('ORDERS_HISTORY_BANK_ACCOUNT_INFO_UPDATED'));
            }
            if ($update_credit_card_info)
            {
                $history[] = new ActionMessage(array('ORDERS_HISTORY_CREDIT_CARD_INFO_UPDATED'));
            }

            $messageResources = &$application->getInstance('MessageResources');
            $history_text = "";
            foreach ($history as $record)
            {
                $history_text .= $messageResources->getMessage($record) . "\n";
            }

            if(!empty($history_text))
            {
                $this->addOrderHistory($order_id, $history_text);
            }
        }

        modApiFunc('EventsManager','throwEvent','OrdersWereUpdated',$statusChanged);

        return $statusChanged;
    }

    /**
     * Copies the part of the updateOrder($data) functionality. It is created          
     * only, because updateOrder is too hungus for the small update.
     */
    function updateOrderPersonDataAttribute($order_id, $person_info_variant_id, $person_attribute_id, $b_encrypted, $encrypted_symmetric_secret_key, $rsa_public_key_asc_format, $new_value)
    {
        global $application;
        $tables = $this->getTables();

        $opd = $tables['order_person_data']['columns'];
        $db_update = new DB_Update('order_person_data');
        $db_update->addUpdateValue($opd['value'], $new_value);
        $db_update->addUpdateValue($opd['encrypted_secret_key'], $encrypted_symmetric_secret_key);
        $db_update->addUpdateValue($opd['rsa_public_key_asc_format'], $rsa_public_key_asc_format);
        $db_update->WhereValue($opd['order_id'], DB_EQ, $order_id);
        $db_update->WhereAND();
        $db_update->WhereValue($opd['attribute_id'], DB_EQ, $person_attribute_id);
        $db_update->WhereAND();
        $db_update->WhereValue($opd['variant_id'], DB_EQ, $person_info_variant_id);
        $db_update->WhereAND();
        $db_update->WhereValue($opd['b_encrypted'], DB_EQ, $b_encrypted ? "1" : "0");
        $application->db->PrepareSQL($db_update);
        $application->db->DB_Exec();
    }
    /**
     *
     *
     * @author Alexandr Girin
     * @param
     * @return
     */
    function setOrdersIDForDelete($ordersId)
    {
        $this->ordersId = $ordersId;
    }

    /**
     *
     *
     * @author Alexandr Girin
     * @param
     * @return
     */
    function getOrdersIDForDelete()
    {
        return $this->ordersId;
    }

    /**
     *
     *
     * @author Alexandr Girin
     * @param
     * @return
     */
    function setDeleteOrdersFlag($value)
    {
        $this->DeleteOrdersFlag = $value;
    }

    /**
     *
     *
     * @author Alexandr Girin
     * @param
     * @return
     */
    function getDeleteOrdersFlag()
    {
        return $this->DeleteOrdersFlag;
    }

    /**
     *
     *
     * @author Alexandr Girin
     * @param
     * @return
     */
    function DeleteOrders($ordersId)
    {
        modApiFunc('EventsManager','throwEvent','OrdersWillBeDeleted',$ordersId);

        global $application;

        $tables = $this->getTables();
        $on = $tables['order_notes']['columns'];
        $opd = $tables['order_person_data']['columns'];
        $opr = $tables['order_prices']['columns'];
        $otx = $tables['order_taxes']['columns'];
        $op = $tables['order_product']['columns'];
        $opca = $tables['order_product_custom_attributes']['columns'];
        $opta = $tables['order_product_to_attributes']['columns'];
        $o = $tables['orders']['columns'];

        $DB_IN_string = "('".implode("', '", $ordersId)."')";

        $query = &new DB_Select();
        $query->addSelectField($op['id'], 'id');
        $query->WhereField($op['order_id'], DB_IN, $DB_IN_string);
        $order_products_id = $application->db->getDB_Result($query);
        foreach ($order_products_id as $key => $order_product_id)
        {
            $order_products_id[$key] = $order_product_id['id'];
        }

        $query = &new DB_Delete('order_notes');
        $query->WhereField($on['order_id'], DB_IN, $DB_IN_string);
        $application->db->getDB_Result($query);

        $query = &new DB_Delete('order_person_data');
        $query->WhereField($opd['order_id'], DB_IN, $DB_IN_string);
        $application->db->getDB_Result($query);

        $query = &new DB_Delete('order_prices');
        $query->WhereField($opr['order_id'], DB_IN, $DB_IN_string);
        $application->db->getDB_Result($query);

        $query = &new DB_Delete('order_taxes');
        $query->WhereField($otx['order_id'], DB_IN, $DB_IN_string);
        $application->db->getDB_Result($query);

        $query = &new DB_Delete('order_product');
        $query->WhereField($op['order_id'], DB_IN, $DB_IN_string);
        $application->db->getDB_Result($query);

        $query = &new DB_Delete('order_product_custom_attributes');
        $query->WhereField($opca['product_id'], DB_IN, "('".implode("', '", $order_products_id)."')");
        $application->db->getDB_Result($query);

        $query = &new DB_Delete('order_product_to_attributes');
        $query->WhereField($opta['product_id'], DB_IN, "('".implode("', '", $order_products_id)."')");
        $application->db->getDB_Result($query);

        $query = &new DB_Delete('orders');
        $query->WhereField($o['id'], DB_IN, $DB_IN_string);
        $application->db->getDB_Result($query);
    }

    /**
     * Returns info about customer from the order with the specified id.
     */
    function getCustomerInfo($order_id, $b_encrypted = false)
    {
        global $application;
        $order = $this->getOrderInfo($order_id, $b_encrypted);

        $tables = $this->getTables();

        $o = $tables['orders']['columns'];
        $opd = $tables['order_person_data']['columns'];
        $pa = $tables['person_attributes']['columns'];
        $piv = $tables['person_info_variants']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($query->fMax($o['id']), 'order_id');
        $query->addSelectField($query->fCount($o['id']), 'order_count');
        $query->addSelectField('sum('.$o['total'].')', 'order_total');
        $query->addSelectField($opd['value'], 'email');
        $query->addSelectField($query->fCount($pa['id']), 'dummy');
        $query->Where($opd['order_id'], DB_EQ, $o['id']);
        $query->WhereAND();
        $query->Where($pa['id'], DB_EQ, $opd['attribute_id']);
        $query->WhereAND();
        $query->WhereValue($piv['type_id'], DB_EQ, 1);
        $query->WhereAND();
        $query->WhereValue($opd['attribute_id'], DB_EQ, 3);
        if(isset($order['Customer']['attr']['Email']))
        {
            $query->WhereAND();
            $query->WhereValue($opd['value'], DB_EQ, $order['Customer']['attr']['Email']['value']);
        }
        $query->addLeftJoin('person_info_variants', $opd['variant_id'], DB_EQ, $piv['id']);
        $query->SelectGroup($opd['value']);
        $query->SelectOrder($o['date'], 'DESC');
        $query->SelectOrder($o['id'], 'DESC');

        $result = $application->db->getDB_Result($query);

        $query = &new DB_Select();
        $query->addSelectField($o['id'], 'order_id');
        $query->Where($opd['order_id'], DB_EQ, $o['id']);
        $query->WhereAND();
        $query->Where($pa['id'], DB_EQ, $opd['attribute_id']);
        $query->WhereAND();
        $query->WhereValue($piv['type_id'], DB_EQ, 1);
        $query->WhereAND();
        $query->WhereValue($opd['attribute_id'], DB_EQ, 3);
        if(isset($order['Customer']['attr']['Email']))
        {
            $query->WhereAND();
            $query->WhereValue($opd['value'], DB_EQ, $order['Customer']['attr']['Email']['value']);
        }
        $query->addLeftJoin('person_info_variants', $opd['variant_id'], DB_EQ, $piv['id']);
        $query->SelectOrder($o['date'], 'ASC');
        $query->SelectOrder($o['id'], 'ASC');
        $_orders = $application->db->getDB_Result($query);
        $orders = array();
        for ($i = 0; $i < count($_orders); $i++)
        {
            $orders[] = $this->getOrderInfo($_orders[$i]['order_id'], $b_encrypted);
            $last_order_id = $_orders[$i]['order_id'];
        }

        //last order
        $last_order = $this->getOrderInfo($last_order_id, $b_encrypted);

        $customer = array(
            'ID' => $last_order['ID']
           ,'Customer' => $last_order['Customer']
           ,'Billing' => $last_order['Billing']
           ,'Shipping' => $last_order['Shipping']
           ,'CreditCard' => $last_order['CreditCard']

           ,'TotalOrders' => $result[0]['order_count']
           ,'TotalAmount' => $result[0]['order_total']

           ,'Orders' => $orders
        );

        return $customer;
    }

    function getCustomerAttributeValue($order_id, $person_info_type_tag, $person_info_variant_tag, $person_attribute)
    {
        global $application;
        $tables = $this->getTables();
        $opd = $tables['order_person_data']['columns'];
        $pa = $tables['person_attributes']['columns'];
        $piv = $tables['person_info_variants']['columns'];
        $pit = $tables['person_info_types']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($opd["value"], "value");
        $query->addLeftJoin('person_attributes', $opd['attribute_id'], DB_EQ, $pa['id']);
        $query->addLeftJoin('person_info_variants', $opd['variant_id'], DB_EQ, $piv['id']);
        $query->addLeftJoin('person_info_types', $piv['type_id'], DB_EQ, $pit['id']);
        $query->WhereValue($pit['tag'], DB_EQ, $person_info_type_tag);
        $query->WhereAND();
        $query->WhereValue($piv['tag'], DB_EQ, $person_info_variant_tag);
        $query->WhereAND();
        $query->WhereValue($pa['tag'], DB_EQ, $person_attribute);
        $query->WhereAND();
        $query->WhereValue($opd['order_id'], DB_EQ, $order_id);
        $result = $application->db->getDB_Result($query);

        return isset($result[0]["value"])? $result[0]["value"]:"";
    }

    /**
     * Returns a list of customers from the order list.
     * Customers are groupped by their emails.
     *
     * @todo both ids type_id and person_attribute_id are strictly specified.
     * @deprecated 1.8.0 - 29.10.2007 by egor
     */
    function getCustomerList()
    {
        global $application;

        $tables = $this->getTables();

        $o = $tables['orders']['columns'];
        $opd = $tables['order_person_data']['columns'];
        $pa = $tables['person_attributes']['columns'];
        $piv = $tables['person_info_variants']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($query->fMax($o['id']), 'order_id');
        $query->addSelectField($query->fCount($o['id']), 'order_count');
        $query->addSelectField('sum('.$o['total'].')', 'order_total');
        $query->addSelectField($opd['value'], 'email');
        $query->addSelectField($query->fCount($pa['id']), 'dummy');
        $query->Where($opd['order_id'], DB_EQ, $o['id']);
        $query->WhereAND();
        $query->Where($pa['id'], DB_EQ, $opd['attribute_id']);
        $query->WhereAND();
        $query->WhereValue($piv['type_id'], DB_EQ, 1);
        $query->WhereAND();
        $query->WhereValue($opd['attribute_id'], DB_EQ, 3);
        $query->addLeftJoin('person_info_variants', $opd['variant_id'], DB_EQ, $piv['id']);
        $query->SelectOrder($o['id'], 'DESC');
        $query->SelectGroup($opd['value']);
        $result = $application->db->getDB_Result($query);

        $customers = array();
        for ($i=0; $i<sizeof($result); $i++)
        {
//            $customer = $this->getCustomerInfo($result[$i]['order_id']);
            #  use a filter
            $filter = $this->customer_search_filter;
            if ($filter['search_by'] == 'letter' && !empty($filter['letter']))
            {
                // the customer name should begin with the specified letter
                $letter = substr(strtolower($filter['letter']), 0, 1);
                $name = substr(strtolower($this->getCustomerAttributeValue($result[$i]['order_id'], "customerinfo", "default", "lastname")), 0, 1);
                if ($letter != $name)
                    continue;
            }
            elseif ($filter['search_by'] == 'field' && !empty($filter['field_name']) && !empty($filter['field_value']))
            {
                $value = strtolower($filter['field_value']);
                if ($filter['field_name'] == 'name')
                {
                    $name = strtolower($this->getCustomerAttributeValue($result[$i]['order_id'], "customerinfo", "default", "last".$filter['field_name']));
                    if (strpos($name, $value) === false)
                    {
                        $name = strtolower($this->getCustomerAttributeValue($result[$i]['order_id'], "customerinfo", "default", "first".$filter['field_name']));
                        if (strpos($name, $value) === false)
                            continue;
                    }
                }
                elseif ($filter['field_name'] == 'Email')
                {
                    $email = strtolower($this->getCustomerAttributeValue($result[$i]['order_id'], "customerinfo", "default", $filter['field_name']));
                    if (strpos($email, $value) === false)
                        continue;
                }
            }
            array_push($customers, $result[$i]['order_id']); // . .          customer accounts,    customer info           order_id
        }
        $customers = modApiFunc('paginator', 'setQuery', $customers, sizeof($customers));
        return $customers;
    }

    /**
     * Adds a record to the change backlog.
     *
     * @param $history - the array of ActionMessage objects
     */
    function addOrderHistory($order_id, $history_text)
    {
        global $application;

//        if (!is_array($history) || count($history) == 0)
//        {
//            return;
//        }
        $tables = $this->getTables();

        //$text = "";
        //foreach ($history as $record)
        //{
        //    $text .= $messageResources->getMessage($record) . "\n";
        //}

        $on = $tables['order_notes']['columns'];
        $query = &new DB_Insert('order_notes');
        $query->addInsertValue($order_id, $on['order_id']);
        $query->addInsertValue(date('Y-m-d H:i:s', time()/*getServerTime()*/), $on['date']);
        $query->addInsertValue($history_text, $on['content']);
        $query->addInsertValue('history', $on['type']);
        $application->db->getDB_Result($query);
     }

    /**
     * Sends a report to each of the recipient.
     */
    function asc_send($subj, $body, $to_address)
    {
        loadCoreFile('htmlMimeMail.php');
        $mail = new htmlMimeMail();
        $mail->setText($body);
        $mail->setSubject($subj);
        $mail->setFrom($to_address);
        $mail->send(array($to_address));
    }


    /**
     * The former log_decrypted_credit_card_info_review.
     * The order, that includes Credit Card Info. For now, log is included to
     * the order info. No special log exists for the credit card now.
     */
    function log_decrypted_credit_card_info_review($order_id)
    {
        /**
         * Get info about the current viewing customer.
         */

        /**
         * IP address.
         * It is taken from users_api.php
         */
        $ip = $_SERVER['REMOTE_ADDR'];

        //Name
        $admin_info = modApiFunc("Users", "getUserInfo", modApiFunc("Users", "getCurrentUserID"));
        $admin_full_name = $admin_info['firstname'] . " " .
                           $admin_info['lastname'];

        //Time, Date
        $date_time = modApiFunc("Localization", "timestamp_date_format", time())
              . " " .modApiFunc("Localization", "timestamp_time_format", time());

        global $application;
        $MessageResources = &$application->getInstance('MessageResources',"payment-module-offline-messages", "AdminZone");

        $order_history_msg = $MessageResources->getMessage('MODULE_PAYMENT_OFFLINE_CC_MSG_LOG_MSG_001') . " " . $admin_full_name . "\n"
                           . $date_time . "\n"
                           . $MessageResources->getMessage('MODULE_PAYMENT_OFFLINE_CC_MSG_LOG_MSG_002') . " " . $ip;

        modApiFunc("Checkout", "addOrderHistory", $order_id, $order_history_msg);
    }

    /**
     * The order, that includes Credit Card Info. For now, log is included to
     * the order info. No special log exists for the credit card now.
     */
    function log_person_info_removal($order_id, $person_info_variant_id)
    {
        /**
         * Get info about the current deleting customer.
         */

        /**
         * IP address.
         * It is taken from users_api.php
         */
        $ip = $_SERVER['REMOTE_ADDR'];

        //Name
        $admin_info = modApiFunc("Users", "getUserInfo", modApiFunc("Users", "getCurrentUserID"));
        $admin_full_name = $admin_info['firstname'] . " " .
                           $admin_info['lastname'];

        //Time, Date
        $date_time = modApiFunc("Localization", "timestamp_date_format", time())
              . " " .modApiFunc("Localization", "timestamp_time_format", time());

        global $application;
        $MessageResources = &$application->getInstance('MessageResources');

        //Get Person Info name
        $person_variant_info = $this->getPersonVariantInfo($person_info_variant_id);
        $order_history_msg = $person_variant_info["type_visible_name"]
                           . " "
                           . $MessageResources->getMessage('MODULE_CHECKOUT_MSG_LOG_MSG_003') . " " . $admin_full_name . "\n"
                           . $date_time . "\n"
                           . $MessageResources->getMessage('MODULE_CHECKOUT_MSG_LOG_MSG_002') . " " . $ip;

        modApiFunc("Checkout", "addOrderHistory", $order_id, $order_history_msg);
    }

   /**
    * The former Action 'PurgeCVVFromStoredCreditCardInfo'.
    *
    * Note! This function shouldn't output anything and shouldn't
    * call exit(), because it is called in the special Action.
    *
    * It repalces the value CVV for this credit card in the database
    * with the string "**purged**".
    * Te following values come from the form:
    *     order_id
    *     person_info_variant_id (just in case, if it is
    *         addmissible to save more than one credit card).
    * The following values are defined by them automatically:
    *     person_attribute_id (for CVV is 14)
    *     encrypted_secret_key
    *
    * The encrypted_secret_key couldn't be used at all, just to
    * generate a new blowfish key and to encrypt it.
    * But then the decryption would take much time (twice as much),
    * as it would have to call RSA decryption for two blowfish keys
    * instead of one.
    */
    function PurgeCVVFromStoredCreditCardInfo($order_id, $person_info_variant_id, $rsa_private_key)
    {
        global $application;
        //The code is not correct. Clear the value CVV.
        //It would be better to search the id by the tag in the table person_attributes.
        $perston_attribute_id = 14;

        //Gets from the base encrypted_secret_key
        //You have to get the whole order for it.
        $order_info = modApiFunc('Checkout', 'getOrderInfo', $order_id, true);
        //Encrypted data should be less in volume, than not encrypted ones.
        foreach($order_info as $key => $info_block)
        {
            if(is_array($info_block))
            {
                if(isset($info_block['person_info_variant_id']))
                {
                    //Person Info
                    if($info_block['person_info_variant_id'] == $person_info_variant_id)
                    {
                        //Creditcard info
                        foreach($info_block['attr'] as $attr)
                        {
                            if($attr['person_attribute_id'] == $perston_attribute_id)
                            {
                                //CVV
                                //The required encrypted blowfish key:
                                $encrypted_secret_key = $attr['encrypted_secret_key'];
                                $old_rsa_public_key_asc_format = $attr['rsa_public_key_asc_format'];

                                /*
                                 If the loaded Private key and the Public key stored
                                 in the database for this attribute don't match - output an
                                 error message. Do not rewrite anything in the dtatabase.
                                 */
                                $old_rsa_public_key_cryptrsa_format = modApiFunc("Crypto", "convert_rsa_public_key_from_asc_into_cryptrsa_format", $old_rsa_public_key_asc_format);
                                if(modApiFunc("Crypto","rsa_do_public_key_match_private_key", $old_rsa_public_key_cryptrsa_format, $rsa_private_key) === true)
                                {
                                    //Decrypt it and rewrite CVV with the message "**purged**":
                                    $rsa_obj = new Crypt_RSA;
                                    $decrypted_secret_key = $rsa_obj->decrypt($encrypted_secret_key, $rsa_private_key);

                                    $MessageResources = &$application->getInstance('MessageResources');
                                    $cvv_purged__msg = $MessageResources->getMessage("CHECKOUT_ORDER_INFO_CVV_PURGED_MSG");
                                    $new_value = $cvv_purged__msg;
                                    $new_value_encrypted =  base64_encode(modApiFunc("Crypto", "blowfish_encrypt", $new_value, $decrypted_secret_key));

                                    $rsa_public_key_asc_format = $old_rsa_public_key_asc_format;
                                    //Save both obfuscated and encrypted value to the database
                                    modApiFunc('Checkout', 'updateOrderPersonDataAttribute', $order_id, $person_info_variant_id, $perston_attribute_id, false, $encrypted_secret_key, $rsa_public_key_asc_format, $new_value);

                                    modApiFunc('Checkout', 'updateOrderPersonDataAttribute', $order_id, $person_info_variant_id, $perston_attribute_id, true,  $encrypted_secret_key, $rsa_public_key_asc_format, $new_value_encrypted);
                                }
                                else
                                {
                                    //              :                                  .
                                    //Output an error: the keys don't match.
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * Deletes data of specified variant_info from the order.
     */
    function removePersonInfoOrderData($order_id, $person_info_variant_id)
    {
        global $application;
        $tables = $this->getTables();
        $opd = $tables['order_person_data']['columns'];
        $query = &new DB_Delete("order_person_data");
        $query->WhereValue($opd["order_id"], DB_EQ, (int)$order_id);
        $query->WhereAnd();
        $query->WhereValue($opd["variant_id"], DB_EQ, (int)$person_info_variant_id);
        $application->db->getDB_Result($query);
    }

    /**
     * Creates an order in the database.
     *
     * @author Alexander Girin
     * @todo finish the function
     * @param
     * @return
     */
    function createOrderInDB()
    {
        $payment_module_id = modApiFunc("Checkout", "getChosenPaymentModuleIdCZ");
        if($payment_module_id === NULL)
        {
            //Use "AllInactive" Payment module.
            $payment_module_id = Checkout::getAllInactiveModuleId("payment");
        }
        $payment_module_info = Checkout::getPaymentModuleInfo($payment_module_id);
        $payment_method_name = $payment_module_info["Name"];

        $shipping_module_id = modApiFunc("Checkout", "getChosenShippingModuleIdCZ");
        if($shipping_module_id === NULL)
        {
            //Use "AllInactive" Shipping module.
            $shipping_module_id = Checkout::getAllInactiveModuleId("shipping");
            $shipping_module_info = Checkout::getShippingModuleInfo($shipping_module_id);
            $shipping_method_name = $shipping_module_info['Name'];
        }
        else
        {
            $shipping_method_id = modApiFunc("Checkout", "getChosenShippingMethodIdCZ");
            $shipping_method_info = modApiFunc("Checkout", "getShippingMethodInfo", $shipping_module_id, $shipping_method_id);
            $shipping_method_name = $shipping_method_info['method_name'];
        }

        $order_subtotal = modApiFunc("Checkout", "getOrderPrice", "Subtotal");

        $subtotal_global_discount = modApiFunc("Checkout", "getOrderPrice", "SubtotalGlobalDiscount");

        $subtotal_promo_code_discount = modApiFunc("Checkout", "getOrderPrice", "SubtotalPromoCodeDiscount");

        $quantity_discount = modApiFunc("Checkout", "getOrderPrice", "QuantityDiscount");

        //FIXME: Total, Shipping and Tax MUST be set. If "Total" and "Tax" depend
        // not only on shipping parameters (checked in
        //  the beginning of addOrder(), then those additional dependencies should be
        //  checked too.
        $order_total = modApiFunc("Checkout", "getOrderPrice", "Total");

        $order_shipping_method_cost = modApiFunc("Checkout", "getOrderPrice", "ShippingMethodCost");

        $order_shipping_cost = modApiFunc("Checkout", "getOrderPrice", "TotalShippingAndHandlingCost");

        $order_order_tax_total_cost = modApiFunc("Checkout", "getOrderPrice", "Tax");

        $order_not_included_tax_total = modApiFunc("Checkout", "getOrderPrice", "NotIncludedTax");

        $order_discounted_subtotal_cost = modApiFunc("Checkout", "getOrderPrice", "DiscountedSubtotal");

        $payment_processor_order_id ="";
        $oid = modApiFunc("Checkout", "addOrder", array('status_id' => 1, // New Order
                                                        'payment_status_id' => 1, //Waiting
                                                        'payment_method' => $payment_method_name,
                                                        'payment_module_id' => $payment_module_id,
                                                        'payment_method_detail' => "NULL",
                                                        'payment_processor_order_id' => $payment_processor_order_id,//"NULL", //$payment_module_id,
                                                        'shipping_method' => $shipping_method_name,
                                                        'track_id' => "",//"NULL", //?
                                                        'subtotal' => $order_subtotal,
                                                        'total' => $order_total,
                                                        'person_id' => '0'//"NULL"//Checkout
                                                        //without registration, so:
                                                        // NO "registered person" id.
                                                       )
                         );

        $order_prices = array(
                // General Shipping Settings
                array('type' => "MinimumShippingCost",
                      'value' => modApiFunc("Checkout", "getOrderPrice", "MinimumShippingCost")),
                array('type' => "FreeShippingForOrdersOver",
                      'value' => modApiFunc("Checkout", "getOrderPrice", "FreeShippingForOrdersOver")),
                array('type' => "FreeHandlingForOrdersOver",
                      'value' => modApiFunc("Checkout", "getOrderPrice", "FreeHandlingForOrdersOver")),
                // Shipping prices
                array('type' => "PerItemShippingCostSum",
                      'value' => modApiFunc("Checkout", "getOrderPrice", "PerItemShippingCostSum")),
                array('type' => "PerOrderShippingFee",
                      'value' => modApiFunc("Checkout", "getOrderPrice", "PerOrderShippingFee")),
                array('type' => "ShippingMethodCost",
                      'value' => $order_shipping_method_cost),
                array('type' => "TotalShippingCharge",
                      'value' => modApiFunc("Checkout", "getOrderPrice", "TotalShippingCharge")),
                array('type' => "PerOrderPaymentModuleShippingFee",
                      'value' => modApiFunc("Checkout", "getOrderPrice", "PerOrderPaymentModuleShippingFee")),
                // handling prices
                array('type' => "PerItemHandlingCostSum",
                      'value' => modApiFunc("Checkout", "getOrderPrice", "PerItemHandlingCostSum")),
                array('type' => "PerOrderHandlingFee",
                      'value' => modApiFunc("Checkout", "getOrderPrice", "PerOrderHandlingFee")),
                array('type' => "TotalHandlingCharge",
                      'value' => modApiFunc("Checkout", "getOrderPrice", "TotalHandlingCharge")),
                // total shipping and handling
                array('type' => "TotalShippingAndHandlingCost",
                      'value' => $order_shipping_cost),
                // total tax
                array('type' => "OrderTaxTotal",
                      'value' => $order_order_tax_total_cost),
                array('type' => "SubtotalGlobalDiscount",
                      'value' => $subtotal_global_discount),
                array('type' => "SubtotalPromoCodeDiscount",
                      'value' => $subtotal_promo_code_discount),
                array('type' => "QuantityDiscount",
                      'value' => $quantity_discount),
                array('type' => "DiscountedSubtotal",
                      'value' => $order_discounted_subtotal_cost),
                array('type' => "OrderNotIncludedTaxTotal",
                      'value' => $order_not_included_tax_total)
                );

        $tax_array = modApiFunc("Taxes", "getTax");
        foreach ($tax_array['TaxSubtotalAmountView'] as $taxView)
        {
            $order_prices[] = array('type' => "Taxes['".mysql_real_escape_string($taxView['view'])."']",
                                    'value' => $taxView['value']);
        }
        modApiFunc("Checkout", "addOrderPrices", $oid, $order_prices);
        //add product info
        modApiFunc("Checkout", "addOrderProducts", $oid, modApiFunc("Cart", "getCartContent"));

        //If the customer is not registered
        if(true)
        {
            modApiFunc("Checkout", "addOrderPerson", $oid);
        }
        else
        {
        }

        modApiFunc("Checkout", "setLastPlacedOrderID", $oid);
        modApiFunc("Checkout", "saveState");

       /*
          Clear all encrypted blowfish-data       , and prerequisitesValidationResults as well. At the current moment
          it can be stored in the session and in the object this. The function, that deletes such data is performed
          in the session. It was used to delete data, when the blowfish key is lost. The key is passed in to the Checkout CZ
          as a GET/POST parameter.
          NOTE! The call "saveState" is required! As it is executed (2007 03 14) a string above, it is commented here.
        */
        //modApiFunc("Checkout", "saveState");
        Checkout::clear_all_checkout_cz_blowfish_encoded_data();
        modApiFunc("Checkout", "loadState");

        // throw event
        modApiFunc('EventsManager','throwEvent','OrderCreated',$oid);

        return $oid;
    }

    /*
     *                     ,                                     ,                .
     *            order_id.
     */
    function createOrderInDBAndUpdateEnvironment()
    {
        //Create order
        $order_id = modApiFunc("Checkout", "createOrderInDB");
        //             -                   
        modApiFunc("Catalog", "updateProductsQuantity", modApiFunc("Cart", "getCartContent"));
        //                .
        //                                
        modApiFunc("Cart", "removeAllFromCart");
        return $order_id;
    }

     function UpdatePaymentStatusInDB($order_id, $payment_status_id, $comment)
     {
        global $application;
        $statusChanged = array("payment_status" => array());

        $tables = $this->getTables();

        $o = $tables['orders']['columns'];
        $on = $tables['order_notes']['columns'];


        #  get current order info
        $order = $this->getOrderInfo($order_id);
        #     check if it is necessary to update status
        $update_payment_status_id = $payment_status_id != $order['PaymentStatusId'];

        if($update_payment_status_id)
        {
            $statusChanged["payment_status"][$order_id] = array("old_status" => $order['PaymentStatusId'], "new_status" => $payment_status_id);
        }

        $db_update = new DB_Update('orders');
        $db_update->addUpdateValue($o['payment_status_id'], $payment_status_id);
        $db_update->WhereValue($o['id'], DB_EQ, $order_id);
        $application->db->PrepareSQL($db_update);
        $application->db->DB_Exec();

        #    add comment, if it is specified
        if (!empty($comment))
        {
            $this->addOrderHistory($order_id, $comment);
        }

        modApiFunc('EventsManager','throwEvent','OrdersWereUpdated',$statusChanged);

        return $statusChanged;
     }

    /**
     * Returns a list of Checkout-related store blocks, which were viewed on
     * the privious View, and data which is expected to be in GET/POST.
     * Each block can appear by outputting some hidden field to the View.
     * For example, SubmitedCheckoutStoreBlocksList['shipping-info'].
     */
    function getLastViewSubmitedCheckoutStoreBlocksList()
    {
        global $application;
        $request = $application->getInstance('Request');
        if($request->getValueByKey('SubmitedCheckoutStoreBlocksList') != NULL)
        {
            return $request->getValueByKey('SubmitedCheckoutStoreBlocksList');
        }
        else
        {
            return array();
        }
    }

    /**
     * Returns prerequisite contents (with all field values) for the current
     * step.
     *
     * @param $step_id - a number of the current step.
     *
     * @author Alexandr Girin
     * @todo
     * @param
     * @return
     */
    function getPrerequisitesValidationResultsForCurrentStep($step_id)
    {
        $prerequisites = modApiFunc('Checkout', 'getPrerequisitesListForStep', $step_id);
        $PrerequisitesValidationResultsForCurrentStep = array();
        foreach($prerequisites as $prerequisite)
        {
            $PrerequisiteValidationResults = modApiFunc('Checkout', 'getPrerequisiteValidationResults', $prerequisite);
            $PrerequisitesValidationResultsForCurrentStep[$prerequisite] = $PrerequisiteValidationResults;
        }
        return $PrerequisitesValidationResultsForCurrentStep;
    }


    function redirectBackBecauseOfErrors()
    {
    }

    function getStepIDtoRedirectToAfterPrerequisitesValidationErrors($target_step_id)
    {
        $PrerequisiteErrorToStepIDTable = modApiFunc('Checkout', 'getPrerequisiteErrorToStepIDTable');
        $PrerequisitesValidationResults = $this->getPrerequisitesValidationResultsForCurrentStep($target_step_id);

        //                            .          ,                         ("isMet" = false)           
        //            ,                                      checkout                      .
        $min_step_id_to_redirect_to = $this->currentStepID;
        foreach($PrerequisitesValidationResults as $prerequisite_name => $validation_results_array)
        {
            //                            CheckoutFormEditor' ,                              .
            //         ,     PersonInfoTypeTag             PrerequisiteName.
            if($validation_results_array['isMet'] == false &&
               Checkout::arePersonInfoTypesActive(array($prerequisite_name)) === true)
            {
                if($PrerequisiteErrorToStepIDTable[$prerequisite_name] < $min_step_id_to_redirect_to)
                {
                    $min_step_id_to_redirect_to = $PrerequisiteErrorToStepIDTable[$prerequisite_name];
                }
            }
        }

        return $min_step_id_to_redirect_to;
    }

    /**
    * Common sub algo for two checkout actions:
    * setCurrentStepID() and
    * updateOrderStatusID()
    *
    * Perform checks: is it possible to set new checkout step id.
    * If possible (prerequisites are met) - set it.
    */
    function ProcessNewStepID($step_id)
    {
        if ($step_id != NULL)
        {
            //check if prerequisites are executed

            // get a list of prerequisites
            $prerequisites = modApiFunc('Checkout', 'getPrerequisitesListForStep', $step_id);

            //Overview the list of prerequisites and do the following for each of them:
            //    If it hasn't been executed yet, then see the compatability table
            //         "prerequisite => store block",
            //         check if data came from this store block.
            //             If it did, call validation function
            //             Otherwise, the prerequisite is not executed.

            foreach($prerequisites as $prerequisite)
            {
                $PrerequisitePreviousValidationResults = modApiFunc('Checkout', 'getPrerequisiteValidationResults', $prerequisite);
                //FIXME: Perhaps there is no need to double check data that just came
                //  (if, for example, this data once were checked successfully).

                //Get a name of store block matching the given prerequisite.
                $PrerequisiteStoreBlock = modApiFunc('Checkout', 'getPrerequisiteStoreBlock', $prerequisite);
                //check, if some GET/POST data came from this block, i.e. if it was outputted
                // on the previous View.
                $LastViewSubmitedCheckoutStoreBlocksList = $this->getLastViewSubmitedCheckoutStoreBlocksList();

                if(($PrerequisitePreviousValidationResults['isMet'] == true) && !isset($LastViewSubmitedCheckoutStoreBlocksList[$PrerequisiteStoreBlock]))
                {
                }
                else
                {
                    if(isset($LastViewSubmitedCheckoutStoreBlocksList[$PrerequisiteStoreBlock]))
                    {
                        //Validate data, that came from Store block, and if it is
                        // done successfully, specify the prerequisite as a executed
                        // condition.
                        ////$validationFunctionName = modApiFunc('Checkout', 'getPrerequisiteValidationFunction', $prerequisite);

                        /**
                         * Call the validation function.
                         * Validation results are saved in the session.
                         */

                        modApiFunc('Checkout', 'validateInputForPrerequisite', $prerequisite);
                    }
                    else
                    {
                        //Example: it specified in the configuration file that
                        // customer info was on the previous step, but in the POST data
                        // it doesn't exist. The case, when a block label exists
                        // but data doesn't (e.g. there are available shipping methods,
                        // but the customer didn't choose any), is described in
                        // validateInputForPrerequisite().

                        //The data didn't come (or Store block didn't specify, that its form data
                        // was filled out and sent). Prerequisite wasn't executed.
                        // Specify it as not executed. After checking other prerequisites
                        // redirect the customer to the previous (or any other,
                        // specified in the list of the pages, to be redirected if an error occurs)
                        // page.
                        //TODO output a detailed error message: POST/GET data of such-and-such
                        // store block.
                        //FIXME: use Checkout API function to get "visibleName" and set Error data.

                        $this->PrerequisitesValidationResults[$prerequisite]["isMet"] = false;

                        $this->PrerequisitesValidationResults[$prerequisite]["error_code"] = 'CHECKOUT_ERR_004_NO_POST_DATA_FOR_PREREQUISITE';
                        $this->PrerequisitesValidationResults[$prerequisite]["error_message_parameters"] = array($this->PrerequisitesValidationResults[$prerequisite]["visibleName"], $prerequisite);
                        //break;
                    }
                }
            }
            //End check if prerequisites are executed

            //Perform required additional actions
            //END Perform required additional actions

            //If not all prerequisites are met, redirect to the page,
            //specified as an error for this step.
            $all_prerequisites_are_met = true;
            $all_active_in_checkout_editor_prerequisites_are_met = true;
            foreach($prerequisites as $prerequisite)
            {
                $PrerequisiteValidationResults = modApiFunc('Checkout', 'getPrerequisiteValidationResults', $prerequisite);
                if($PrerequisiteValidationResults['isMet'] == false)
                {
                    $all_prerequisites_are_met = false;
                    //                               (turned off)   Checkout Form Editor' 
                    //                                                                   
                    //        . isMet                   false!
                    if(Checkout::arePersonInfoTypesActive(array($prerequisite)) === true)
                    {
                        //FIXME:                                               .
                        //                        ,    -   CreditCardInfo   BankAccountInfo      
                        //                             - UIN'                     .
                        //                                   ,                ,                  
                        //                     .
                        //                 if'                           :
                        //     _             = person_info_type_tag
                        $all_active_in_checkout_editor_prerequisites_are_met = false;
                    }
                }
            }

            if($all_active_in_checkout_editor_prerequisites_are_met === true)
            {
                //Define a number of the step, where can be the checkout process.
                $retval = modApiFunc('Checkout', 'setCurrentStepID', $step_id);
                //END Define a number of the step, where can be the checkout process.
            }
            else
            {
                //Redirect to the page, specified as an error for this step.

                //FIXME : fill the error array
                //Fill the error array. Only the block list, where an error occurred
                // and an error message, which is returned by the validation function.

                //$this->redirectBackBecauseOfErrors();
                $step_id_to_redirect_to = modApiFunc('Checkout', 'getStepIDtoRedirectToAfterPrerequisitesValidationErrors', $step_id);

                $retval = modApiFunc('Checkout', 'setCurrentStepID', $step_id_to_redirect_to);
            }
        }
    }

    function isShippingModulesListEmpty($groups = null)
    {
        /*
        global $application;

        $SelectedShippingModules = $this->getSelectedModules("shipping");

        $sm_list = Checkout::getInstalledAndActiveModulesListData("shipping", $groups);

        $items = array();

        foreach ($sm_list as $sm_item)
        {
            //FIXME create/use some mm function to convert class names.
            $name = strtolower($sm_item->name);

            $smInfo = modApiFunc($name, "getInfo");

            //FIXME : check if function exists
            $module_uid = $smInfo['GlobalUniqueShippingModuleID'];

            if (array_key_exists($module_uid, $SelectedShippingModules) == true)
            {
               /* Check, if for the current address even one method works.
                  Otherwise output a special template "No shipping method
                  available for the inputted address".
                  What should be outputted, if the address hasn't been inputted yet?
                */
                /*
                $methods_info_list = modApiFunc($smInfo['APIClassName'], "getShippingMethods", "AVAILABLE", true);
                if(!empty($methods_info_list))
                {
                    return false;
                }
            }
        }
        */

        $formatted_cart = modApiFunc("Shipping_Cost_Calculator","formatCart",modApiFunc("Cart","getCartContent"));
        modApiFunc("Shipping_Cost_Calculator","setShippingInfo",modApiFunc("Checkout", "getPrerequisiteValidationResults", "shippingInfo"));
        modApiFunc("Shipping_Cost_Calculator","setCart",$formatted_cart);
        $_modules_and_methods = modApiFunc("Shipping_Cost_Calculator","calculateShippingCost");

        if(!empty($_modules_and_methods) and !isset($_modules_and_methods[modApiFunc("Checkout","getAllInactiveModuleClassAPIName","shipping")]))
            return false;

        return true;
    }

    function isPaymentModulesListEmpty($groups = null)
    {
        global $application;

        $SelectedPaymentModules = $this->getSelectedModules("payment");

        $pm_list = Checkout::getInstalledAndActiveModulesListData("payment", $groups);

        $items = array();
        $new_selected_module_sort_order = 0;
        foreach ($pm_list as $pm_item)
        {
            //FIXME create/use some mm function to convert class names.
            $name = strtolower($pm_item->name);

            $pmInfo = modApiFunc($name, "getInfo");

            //FIXME : check if function exists
            $module_uid = $pmInfo['GlobalUniquePaymentModuleID'];
            if (array_key_exists($module_uid, $SelectedPaymentModules) == true)
            {
                return false;
            }
        }
        return true;
    }

    function getOrderProductInfo($order_product_id)
    {
        global $application;
        $tables = $this->getTables();
        $op_table = $tables['order_product']['columns'];

        $query = &new DB_Select();
        $query->addSelectTable('order_product');
        $query->addSelectField('*');
        $query->WhereValue($op_table['id'], DB_EQ, $order_product_id);

        $res = $application->db->getDB_Result($query);

        if(count($res)!=1)
            return null;
        else
            return array_shift($res);
    }

    function getOrderProductsIDs($order_id)
    {
        global $application;
        $tables = $this->getTables();
        $op_table = $tables['order_product']['columns'];

        $query = &new DB_Select();
        $query->addSelectField($op_table['id'],'id');
        $query->WhereValue($op_table['order_id'], DB_EQ, $order_id);

        $res = $application->db->getDB_Result($query);

        $opids = array();
        for($i=0;$i<count($res);$i++)
            $opids[] = $res[$i]['id'];

        return $opids;
    }

    function getNotNeedShippingModuleID()
    {
        return "5BED3FCA-82F4-4A5D-A578-D6022994BC2F";
    }

    function getNotNeedShippingMethodID()
    {
        return 1;
    }

    function getNotNeedShippingMethodName()
    {
        return "Shipping is not needed"; //TODO: move to resources?
    }

    //                                "OrderCreated"
    function getPaymentMethodText($pm_uid)
    {
        global $application;
        $pm_info = modApiFunc("Checkout", "getPaymentModuleInfo", $pm_uid);
        $mmObj = &$application->getInstance('Modules_Manager');
        /* Load required file */
        $mmObj->includeAPIFileOnce($pm_info['APIClassName']);
        /* This condition can be checked only after loading */
        if(is_callable(array($pm_info['APIClassName'],"getPaymentMethodText")))
        {
            $text = modApiFunc($pm_info['APIClassName'], "getPaymentMethodText");
        }
        else
        {
            $text = "";
        }
        return $text;
    }

    //                    Auto_Increment,                           ID
    //                  ,                   .
    function getNextOrderId()
    {
        global $application;

        $query = "SHOW TABLE STATUS LIKE '" . $application->db->table_prefix . "orders'";
        $result = $application->db->DB_Query($query);
        if(mysql_numrows($result) == 1)
        {
            $result = mysql_fetch_assoc($result);
            if(isset($result['Auto_increment']))
            {
                return $result['Auto_increment'];
            }
            else
            {
                //TODO: report error
                return false;
            }
        }
        else
        {
            //TODO: report error
            return false;
        }
    }

    //                       Auto_Increment      order_id          
    // orders.
    function setNextOrderId($new_id)
    {
        global $application;

        $query = "ALTER TABLE " . $application->db->table_prefix . "orders AUTO_INCREMENT = " . $new_id;
        $result = $application->db->DB_Query($query);
    }

    /**
     *                             .         , ShippingInfo['State'].                 ,      
     *                                               .         ,                                . .
     *          $options                                                        .         , "TEXT",    
     * "ID"          .
     */
    function getPrAttr($variant_tag, $attribute_tag, $options = array())
    {
    	$pvr = &$this->PrerequisitesValidationResults;
    	if(!isset($pvr[$variant_tag]))
    	{
    		return NULL;
    	}
    	else
    	{
    		if(!isset($pvr[$variant_tag]['validatedData'][$attribute_tag]))
    		{
    			return NULL;
    		}
    		else
    		{
    			return $pvr[$variant_tag]['validatedData'][$attribute_tag]['value'];
    		}
    	}
    }

    /**
     * The list of Checkout views.
     */
    var $ViewsList;

    /**
     * Settings from DB.
     */
    var $Settings = NULL;

    /**
     * Current Checkout Step ID
     */
    var $currentStepID = NULL;

    /**
     * Current Selected Order.
     */
    var $currentOrderID = NULL;

    /**
     * Last Placed Order
     */
    var $lastPlacedOrderID = NULL;

    /**
     * Current Selected Customer.
     */
    var $currentCustomerID = NULL;

    /**
     * List of action handlers.
     */
    var $ActionHandlersList;

    /**
     * A table of compatability of prerequisite and store block.
     */
    var $PrerequisiteToStoreBlockTable = array
        (
        //'customerInfo'     => 'customer-info-input',
        'shippingInfo' => 'shipping-info-input',
        'shippingModuleAndMethod'  => 'shipping-method-list-input',
        'billingInfo'  => 'billing-info-input',
        'paymentModule'   => 'payment-method-list-input'
        );

    /**
     * The table, that specifies the functions to check prerequisites.
     * Data form blocks is coming through GET or POST. Every block contains its
     * name. As mentioned above, each block matches the prerequisite. Define the
     * prerequisite by the block name and call matching it validation function.
     */
    /* prerequisite => validation-function */
/*    var $PrerequisitesValidationFunctionsTable = array
        (
        'customerInfo'     => 'validateInputForStep',
        'shippingInfo' => 'validateInputForStep',
        'shippingModuleAndMethod'  => 'validateInputForStep', //Calls the validation function matching a shipping module. It returns true/false and a formatted array with shipping data. See a long paper.
        'billingInfo'  => 'validateInputForStep',
        'paymentMethod'   => 'validateInputForStep'
        );
*/

    /**
     *                                                       prerequisite'  .                
     *             ,                        ,    'isMet' = false,       =true.
     */
    var $PrerequisitesValidationResults; /* = array
        (
        'customer-info'     => array('isMet' => false,
                                     'error_code' => 'CHECKOUT_ERR_004',
                                     'error_message_parameters' => array($this->PrerequisitesValidationResults[$prerequisite]["visibleName"], $prerequisite)
                                     'validatedData' => array( array(                 'view_tag'=>'',
                                                                                      'value'=>'',
                                                                                      'pattern_id' => '',
                                                                                      'error_code_short'    =>'',
                                                                                      'error_code_full'    =>'',
                                                                                       ... ),
                                                                                array('view_tag'=>'',
                                                                                      'value'=>'',
                                                                                      'pattern_id' => '',
                                                                                      'error_code_short'    =>'',
                                                                                      'error_code_full'    =>'',
                                                                                       ... )
                                                                              ),
        'shipping-info' => array('isMet' => false, 'validatedData' => array()),
        'shipping-method'  => array('isMet' => false, 'validatedData' => array()),
        'billing-info'  => array('isMet' => false, 'validatedData' => array()),
        'payment-method'   => array('isMet' => false, 'validatedData' => array())
        );
        */

    var $order_search_filter = array();

    var $customer_search_filter = array();

    /**
     * View, which is responsible for editing the current selected
     * Payment module.
     */
    var $CurrentPaymentModuleSettingsViewName = NULL;


    /**
     * Some HTML code, to be viewed, that came as answer form the payment
     * gateway.
     */
    var $CustomPaymentGatewayPageContents = NULL;

    /**
     * View, which is responsible for editting the current selected
     * Shipping module.
     */
    var $CurrentShippingModuleSettingsViewName = NULL;

    var $MessageResources;

    var $DeleteOrdersFlag;
    /**#@-*/

}
?>